package adrian.master.knowledgewar.server;

import java.io.IOException;
import java.io.InputStream;
import java.io.PrintWriter;
import java.sql.Connection;
import java.sql.SQLException;

import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NamingException;
import javax.servlet.ServletConfig;
import javax.servlet.ServletContext;
import javax.servlet.ServletException;
import javax.servlet.annotation.WebServlet;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.sql.DataSource;

import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;

import adrian.master.knowledgewar.server.constants.Constants;
import adrian.master.knowledgewar.server.constants.SQLStatements;
import adrian.master.knowledgewar.server.gameconcepts.BuildingFactory;
import adrian.master.knowledgewar.server.logic.AnswerHandler;
import adrian.master.knowledgewar.server.logic.DatabaseConnection;
import adrian.master.knowledgewar.server.logic.GameLogic;

/**
 * Servlet implementation class BattleServlet
 */
@WebServlet("/BattleServlet")
public class BattleServlet extends HttpServlet {
	private static final long serialVersionUID = 1L;
	DataSource datasource;

	/**
	 * @see HttpServlet#HttpServlet()
	 */
	public BattleServlet() {
		super();
	}

	public void init(ServletConfig config) throws ServletException {
		super.init(config);
		try {
			// Look up the JNDI data source only once at init time
			Context envCtx = (Context) new InitialContext().lookup("java:comp/env");
			datasource = (DataSource)envCtx.lookup("jdbc/knowledgewar");
		}
		catch (NamingException e) {
			e.printStackTrace();
		}
	}

	private Connection getConnection() throws SQLException {
		return datasource.getConnection();
	}

	/**
	 * @see HttpServlet#doGet(HttpServletRequest request, HttpServletResponse response)
	 */
	protected void doGet(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException {
		response.setCharacterEncoding("UTF-8");
		PrintWriter responseWriter = response.getWriter();
		responseWriter.print("BattleServlet fungerer");
		responseWriter.flush();
	}

	/**
	 * @see HttpServlet#doPost(HttpServletRequest request, HttpServletResponse response)
	 */
	protected void doPost(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException {
		Connection connection = null;
		String result = "";

		try {
			connection=getConnection();

			if(GameLogic.verifyLogin(request, connection)){
				String command = request.getParameter(Constants.COMMAND);

				if(command.equals(Constants.COMMAND_ANSWER)){
					result = handleAnswer(connection, request);
				}else if(command.equals(Constants.COMMAND_FINISH)){
					result = handlePlayerFinished(connection, request);
				}else if(command.equals(Constants.COMMAND_INFO)){
					result = getBattleInfo(connection, request);
				}else if(command.equals(Constants.COMMAND_GETQUESTIONS)){
					result = getBattleQuestions(connection, request);
				}else if(command.equals(Constants.COMMAND_ISFINISHED)){
					result = isBattleOver(connection, request);
				}
//				else if(command.equals(Constants.COMMAND_BATTLE_RESULT)){
//					result = getBattleResult(connection, request);
//				}

				response.setCharacterEncoding("UTF-8");
				PrintWriter responseWriter = response.getWriter();
				responseWriter.print(result);
				responseWriter.flush();
			}
		} catch (SQLException e) {
			// TODO Handle SQLException
			e.printStackTrace();
		} catch (JSONException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}

	private String handleAnswer(Connection con, HttpServletRequest request) throws SQLException, IOException{
		String account = request.getParameter(Constants.PARAMETER_ACCOUNT);
		String battleNr = request.getParameter(Constants.PARAMETER_BATTLENR);
		String newDebuff = request.getParameter(Constants.PARAMETER_ATTACK);

		String result = AnswerHandler.handleAnswer(con, account, battleNr, newDebuff);

		return result;
	}

	private String handlePlayerFinished(Connection con, HttpServletRequest request) throws SQLException, JSONException{
		String account = request.getParameter(Constants.PARAMETER_ACCOUNT);
		String battleNr = request.getParameter(Constants.PARAMETER_BATTLENR);
		String score = request.getParameter(Constants.PARAMETER_SCORE);
		String weaponcost = request.getParameter(Constants.PARAMETER_WEAPONCOST);
		
//		File tempDir = (File)getServletContext().getAttribute("javax.servlet.context.tempdir");
//		File newFile = new File(tempDir, "buildings.xml");
		InputStream catDoc = getServletContext().getResourceAsStream("/WEB-INF/buildings.xml");
		BuildingFactory.parseXML(catDoc);

		boolean battleOver = AnswerHandler.handleFinished(con, account, battleNr, score, weaponcost);
		GameLogic.markPlayerAsBusy(con, account, false);
		String result="";

		if(battleOver){
			result = Constants.SERVER_RESPONSE_BATTLEOVER;
		}else{
			result = Constants.SERVER_RESPONSE_WAITING;
		}
		return result;
	}

	private String getBattleInfo(Connection con, HttpServletRequest req) throws SQLException, JSONException{
		String battleNr = req.getParameter(Constants.PARAMETER_BATTLENR);
		String account = req.getParameter(Constants.PARAMETER_ACCOUNT);
		String categoryID = req.getParameter(Constants.PARAMETER_CATEGORY);
		JSONArray result = DatabaseConnection.getBattleInfo(con, battleNr);
		JSONObject opponent = DatabaseConnection.getOpponent(con, battleNr, account);
		JSONObject category = DatabaseConnection.getCategoryNameFromId(con, categoryID);
		result.put(opponent);
		result.put(category);
		System.out.println("BattleInfo: "+result.toString());
		return result.toString();
	}
	
	private String getBattleQuestions(Connection con, HttpServletRequest req) throws SQLException, JSONException{
		String battleNr = req.getParameter(Constants.PARAMETER_BATTLENR);
		JSONArray result = DatabaseConnection.getQuestionsForBattle(con, battleNr);
		return result.toString();
	}
	
	private String isBattleOver(Connection con, HttpServletRequest req) throws SQLException{
		String battleNr = req.getParameter(Constants.PARAMETER_BATTLENR);
		boolean battleOver = DatabaseConnection.isBattleFinished(con, battleNr);
		String result="";
		if(battleOver){
			result=Constants.COMMAND_BATTLEOVER;
		}
		return result;
	}
	
//	private String getBattleResult(Connection con, HttpServletRequest req) throws SQLException, JSONException{
//		String battleNr = req.getParameter(Constants.PARAMETER_BATTLENR);
//		String account = req.getParameter(Constants.PARAMETER_ACCOUNT);
//		JSONArray ar = DatabaseConnection.getBattleResult(con, battleNr);
//		
//		int ownScore=0;
//		int opponentScore=0;
//		
//		for(int i=0;i<2;i++){
//			JSONObject temp = ar.getJSONObject(i);
//			int tempInt =Integer.parseInt(temp.get(SQLStatements.BATTLES_IN_SCORE).toString());
//			if(temp.get(SQLStatements.BATTLES_IN_PLAYER).toString().equals(account)){
//				ownScore=tempInt;
//			}else{
//				opponentScore=tempInt;
//			}
//		}
//		
//		if(ownScore>opponentScore){
//			return "victory";
//		}else if(ownScore==opponentScore){
//			return "draw";
//		}
//		
//		return "loss";
//	}

}
