package adrian.master.knowledgewar.server;

import java.io.IOException;
import java.io.InputStream;
import java.io.PrintWriter;
import java.sql.Connection;
import java.sql.SQLException;

import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NamingException;
import javax.servlet.ServletConfig;
import javax.servlet.ServletException;
import javax.servlet.annotation.WebServlet;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.sql.DataSource;

import org.json.JSONArray;
import org.json.JSONException;

import adrian.master.knowledgewar.server.constants.Constants;
import adrian.master.knowledgewar.server.gameconcepts.BuildingFactory;
import adrian.master.knowledgewar.server.logic.DatabaseConnection;
import adrian.master.knowledgewar.server.logic.GCMMessenger;
import adrian.master.knowledgewar.server.logic.GameLogic;

/**
 * Servlet implementation class ChallengeServlet
 */
@WebServlet("/ChallengeServlet")
public class ChallengeServlet extends HttpServlet {
	private static final long serialVersionUID = 1L;
	DataSource datasource;

	/**
	 * @see HttpServlet#HttpServlet()
	 */
	public ChallengeServlet() {
		super();
	}

	public void init(ServletConfig config) throws ServletException {
		super.init(config);
		try {
			// Look up the JNDI data source only once at init time
			Context envCtx = (Context) new InitialContext().lookup("java:comp/env");
			datasource = (DataSource)envCtx.lookup("jdbc/knowledgewar");
		}
		catch (NamingException e) {
			e.printStackTrace();
		}
	}

	private Connection getConnection() throws SQLException {
		return datasource.getConnection();
	}

	/**
	 * @see HttpServlet#doGet(HttpServletRequest request, HttpServletResponse response)
	 */
	protected void doGet(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException {
	}

	/**
	 * @see HttpServlet#doPost(HttpServletRequest request, HttpServletResponse response)
	 */
	protected void doPost(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException {
		Connection connection=null;
		response.setCharacterEncoding("UTF-8");
		try {
			connection = getConnection();
			String result="";
			//Verify that the request came from someone with valid credentials
			if(GameLogic.verifyLogin(request, connection)){
				//Determine what the servlet is asked to do, and call correct method
				String command = request.getParameter(Constants.COMMAND);

				if(command.equals(Constants.COMMAND_CHALLENGE)){
					result = sendChallenge(connection, request);
				}else if(command.equals(Constants.COMMAND_ACCEPT)){
					challengeAccepted(connection, request);
				}else if(command.equals(Constants.COMMAND_DECLINE)){
					challengeDeclined(connection, request);
				}else if(command.equals(Constants.COMMAND_GETQUESTIONCATEGORIES)){
					result = getQuestionCategories(connection, request);
				}
//				System.out.println("CHallengeservletresponse: "+result);
				response.setCharacterEncoding("UTF-8");
				PrintWriter responseWriter = response.getWriter();
				responseWriter.print(result);
				responseWriter.flush();
			}
		}catch (SQLException e) {
			// TODO Handle SQLException
			e.printStackTrace();
		} catch (JSONException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}


	private String sendChallenge(Connection con, HttpServletRequest request) throws SQLException{
		String challengerName = request.getParameter(Constants.PARAMETER_ACCOUNT);
		String challengeeName = request.getParameter(Constants.PARAMETER_CHALLENGEE);
		String risk = request.getParameter(Constants.PARAMETER_RISK);
		String size = request.getParameter(Constants.PARAMETER_SIZE);
		String challengeeRegId = getRegId(con, challengeeName);
		String categoryId = request.getParameter(Constants.PARAMETER_CATEGORY);
		System.out.println("Challengeservlet categoryID: "+categoryId);
		if(!DatabaseConnection.isPlayerBusy(con, challengeeName)){
			String battleNr = GameLogic.createNewBattle(con, risk, size, challengerName, challengeeName,categoryId);
//			System.out.println("Challengeservlet: battle has been created");
			GCMMessenger.sendChallenge(challengeeRegId, challengeeName, challengerName, risk, battleNr, categoryId);
			return Constants.SERVER_RESPONSE_PLAYER_AVAILABLE;
		}
		return Constants.SERVER_RESPONSE_PLAYER_BUSY;
	}
	
	private String getQuestionCategories(Connection con, HttpServletRequest request) throws SQLException, JSONException{
		JSONArray ar = DatabaseConnection.getQuestionCategories(con);
		return ar.toString();
	}

	private void challengeAccepted(Connection con, HttpServletRequest request) throws SQLException, JSONException{
		String battleNr = request.getParameter(Constants.PARAMETER_BATTLENR);
		String challengee = request.getParameter(Constants.PARAMETER_CHALLENGEE);
		String challenger = request.getParameter(Constants.PARAMETER_CHALLENGER);
		
		String challengeeRegID = DatabaseConnection.getRegId(con, challengee);
		String challengerRegID = DatabaseConnection.getRegId(con, challenger);
		
//		System.out.println("Challenge: "+challengee+" Challenger: "+challenger+" Regid: "+challengeeRegID+" Regid2: "+challengerRegID);

		GameLogic.markPlayerAsBusy(con, challengee, true);
		GameLogic.markChallengeAsHandled(con, battleNr);
//		GameLogic.startBattleTimer(battleNr); //TODO Remove comment to implement overall timer for a single battle
		GCMMessenger.startRound(challengee,challengeeRegID, challenger, challengerRegID, battleNr);
		GameLogic.declineOtherUnstartedDuels(con, battleNr, challengee);
	}

	private void challengeDeclined(Connection con, HttpServletRequest request) throws SQLException, JSONException{
		
		InputStream catDoc = getServletContext().getResourceAsStream("/WEB-INF/buildings.xml");
		BuildingFactory.parseXML(catDoc);
		
		String battleNr = request.getParameter(Constants.PARAMETER_BATTLENR);
		String challengee = request.getParameter(Constants.PARAMETER_CHALLENGEE);
		String challenger = request.getParameter(Constants.PARAMETER_CHALLENGER);
		
		GameLogic.markChallengeAsHandled(con, battleNr);
		GameLogic.battleWasDeclined(con, challenger, challengee, battleNr);

	}

	private String getRegId(Connection con, String account) throws SQLException{
		String regId = DatabaseConnection.getRegId(con, account);
		return regId;
	}
	
	


}
