package adrian.master.knowledgewar.server;

import java.sql.Connection;
import java.sql.SQLException;

import javax.servlet.ServletContext;
import javax.servlet.ServletContextEvent;
import javax.servlet.ServletContextListener;

import adrian.master.knowledgewar.server.constants.Constants;
import adrian.master.knowledgewar.server.logic.DatabaseConnection;

public  class ContextConfiguration implements ServletContextListener {

	private static ContextConfiguration _instance;
	private static Thread mUpdater=null;
	private static boolean mRunning;

	private ServletContext context = null;

	//This method is invoked when the Web Application
	//is ready to service requests
	public void contextInitialized(ServletContextEvent event) {
		this.context = event.getServletContext();

		//initialize the static reference _instance
		_instance=this;
		
		if(mUpdater==null){
			mRunning=true;
			Updater run = new Updater();
			mUpdater = new Thread(run);
			mUpdater.start();
			
		}
	}

	/*This method is invoked when the Web Application has been removed 
	  and is no longer able to accept requests
	 */
	public void contextDestroyed(ServletContextEvent event) {
		this.context = null;
		mRunning = false;
		mUpdater.interrupt();
		mUpdater=null;

	}

	/* Provide a method to get the context values */
	public String getContextParameter(String key) {
		return this.context.getInitParameter(key);
	}

	public ServletContext getContext(){
		return context;
	}

	//now, provide an static method to allow access from anywere on the code:
	public static ContextConfiguration getInstance() {
		return _instance;
	}
	
	private class Updater implements Runnable{
		
		@Override
		public void run() {
			Thread thisThread = Thread.currentThread();
			System.out.println("Loginservlet: run() called.");
			while(mUpdater==thisThread && mRunning){			
				try {
					Connection con = DatabaseConnection.getConnection();
					DatabaseConnection.incrementHeartbeatCountAndLogout(con);
					System.out.println("LoginServlet: Hearbeatcount incremented, logging out players.");
					Thread.sleep(Constants.HEARTBEAT_PROCESS_SLEEP_LENGTH);
				} catch (SQLException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				} catch (InterruptedException e) {
					mRunning=false;
					e.printStackTrace();
				} catch (ClassNotFoundException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}
			}
			System.out.println("Loginservlet: run() ended. Thread stopped.");
			
		}
		
	}
}

