package adrian.master.knowledgewar.server;

import java.io.IOException;
import java.sql.Connection;
import java.sql.SQLException;

import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NamingException;
import javax.servlet.ServletConfig;
import javax.servlet.ServletException;
import javax.servlet.annotation.WebServlet;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.sql.DataSource;

import adrian.master.knowledgewar.server.constants.Constants;
import adrian.master.knowledgewar.server.logic.DatabaseConnection;
import adrian.master.knowledgewar.server.logic.GameLogic;

/**
 * Servlet implementation class GCMServlet
 */
@WebServlet("/GCMServlet")
public class GCMServlet extends HttpServlet {
	private static final long serialVersionUID = 1L;
	DataSource datasource;

	/**
	 * @see HttpServlet#HttpServlet()
	 */
	public GCMServlet() {
		super();
	}

	public void init(ServletConfig config) throws ServletException {
		super.init(config);
		try {
			// Look up the JNDI data source only once at init time
			Context envCtx = (Context) new InitialContext().lookup("java:comp/env");
			datasource = (DataSource)envCtx.lookup("jdbc/knowledgewar");
		}
		catch (NamingException e) {
			e.printStackTrace();
		}
	}

	private Connection getConnection() throws SQLException {
		return datasource.getConnection();
	}

	/**
	 * @see HttpServlet#doGet(HttpServletRequest request, HttpServletResponse response)
	 */
	protected void doGet(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException {
	}

	/**
	 * @see HttpServlet#doPost(HttpServletRequest request, HttpServletResponse response)
	 */
	protected void doPost(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException {
		Connection con = null;
		response.setCharacterEncoding("UTF-8");
		try {
			con = getConnection();

			if(GameLogic.verifyLogin(request, con)){
				String command = request.getParameter(Constants.COMMAND);

				if(command.equals(Constants.COMMAND_VERIFYGCM)){
					verifyGCM(con, request, response);
				}else if (command.equals(Constants.COMMAND_REGISTERGCM)){
					registerGCM(con, request, response);
				}
			}
		} catch (SQLException e) {
			// TODO Handle SQLException
			e.printStackTrace();
		}
	}

	private void verifyGCM(Connection con, HttpServletRequest request, HttpServletResponse response) throws SQLException{
		String accountname = request.getParameter(Constants.PARAMETER_ACCOUNT);
		String regId = request.getParameter(Constants.PARAMETER_REGID);
		DatabaseConnection.verifyRegId(con, accountname, regId);
	}

	private void registerGCM(Connection con, HttpServletRequest request, HttpServletResponse response) throws SQLException{
		String accountname = request.getParameter(Constants.PARAMETER_ACCOUNT);
		String regId = request.getParameter(Constants.PARAMETER_REGID);
		DatabaseConnection.registerRegId(con, accountname, regId);
	}

}
