package adrian.master.knowledgewar.server;

import java.io.IOException;
import java.io.InputStream;
import java.io.PrintWriter;
import java.sql.Connection;
import java.sql.SQLException;

import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NamingException;
import javax.servlet.ServletConfig;
import javax.servlet.ServletException;
import javax.servlet.annotation.WebServlet;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.sql.DataSource;

import adrian.master.knowledgewar.server.constants.Constants;
import adrian.master.knowledgewar.server.gameconcepts.BuildingFactory;
import adrian.master.knowledgewar.server.logic.DatabaseConnection;

/**
 * Servlet implementation class LoginServlet
 */
@WebServlet("/LoginServlet")
public class LoginServlet extends HttpServlet implements Runnable {
	private static final long serialVersionUID = 1L;
	private DataSource datasource;
	private volatile Thread mUpdater;
	private boolean mRunning;

	/**
	 * @see HttpServlet#HttpServlet()
	 */
	public LoginServlet() {
		super();
	}

	public void init(ServletConfig config) throws ServletException {
		super.init(config);
		try {
			// Look up the JNDI data source only once at init time
			Context envCtx = (Context) new InitialContext().lookup("java:comp/env");
			datasource = (DataSource)envCtx.lookup("jdbc/knowledgewar");
			//			mRunning = true;
			//			mUpdater = new Thread(this);
			//			mUpdater.setPriority(Thread.MIN_PRIORITY);
			//			mUpdater.start();
			InputStream catDoc = getServletContext().getResourceAsStream("/WEB-INF/buildings.xml");
			BuildingFactory.parseXML(catDoc);
		}
		catch (NamingException e) {
			e.printStackTrace();
		}
	}

	private Connection getConnection() throws SQLException {
		return datasource.getConnection();
	}

	/**
	 * @see HttpServlet#doGet(HttpServletRequest request, HttpServletResponse response)
	 */
	protected void doGet(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException {
		//TODO Remove testing-code
		PrintWriter responseWriter = response.getWriter();
		responseWriter.print("I appreciate that you tried checking an answer, but this question does not have an answer associated with it. Sorry about that.");
		responseWriter.flush();
	}

	/**
	 * @see HttpServlet#doPost(HttpServletRequest request, HttpServletResponse response)
	 */
	protected void doPost(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException {
		response.setCharacterEncoding("UTF-8");
		Connection connection=null;
		boolean success = false;
		String result="";
		PrintWriter responseWriter = response.getWriter();
		try {
			connection = getConnection();


			//Determine what the servlet is asked to do, and call correct method
			String command = request.getParameter(Constants.COMMAND);

			if(command.equals(Constants.COMMAND_REGISTER)){
				result = registerPlayer(connection, request, response);
				if(result.equals(Constants.SERVER_RESPONSE_REGISTRATION_SUCCESSFUL)){
					loginPlayer(connection, request, response);
				}
			}else if(command.equals(Constants.COMMAND_LOGIN)){
				result = loginPlayer(connection, request, response);
			}else if(command.equals(Constants.COMMAND_LOGOUT)){
				logoutPlayer(connection, request);
			}else if(command.equals(Constants.COMMAND_UPDATEPOSITION)){
				updatePosition(connection, request);
			}else{

			}

			//Respond to the client whether or not the database-interaction were successful

			responseWriter.print(result);
			responseWriter.flush();				
		} catch (SQLException e) {
			responseWriter.print("SQLException thrown at LoginServlet.");
			responseWriter.flush();	
			e.printStackTrace();
		}

	}

	private String registerPlayer(Connection con, HttpServletRequest request, HttpServletResponse response) throws IOException, SQLException{

		String account_name = request.getParameter(Constants.PARAMETER_ACCOUNT);
		String password = request.getParameter(Constants.PARAMETER_PASSWORD);

		boolean success = DatabaseConnection.registerPlayer(con, account_name, password);

		if(success){
			return Constants.SERVER_RESPONSE_REGISTRATION_SUCCESSFUL;
		}else{
			return Constants.SERVER_RESPONSE_REGISTRATION_FAILURE;
		}

	}

	private String loginPlayer(Connection con, HttpServletRequest request, HttpServletResponse response) throws IOException, SQLException{
		String account_name = request.getParameter(Constants.PARAMETER_ACCOUNT);
		String password = request.getParameter(Constants.PARAMETER_PASSWORD);

		boolean success = DatabaseConnection.loginUser(con, account_name, password);
		System.out.println("User logged in: "+account_name);
		if(success){
			return Constants.SERVER_RESPONSE_LOGIN_SUCCESSFUL;
		}else{
			return Constants.SERVER_RESPONSE_LOGIN_FAILURE;
		}

	}

	private void logoutPlayer(Connection con, HttpServletRequest request) throws SQLException{
		String account = request.getParameter(Constants.PARAMETER_ACCOUNT);
		System.out.println("User logged out: "+account);
		DatabaseConnection.logoutUser(con, account);
	}

	private void updatePosition(Connection con, HttpServletRequest request) throws SQLException{
		String latitude = request.getParameter(Constants.PARAMETER_LATITUDE);
		String longitude = request.getParameter(Constants.PARAMETER_LONGITUDE);
		String account = request.getParameter(Constants.PARAMETER_ACCOUNT);

		DatabaseConnection.updatePlayerPosition(con, account, longitude, latitude);

	}

	@Override
	public void run() {
		Thread thisThread = Thread.currentThread();
		System.out.println("Loginservlet: run() called.");
		while(mUpdater==thisThread && mRunning){			
			try {
				Connection con = getConnection();
				DatabaseConnection.incrementHeartbeatCountAndLogout(con);
				System.out.println("LoginServlet: Hearbeatcount incremented, logging out players.");
				Thread.sleep(Constants.HEARTBEAT_PROCESS_SLEEP_LENGTH);
			} catch (SQLException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			} catch (InterruptedException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		}
		System.out.println("Loginservlet: run() ended. Thread stopped.");

	}

	@Override
	public void destroy() {
		mRunning=false;
		mUpdater=null;
		super.destroy();
	}

}
