package adrian.master.knowledgewar.server;

import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.PrintWriter;
import java.io.Reader;
import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.SQLException;

import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NamingException;
import javax.servlet.ServletConfig;
import javax.servlet.ServletException;
import javax.servlet.annotation.WebServlet;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.sql.DataSource;

import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;

import adrian.master.knowledgewar.server.constants.Constants;
import adrian.master.knowledgewar.server.gameconcepts.BuildingFactory;
import adrian.master.knowledgewar.server.logic.DatabaseConnection;
import adrian.master.knowledgewar.server.logic.GameLogic;
import adrian.master.knowledgewar.server.logic.ResultSetConverter;


/**
 * Servlet implementation class PlayerServlet
 */
@WebServlet("/PlayerServlet")
public class PlayerServlet extends HttpServlet {
	private static final long serialVersionUID = 1L;
	private DataSource datasource;

	/**
	 * @see HttpServlet#HttpServlet()
	 */
	public PlayerServlet() {
		super();
	}

	public void init(ServletConfig config) throws ServletException {
		super.init(config);
		try {
			// Look up the JNDI data source only once at init time
			Context envCtx = (Context) new InitialContext().lookup("java:comp/env");
			datasource = (DataSource)envCtx.lookup("jdbc/knowledgewar");
		}
		catch (NamingException e) {
			e.printStackTrace();
		}
	}

	private Connection getConnection() throws SQLException {
		return datasource.getConnection();
	}

	/**
	 * @see HttpServlet#doGet(HttpServletRequest request, HttpServletResponse response)
	 */
	protected void doGet(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException {
	}

	/**
	 * @see HttpServlet#doPost(HttpServletRequest request, HttpServletResponse response)
	 */
	protected void doPost(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException {

		Connection connection=null;
		response.setCharacterEncoding("UTF-8");
		PrintWriter responseWriter = response.getWriter();
		try {
			connection = getConnection();

			if(GameLogic.verifyLogin(request, connection)){
				
//				System.out.println(RequestPrinter.debugString(request));

				//Determine what the servlet is asked to do, and call correct method
				String command = request.getParameter(Constants.COMMAND);
				String res = "";
				if(command.equals(Constants.COMMAND_GETPLAYERS)){
					String account = request.getParameter(Constants.PARAMETER_ACCOUNT);
					JSONArray result = getPlayers(connection, account);
					res = result.toString();
				}else if(command.equals(Constants.COMMAND_GET_OWN_STATS)){
					String account = request.getParameter(Constants.PARAMETER_ACCOUNT);
					res = getStats(connection, account);
				}else if(command.equals(Constants.COMMAND_UPDATESTATS)){
					res = setStats(connection, request);
				}else if(command.equals(Constants.COMMAND_PLACEBUILDING)){
					res = placeBuilding(connection, request);
				}else if(command.equals(Constants.COMMAND_UPGRADEBUILDING)){
					res = upgradeBuilding(connection, request);
				}else if(command.equals(Constants.COMMAND_REMOVEBUILDING)){
					res = removeBuilding(connection, request);
				}

				if(res!=null){
					responseWriter.print(res);
					responseWriter.flush();				
				}else{
					System.out.println("Dette var en failure");
					responseWriter.print("test");
					responseWriter.flush();
				}
			}

		}catch (SQLException e) {
			responseWriter.print("SQLException thrown at PlayerServlet.");
			responseWriter.flush();	
			e.printStackTrace();
		} catch (JSONException e) {
			responseWriter.print("JSONException thrown at PlayerServlet.");
			responseWriter.flush();
			e.printStackTrace();
		}

	}

	private JSONArray getPlayers(Connection con, String account) throws SQLException, JSONException{

		JSONArray result = DatabaseConnection.getClosePlayers(con, account);
		//		JSONArray result = DatabaseConnection.getPlayers(con, account);

		return result;
	}

	private String getStats(Connection con, String account) throws SQLException, JSONException{
		JSONObject result = DatabaseConnection.getPlayerStats(con, account);
		return result.toString();
	}

	private String setStats(Connection con, HttpServletRequest req) throws SQLException{
		String account = req.getParameter(Constants.PARAMETER_ACCOUNT);
		String gold = req.getParameter(Constants.PARAMETER_GOLD);
		String area = req.getParameter(Constants.PARAMETER_AREA);
		DatabaseConnection.updatePlayerStats(con, account, gold, area);

		return Constants.SERVER_RESPONSE_SUCCESS;
	}

	private String placeBuilding(Connection con, HttpServletRequest req) throws SQLException, JSONException{

		String account = req.getParameter(Constants.PARAMETER_ACCOUNT);
		String building = req.getParameter(Constants.PARAMETER_BUILDINGTYPE);
		String x = req.getParameter(Constants.PARAMETER_BUILDINGX);
		String y = req.getParameter(Constants.PARAMETER_BUILDINGY);

		boolean success = GameLogic.placeNewBuilding(con, account, building, x, y);

		if(success){
			return Constants.SERVER_RESPONSE_SUCCESS;			
		}
		return Constants.SERVER_RESPONSE_FAILURE;
	}

	private String upgradeBuilding(Connection con, HttpServletRequest req) throws JSONException, SQLException{
		String account = req.getParameter(Constants.PARAMETER_ACCOUNT);
		String building = req.getParameter(Constants.PARAMETER_BUILDINGTYPE);
		String buildingRank = req.getParameter(Constants.PARAMETER_BUILDINGRANK);
		String x = req.getParameter(Constants.PARAMETER_BUILDINGX);
		String y = req.getParameter(Constants.PARAMETER_BUILDINGY);

		boolean success = GameLogic.upgradeBuilding(con, account, building, buildingRank, x, y);

		if(success){
			return Constants.SERVER_RESPONSE_SUCCESS;
		}
		return Constants.SERVER_RESPONSE_FAILURE;
	}

	private String removeBuilding(Connection con, HttpServletRequest req) throws SQLException, JSONException{
		String account = req.getParameter(Constants.PARAMETER_ACCOUNT);
		String x = req.getParameter(Constants.PARAMETER_BUILDINGX);
		String y = req.getParameter(Constants.PARAMETER_BUILDINGY);

		boolean success = GameLogic.removeBuilding(con, account, x, y);

		if(success){
			return Constants.SERVER_RESPONSE_SUCCESS;
		}
		return Constants.SERVER_RESPONSE_FAILURE;
	}

}
