package adrian.master.knowledgewar.server.gameconcepts;

import adrian.master.knowledgewar.server.constants.Constants;

public class Battle {
	
	private Player attacker;
	private Player defender;
	private String risk;
	private int battleSize, numberOfTiles;
	private int winnerProfits, loserLoss;
	
	//Variables used in calculating the result of the battle. Stands for BattleAreaX and BattleAreaY 
	private int[] attackerXRange = new int[2], attackerYRange = new int[2], 
			defenderXRange = new int[2], defenderYRange = new int[2];
	
	public Battle(Player att, Player def, String risk, int size){
		attacker = att;
		defender = def;
		battleSize = size;
		numberOfTiles = calculateAreaRanges(risk);
	}
	
	public int getWinnerProfits(){
		return winnerProfits;
	}
	
	public int getLoserLoss(){
		return loserLoss;
	}
	
	//Risk in the database is represented as a String in this format "x1-x2,y1-y2;x1-x2,y1-y2" where the numbers before the 
	//semicolon represent the area of the attacker that is involved in the combat, while the numbers after the semicolons represent
	//the area of the defender. This method simply parses this String and places the values in 4 different arrays
	//so that the numbers are easy to use later in the process.
	private int calculateAreaRanges(String risk){
		risk = risk.substring(risk.indexOf(';')+1);
		String attackerString = risk.substring(0, risk.indexOf(';'));
		String defenderString = risk.substring(risk.indexOf(';')+1, risk.length());
		
		//Calculate attackers area
		String xValues = attackerString.substring(0, attackerString.indexOf(','));
		String yValues = attackerString.substring(attackerString.indexOf(',')+1, attackerString.length());
		attackerXRange[0]=Integer.parseInt(xValues.substring(0, xValues.indexOf('-')));
		attackerXRange[1]=Integer.parseInt(xValues.substring(xValues.indexOf('-')+1, xValues.length()));	
		attackerYRange[0]=Integer.parseInt(yValues.substring(0, yValues.indexOf('-')));
		attackerYRange[1]=Integer.parseInt(yValues.substring(yValues.indexOf('-')+1, yValues.length()));
		attacker.xRange=attackerXRange;
		attacker.yRange=attackerYRange;
		
		//Calculate defenders area
		xValues = defenderString.substring(0, defenderString.indexOf(','));
		yValues = defenderString.substring(defenderString.indexOf(',')+1, defenderString.length());
		defenderXRange[0]=Integer.parseInt(xValues.substring(0, xValues.indexOf('-')));
		defenderXRange[1]=Integer.parseInt(xValues.substring(xValues.indexOf('-')+1, xValues.length()));	
		defenderYRange[0]=Integer.parseInt(yValues.substring(0, yValues.indexOf('-')));
		defenderYRange[1]=Integer.parseInt(yValues.substring(yValues.indexOf('-')+1, yValues.length()));
		defender.xRange=defenderXRange;
		defender.yRange=defenderYRange;
		
		int nrTiles = (defenderXRange[1]-defenderXRange[0]+1)*(defenderYRange[1]-defenderYRange[0]+1);
//		System.out.println("Number of tiles in battle: "+nrTiles);
		return nrTiles;
	}
	
	public boolean calculateWinner(){
		boolean draw = false;
		if(attacker.getScore()>defender.getScore()){ //Attacker wins
			attacker.winner=true;
			calculateBattleResults(attacker, defender);
		}else if(attacker.getScore()==defender.getScore()){ //Draw
			calculateDraw();
			draw = true;
		}else{ //Defender wins
			defender.winner=true;
			calculateBattleResults(defender, attacker);
		}
		
		return draw;
	}
	
	public void battleDeclined(){
		attacker.calculateBonuses();
		defender.calculateBonuses();
		float[] battleValues = calculateBattleValue();
		winnerProfits = (int)(Constants.BATTLE_DECLINED_PROFITS_MODIFIER*calculateWinnerProfits(battleValues, 0, attacker, defender));
		loserLoss = (int)(Constants.BATTLE_DECLINED_PROFITS_MODIFIER*calculateLoserLoss(battleValues, attacker, defender));
		
		attacker.modifyGold(winnerProfits);
		defender.modifyGold(loserLoss);
	}
	
	
	private void calculateBattleResults(Player winner, Player loser){
		//TODO Might be necessary to reorder these. Should loser's buildings be destroyed before bonuses are calculated?
		winner.calculateBonuses();
		loser.calculateBonuses();
		
		int value = destroyBuildings(winner.getDamageBonus(), loser);
		float[] battleValues = calculateBattleValue();
		System.out.println("\t Value of tiles: "+battleValues[0]+". Battlesizebonus: "+battleValues[1]);
		winnerProfits = calculateWinnerProfits(battleValues, value, winner, loser);
		loserLoss = calculateLoserLoss(battleValues, winner, loser);
		
		winner.modifyGold(winnerProfits-winner.getWeaponCost());
		winner.goldResult=winnerProfits;
		loser.modifyGold(loserLoss-loser.getWeaponCost());
		loser.goldResult=loserLoss;
	}
	
	private void calculateDraw(){
		float[] battleValues = calculateBattleValue();
		
		winnerProfits = (int)(battleValues[0]*battleValues[1]*Constants.BATTLE_DRAW_PROFITS_PENALTY);
		attacker.modifyGold(winnerProfits - attacker.getWeaponCost());
		defender.modifyGold(winnerProfits - defender.getWeaponCost());
		attacker.goldResult = winnerProfits;
		defender.goldResult = winnerProfits;
	}
	
	
	
	private int destroyBuildings(int opponentDamageBonus, Player player){
		int totalDamage = opponentDamageBonus+battleSize;
		System.out.println("\t Loser is "+player.getName()+" and received "+totalDamage+" damage.");
		int result = player.destroyBuildings(totalDamage);
		System.out.println("\t Total value of destroyed buildings: "+result);
		return result;
	}
	
	private float[] calculateBattleValue(){
		float battleValue = numberOfTiles * Constants.TILE_VALUE;
		float battleSizeBonus = (battleSize/10f)+1;
		float[] result = {battleValue, battleSizeBonus};
		return result;
	}
	
	private int calculateWinnerProfits(float[] battleValues, int buildingValues, Player winner, Player loser){
		float profit = (battleValues[0]+ buildingValues) * battleValues[1];
		float modifier = 1+(winner.getGoldBonus() - loser.getGoldPenalty());
		int total = (int)(profit*modifier);
		return total;
	}
	
	private int calculateLoserLoss(float[] battleValues,  Player winner, Player loser){
		float loss = (battleValues[0]*battleValues[1]);
		float modifier = 1+(winner.getGoldPenalty());
		int total = -(int)(loss*modifier);
		return total;
	}

}
