package adrian.master.knowledgewar.server.gameconcepts;

import java.io.IOException;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.Iterator;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;
import org.xml.sax.SAXException;

public class BuildingFactory {
	
	private static ArrayList<Building> buildingTemplates = new ArrayList<Building>();
	
	public static void parseXML(InputStream catDoc){
		DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
		Document dom;
		try {
			
			DocumentBuilder db = dbf.newDocumentBuilder();
			dom = db.parse(catDoc, null);
			
			Element doc = dom.getDocumentElement();
			NodeList nl = doc.getElementsByTagName("Building");
			
			if(nl!=null){
				for(int i=0;i<nl.getLength();i++){
					Element el = (Element)nl.item(i);
					constructBuildingTemplate(el);
				}
			}
		} catch (ParserConfigurationException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (SAXException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}
	
	private static void constructBuildingTemplate(Element el){
		String type = getNodeValue(el, "Type");
		String name = getNodeValue(el, "Name");
		NodeList nl = el.getElementsByTagName("Rank");
		if(nl != null){
			for(int i=0;i<nl.getLength();i++){
				Element ele = (Element)nl.item(i);
				int number = Integer.parseInt(getNodeValue(ele, "Number"));
				int value = Integer.parseInt(getNodeValue(ele, "Value"));
				int health = Integer.parseInt(getNodeValue(ele, "Health"));
				boolean global = getNodeValue(ele, "Global").equals("true");
				float goldBonus = Float.parseFloat(getNodeValue(ele, "GoldBonus"));
				int damageBonus = Integer.parseInt(getNodeValue(ele, "DamageBonus"));
				int healthBonus = Integer.parseInt(getNodeValue(ele, "HealthBonus"));
				float goldPenalty = Float.parseFloat(getNodeValue(ele, "GoldPenalty"));
				int upgradeCost = Integer.parseInt(getNodeValue(ele, "UpgradeCost"));
				Building b = new Building(type, name, number, value, health, global, 
						goldBonus, damageBonus, healthBonus, goldPenalty, upgradeCost);
				buildingTemplates.add(b);
			}
		}
	}
	
	private static String getNodeValue(Element el, String tagName){
		String value="";
		
		NodeList nl = el.getElementsByTagName(tagName);
		if(nl!=null){
			Element ele = (Element)nl.item(0);
			value = ele.getFirstChild().getNodeValue();
		}
		
		return value;
	}
	
	private static Building getNewBuildingFromTemplate(int x, int y, String type, int rank){
		Iterator<Building> it = buildingTemplates.iterator();
		Building result=null;
		while(it.hasNext()){
			Building b = it.next();
			if(b.getType().equals(type) && b.getRank()==rank){
				result = Building.getNewBuilding(b);
				result.setX(x);
				result.setY(y);
				break;
			}
		}
		return result;
	}
	
	public static Building constructBuilding(String input){
		Building result = parseInputString(input);
		return result;
	}
	
	public static Building constructNewBuilding(String type){
		Building result = getNewBuildingFromTemplate(0, 0, type, 1);
		return result;
	}
	
	public static Building getBuildingofSpecificTypeAndRank(String type, int rank, int x, int y){
		Building result = getNewBuildingFromTemplate(x,y,type, rank);
		return result;
	}
	
	private static Building parseInputString(String input){
		String[] substrings = new String[3];
		
		for(int i=0; i<substrings.length;i++ ){
			int index = input.indexOf(':');
			if(index!=-1){
				substrings[i] = input.substring(0, index);				
			}else{
				substrings[i] = input;
			}
			input = input.substring(index+1);
		}
		
		String type = substrings[0];
		int rank = Integer.parseInt(substrings[1]);
		
		int indexOfCoordinateSeparator = substrings[2].indexOf(',');
		int x=Integer.parseInt(substrings[2].substring(0,indexOfCoordinateSeparator));
		int y=Integer.parseInt(substrings[2].substring(indexOfCoordinateSeparator+1));
		
		return getNewBuildingFromTemplate(x, y, type, rank);
	}
	
//	private static void giveBuildingBonuses(Building building){
//		String type = building.getType();
//		
//		double ownGoldModifier=1;
//		double opponentGoldModifier=1;
//		int buildingHealthBonus=0;
//		int damageBonus=0;
//		int buildingHealth=5;
//		int buildingValue=5;
//		boolean global=false;
//		
//		switch(type){
//		case "A": 
//			ownGoldModifier=1.5;
//			buildingHealth=1;
//			global=true;break;
//		case "B":
//			opponentGoldModifier=0.15;break;
//		case "C":
//			buildingHealthBonus=1;
//			buildingHealth=4;break;
//		case "D":
//			damageBonus=1;
//			global=true;break;
//			
//		}
//		building.setBonuses(buildingHealth, buildingHealthBonus, damageBonus, ownGoldModifier, opponentGoldModifier, buildingValue, global);
//	}


}
