package adrian.master.knowledgewar.server.gameconcepts;

import java.util.ArrayList;
import java.util.Iterator;

import org.json.JSONException;
import org.json.JSONObject;

import adrian.master.knowledgewar.server.constants.Constants;
import adrian.master.knowledgewar.server.constants.SQLStatements;

public class Player {

	private int x,y;
	private String account;
	private int gold;
	private int xp;
	private int score;
	private ArrayList<Building> buildings;
	private String mGcm;
	private int weaponCost;

	private float totalGoldModifier=0;
	private float totalExpModifier=0;
	private float totalOpponentGoldModifier=0;
	private float totalOpponentExpModifier=0;
	private int totalDamageBonus=0;
	private int totalHealthBonus=0;

	public int[] xRange, yRange;
	
	public int goldResult =0;
	public boolean winner=false;

	public Player(String acc, String areaBuildings, String sc, String gold, String gcm, String weaponcost){
		this.buildings = new ArrayList<Building>();
		this.account = acc;
		this.score = Integer.parseInt(sc);
		this.gold = Integer.parseInt(gold);
		this.weaponCost = Integer.parseInt(weaponcost);
//		this.xp = Integer.parseInt(xp);
		this.mGcm = gcm;

		parseAreaBuildings(areaBuildings);
	}
	
	public Player(String acc, String areaBuildings, String sc, String gold, String gcm){
		this.buildings = new ArrayList<Building>();
		this.account = acc;
		this.score = Integer.parseInt(sc);
		this.gold = Integer.parseInt(gold);
//		this.xp = Integer.parseInt(xp);
		this.mGcm = gcm;

		parseAreaBuildings(areaBuildings);
	}
	
	public Player(String acc, String areaBuildings, String gold){
		this.buildings = new ArrayList<Building>();
		this.account = acc;
		this.gold = Integer.parseInt(gold);

		parseAreaBuildings(areaBuildings);
	}

	private void parseAreaBuildings(String areaBuildings){
		
		ArrayList<String> strings = new ArrayList<String>();
		String area2 = areaBuildings;
		int index=0;
		do{
			index=area2.indexOf(';');
			String temp="";
			if(index!=-1){
				temp = area2.substring(0, index);
				area2 = area2.substring(index+1);
			}else{
				temp = area2;
			}
			strings.add(temp);
		}while(index!=-1);
		
		
		int indexOfSeparator = strings.get(0).indexOf('x');
		x=Integer.parseInt(strings.get(0).substring(0, indexOfSeparator));
		y=Integer.parseInt(strings.get(0).substring(indexOfSeparator+1));

		
		for(int i=1; i<strings.size();i++){
			
			Building temp = BuildingFactory.constructBuilding(strings.get(i));
			buildings.add(temp);
		}
		
//		System.out.println(createAreaString());
	}

	public void calculateBonuses(){
		for(Building b : buildings){
			if(b!=null && (b.isGlobal() || ((b.getX()>=xRange[0] && b.getX()<=xRange[1]) && (b.getY()>=yRange[0] && b.getY()<=yRange[1])))){
				totalGoldModifier+=b.getGoldBonus();
				totalDamageBonus+=b.getDamageBonus();
				totalHealthBonus+=b.getHealthBonus();
				totalOpponentGoldModifier+=b.getGoldPenalty();
			}
		}
	}

	public int destroyBuildings(int damage){
		Iterator<Building> itr = buildings.iterator();
		int totalValue=0;
		int nrBuildings=0;

		while(itr.hasNext()){
			Building b = itr.next();
			if(((b.getX()>=xRange[0] && b.getX()<=xRange[1]) && (b.getY()>=yRange[0] && b.getY()<=yRange[1]))){
				int health = b.getHealth()+totalHealthBonus;
				System.out.println("\t Destroybuildings: Damage= "+damage+", building health= "+health);
				if(health<=damage){
					totalValue+=b.getValue();
					nrBuildings++;
					itr.remove();
				}
			}
		}
		System.out.println("\t Number of buildings destroyed: "+nrBuildings);
		return (int)(Constants.BATTLE_BUILDING_VALUE_MODIFIER*totalValue);

	}
	
	public void modifyGold(int income){
		gold+=income;
		
		if(gold<0){
			gold=0;
		}
	}
	
	public int getWeaponCost(){
		return weaponCost;
	}


	public float getGoldBonus(){
		return totalGoldModifier;
	}

	public float getGoldPenalty(){
		return totalOpponentGoldModifier;
	}

	public float getExpPenalty(){
		return totalOpponentExpModifier;
	}

	public int getBuildingHealthBonus(){
		return totalHealthBonus;
	}

	public int getDamageBonus(){
		return totalDamageBonus;
	}

	public int getScore(){
		return score;
	}

	public int getX(){
		return x;
	}

	public int getY(){
		return y;
	}
	
	public String getName(){
		return account;
	}
	
	public String getGCM(){
		return mGcm;
	}

	public JSONObject getPlayerInfo() throws JSONException{
		JSONObject result = new JSONObject();
		result.put(SQLStatements.PLAYER_KEY, this.account);
		result.put(SQLStatements.PLAYER_GOLD, this.gold);
//		result.put(SQLStatements.PLAYER_XP, this.xp);
		result.put(SQLStatements.PLAYER_AREA, createAreaString());

		return result;
	}

	public int getNewAreaLength(){
		//TODO temporary method that always returns 5
		return 5;
	}
	
	public boolean addNewBuilding(Building building, int px, int py){
		boolean isTileOccupied = false;
		if(gold>=building.getValue()){
			for(Building b: buildings){
				if(b.getX()==px && b.getY()==py){
					isTileOccupied=true;
				}
			}

			if(!isTileOccupied && 
					(px==0 && py==0 || px==4 && py==4 || px==4 && py==0 || px==0 && py==4 )){
				building.setX(px);
				building.setY(py);
				payGold(building.getValue());
				buildings.add(building);
				return true;			
			}			
		}

		return false;
	}
	
	public boolean upgradeBuilding(Building oldBuilding, Building newBuilding){
		
		if(oldBuilding != null && hasEnoughGold(oldBuilding.getUpgradeCost())){
			Iterator<Building> iter = buildings.iterator();		
			while(iter.hasNext()){
				Building b = iter.next();
				if(b.getX()==oldBuilding.getX() && b.getY()==oldBuilding.getY() && b.getType().equals(oldBuilding.getType())){
					iter.remove();
					buildings.add(newBuilding);
					payGold(oldBuilding.getUpgradeCost());
					return true;
				}
			}			
		}

		return false;
	}
	
	public boolean removeBuilding(int x, int y){
		
		if(x!=2 && y!=2){
			Iterator<Building> iter = buildings.iterator();
			
			while(iter.hasNext()){
				Building b = iter.next();
				if(b.getX()==x && b.getY()==y){
					iter.remove();
					return true;
				}
			}

		}

		return false;
	}
	
	public void payGold(int gold){
		this.gold-=gold;
	}
	
	private boolean hasEnoughGold(int gold){
		return (this.gold-gold)>=0;
	}

	private String createAreaString(){
		String area="";

		area+=x+"x"+y;

		for(Building b : buildings){
			if(b!=null)
			area+=b.toString();
		}

		return area;
	}

	@Override
	public String toString(){
		return "Account: "+account+", x: "+x+", y: "+y+", Number of buildings: "+buildings.size();
	}

}
