package adrian.master.knowledgewar.server.logic;

import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;

import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;

import adrian.master.knowledgewar.server.constants.Constants;
import adrian.master.knowledgewar.server.constants.SQLStatements;


public class DatabaseConnection {

	public static Connection getConnection() throws ClassNotFoundException, SQLException{
		Class.forName("com.mysql.jdbc.Driver");
		Connection connect = DriverManager
				.getConnection("jdbc:mysql://localhost:3306/knowledgewar?"
						+ "user=db_user&password=test");
		return connect;
	}

	public static boolean registerPlayer(Connection con, String account_name, String password) throws SQLException{

		boolean accountAvailable = isAccountnameAvailable(con, account_name);

		if(accountAvailable){
			//con.setAutoCommit(false);
			PreparedStatement register = con.prepareStatement(SQLStatements.REGISTER_STATEMENT);
			register.setString(1, account_name);
			register.setString(2, password);
			register.setString(3, Constants.PLAYER_STARTING_AREA);
			register.setInt(4, Constants.PLAYER_STARTING_GOLD);

			register.executeUpdate();

			return true;
			//			con.commit();
		}
		return false;

	}

	public static boolean verifyLogin(Connection con, String account_name, String password) throws SQLException{
		PreparedStatement verifyLogin = con.prepareStatement(SQLStatements.VERIFY_LOGIN_STATEMENT);
		verifyLogin.setString(1, account_name);
		verifyLogin.setString(2, password);

		ResultSet res = verifyLogin.executeQuery();

		//The condition (res.isBeforeFirst()) evaluates to true if the resultset contains a row. 
		//If that is the case, the account/password-combination exists, and the user can log in.
		if (res.isBeforeFirst() ) {
			return true;
		}

		return false;
	}

	public static boolean loginUser(Connection con, String account, String password) throws SQLException{
		PreparedStatement verifyLogin = con.prepareStatement(SQLStatements.VERIFY_LOGIN_STATEMENT);
		verifyLogin.setString(1, account);
		verifyLogin.setString(2, password);

		ResultSet res = verifyLogin.executeQuery();

		//The condition (res.isBeforeFirst()) evaluates to true if the resultset contains a row. 
		//If that is the case, the account/password-combination exists, and the user can log in.
		if (res.isBeforeFirst() ) {
			PreparedStatement login = con.prepareStatement(SQLStatements.LOGIN_STATEMENT);
			login.setString(1, account);
			login.executeUpdate();
			return true;
		}

		return false;
	}

	private static boolean isAccountnameAvailable(Connection con, String account_name) throws SQLException{

		PreparedStatement checkAccountName = con.prepareStatement(SQLStatements.CHECK_ACCOUNTNAMEAVAILABILITY_STATEMENT);
		checkAccountName.setString(1, account_name);

		ResultSet res = checkAccountName.executeQuery();

		//The condition (!res.isBeforeFirst()) evaluates to true if the resultset is empty. 
		//If that is the case, the accountname is available.
		if (!res.isBeforeFirst() ) {    
			return true;
		}


		return false;
	}

	public static void logoutUser(Connection con, String account) throws SQLException{

		PreparedStatement logout = con.prepareStatement(SQLStatements.LOGOUT_STATEMENT);
		logout.setString(1, account);
		logout.executeUpdate();

	}

	public static JSONObject getPlayerStats(Connection con, String account) throws SQLException, JSONException{
		PreparedStatement playerArea = con.prepareStatement(SQLStatements.GET_PLAYER_STATS);
		playerArea.setString(1, account);

		ResultSet rs = playerArea.executeQuery();
		JSONArray ar = ResultSetConverter.convert(rs);
		return ar.getJSONObject(0);
	}

	public static JSONObject getPlayerInfo(Connection con, String account) throws SQLException, JSONException{
		PreparedStatement player = con.prepareStatement(SQLStatements.GET_PLAYER_INFO);
		player.setString(1, account);

		ResultSet rs = player.executeQuery();
		JSONArray ar = ResultSetConverter.convert(rs);
		return ar.getJSONObject(0);
	}

	public static void updatePlayerPosition(Connection con, String account, String longitude, String latitude) throws SQLException{
		PreparedStatement position = con.prepareStatement(SQLStatements.UPDATE_PLAYER_POSITION);
		double longt = Double.parseDouble(longitude);
		double lat = Double.parseDouble(latitude);

		position.setDouble(1, lat);
		position.setDouble(2, longt);
		position.setString(3, account);
		System.out.println(position.toString());

		position.executeUpdate();
	}

	public static void updatePlayerStats(Connection con, String account, String gold, String area) throws SQLException{
		PreparedStatement stats = con.prepareStatement(SQLStatements.UPDATE_PLAYER_STATS);

		stats.setString(1, gold);
		stats.setString(2, area);
		stats.setString(3, account);

		stats.executeUpdate();
	}

	public static void verifyRegId(Connection con, String accountname, String regId) throws SQLException{

		PreparedStatement checkRegId = con.prepareStatement(SQLStatements.VERIFY_GCM_STATEMENT);
		checkRegId.setString(1, accountname);

		ResultSet res = checkRegId.executeQuery();

		res.next();
		String previousId = res.getString(SQLStatements.PLAYER_GCM);
		System.out.println("GCM. Previous regid= "+previousId);
		System.out.println("GCM. New regid= "+regId);

		if(previousId != null && !previousId.equals(regId)){
			PreparedStatement updateRegId = con.prepareStatement(SQLStatements.UPDATE_GCM_STATEMENT);
			updateRegId.setString(1, regId);
			updateRegId.setString(2, accountname);
			updateRegId.executeUpdate();
		}

	}

	public static boolean registerRegId(Connection con, String accountname, String regId) throws SQLException{

		PreparedStatement updateRegId = con.prepareStatement(SQLStatements.UPDATE_GCM_STATEMENT);
		updateRegId.setString(1, regId);
		updateRegId.setString(2, accountname);
		updateRegId.executeUpdate();
		return true;

	}

	public static JSONArray getPlayers(Connection con, String account) throws SQLException, JSONException{
		PreparedStatement getPlayersStatement = con.prepareStatement(SQLStatements.GET_PLAYERS_STATEMENT);
		getPlayersStatement.setString(1, account);
		ResultSet result = getPlayersStatement.executeQuery();
		JSONArray ar = ResultSetConverter.convert(result);
		//		System.out.println(result.toString());
		return ar;
	}
	
	public static JSONArray getClosePlayers(Connection con, String account) throws SQLException, JSONException{
		PreparedStatement latlong = con.prepareStatement(SQLStatements.GET_PLAYER_LATITUDE_LONGITUDE);
		latlong.setString(1, account);
		ResultSet rs = latlong.executeQuery();
		rs.next();
		double longitude = rs.getDouble(SQLStatements.PLAYER_LONGITUDE);
		double latitude = rs.getDouble(SQLStatements.PLAYER_LATITUDE);
		
		PreparedStatement stmnt = con.prepareStatement(SQLStatements.GET_CLOSE_PLAYERS_STATEMENT);
		stmnt.setString(1, account);
		stmnt.setDouble(2, latitude);
		stmnt.setDouble(3, latitude);
		stmnt.setDouble(4, longitude);
		stmnt.setDouble(5, longitude);
		rs = stmnt.executeQuery();
		JSONArray ar = ResultSetConverter.convert(rs);
		return ar;
	}

	public static String getRegId(Connection con, String accountname) throws SQLException{
		PreparedStatement getRegId;
		getRegId = con.prepareStatement(SQLStatements.GET_PLAYER_REGID);
		getRegId.setString(1, accountname);
		ResultSet result = getRegId.executeQuery();
		result.next();
		return result.getString(SQLStatements.PLAYER_GCM);
	}

	public static JSONArray getMultipleRegIds(Connection con, JSONArray players) throws SQLException, JSONException{
		String sql = SQLStatements.GET_PLAYER_REGID;
		if(players!=null && players.length()!=0){
			for(int i=1; i<players.length();i++){
				sql+=SQLStatements.GET_MULTIPLE_PLAYER_REGID_ADDITION;
			}
			PreparedStatement getPlayers = con.prepareStatement(sql);
			for(int i=0;i<players.length();i++){
				getPlayers.setString(i+1, players.getString(i));
			}


			ResultSet rs = getPlayers.executeQuery();
			JSONArray ar = ResultSetConverter.convert(rs);

			return ar;
		}
		return null;
	}
	
	public static JSONArray getRegIDsforClosePlayers(Connection con, String account) throws SQLException, JSONException{
		PreparedStatement latlong = con.prepareStatement(SQLStatements.GET_PLAYER_LATITUDE_LONGITUDE);
		latlong.setString(1, account);
		ResultSet rs = latlong.executeQuery();
		rs.next();
		double longitude = rs.getDouble(SQLStatements.PLAYER_LONGITUDE);
		double latitude = rs.getDouble(SQLStatements.PLAYER_LATITUDE);
		
		PreparedStatement stmnt = con.prepareStatement(SQLStatements.GET_CLOSE_PLAYERS_REGIDS);
		stmnt.setDouble(1, latitude);
		stmnt.setDouble(2, latitude);
		stmnt.setDouble(3, longitude);
		stmnt.setDouble(4, longitude);
		rs = stmnt.executeQuery();
		JSONArray ar = ResultSetConverter.convert(rs);
		return ar;
		
	}

	public static String getNewBattle(Connection con, String risk, String size) throws SQLException{

		//TODO Implement marking new battle as "unstarted"
		PreparedStatement newBattle, getLastInsertId;

		newBattle=con.prepareStatement(SQLStatements.CREATE_BATTLE_STATEMENT);
		newBattle.setString(1, risk);
		newBattle.setString(2, size);
		newBattle.executeUpdate();

		getLastInsertId = con.prepareStatement(SQLStatements.GET_LAST_INSERT_ID);
		ResultSet result = getLastInsertId.executeQuery();
		result.next();
		String res = result.getString(SQLStatements.LAST_INSERT_ID_FIELD);
		return res;
	}

	public static void challengeIsHandled(Connection con, String battleNr) throws SQLException{
		PreparedStatement startBattle = con.prepareStatement(SQLStatements.START_BATTLE_STATEMENT);

		startBattle.setString(1, battleNr);

		startBattle.executeUpdate();
	}

	public static boolean hasBattleStarted(Connection con, String battleNr) throws SQLException{
		PreparedStatement hasStarted = con.prepareStatement(SQLStatements.HAS_BATTLE_STARTED);
		hasStarted.setString(1, battleNr);
		ResultSet rs = hasStarted.executeQuery();
		rs.next();
		int result = rs.getInt(SQLStatements.BATTLE_STARTED);
		if(result==1){
			return true;
		}
		return false;
	}

	public static String hasBattleFinished(Connection con, String battleNr) throws SQLException{
		PreparedStatement hasFinished = con.prepareStatement(SQLStatements.IS_A_PLAYER_NOT_FINISHED);
		hasFinished.setString(1, battleNr);
		ResultSet rs=hasFinished.executeQuery();

		if (!rs.isBeforeFirst() ) {    
			return null;
		}else{
			rs.next();
			String remainingPlayer = rs.getString(SQLStatements.PLAYER_KEY);
			return remainingPlayer;
		}
	}

	public static void associateBattleWithParticipants(Connection con, String challenger, String challengee, String battleNr) throws SQLException{
		PreparedStatement associatePlayers = con.prepareStatement(SQLStatements.ASSOCIATE_PLAYERS_WITH_BATTLE_STATEMENT);

		associatePlayers.setString(1, challenger);
		associatePlayers.setString(2, battleNr);
		associatePlayers.setInt(3, 1);
		associatePlayers.setString(4, challengee);
		associatePlayers.setString(5, battleNr);
		associatePlayers.setInt(6, 0);

		associatePlayers.executeUpdate();
	}

	public static JSONArray getBattleInfo(Connection con, String battleNr) throws SQLException, JSONException{
		JSONArray result = new JSONArray();

		PreparedStatement battleInfo = con.prepareStatement(SQLStatements.GET_BATTLE_INFO);
		battleInfo.setString(1, battleNr);
		ResultSet rs = battleInfo.executeQuery();
		rs.next();
		result.put(rs.getString(SQLStatements.BATTLE_KEY));
		result.put(rs.getString(SQLStatements.BATTLE_RISK));
		result.put(rs.getString(SQLStatements.BATTLE_SIZE));

		return result;
	}
	
	public static void setBattleFinished(Connection con, String battleNr) throws SQLException{
		PreparedStatement finished = con.prepareStatement(SQLStatements.SET_BATTLE_FINISHED);
		finished.setString(1, battleNr);
		finished.executeUpdate();
	}
	
	public static JSONArray getPlayersInUnstartedDuels(Connection con, String player) throws SQLException, JSONException{
		PreparedStatement otherPlayers = con.prepareStatement(SQLStatements.GET_OTHER_PLAYERS_IN_UNSTARTED_DUELS);
		otherPlayers.setString(1, player);
		otherPlayers.setString(2, player);
		
		ResultSet rs = otherPlayers.executeQuery();
		JSONArray ar = ResultSetConverter.convert(rs);
		
		return ar;
		
	}
	
	public static void setUnstartedDuelsAsFinished(Connection con, String player) throws SQLException{
		PreparedStatement unstarted = con.prepareStatement(SQLStatements.MARK_UNSTARTED_DUELS_AS_FINISHED);
		unstarted.setString(1, player);
		unstarted.executeUpdate();
	}

	public static JSONObject getOpponent(Connection con, String battleNr, String account) throws SQLException, JSONException{
		JSONObject player = new JSONObject();

		PreparedStatement opponent = con.prepareStatement(SQLStatements.GET_OPPONENT_IN_BATTLE);
		opponent.setString(1, battleNr);
		opponent.setString(2, account);
		ResultSet rs = opponent.executeQuery();
		rs.next();
		player.put(SQLStatements.PLAYER_KEY, rs.getString(SQLStatements.PLAYER_KEY));
		player.put(SQLStatements.PLAYER_AREA, rs.getString(SQLStatements.PLAYER_AREA));
		return player;
	}
	

	public static JSONArray getBattleParticipants(Connection con, String battleNr) throws SQLException{
		JSONArray result = new JSONArray();

		PreparedStatement participants = con.prepareStatement(SQLStatements.GET_PLAYERS_IN_BATTLE);
		participants.setString(1, battleNr);
		ResultSet rs = participants.executeQuery();
		while(rs.next()){
			result.put(rs.getString(SQLStatements.PLAYER_GCM));
		}
		return result;
	}

	public static String insertGetAttack(Connection con, String account, String battleNr, String attack) throws SQLException{
		String result ="";

		PreparedStatement insertAttack = con.prepareStatement(SQLStatements.GET_ATTACKS_STATEMENT);
		insertAttack.setString(1, battleNr);
		insertAttack.setString(2, account);

		return result;
	}

	public static String getDebuffs(Connection con, String battleNr, String account) throws SQLException{
		PreparedStatement getDebuffs = con.prepareStatement(SQLStatements.GET_DEBUFFS_STATEMENT);
		getDebuffs.setString(1, account);
		getDebuffs.setString(2, battleNr);
		ResultSet rs = getDebuffs.executeQuery();
		rs.next();
		return rs.getString(SQLStatements.BATTLES_IN_DEBUFFS);
	}

	public static void setDebuffs(Connection con, String battleNr, String account, String debuffs) throws SQLException{
		PreparedStatement setDebuffs = con.prepareStatement(SQLStatements.SET_DEBUFFS_STATEMENT);
		setDebuffs.setString(1, debuffs);
		setDebuffs.setString(2, account);
		setDebuffs.setString(3, battleNr);
		setDebuffs.executeUpdate();
	}

	public static String getAttacks(Connection con, String battleNr, String account) throws SQLException{
		PreparedStatement getAttacks = con.prepareStatement(SQLStatements.GET_ATTACKS_STATEMENT);
		getAttacks.setString(1, account);
		getAttacks.setString(2, battleNr);
		ResultSet rs = getAttacks.executeQuery();
		rs.next();
		return rs.getString(SQLStatements.BATTLES_IN_DEBUFFS);
	}

	public static void setAttacks(Connection con, String battleNr, String account, String attacks) throws SQLException{
		PreparedStatement setAttacks = con.prepareStatement(SQLStatements.SET_ATTACKS_STATEMENT);
		setAttacks.setString(1, attacks);
		setAttacks.setString(2, account);
		setAttacks.setString(3, battleNr);
		setAttacks.executeUpdate();
	}

	public static void playerFinished(Connection con, String account, String battleNr, String score, String weaponcost) throws SQLException{
		PreparedStatement playerFinished = con.prepareStatement(SQLStatements.PLAYER_FINISHED_STATEMENT);
		playerFinished.setString(1, score);
		playerFinished.setString(2, weaponcost);
		playerFinished.setString(3, account);
		playerFinished.setString(4, battleNr);
		playerFinished.executeUpdate();
	}

	public static boolean isOtherPlayerFinished(Connection con, String account, String battleNr) throws SQLException{
		PreparedStatement isOtherPlayerFinished = con.prepareStatement(SQLStatements.IS_PLAYER_FINISHED_QUERY);
		isOtherPlayerFinished.setString(1, battleNr);
		isOtherPlayerFinished.setString(2, account);
		ResultSet rs = isOtherPlayerFinished.executeQuery();
		rs.next();

		if(rs.getString(SQLStatements.BATTLES_IN_FINISHED).equals("1")){
			return true;
		}

		return false;
	}

	public static int getNumberOfQuestionInDatabase(Connection con) throws SQLException{
		PreparedStatement numberOfQuestion = con.prepareStatement(SQLStatements.GET_NUMBER_OF_QUESTIONS);
		ResultSet rs = numberOfQuestion.executeQuery();
		rs.next();
		return rs.getInt(SQLStatements.COUNT_FIELD);
	}
	
	public static ArrayList<Integer> getAllQuestionsInCategory(Connection con, String categoryID) throws SQLException{
		ArrayList<Integer> ar = new ArrayList<Integer>();
		PreparedStatement questions;
		if(Integer.parseInt(categoryID)==0){
			questions = con.prepareStatement(SQLStatements.GET_ALL_QUESTIONS);
		}else{
			questions = con.prepareStatement(SQLStatements.GET_ALL_QUESTIONS_IN_CATEGROY);
			questions.setString(1, categoryID);
		}
		ResultSet rs = questions.executeQuery();
		while(rs.next()){
			ar.add(rs.getInt(SQLStatements.QUESTION_KEY));
		}
		return ar;			
	}
	
	public static JSONArray getQuestionCategories(Connection con) throws SQLException, JSONException{
		PreparedStatement categories = con.prepareStatement(SQLStatements.GET_CATEGORIES);
		ResultSet rs = categories.executeQuery();
		JSONArray ar = ResultSetConverter.convert(rs);
		JSONArray ar2 = new JSONArray();
		JSONObject obj = new JSONObject();
		obj.put("id", 0);
		obj.put("name", "All");
		ar2.put(obj);
		for(int i=0;i<ar.length();i++){
			ar2.put(ar.getJSONObject(i));
		}
		System.out.println("Databaseconnection getcategories: "+ar.toString());
		return ar2;
	}
	
	public static JSONObject getCategoryNameFromId(Connection con, String categoryID) throws SQLException, JSONException{
		JSONObject result = new JSONObject();
		if(Integer.parseInt(categoryID)!=0){
			PreparedStatement category = con.prepareStatement(SQLStatements.GET_CATEGORY_FROM_ID);
			category.setString(1, categoryID);
			ResultSet rs = category.executeQuery();
			rs.next();
			result.put(Constants.PARAMETER_CATEGORY, rs.getString(SQLStatements.QUESTION_CATEGORY_NAME));			
		}else{
			result.put(Constants.PARAMETER_CATEGORY, "All");
		}
//		JSONArray ar = ResultSetConverter.convert(rs);
		return result;
	}

	public static void associateQuestionsWithBattle(Connection con, int[] questions, String battleNr) throws SQLException{
		String insertQuestionValues = SQLStatements.INSERT_QUESTION_BATTLE_VALUES;
		for(int i=1; i<questions.length;i++){
			insertQuestionValues+=SQLStatements.INSERT_ADDITIONAL_QUESTION_BATTLE_VALUES;
		}

		PreparedStatement insertQuestions = con.prepareStatement(insertQuestionValues);
		for(int i=0;i<questions.length;i++){
			insertQuestions.setString(2*i+1, battleNr);
			insertQuestions.setString(2*i+2, ""+questions[i]);
		}

		insertQuestions.executeUpdate();

	}

	public static JSONArray getQuestionsForBattle(Connection con, String battleNr) throws SQLException, JSONException{
		PreparedStatement getQuestions = con.prepareStatement(SQLStatements.GET_QUESTIONS_FOR_BATTLE);
		getQuestions.setString(1, battleNr);
		ResultSet rs = getQuestions.executeQuery();
		JSONArray ar = ResultSetConverter.convert(rs);
		return ar;
	}

	public static boolean isBattleFinished(Connection con, String battleNr) throws SQLException{
		PreparedStatement getPlayersFinished = con.prepareStatement(SQLStatements.IS_PLAYERS_FINISHED);
		getPlayersFinished.setString(1, battleNr);
		ResultSet rs = getPlayersFinished.executeQuery();
		rs.next();
		int temp = rs.getInt(SQLStatements.COUNT_FIELD);
		if(temp==2){
			return true;
		}
		return false;
	}

	public static JSONArray getBattleResult(Connection con, String battleNr) throws SQLException, JSONException{
		PreparedStatement battleResult = con.prepareStatement(SQLStatements.GET_BATTLE_RESULT);
		battleResult.setString(1, battleNr);
		ResultSet rs = battleResult.executeQuery();
		JSONArray ar = ResultSetConverter.convert(rs);
		return ar;
	}

	public static JSONArray getBattleRiskAndSize(Connection con, String battleNr) throws SQLException, JSONException{
		PreparedStatement battleRisk = con.prepareStatement(SQLStatements.GET_BATTLE_RISK_AND_SIZE);
		battleRisk.setString(1, battleNr);
		ResultSet rs = battleRisk.executeQuery();
		JSONArray ar = ResultSetConverter.convert(rs);
		return ar;
	}

	public static JSONArray getPlayerInfoForBattleResult(Connection con, String battleNr) throws SQLException, JSONException{
		PreparedStatement playerInfo = con.prepareStatement(SQLStatements.GET_PLAYER_INFO_FOR_BATTLE);
		playerInfo.setString(1, battleNr);
		ResultSet rs = playerInfo.executeQuery();
		JSONArray ar = ResultSetConverter.convert(rs);
		return ar;
	}

	public static void setUpdatedPlayerInfo(Connection con, JSONObject player) throws JSONException, SQLException{
		String account = player.getString(SQLStatements.PLAYER_KEY);
		String area = player.getString(SQLStatements.PLAYER_AREA);
		String gold = player.getString(SQLStatements.PLAYER_GOLD);

		PreparedStatement playerInfo = con.prepareStatement(SQLStatements.SET_PLAYER_INFO_AFTER_BATTLE);
		playerInfo.setString(1, gold);
		playerInfo.setString(2, area);
		playerInfo.setString(3, account);

		playerInfo.executeUpdate();
	}

	public static void setPlayerBusy(Connection con, String account, boolean isBusy) throws SQLException{
		PreparedStatement playerBusy = con.prepareStatement(SQLStatements.SET_PLAYER_BUSY);

		playerBusy.setBoolean(1, isBusy);
		playerBusy.setString(2, account);

		playerBusy.executeUpdate();
	}
	
	public static boolean isPlayerBusy(Connection con, String account) throws SQLException{
		PreparedStatement playerBusy = con.prepareStatement(SQLStatements.GET_PLAYER_BUSY);
		playerBusy.setString(1, account);
		
		ResultSet rs = playerBusy.executeQuery();
		rs.next();
		boolean result = rs.getBoolean(SQLStatements.PLAYER_BUSY);
		return result;
	}
	
	public static void incrementHeartbeatCountAndLogout(Connection con) throws SQLException{
		PreparedStatement incrementHeartbeat = con.prepareStatement(SQLStatements.INCREMENT_PLAYER_HEARTBEAT);
		incrementHeartbeat.executeUpdate();
		
		PreparedStatement logout = con.prepareStatement(SQLStatements.CHECK_HEARTBEAT_AND_LOGOUT);
		logout.executeUpdate();
	}
}
