package adrian.master.knowledgewar.server.logic;

import java.io.IOException;
import java.util.LinkedList;
import java.util.List;

import org.json.JSONArray;
import org.json.JSONException;

import adrian.master.knowledgewar.server.constants.Constants;
import adrian.master.knowledgewar.server.constants.SQLStatements;

import com.google.android.gcm.server.Message;
import com.google.android.gcm.server.Sender;

public class GCMMessenger {
	
	public static void sendMessage(List<String> ids, Message message){
		try {
			Sender sender = new Sender(Constants.API_KEY);
			
			sender.send(message, ids , 2);
		} catch (IOException e) {
			// TODO Handle IOException
			e.printStackTrace();
		}
	}
	
	public static void sendChallenge(String challengeeRegId, String challengeeAccount, String challengerAccount, 
			String risk, String battleNr, String categoryID){
//		System.out.println("Challenger: "+challengerAccount+" challenge: "+" Risk: "+risk+" Battlenr"+battleNr);
		Message message = new Message.Builder()
		.addData(Constants.COMMAND, Constants.COMMAND_CHALLENGE)
		.addData(Constants.PARAMETER_CHALLENGER, challengerAccount)
		.addData(Constants.PARAMETER_ACCOUNT, challengeeAccount)
		.addData(Constants.PARAMETER_BATTLENR, battleNr)
		.addData(Constants.PARAMETER_RISK, risk)
		.addData(Constants.PARAMETER_CATEGORY, categoryID)
		.build();
		
		List<String> devices = new LinkedList<String>();
		devices.add(challengeeRegId);
//		System.out.println("GCM Info: "+challengerAccount+challengeeAccount+battleNr+risk);
		System.out.println("Sending GCM message: "+Constants.COMMAND_CHALLENGE+", "+challengeeAccount);
		sendMessage(devices, message);
	}
	
	public static void startRound(String challengee, String challengeeRegID, String challenger, String challengerRegID, String battleNr){
		Message message1 = new Message.Builder()
		.addData(Constants.COMMAND, Constants.COMMAND_STARTROUND)
		.addData(Constants.PARAMETER_BATTLENR, battleNr)
		.addData(Constants.PARAMETER_ACCOUNT, challengee)
		.build();
		System.out.println("Challengee: "+challengee+" Regid: "+challengeeRegID);
		
		List<String> devices1 = new LinkedList<String>();
		devices1.add(challengeeRegID);
		sendMessage(devices1, message1);
		
		Message message2 = new Message.Builder()
		.addData(Constants.COMMAND, Constants.COMMAND_STARTROUND)
		.addData(Constants.PARAMETER_BATTLENR, battleNr)
		.addData(Constants.PARAMETER_ACCOUNT, challenger)
		.build();
		System.out.println("Challenger: "+challenger+" regid: "+challengerRegID);
		
		List<String> devices2 = new LinkedList<String>();
		devices2.add(challengerRegID);
		sendMessage(devices2, message2);
		
	}
	
	public static void challengeDeclined(String challengerId, String challengerName, String profits, 
			String challengeeId, String challengeeName, String loss, String battleNr){
		Message toWinner = new Message.Builder()
		.addData(Constants.COMMAND, Constants.COMMAND_CHALLENGE_DECLINED)
		.addData(Constants.PARAMETER_CHALLENGEE, challengeeName)
		.addData(Constants.PARAMETER_ACCOUNT, challengerName)
		.addData(Constants.PARAMETER_RESULT, profits)
		.addData(Constants.PARAMETER_BATTLENR, battleNr)
		.build();
		
		Message toLoser = new Message.Builder()
		.addData(Constants.COMMAND, Constants.COMMAND_YOU_DECLINED_CHALLENGE)
		.addData(Constants.PARAMETER_CHALLENGER, challengerName)
		.addData(Constants.PARAMETER_ACCOUNT, challengeeName)
		.addData(Constants.PARAMETER_RESULT, loss)
		.addData(Constants.PARAMETER_BATTLENR, battleNr)
		.build();
		
		List<String> devices1 = new LinkedList<String>();
		devices1.add(challengerId);
		System.out.println("Sending GCM message: "+Constants.COMMAND_CHALLENGE_DECLINED+", "+challengerName);
		sendMessage(devices1, toWinner);
		
		List<String> devices2 = new LinkedList<String>();
		devices2.add(challengeeId);
		System.out.println("Sending GCM message: "+Constants.COMMAND_YOU_DECLINED_CHALLENGE+", "+challengeeName);
		sendMessage(devices2, toLoser);
	}
	
	public static void battleOver(String playerAccount, String playerId, String playerGoldProfit,int score,int opponentScore, String winner, String battleNr){
//		String winnerString;
//		if(winner){
//			winnerString=Constants.PARAMETER_WINNER;
//		}else{
//			winnerString=Constants.PARAMETER_LOSER;
//		}
		
		Message message = new Message.Builder()
		.addData(Constants.COMMAND, Constants.COMMAND_FINISH)
		.addData(Constants.PARAMETER_ACCOUNT, playerAccount)
		.addData(Constants.PARAMETER_BATTLENR, battleNr)
		.addData(Constants.PARAMETER_GOLD, playerGoldProfit)
		.addData(Constants.PARAMETER_RESULT, winner)
		.addData(Constants.PARAMETER_SCORE, score+"")
		.addData(Constants.PARAMETER_OPPONENTSCORE, opponentScore+"")
		.build();
		
		List<String> devices = new LinkedList<String>();
		devices.add(playerId);
		
		sendMessage(devices, message);
	}
	
	public static void sendChatMessage(String message, JSONArray regIds) throws JSONException{
		for(int i=0;i<regIds.length();i++){
			Message mes = new Message.Builder()
			.addData(Constants.COMMAND, Constants.COMMAND_CHAT)
			.addData(Constants.PARAMETER_MESSAGE, message)
			.addData(Constants.PARAMETER_ACCOUNT, regIds.getJSONObject(i).getString(SQLStatements.PLAYER_KEY))
			.build();
			
			List<String> devices = new LinkedList<String>();
			devices.add(regIds.getJSONObject(i).getString(SQLStatements.PLAYER_GCM));
			sendMessage(devices, mes);
		}
	}
	
	public static void declineOtherDuels(JSONArray players) throws JSONException{
		Message mes = new Message.Builder()
		.addData(Constants.COMMAND, Constants.COMMAND_CHALLENGE_IMPOSSIBLE)
		.build();
		
		List<String> devices = new LinkedList<String>();
		for(int i=0;i<players.length();i++){
			devices.add(players.getJSONObject(i).getString(SQLStatements.PLAYER_GCM));
		}
		
		sendMessage(devices, mes);
	}

}
