package adrian.master.knowledgewar.server.logic;

import java.io.InputStream;
import java.sql.Connection;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.Random;

import javax.servlet.http.HttpServletRequest;

import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;

import adrian.master.knowledgewar.server.ContextConfiguration;
import adrian.master.knowledgewar.server.constants.Constants;
import adrian.master.knowledgewar.server.constants.SQLStatements;
import adrian.master.knowledgewar.server.gameconcepts.Battle;
import adrian.master.knowledgewar.server.gameconcepts.Building;
import adrian.master.knowledgewar.server.gameconcepts.BuildingFactory;
import adrian.master.knowledgewar.server.gameconcepts.Player;

public class GameLogic {
	
	private static Random mRand;
	
	public static boolean verifyLogin(HttpServletRequest request, Connection con) throws SQLException{
		String account = request.getParameter(Constants.PARAMETER_ACCOUNT);
		String password = request.getParameter(Constants.PARAMETER_PASSWORD);
		return DatabaseConnection.verifyLogin(con, account, password);
	}
	
	public static int[] createBattleQuestionSet(Connection con, String battleSize, String categoryID) throws SQLException{
		//TODO There is a possibility that questions are removed from the database. Add check that makes sure that
		//all selected questions are available.
//		int numberOfQuestions = DatabaseConnection.getNumberOfQuestionInDatabase(con);
		ArrayList<Integer> questions = DatabaseConnection.getAllQuestionsInCategory(con, categoryID);
		int numberOfQuestions = questions.size();
		int sizeofBattle = Integer.parseInt(battleSize);
		if(numberOfQuestions<sizeofBattle){
			sizeofBattle = numberOfQuestions;
		}
		int[] questionIds = new int[sizeofBattle];
		Random rand = getRNG();
		int i=0;
		
		do{
			boolean questionAlreadyUsed = false;
			int temp = rand.nextInt(numberOfQuestions);
			int tempQuestion = questions.get(temp);
			for(int j=0; j<i;j++){
				if(questionIds[j]==tempQuestion) {
					questionAlreadyUsed=true;
				}
			}
			if(!questionAlreadyUsed){
//				questionIds[i]=temp;
//				i++;
				questionIds[i]=tempQuestion;
				i++;
			}
		}while(i<sizeofBattle);
		
		return questionIds;
		
	}
	
	public static void calculateBattleResult(Connection con, String battleNr) throws SQLException, JSONException{
//		JSONArray battleResult = DatabaseConnection.getBattleResult(con, battleNr);
		JSONArray players = DatabaseConnection.getPlayerInfoForBattleResult(con, battleNr);
		JSONArray ar = DatabaseConnection.getBattleRiskAndSize(con, battleNr);
		String risk = ar.getJSONObject(0).getString(SQLStatements.BATTLE_RISK);
		int size = Integer.parseInt(ar.getJSONObject(0).getString(SQLStatements.BATTLE_SIZE));
		
		Player p1 = new Player(players.getJSONObject(0).getString(SQLStatements.PLAYER_KEY), 
				players.getJSONObject(0).getString(SQLStatements.PLAYER_AREA),
				players.getJSONObject(0).getString(SQLStatements.BATTLES_IN_SCORE),
				players.getJSONObject(0).getString(SQLStatements.PLAYER_GOLD),
				players.getJSONObject(0).getString(SQLStatements.PLAYER_GCM),
				players.getJSONObject(0).getString(SQLStatements.BATTLES_IN_WEAPONCOST));
		Player p2 = new Player(players.getJSONObject(1).getString(SQLStatements.PLAYER_KEY), 
				players.getJSONObject(1).getString(SQLStatements.PLAYER_AREA),
				players.getJSONObject(1).getString(SQLStatements.BATTLES_IN_SCORE),
				players.getJSONObject(1).getString(SQLStatements.PLAYER_GOLD),
				players.getJSONObject(1).getString(SQLStatements.PLAYER_GCM),
				players.getJSONObject(1).getString(SQLStatements.BATTLES_IN_WEAPONCOST));
		
		Battle battle;
		if(players.getJSONObject(0).getBoolean(SQLStatements.BATTLES_IN_ATTACKER)){
			battle = new Battle(p1, p2, risk, size);
		}else{
			battle = new Battle(p2, p1, risk, size);
		}
		
		System.out.println("Calculating result of battle: "+battleNr); 
		boolean drawGame = battle.calculateWinner();
		
		System.out.println("\t Result is: Player 1 is "+p1.getName()+" and got "+p1.goldResult+". Player 2 is " +
				p2.getName()+" and got "+p2.goldResult);
		DatabaseConnection.setBattleFinished(con, battleNr);
		DatabaseConnection.setUpdatedPlayerInfo(con, p1.getPlayerInfo());
		DatabaseConnection.setUpdatedPlayerInfo(con, p2.getPlayerInfo());
		
		if(drawGame){
			GCMMessenger.battleOver(p1.getName(),p1.getGCM(), p1.goldResult+"",p1.getScore(),p2.getScore(), Constants.PARAMETER_DRAW, battleNr);
			GCMMessenger.battleOver(p2.getName(),p2.getGCM(), p2.goldResult+"",p2.getScore(),p1.getScore(), Constants.PARAMETER_DRAW, battleNr);	
		}else{
			String p1Result, p2Result;
			if(p1.winner){
				p1Result=Constants.PARAMETER_WINNER;
				p2Result= Constants.PARAMETER_LOSER;
			}else{
				p1Result= Constants.PARAMETER_LOSER;
				p2Result=Constants.PARAMETER_WINNER;
			}
			GCMMessenger.battleOver(p1.getName(),p1.getGCM(), p1.goldResult+"",p1.getScore(),p2.getScore(), p1Result, battleNr);
			GCMMessenger.battleOver(p2.getName(),p2.getGCM(), p2.goldResult+"",p2.getScore(),p1.getScore(), p2Result, battleNr);			
		}
		
	}
	
	public static void battleWasDeclined(Connection con, String challenger, String coward, String battleNr) throws SQLException, JSONException{
		
		DatabaseConnection.playerFinished(con, challenger, battleNr, "10", "0");
		DatabaseConnection.playerFinished(con, coward, battleNr, "0", "0");
		
		JSONArray ar = DatabaseConnection.getBattleRiskAndSize(con, battleNr);
		String risk = ar.getJSONObject(0).getString(SQLStatements.BATTLE_RISK);
		int size = Integer.parseInt(ar.getJSONObject(0).getString(SQLStatements.BATTLE_SIZE));
		
		JSONObject win = DatabaseConnection.getPlayerInfo(con, challenger);
		JSONObject lose = DatabaseConnection.getPlayerInfo(con, coward);
		Player winner = new Player(win.getString(SQLStatements.PLAYER_KEY),
				win.getString(SQLStatements.PLAYER_AREA),
				"10", //Fake score because battle was never played
				win.getString(SQLStatements.PLAYER_GOLD),
				win.getString(SQLStatements.PLAYER_GCM));
		
		Player loser = new Player(lose.getString(SQLStatements.PLAYER_KEY),
				lose.getString(SQLStatements.PLAYER_AREA),
				"0", //Fake score because battle was never played
				lose.getString(SQLStatements.PLAYER_GOLD),
				lose.getString(SQLStatements.PLAYER_GCM));
		
		Battle battle = new Battle(winner, loser, risk, size);
		battle.battleDeclined();
		
		DatabaseConnection.setUpdatedPlayerInfo(con, winner.getPlayerInfo());
		markPlayerAsBusy(con, winner.getName(), false);
		DatabaseConnection.setUpdatedPlayerInfo(con, loser.getPlayerInfo());
		DatabaseConnection.setBattleFinished(con, battleNr);
		
		markChallengeAsHandled(con, battleNr);
		GCMMessenger.challengeDeclined(winner.getGCM(), winner.getName(), battle.getWinnerProfits()+"", 
				loser.getGCM(), loser.getName(), battle.getLoserLoss()+"", battleNr);
	}
	
	public static Random getRNG(){
		if(mRand==null){
			mRand = new Random();
		}
		return mRand;
	}
	
	public static String createNewBattle(Connection con, String risk, String size, String challenger, String challengee, String categoryID) throws SQLException{
		String battleNr = DatabaseConnection.getNewBattle(con, risk, size);
		DatabaseConnection.associateBattleWithParticipants(con, challenger, challengee, battleNr);
		int[] questions = GameLogic.createBattleQuestionSet(con, size, categoryID);
		DatabaseConnection.associateQuestionsWithBattle(con, questions, battleNr);
		
		markPlayerAsBusy(con, challenger, true);
		
		BattleChecker checker = new GameLogic.BattleChecker(challenger, challengee,battleNr);
		new Thread(checker).start();

		return battleNr;
	}
	
	public static void markPlayerAsBusy(Connection con, String account, boolean busy) throws SQLException{
		DatabaseConnection.setPlayerBusy(con, account, busy);
	}
	
	public static void markChallengeAsHandled(Connection con, String battleNr) throws SQLException{
		DatabaseConnection.challengeIsHandled(con, battleNr);
	}
	
	public static void declineOtherUnstartedDuels(Connection con, String battleNr, String player) throws SQLException, JSONException{
		JSONArray otherPlayers = DatabaseConnection.getPlayersInUnstartedDuels(con, player);
		DatabaseConnection.setUnstartedDuelsAsFinished(con, player);
		GCMMessenger.declineOtherDuels(otherPlayers);
	}
	
	private static void parseBuildingsAndSuch(){
		ContextConfiguration config=ContextConfiguration.getInstance();
		InputStream catDoc = config.getContext().getResourceAsStream("/WEB-INF/buildings.xml");
		BuildingFactory.parseXML(catDoc);
	}
	
	public static void startBattleTimer(String battleNr){
		BattleTimeouter timer = new BattleTimeouter(battleNr);
		new Thread(timer).start();
	}
	
	public static boolean placeNewBuilding(Connection con, String account, String buildingType, String x, String y) throws SQLException, JSONException{
		JSONObject playerObject = DatabaseConnection.getPlayerInfo(con, account);
		Player player = new Player(playerObject.getString(SQLStatements.PLAYER_KEY), 
				playerObject.getString(SQLStatements.PLAYER_AREA), 
				playerObject.getString(SQLStatements.PLAYER_GOLD));
		
		Building newBuilding = BuildingFactory.constructNewBuilding(buildingType);
		int intX = Integer.parseInt(x);
		int intY = Integer.parseInt(y);
		
		boolean success = player.addNewBuilding(newBuilding, intX, intY);
		
		if(success){
			DatabaseConnection.setUpdatedPlayerInfo(con, player.getPlayerInfo());
		}
		return success;
	}
	
	public static boolean upgradeBuilding(Connection con, String account, String buildingType, String buildingRank, String px, String py) throws JSONException, SQLException{
		JSONObject playerObject = DatabaseConnection.getPlayerInfo(con, account);
		Player player = new Player(playerObject.getString(SQLStatements.PLAYER_KEY), 
				playerObject.getString(SQLStatements.PLAYER_AREA), 
				playerObject.getString(SQLStatements.PLAYER_GOLD));
		
		int rank = Integer.parseInt(buildingRank);
		int x = Integer.parseInt(px);
		int y = Integer.parseInt(py);
		
		Building b = BuildingFactory.getBuildingofSpecificTypeAndRank(buildingType, rank,x,y);
		Building newBuilding = BuildingFactory.getBuildingofSpecificTypeAndRank(buildingType, (rank+1),x,y);
		
		boolean success = player.upgradeBuilding(b, newBuilding);
		
		if(success){
			DatabaseConnection.setUpdatedPlayerInfo(con, player.getPlayerInfo());
		}
		
		return success;
	}
	
	public static boolean removeBuilding(Connection con, String account, String px, String py) throws SQLException, JSONException{
		int x = Integer.parseInt(px);
		int y = Integer.parseInt(py);
		
		JSONObject playerObject = DatabaseConnection.getPlayerInfo(con, account);
		Player player = new Player(playerObject.getString(SQLStatements.PLAYER_KEY), 
				playerObject.getString(SQLStatements.PLAYER_AREA), 
				playerObject.getString(SQLStatements.PLAYER_GOLD));
		
		boolean success = player.removeBuilding(x, y);
		
		if(success){
			DatabaseConnection.setUpdatedPlayerInfo(con, player.getPlayerInfo());
		}
		
		return success;
	}
	
	private static class BattleChecker implements Runnable{
		
		String battleNr, challenger, coward;
		
		public BattleChecker(String challenger, String coward, String battle){
			battleNr = battle;
			this.challenger = challenger;
			this.coward = coward;
		}

		@Override
		public void run() {
			try {
				parseBuildingsAndSuch();
				Thread.sleep(Constants.DELAY_BEFORE_CHALLENGE_IS_DECLINED_BY_DEFAULT);
				Connection con = DatabaseConnection.getConnection();
				boolean isBattleStarted = DatabaseConnection.hasBattleStarted(con, battleNr);
				
//				String paramValue=config.getContextParameter("parameterKey");
				
				if(!isBattleStarted){
					GameLogic.battleWasDeclined(con, challenger, coward, battleNr);
				}else{
					System.out.println("GameLogic: Challenge was handled.");
				}
				con.close();
			} catch (InterruptedException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			} catch (SQLException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			} catch (JSONException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			} catch (ClassNotFoundException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		}
		
	}
	
	private static class BattleTimeouter implements Runnable{
		
		String mBattleNr;
		
		public BattleTimeouter(String battleNr){
			mBattleNr=battleNr;
		}

		@Override
		public void run() {
			try {
				System.out.println("Battletimer started");
				Thread.sleep(Constants.DELAY_BEFORE_BATTLE_IS_DECIDED_BY_TIMEOUT);
				Connection con = DatabaseConnection.getConnection();
				String remainingPlayer = DatabaseConnection.hasBattleFinished(con, mBattleNr);
				if(remainingPlayer!=null){
					DatabaseConnection.playerFinished(con, remainingPlayer, mBattleNr, "0", "0");
					calculateBattleResult(con, mBattleNr);
				}
				System.out.println("Battletimer ended");
			} catch (InterruptedException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			} catch (ClassNotFoundException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			} catch (SQLException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			} catch (JSONException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		}
		
	}

}
