package adrian.master.knowledgewar.activities;

import adrian.master.knowledgewar.R;
import adrian.master.knowledgewar.gameactivities.MainActivity;
import adrian.master.knowledgewar.gameconcepts.BuildingFactory;
import adrian.master.knowledgewar.gameconcepts.WeaponFactory;
import adrian.master.knowledgewar.logic.Constants;
import adrian.master.knowledgewar.logic.Server;
import adrian.master.knowledgewar.logic.UserFunctions;
import android.app.Activity;
import android.content.Intent;
import android.os.AsyncTask;
import android.os.Bundle;
import android.text.Editable;
import android.text.TextWatcher;
import android.view.Menu;
import android.view.View;
import android.view.View.OnClickListener;
import android.widget.Button;
import android.widget.EditText;
import android.widget.Toast;

public class LoginActivity extends Activity {

	private Button login, register;
	private EditText accountname, password,ipAddress;

	@Override
	public void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		setContentView(R.layout.activity_login);
		getWindow().setBackgroundDrawableResource(R.drawable.dune);

		UserFunctions.setApplicationContext(getApplicationContext());
		//Find the necessary views in the activity
		login = (Button)findViewById(R.id.login_button_login);
		register = (Button)findViewById(R.id.login_button_register);

		login.setEnabled(false);

		accountname = (EditText)findViewById(R.id.login_accountname);
		password = (EditText)findViewById(R.id.login_password);
		ipAddress = (EditText)findViewById(R.id.login_ip); 

		checkIfAlreadyLoggedIn();
		isLoginActivated();
		//Attach listeners to buttons
		login.setOnClickListener(new OnClickListener(){
			public void onClick(View arg0) {
				userLogin();
			}
		});

		register.setOnClickListener(new OnClickListener(){
			public void onClick(View v) {
				startRegisterActivity();
			}
		});


		//Attach listener to textfield
		accountname.addTextChangedListener(new TextWatcher(){

			public void afterTextChanged(Editable arg0) {
				isLoginActivated();
			}
			public void beforeTextChanged(CharSequence arg0, int arg1,
					int arg2, int arg3) {
			}
			public void onTextChanged(CharSequence arg0, int arg1, int arg2,
					int arg3) {
			}
		});

		password.addTextChangedListener(new TextWatcher(){

			public void afterTextChanged(Editable s) {
				isLoginActivated();
			}
			public void beforeTextChanged(CharSequence s, int start, int count,
					int after) {
			}
			public void onTextChanged(CharSequence s, int start, int before,
					int count) {
			}
		});
		
		ipAddress.setText(UserFunctions.getServerIp());
	}

	@Override
	public boolean onCreateOptionsMenu(Menu menu) {
		getMenuInflater().inflate(R.menu.activity_login, menu);
		return true;
	}

	public void userLogin(){
		String accname = accountname.getText().toString();
		String pw = password.getText().toString();
		String ip = ipAddress.getText().toString();

		LoginTask task = new LoginTask(accname, pw, ip);
		task.execute();

	}

	public void isLoginActivated(){
		if(accountname.getText().toString().equals("") || password.getText().toString().equals("")){
			login.setEnabled(false);
		}else{
			login.setEnabled(true);
		}
	}

	public void startRegisterActivity(){
		Intent intent = new Intent(this, RegisterActivity.class);
		startActivity(intent);
	}

	public void successfulLogin(String account, String pass){
		UserFunctions.logInUser(getApplicationContext(), account, pass);
		UserFunctions.registerGCM(getApplicationContext());

		startPlayerActivity();
	}

	private void startPlayerActivity(){
		UserFunctions.startLocationPoller(getApplicationContext());
		Intent dashboard = new Intent(getApplicationContext(), MainActivity.class);
		dashboard.addFlags(Intent.FLAG_ACTIVITY_CLEAR_TOP);
		startActivity(dashboard);
		finish();
	}


	private void checkIfAlreadyLoggedIn(){
		if(UserFunctions.isUserLoggedIn(getApplicationContext())){
			accountname.setEnabled(false);
			password.setEnabled(false);
			LoginTask task = new LoginTask(UserFunctions.getUserAccount(this), UserFunctions.getUserPassword(this), UserFunctions.getServerIp());
			task.execute();
		}
	}

	private class LoginTask extends AsyncTask<Void, Void, String>{

		String accountname="";
		String password="";
		String ip="";

		public LoginTask(String name, String pass, String ip){
			this.accountname=name;
			this.password=pass;
			this.ip = ip;
		}

		@Override
		protected String doInBackground(Void... params) {
			UserFunctions.storeServerIP(getApplicationContext(), ip);
			String success = Server.userLogin(this.accountname, this.password);
			return success;
		}

		@Override
		protected void onPostExecute(String result) {
			String message="";
			if(result.contains(Constants.SERVER_RESPONSE_LOGIN_SUCCESSFUL)){
				message = getString(R.string.PLAYER_LOGIN_SUCCESS);
				successfulLogin(this.accountname, this.password);
			}else if(result.contains(Constants.SERVER_RESPONSE_LOGIN_FAILURE)){
				message =getString(R.string.PLAYER_LOGIN_FAILURE);
				LoginActivity.this.accountname.setEnabled(true);
				LoginActivity.this.password.setEnabled(true);
			}else if(result.contains(Constants.HTTP_EXECUTE_FAILURE)){
				message = result;
				LoginActivity.this.accountname.setEnabled(true);
				LoginActivity.this.password.setEnabled(true);
			}
			Toast.makeText(LoginActivity.this, message, Toast.LENGTH_SHORT).show();
		}

	}

}
