package adrian.master.knowledgewar.activities;

import adrian.master.knowledgewar.R;
import adrian.master.knowledgewar.gameactivities.MainActivity;
import adrian.master.knowledgewar.logic.Constants;
import adrian.master.knowledgewar.logic.Server;
import adrian.master.knowledgewar.logic.UserFunctions;
import android.app.Activity;
import android.content.Intent;
import android.os.AsyncTask;
import android.os.Bundle;
import android.text.Editable;
import android.text.TextWatcher;
import android.view.Menu;
import android.view.View;
import android.view.View.OnClickListener;
import android.widget.Button;
import android.widget.EditText;
import android.widget.Toast;

public class RegisterActivity extends Activity {

	EditText accountname, password, rPassword, rIP;
	Button registerButton;

	@Override
	public void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		setContentView(R.layout.activity_register);
		getWindow().setBackgroundDrawableResource(R.drawable.dune);

		//Find the relevant views
		registerButton = (Button)findViewById(R.id.register_button);
		accountname = (EditText)findViewById(R.id.register_accountname);
		password = (EditText)findViewById(R.id.register_password);
		rPassword = (EditText)findViewById(R.id.register_repeat_password);
		rIP = (EditText)findViewById(R.id.register_ip);

		registerButton.setEnabled(false);

		registerButton.setOnClickListener(new OnClickListener(){
			public void onClick(View v) {
				boolean passwordMatch = (password.getText().toString()).equals(rPassword.getText().toString());
				if(passwordMatch){
					String[] registerParams = {accountname.getText().toString(), 
							password.getText().toString()};
					UserFunctions.storeServerIP(getApplicationContext(), rIP.getText().toString());
					RegisterTask task = new RegisterTask(registerParams[0], registerParams[1]);
					task.execute();					
				}else{
					Toast.makeText(getApplicationContext(), getString(R.string.register_password_not_matching), Toast.LENGTH_SHORT).show();
				}
			}
		});

		TextWatcher enabler = new RegisterButtonEnabler();
		accountname.addTextChangedListener(enabler);
		password.addTextChangedListener(enabler);
		rPassword.addTextChangedListener(enabler);
		
		
	}

	@Override
	public boolean onCreateOptionsMenu(Menu menu) {
		getMenuInflater().inflate(R.menu.activity_register, menu);
		return true;
	}

	private void isRegisterButtonReady(){
		boolean ac = accountname.getText().toString().equals("");
		boolean pass = password.getText().toString().equals("");
		boolean rpass = rPassword.getText().toString().equals("");

		if(ac || pass || rpass){
			registerButton.setEnabled(false);
		}else{
			registerButton.setEnabled(true);
		}
	}

	private void successfulRegister(String account, String pass){
		UserFunctions.logInUser(this, account, pass);
		UserFunctions.registerGCM(this);
		UserFunctions.startLocationPoller(getApplicationContext());
		Intent dashboard = new Intent(getApplicationContext(), MainActivity.class);
		dashboard.addFlags(Intent.FLAG_ACTIVITY_CLEAR_TOP);
		startActivity(dashboard);
		RegisterActivity.this.finish();
	}

	/**
	 * Background task that handles communication with the server.
	 * @author Adrian
	 *
	 */
	private class RegisterTask extends AsyncTask<Void, Void, String>{

		String accountname="";
		String password="";

		public RegisterTask(String name, String pw){
			this.accountname=name;
			this.password=pw;
		}

		@Override
		protected String doInBackground(Void... params) {
			String result = Server.registerPlayer(accountname, password);
			return result;
		}

		@Override
		protected void onPostExecute(String result) {
			String message="";
			if(result.contains(Constants.SERVER_RESPONSE_REGISTRATION_SUCCESSFUL)){
				message = getString(R.string.PLAYER_REGISTER_SUCCESS);
				successfulRegister(accountname, password);
			}else if(result.contains(Constants.SERVER_RESPONSE_REGISTRATION_FAILURE)){
				message = getString(R.string.PLAYER_REGISTER_FAILURE);
			}else if(result.equals(Constants.HTTP_EXECUTE_FAILURE)){
				message = result; 
			}
			Toast.makeText(RegisterActivity.this, message, Toast.LENGTH_SHORT).show();				
		}

	}

	/**
	 * Textwatcher that checks that all textfields are filled out, and enables the Register-button.
	 * @author Adrian
	 *
	 */
	private class RegisterButtonEnabler implements TextWatcher{

		public void afterTextChanged(Editable s) {
			isRegisterButtonReady();
		}

		public void beforeTextChanged(CharSequence s, int start, int count,
				int after) {
		}

		public void onTextChanged(CharSequence s, int start, int before,
				int count) {
		}

	}
}
