package adrian.master.knowledgewar.gameactivities;

import org.andengine.engine.camera.ZoomCamera;
import org.andengine.engine.options.EngineOptions;
import org.andengine.engine.options.ScreenOrientation;
import org.andengine.engine.options.resolutionpolicy.RatioResolutionPolicy;
import org.andengine.entity.scene.Scene;
import org.andengine.opengl.font.Font;
import org.andengine.opengl.font.FontFactory;
import org.andengine.opengl.texture.TextureOptions;
import org.andengine.opengl.texture.atlas.bitmap.BitmapTextureAtlas;
import org.andengine.opengl.texture.atlas.bitmap.BitmapTextureAtlasTextureRegionFactory;
import org.andengine.opengl.texture.region.TextureRegion;
import org.andengine.opengl.texture.region.TiledTextureRegion;
import org.andengine.opengl.vbo.VertexBufferObjectManager;
import org.andengine.ui.activity.SimpleBaseGameActivity;
import org.json.JSONArray;
import org.json.JSONException;

import adrian.master.knowledgewar.R;
import adrian.master.knowledgewar.gameconcepts.Battle;
import adrian.master.knowledgewar.gameconcepts.Player;
import adrian.master.knowledgewar.gameconcepts.Weapon;
import adrian.master.knowledgewar.gameconcepts.WeaponFactory;
import adrian.master.knowledgewar.interfaces.BattleActivityInterface;
import adrian.master.knowledgewar.logic.Constants;
import adrian.master.knowledgewar.logic.QuestionSceneFactory;
import adrian.master.knowledgewar.logic.Server;
import adrian.master.knowledgewar.logic.UserFunctions;
import adrian.master.knowledgewar.scenes.BaseDisplayQuestionScene;
import adrian.master.knowledgewar.scenes.BattleResultScene;
import adrian.master.knowledgewar.scenes.ChallengeAcceptedScene;
import adrian.master.knowledgewar.scenes.ChallengeSentScene;
import adrian.master.knowledgewar.scenes.DisplayAttackScene;
import adrian.master.knowledgewar.scenes.RightAnswerScene;
import adrian.master.knowledgewar.scenes.TransitionScene;
import adrian.master.knowledgewar.scenes.WrongAnswerScene;
import adrian.master.knowledgewar.subscenes.LoadingHUD;
import android.content.Intent;
import android.content.res.Resources;
import android.graphics.Typeface;
import android.net.Uri;
import android.os.AsyncTask;
import android.os.Bundle;
import android.util.Log;
import android.widget.Toast;

public class BattleActivity extends SimpleBaseGameActivity {

	public int CAMERA_HEIGHT=Constants.CAMERA_HEIGHT, CAMERA_WIDTH=Constants.CAMERA_WIDTH;

	public static BattleActivity mThis;

	private BitmapTextureAtlas mDuneBackground, mBigDuneBackground;
	private BitmapTextureAtlas mSandBackground;
	private BitmapTextureAtlas mButton, mWhiteButton;
	private BitmapTextureAtlas mRedBar, mGreenBar, mBlack;
	private BitmapTextureAtlas mHorizontalLine, mTimerBorder, mAwesomeFace, mBibleThump;
	private BitmapTextureAtlas mChicken, mCow, mNinja1,mNinja2,mFan1,mFan2,mWand1,mWand2;
	private BitmapTextureAtlas mLeatherBackground, mPaperBackground;
	private BitmapTextureAtlas mRedX, mGreenCheck, mCube, mMoney;

	public TextureRegion mDuneTexture, mBigDuneTexture;
	public TextureRegion mSandTexture;
	public TextureRegion mButtonTexture, mWhiteButtonTexture;
	public TextureRegion mRedBarTexture, mGreenBarTexture;
	public TextureRegion mBlackTexture;
	public TextureRegion mHorizontalLineTexture, mTimerBorderTexture, mAwesomeFaceTexture, mBibleThumpTexture;
	public TextureRegion mChickenTexture, mCowTexture, mNinja1Texture,mNinja2Texture,mFan1Texture,mFan2Texture,mWand1Texture,mWand2Texture;
	public TextureRegion mLeatherTexture, mPaperTexture;
	public TextureRegion mRedXTexture, mGreenCheckTexture, mMoneyTexture;
	public TiledTextureRegion mCubeTexture;

	public Font mFont,mFontBold, mBigFont, mSmallFont, mSmallFontBold;

	public ZoomCamera mCamera;
	public LoadingHUD mLoadingHUD;
	public VertexBufferObjectManager mMgr;
	public Player mSelf;
	public Resources mRes;
	private EngineOptions mEngineOptions;

	public Battle mCurrentBattle;
	private String mBattleNr, mSelfName, mOpponentName;
	private String mCurrentDebuff="";
	private boolean isBackPressedOnce = false, isFinished=false, isBattleStarted=false;
	private boolean mSentChallenge;
	private float mCameraCenterX, mCameraCenterY;

	private ChallengeSentScene mChallengeSentScene;
	private ChallengeAcceptedScene mChallengeAcceptedScene;
	private BaseDisplayQuestionScene mCurrentQuestionScene;
	private BattleResultScene mBattleResultScene;
	private BattleActivityInterface mBetweenQuestionsScene;
	private BattleActivityInterface mCurrentScene;
	
	private AsyncTask mActiveTask;

//	private Database mQuestionDatabase;


	public void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		mThis = this;

		//The activity needs to do different things depending on whether the player SENT or RECEIVED a challenge. If it has RECEIVED a challenge, it needs
		//to get the identifier for the battle and the name of the opponent from the Intent.
		Bundle b = getIntent().getExtras();
		if(b!=null){
			mSentChallenge = b.getBoolean(Constants.PARAMETER_SENT_CHALLENGE);
			if(!mSentChallenge){
				mBattleNr = b.getString(Constants.PARAMETER_BATTLENR);
				mSelfName = b.getString(Constants.PARAMETER_SELF);
				mOpponentName = b.getString(Constants.PARAMETER_OPPONENT);
			}
		}
		mSelf = UserFunctions.self;
		UserFunctions.setUserBusy(getApplicationContext(), false);
		this.mRes = getResources();
		
//		mQuestionDatabase = Database.getInstance(getApplicationContext());
//		mQuestionDatabase.open();
		
	}

	@Override
	public EngineOptions onCreateEngineOptions() {
		mCamera = new ZoomCamera(0,0,CAMERA_WIDTH,CAMERA_HEIGHT );
		mEngineOptions = new EngineOptions(false, ScreenOrientation.PORTRAIT_FIXED, 
				new RatioResolutionPolicy(CAMERA_WIDTH, CAMERA_HEIGHT), this.mCamera);
		return mEngineOptions;
	}

	@Override
	protected void onCreateResources() {

		BitmapTextureAtlasTextureRegionFactory.setAssetBasePath("gfx/");

		mDuneBackground = new BitmapTextureAtlas(this.getTextureManager(), 1280, 480, TextureOptions.BILINEAR_PREMULTIPLYALPHA);
		mDuneTexture = BitmapTextureAtlasTextureRegionFactory.createFromAsset(mDuneBackground, this, "dune.png",0,0);
		mDuneBackground.load();

		mBigDuneBackground = new BitmapTextureAtlas(this.getTextureManager(), 1920, 720, TextureOptions.BILINEAR_PREMULTIPLYALPHA);
		mBigDuneTexture = BitmapTextureAtlasTextureRegionFactory.createFromAsset(mBigDuneBackground, this, "dunebackground.png",0,0);
		mBigDuneBackground.load();

		mSandBackground = new BitmapTextureAtlas(this.getTextureManager(), 512 , 512, TextureOptions.REPEATING_BILINEAR_PREMULTIPLYALPHA);
		mSandTexture = BitmapTextureAtlasTextureRegionFactory.createFromAsset(mSandBackground, this, "sand.png",0,0);
		mSandBackground.load();

		mButton = new BitmapTextureAtlas(this.getTextureManager(), 512, 256, TextureOptions.BILINEAR_PREMULTIPLYALPHA);
		mButtonTexture = BitmapTextureAtlasTextureRegionFactory.createFromAsset(mButton, this, "button_blank.png",0,0);
		mButton.load();

		mWhiteButton = new BitmapTextureAtlas(this.getTextureManager(), 512, 256, TextureOptions.BILINEAR);
		mWhiteButtonTexture = BitmapTextureAtlasTextureRegionFactory.createFromAsset(mWhiteButton, this, "button_white.png",0,0);
		mWhiteButton.load();

		mRedBar = new BitmapTextureAtlas(this.getTextureManager(), 32, 32, TextureOptions.REPEATING_BILINEAR_PREMULTIPLYALPHA);
		mRedBarTexture = BitmapTextureAtlasTextureRegionFactory.createFromAsset(mRedBar, this, "redbar.png",0,0);
		mRedBar.load();

		mGreenBar = new BitmapTextureAtlas(this.getTextureManager(), 32, 32, TextureOptions.REPEATING_BILINEAR_PREMULTIPLYALPHA);
		mGreenBarTexture = BitmapTextureAtlasTextureRegionFactory.createFromAsset(mGreenBar, this, "greenbar.png",0,0);
		mGreenBar.load();

		this.mHorizontalLine = new BitmapTextureAtlas(this.getTextureManager(), Constants.TILE_SIZE, 2, TextureOptions.REPEATING_BILINEAR_PREMULTIPLYALPHA);
		this.mHorizontalLineTexture = BitmapTextureAtlasTextureRegionFactory.createFromAsset(this.mHorizontalLine, this, "horizontalline.png", 0,0);
		this.mHorizontalLine.load();

		this.mChicken = new BitmapTextureAtlas(this.getTextureManager(), 256, 256, TextureOptions.BILINEAR);
		this.mChickenTexture = BitmapTextureAtlasTextureRegionFactory.createFromAsset(this.mChicken, this, "chicken.png",0,0);
		this.mChicken.load();

		this.mCow = new BitmapTextureAtlas(this.getTextureManager(), 125, 125, TextureOptions.BILINEAR);
		this.mCowTexture = BitmapTextureAtlasTextureRegionFactory.createFromAsset(this.mCow, this, "cow.png",0,0);
		this.mCow.load();

		this.mNinja1 = new BitmapTextureAtlas(this.getTextureManager(), 200, 200, TextureOptions.BILINEAR);
		this.mNinja1Texture = BitmapTextureAtlasTextureRegionFactory.createFromAsset(this.mNinja1, this, "ninja1.png",0,0);
		this.mNinja1.load();

		this.mNinja2 = new BitmapTextureAtlas(this.getTextureManager(), 200, 200, TextureOptions.BILINEAR);
		this.mNinja2Texture = BitmapTextureAtlasTextureRegionFactory.createFromAsset(this.mNinja2, this, "ninja2.png",0,0);
		this.mNinja2.load();

		this.mFan1 = new BitmapTextureAtlas(this.getTextureManager(), 225, 225, TextureOptions.BILINEAR);
		this.mFan1Texture = BitmapTextureAtlasTextureRegionFactory.createFromAsset(this.mFan1, this, "fan1.png",0,0);
		this.mFan1.load();

		this.mFan2 = new BitmapTextureAtlas(this.getTextureManager(), 256, 256, TextureOptions.BILINEAR);
		this.mFan2Texture = BitmapTextureAtlasTextureRegionFactory.createFromAsset(this.mFan2, this, "fan2.png",0,0);
		this.mFan2.load();

		this.mWand1 = new BitmapTextureAtlas(this.getTextureManager(), 256, 256, TextureOptions.BILINEAR);
		this.mWand1Texture = BitmapTextureAtlasTextureRegionFactory.createFromAsset(this.mWand1, this, "magicwand1.png",0,0);
		this.mWand1.load();

		this.mWand2 = new BitmapTextureAtlas(this.getTextureManager(), 256, 256, TextureOptions.BILINEAR);
		this.mWand2Texture = BitmapTextureAtlasTextureRegionFactory.createFromAsset(this.mWand2, this, "magicwand2.png",0,0);
		this.mWand2.load();

		this.mLeatherBackground = new BitmapTextureAtlas(this.getTextureManager(), 1100, 850, TextureOptions.BILINEAR);
		this.mLeatherTexture = BitmapTextureAtlasTextureRegionFactory.createFromAsset(this.mLeatherBackground, this, "leather_scroll2.png",0,0);
		this.mLeatherBackground.load();

		this.mPaperBackground = new BitmapTextureAtlas(this.getTextureManager(), 1023, 601, TextureOptions.BILINEAR);
		this.mPaperTexture = BitmapTextureAtlasTextureRegionFactory.createFromAsset(this.mPaperBackground, this, "alternative_background.png",0,0);
		this.mPaperBackground.load();

		this.mRedX = new BitmapTextureAtlas(this.getTextureManager(), 300, 300, TextureOptions.BILINEAR);
		this.mRedXTexture = BitmapTextureAtlasTextureRegionFactory.createFromAsset(this.mRedX, this, "red_x.png",0,0);
		this.mRedX.load();

		this.mGreenCheck = new BitmapTextureAtlas(this.getTextureManager(), 300, 300, TextureOptions.BILINEAR);
		this.mGreenCheckTexture = BitmapTextureAtlasTextureRegionFactory.createFromAsset(this.mGreenCheck, this, "green_check.png",0,0);
		this.mGreenCheck.load();

		this.mTimerBorder = new BitmapTextureAtlas(this.getTextureManager(), 480, 40, TextureOptions.BILINEAR);
		this.mTimerBorderTexture = BitmapTextureAtlasTextureRegionFactory.createFromAsset(this.mTimerBorder, this, "timerborder.png",0,0);
		this.mTimerBorder.load();

		this.mAwesomeFace = new BitmapTextureAtlas(this.getTextureManager(), 219, 219, TextureOptions.BILINEAR);
		this.mAwesomeFaceTexture = BitmapTextureAtlasTextureRegionFactory.createFromAsset(this.mAwesomeFace, this, "awesomeface.png",0,0);
		this.mAwesomeFace.load();

		this.mBibleThump = new BitmapTextureAtlas(this.getTextureManager(), 64, 64, TextureOptions.BILINEAR);
		this.mBibleThumpTexture = BitmapTextureAtlasTextureRegionFactory.createFromAsset(this.mBibleThump, this, "BibleThump.png",0,0);
		this.mBibleThump.load();

		this.mBlack = new BitmapTextureAtlas(this.getTextureManager(), Constants.TILE_SIZE, Constants.TILE_SIZE, TextureOptions.REPEATING_BILINEAR_PREMULTIPLYALPHA);
		this.mBlackTexture = BitmapTextureAtlasTextureRegionFactory.createFromAsset(this.mBlack, this, "black.png", 0,0);
		this.mBlack.load();

		this.mCube = new BitmapTextureAtlas(this.getTextureManager(), 300, 360, TextureOptions.BILINEAR);
		this.mCubeTexture = BitmapTextureAtlasTextureRegionFactory.createTiledFromAsset(mCube, this, "cube.png",0,0,5,6);
		this.mCube.load();
		
		this.mMoney = new BitmapTextureAtlas(this.getTextureManager(), 256, 256, TextureOptions.BILINEAR);
		this.mMoneyTexture = BitmapTextureAtlasTextureRegionFactory.createFromAsset(this.mMoney, this, "money2.png",0,0);
		this.mMoney.load();

		this.mFont = FontFactory.create(this.getFontManager(), this.getTextureManager(), 256, 256, TextureOptions.BILINEAR, Typeface.create(Typeface.DEFAULT, Typeface.NORMAL),24);
		this.mFont.load();

		this.mBigFont = FontFactory.create(this.getFontManager(), this.getTextureManager(), 256, 256, TextureOptions.BILINEAR, Typeface.create(Typeface.DEFAULT, Typeface.BOLD),36);
		this.mBigFont.load();

		this.mSmallFont = FontFactory.create(this.getFontManager(), this.getTextureManager(), 256, 256, TextureOptions.BILINEAR, Typeface.create(Typeface.DEFAULT, Typeface.NORMAL),16);
		this.mSmallFont.load();

		this.mSmallFontBold = FontFactory.create(this.getFontManager(), this.getTextureManager(), 256, 256, TextureOptions.BILINEAR, Typeface.create(Typeface.DEFAULT, Typeface.BOLD),16);
		this.mSmallFontBold.load();

		this.mFontBold = FontFactory.create(this.getFontManager(), this.getTextureManager(), 256, 256, TextureOptions.BILINEAR, Typeface.create(Typeface.DEFAULT, Typeface.BOLD),24);
		this.mFontBold.load();

		mMgr = this.getVertexBufferObjectManager();

	}

	@Override
	protected Scene onCreateScene() {
		mLoadingHUD = new LoadingHUD(mBlackTexture, mCubeTexture, mMgr);
		mCamera.setHUD(mLoadingHUD);
		mCameraCenterX = mCamera.getCenterX();
		mCameraCenterY = mCamera.getCenterY();
		mBattleResultScene = new BattleResultScene(this);

		//The activity needs to do different things depending on whether the player SENT or RECEIVED a challenge.
		if(mSentChallenge){
			mChallengeSentScene = new ChallengeSentScene(this,mEngine);
			mCurrentScene=mChallengeSentScene;
			return mChallengeSentScene;			
		}else{
			mChallengeAcceptedScene = new ChallengeAcceptedScene(this);
			mCurrentScene = mChallengeAcceptedScene;
			UserFunctions.acceptChallenge(mBattleNr, mSelfName, mOpponentName, getApplicationContext());
			return mChallengeAcceptedScene;
		}
	}

	/**
	 * Method that starts the impending battle. Creates battle-object and contacts the server to find the questions.
	 * @param battleNr The identifier for the battle. Used to find necessary information on the server.
	 */
	public void startBattle(String battleNr){
		if(mSentChallenge){
			mChallengeSentScene.cancelTimer();
		}
		mCurrentBattle = new Battle(battleNr);
		mBattleNr = battleNr;
		QuestionFinder finder = new QuestionFinder();
		finder.execute();
	}

	/**
	 * Called if the challenged player chooses to reject the challenge sent by this player.
	 * @param challengee Name of the player that rejected the challenge
	 * @param profits Amount of money this player won
	 */
	public void challengeDeclined(String challengee, String profits){
		mChallengeSentScene.battleDeclined(challengee, profits);
	}
	
	public void opponentStartedOtherBattle(String challengee){
		mChallengeSentScene.opponentStartedOtherBattle(challengee);
	}

	/**
	 * Called after the player has answered a question during a battle. Updates the battle-object with the result and score, and creates
	 * the scene that appears between questions depending on whether the answer was correct or not.
	 * @param score The score that is added to the total in this battle. Calculated based on time left when the player answered.
	 */
	public void questionAnswered(int score){
		restoreCameraSettings();
		mCamera.setHUD(mLoadingHUD);
		mLoadingHUD.displayLoadingAnimation(true);
		mCurrentBattle.addScore(score);

		if(mCurrentQuestionScene.isAnswerCorrect()){
			mCurrentBattle.updateResult(true);
			mBetweenQuestionsScene = new RightAnswerScene(this, mEngine);
		}else{
			mCurrentBattle.updateResult(false);
			mBetweenQuestionsScene = new WrongAnswerScene(this);
		}
		mLoadingHUD.displayLoadingAnimation(false);
		new TransitionScene(this,mCamera,mEngine,(Scene)mCurrentScene, (Scene)mBetweenQuestionsScene);
		mCurrentScene=mBetweenQuestionsScene;
	}

	/**
	 * Uses an AsyncTask to contact the server and check if the player is being attacked. Also puts the players own attack into the opponents debuff-queue.
	 * @param attack String that contains information about the debuff the player wishes to inflict on the opponent.
	 */
	public void nextQuestion(String attack){
		AnswerHandler answer = new AnswerHandler();
		answer.execute(attack);
	}

	/**
	 * Called when the player returns to the main menu from the scene displaying the result of the battle.
	 */
	public void doneWithBattle(){
		//		UserFunctions.getOwnStatsFromServer(getApplicationContext());
		//		MainActivity.instance.updateBusyText();
		MainActivity.instance.updateStats();
		mThis=null;
		finish();
	}

	/**
	 * Creates the scene that displays whether or not the player is being attacked between each question.
	 */
	public void createTransitionScene(){
		Weapon w = WeaponFactory.getWeaponBasedOnDebuff(mCurrentDebuff);
		boolean isBeingAttacked;
		if(w==null){
			isBeingAttacked=false;
		}else{
			isBeingAttacked=true;
		}
		DisplayAttackScene scene = new DisplayAttackScene(this,w, isBeingAttacked);
		new TransitionScene(this,mCamera,mEngine,(Scene)mCurrentScene, scene);
		mCurrentScene=scene;
	}

	/**
	 * Finds the next question in the battle and creates the scene displaying the question. Also checks if there are no more questions, and if there
	 * isn't, tells the server that the player is finished.
	 */
	public void createNextQuestionScene(){
		if(mCurrentBattle.next()){
			this.mCurrentQuestionScene = QuestionSceneFactory.createNextQuestion(this, 
					mCurrentBattle.getCurrentQuestion(), mCurrentDebuff);

			new TransitionScene(this,mCamera,mEngine,(Scene)mCurrentScene, (Scene)mCurrentQuestionScene);
			mCurrentScene=mCurrentQuestionScene;
			mCurrentQuestionScene.startTimer();

		}else{
			isFinished=true;
			BattleFinisher finisher = new BattleFinisher();
			finisher.execute();

		}
	}

	/**
	 * Method necessary because of the way AndEngine and the game uses the camera. If the camera is moved this must be called to make sure the
	 * question/alternatives can be viewed during next question.
	 */
	private void restoreCameraSettings(){
		mCamera.setBoundsEnabled(false);
		mCamera.setCenter(CAMERA_WIDTH/2, CAMERA_HEIGHT/2);
	}

	/**
	 * Creates the scene that displays the results of the battle. This scene will be updated when the opponent finishes, displaying whether the player
	 * won or lost, and potential profits.
	 */
	private void createBattleFinishedScene(){
		if(mBattleResultScene!=null && this.mEngine!=null){
			mBattleResultScene.initialize();
			mSelf.payGold(mCurrentBattle.getWeaponValue());
			new TransitionScene(this,mCamera,mEngine,(Scene)mCurrentScene, (Scene)mBattleResultScene);
			mCurrentScene=mBattleResultScene;
			
//			for(int i=0; i<mCurrentBattle.mSize;i++){
//				Question q = mCurrentBattle.getQuestion(i);
//				mQuestionDatabase.insertQuestion(q);
//			}
			
		}
	}

	/**
	 * Called from the class PushMessageHandler when the game receives a message from the server that the other player has finished the battle. 
	 * Updates the BattleResultScene to display whether the player won or not, and how much they won or lost.
	 * @param winner Tells the game whether the player won or lost.
	 * @param score How many points the player got in this battle.
	 * @param gold How much gold the player won or lost.
	 */
	public void updateBattleResultScene(String winner, String score,String opponentScore, String gold){
		mBattleResultScene.updateScreenWithResults(winner, score,opponentScore,gold, mCurrentBattle.getWeaponValue());
	}

	/**
	 * Called if the player presses the "Extra Information"-button that is next to each question on the BattleResultScene. Opens a web-browser to show
	 * a website where the player can find more information about or answer to a question.
	 * @param address The URL to the site that contains more information about a question. Is stored in the database along with the question itself.
	 */
	public void startBrowser(String address){
		if(address== null || address.equals("")){
			address = Constants.LOGIN_SERVLET;
			String ip = UserFunctions.getServerIp();
			address = address.replace("?", ip);
		}
		Intent i = new Intent(Intent.ACTION_VIEW);
		i.setData(Uri.parse(address));
		startActivity(i);
	}

	/**
	 * When the player uses a weapon, the amount of money used is recorded.
	 * @param cost The cost of the weapon used.
	 */
	public void payCostForWeapon(int cost){
		mCurrentBattle.addToWeaponValue(cost);
	}
	
	public int getMoneySpentOnWeapons(){
		return mCurrentBattle.getWeaponValue();
	}

	/**
	 * Utility-method used to give the icon for each weapon the proper texture
	 * @param w The Weapon that needs an icon
	 * @return The appropriate texture that can be used on the Weapon-button.
	 */
	public TextureRegion getTextureBasedonWeapon(Weapon w){
		String c = w.getmClass()+w.getmRank();
		if(c.equals("B1")){
			return mNinja1Texture;
		}else if(c.equals("B3")){
			return mNinja2Texture;
		}else if(c.equals("C1")){
			return mFan1Texture;
		}else if(c.equals("C3")){
			return mFan2Texture;
		}else if(c.equals("E1")){
			return mWand1Texture;
		}else if(c.equals("E3")){
			return mWand2Texture;
		}else if(c.equals("F1")){
			return mChickenTexture;
		}else if(c.equals("F3")){
			return mCowTexture;
		}
		return mButtonTexture;
	}

	/**
	 * At the beginning of a battle, this class contacts the server to get the questions for the current battle. It extends AsyncTask so that the 
	 * network-access can be done in the background, and not impact the responsiveness of the UI.
	 * @author Adrian
	 *
	 */
	private class QuestionFinder extends AsyncTask<Void, Void, String>{
		@Override
		protected String doInBackground(Void... arg0) {
			String result="";
			try {
				do{
					result = Server.getQuestions(BattleActivity.this.mBattleNr, getApplicationContext());
					if(result.equals(Constants.HTTP_EXECUTE_FAILURE)){
						publishProgress();
						Thread.sleep(Constants.STANDARD_CONNECTION_WAIT_TIME);
					}
				}while(result.equals(Constants.HTTP_EXECUTE_FAILURE));
			} catch (InterruptedException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}

			return result;
		}

		@Override
		protected void onProgressUpdate(Void... values) {
			Toast.makeText(getApplicationContext(), getString(R.string.server_could_not_contact), Toast.LENGTH_SHORT).show();
		}

		protected void onPostExecute(String result){
			try {
				JSONArray questions = new JSONArray(result);
				mCurrentBattle.populateQuestionList(questions);
				createTransitionScene();
			} catch (JSONException e) {
				Log.v("Knowledgewar","Battleactivity: exception caught "+e.getLocalizedMessage());
				e.printStackTrace();
			}
		}
	}


	/**
	 * Called after the player has either selected a weapon in RightAnswerScene, or accepted that he doesn't get to choose a 
	 * weapon in WrongAnswerScene
	 * @author Adrian
	 *
	 */
	private class AnswerHandler extends AsyncTask<String, Void, String>{

		@Override
		protected void onPreExecute(){
			mLoadingHUD.displayLoadingAnimation(true);
		}

		@Override
		protected String doInBackground(String... arg0) {
			String result="";
			try {
				do{
					result = Server.getSetDebuffs(getApplicationContext(), mBattleNr, arg0[0]);	
					if(result.equals(Constants.HTTP_EXECUTE_FAILURE)){
						publishProgress();
						Thread.sleep(Constants.STANDARD_CONNECTION_WAIT_TIME);
					}
				}while(result.equals(Constants.HTTP_EXECUTE_FAILURE));
			} catch (InterruptedException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
			return result;
		}

		@Override
		protected void onProgressUpdate(Void... values) {
			Toast.makeText(getApplicationContext(), getString(R.string.server_could_not_contact), Toast.LENGTH_SHORT).show();
		}

		protected void onPostExecute(String result){
			if(result!= null && !result.equals("")){
				mCurrentDebuff = result;				
			}else{
				mCurrentDebuff = "No debuff.";
			}
			mLoadingHUD.displayLoadingAnimation(false);
			if(mCurrentBattle.hasNext()){
				createTransitionScene();				
			}else{
				createNextQuestionScene();
			}
		}

	}

	/**
	 * When the player has finished the battle, this class contacts the server and updates the database with the results. Done in background in order
	 * to not impact responsiveness of UI.
	 * @author Adrian
	 *
	 */
	private class BattleFinisher extends AsyncTask<Void, Void, String>{

		@Override
		protected String doInBackground(Void... arg0) {
			String battleResult = "";
			try {
				do{
					battleResult=Server.playerFinished(getApplicationContext(), mBattleNr, mCurrentBattle.getScore(), mCurrentBattle.getWeaponValue());
					if(battleResult.equals(Constants.HTTP_EXECUTE_FAILURE)){
						publishProgress();
						Thread.sleep(Constants.STANDARD_CONNECTION_WAIT_TIME);
					}
				}while(battleResult.equals(Constants.HTTP_EXECUTE_FAILURE));
			} catch (InterruptedException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}

			return battleResult;
		}

		@Override
		protected void onProgressUpdate(Void... values) {
			Toast.makeText(getApplicationContext(), getString(R.string.server_could_not_contact), Toast.LENGTH_SHORT).show();
		}

		protected void onPostExecute(String result){
			createBattleFinishedScene();
			UserFunctions.setUserBusy(getApplicationContext(), false);
		}

	}

	/**
	 * Leaving the activity during a battle automatically forfeits, so this class is used to make sure that the player needs to press Back twice within
	 * 3 seconds in order to leave the Activity. 
	 * @author Adrian
	 *
	 */
	private class BackButtonTimer extends AsyncTask<Void, Void, Void>{

		@Override
		protected Void doInBackground(Void... arg0) {
			try {
				Thread.sleep(Constants.STANDARD_CONNECTION_WAIT_TIME);
				isBackPressedOnce=false;
			} catch (InterruptedException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
			return null;
		}

	}

	/**
	 * Overrides the Back-button functionality in order to display a warning that leaving the Activity will forfeit the current battle. The
	 * player has to press back twice within 3 seconds to leave the Activity.
	 */
	@Override
	public void onBackPressed() {
		if(mCurrentScene.onBackButtonPressed()){
			
		}else if(!isBackPressedOnce){
			toastOnUIThread(mRes.getString(R.string.battle_back_button_message), Toast.LENGTH_SHORT);
			isBackPressedOnce=true;
			BackButtonTimer timer = new BackButtonTimer();
			timer.execute();
		}else{
			finish();
		}
	}

	/**
	 * An Activity can not interfere with the functionality of the Home-button, so if a player presses that, the battle is not immediately forfeited
	 * in case the button was pressed on accident. Only if the activity is killed by the OS is the battle forfeited. Not ideal, but a necessary compromise.
	 */
	@Override
	protected void onPause() {
		//		if(this.mCurrentQuestionScene!= null){
		//			mCurrentQuestionScene.cancelTimer();
		//		}
		if(!isBattleStarted){

		}else if(!isFinished){
			finish();
		}
		Log.v("Knowledgewar", "Battleacitivity: onpause called");
		super.onPause();
	}

	/**
	 * Make sure the battle if forfeited if the Activity is killed by the player or the OS.
	 */
	@Override
	protected void onStop() {
		Log.v("Knowledgewar", "Battleacitivity: onstop called");
		if(mCurrentBattle!=null && !isFinished){
			mCurrentBattle.forfeitBattle();
			BattleFinisher finisher = new BattleFinisher();
			finisher.execute();	
			if(this.mCurrentQuestionScene!= null){
				mCurrentQuestionScene.cancelTimer();
			}
		}
		super.onStop();
	}


}
