package adrian.master.knowledgewar.gameactivities;

import org.andengine.engine.camera.ZoomCamera;
import org.andengine.engine.options.EngineOptions;
import org.andengine.engine.options.ScreenOrientation;
import org.andengine.engine.options.resolutionpolicy.RatioResolutionPolicy;
import org.andengine.entity.scene.Scene;
import org.andengine.opengl.font.Font;
import org.andengine.opengl.font.FontFactory;
import org.andengine.opengl.texture.TextureOptions;
import org.andengine.opengl.texture.atlas.bitmap.BitmapTextureAtlas;
import org.andengine.opengl.texture.atlas.bitmap.BitmapTextureAtlasTextureRegionFactory;
import org.andengine.opengl.texture.region.TextureRegion;
import org.andengine.opengl.texture.region.TiledTextureRegion;
import org.andengine.ui.activity.SimpleBaseGameActivity;
import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;

import adrian.master.knowledgewar.R;
import adrian.master.knowledgewar.gameconcepts.Player;
import adrian.master.knowledgewar.logic.Constants;
import adrian.master.knowledgewar.logic.Server;
import adrian.master.knowledgewar.logic.UserFunctions;
import adrian.master.knowledgewar.scenes.ReceiveChallengeScene;
import adrian.master.knowledgewar.subscenes.LoadingHUD;
import android.content.Intent;
import android.content.res.Resources;
import android.graphics.Typeface;
import android.os.AsyncTask;
import android.os.Bundle;
import android.util.Log;
import android.widget.Toast;

public class ReceiveChallengeActivity extends SimpleBaseGameActivity {
	
	public int CAMERA_WIDTH = Constants.CAMERA_WIDTH;
	public int CAMERA_HEIGHT = Constants.CAMERA_HEIGHT;
	
	public ZoomCamera mZoomCamera;
	
	private BitmapTextureAtlas mEnemyTile, mRedTile, mYellowTile, mGreenTile;
	private BitmapTextureAtlas mSandBackground;
	private BitmapTextureAtlas mOwnTile;
	private BitmapTextureAtlas mCart, mCartStar, mCartSquare, mCartTriangle, mCartCircle, mCartPentagon;;
	private BitmapTextureAtlas mDune;
	private BitmapTextureAtlas mButton, mWhiteButton;
	private BitmapTextureAtlas mBlankTile;
	private BitmapTextureAtlas mVerticalLine;
	private BitmapTextureAtlas mRedBar, mBlueBar;
	private BitmapTextureAtlas mLeatherBackground, mPaperBackground, mBlack, mCube;

	public TextureRegion mEnemyTileTexture, mRedTileTexture, mYellowTileTexture, mGreenTileTexture;
	public TextureRegion mSandBackgroundTexture;
	public TextureRegion mOwnTileTexture;
	public TextureRegion mCartTexture, mCartStarTexture, mCartSquareTexture, mCartTriangleTexture, mCartCircleTexture, mCartPentagonTexture;
	public TextureRegion mDuneTexture;
	public TextureRegion mButtonTexture, mWhiteButtonTexture;
	public TextureRegion mBlankTileTexture;
	public TextureRegion mVerticalLineTexture;
	public TextureRegion mRedBarTexture, mBlueBarTexture;
	public TextureRegion mLeatherTexture, mPaperTexture, mBlackTexture;
	public TiledTextureRegion mCubeTexture;
	
	public Font mFont, mSmallFont, mBigFontBold;
	public Font mMediumFont, mMediumFontBold;
	public Resources mRes;
	
	public Player mSelf, mOpponent;
	public String battleNr, mBattleSize, mCategory,mCategoryName;
	public int x,y;
	private boolean isBackPressedOnce=false;
	private LoadingHUD mHud;
	
	private ReceiveChallengeScene mScene;
	
	public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        Bundle b = getIntent().getExtras();
        if(b!=null){
        	battleNr = b.getString(Constants.PARAMETER_BATTLENR);   
        	mCategory = b.getString(Constants.PARAMETER_CATEGORY);
        }else{
        	battleNr = "Finner ikke b";
        }
        
        this.mRes = getResources();
        
	}

	@Override
	public EngineOptions onCreateEngineOptions() {
		mZoomCamera = new ZoomCamera(0, 0, CAMERA_WIDTH, CAMERA_HEIGHT);
		final EngineOptions engineOptions = new EngineOptions(false, ScreenOrientation.PORTRAIT_FIXED, 
				new RatioResolutionPolicy(CAMERA_WIDTH, CAMERA_HEIGHT), this.mZoomCamera);
		return engineOptions;
	}

	@Override
	protected void onCreateResources() {
		BitmapTextureAtlasTextureRegionFactory.setAssetBasePath("gfx/");
		
		this.mEnemyTile = new BitmapTextureAtlas(this.getTextureManager(), Constants.TILE_SIZE, Constants.TILE_SIZE, TextureOptions.REPEATING_BILINEAR_PREMULTIPLYALPHA);
		mEnemyTileTexture =BitmapTextureAtlasTextureRegionFactory.createFromAsset(this.mEnemyTile, this, "redtile.png", 0, 0);
		mEnemyTileTexture.setTextureSize(Constants.TILE_SIZE,Constants.TILE_SIZE);
		this.mEnemyTile.load();
		
		this.mRedTile = new BitmapTextureAtlas(this.getTextureManager(), Constants.TILE_SIZE, Constants.TILE_SIZE, TextureOptions.REPEATING_BILINEAR);
		mRedTileTexture =BitmapTextureAtlasTextureRegionFactory.createFromAsset(this.mRedTile, this, "redtile.png", 0, 0);
		this.mRedTile.load();

		this.mYellowTile = new BitmapTextureAtlas(this.getTextureManager(), Constants.TILE_SIZE, Constants.TILE_SIZE, TextureOptions.REPEATING_BILINEAR);
		mYellowTileTexture =BitmapTextureAtlasTextureRegionFactory.createFromAsset(this.mYellowTile, this, "yellowtile.png", 0, 0);
		this.mYellowTile.load();

		this.mGreenTile = new BitmapTextureAtlas(this.getTextureManager(), Constants.TILE_SIZE, Constants.TILE_SIZE, TextureOptions.REPEATING_BILINEAR);
		mGreenTileTexture =BitmapTextureAtlasTextureRegionFactory.createFromAsset(this.mGreenTile, this, "greentile.png", 0, 0);
		this.mGreenTile.load();
		
		this.mBlankTile = new BitmapTextureAtlas(this.getTextureManager(), Constants.TILE_SIZE, Constants.TILE_SIZE, TextureOptions.REPEATING_BILINEAR_PREMULTIPLYALPHA);
		mBlankTileTexture = BitmapTextureAtlasTextureRegionFactory.createFromAsset(this.mBlankTile, this, "blanktile.png", 0, 0);
		mBlankTileTexture.setTextureSize(Constants.TILE_SIZE,Constants.TILE_SIZE);
		this.mBlankTile.load();
		
		this.mSandBackground = new BitmapTextureAtlas(this.getTextureManager(), Constants.TILE_SIZE*8, Constants.TILE_SIZE*8, TextureOptions.REPEATING_BILINEAR_PREMULTIPLYALPHA);
		this.mSandBackgroundTexture = BitmapTextureAtlasTextureRegionFactory.createFromAsset(this.mSandBackground, this, "sand.png", 0,0);
		this.mSandBackground.load();
		
		this.mVerticalLine = new BitmapTextureAtlas(this.getTextureManager(), 2,Constants.TILE_SIZE,  TextureOptions.REPEATING_BILINEAR_PREMULTIPLYALPHA);
		this.mVerticalLineTexture = BitmapTextureAtlasTextureRegionFactory.createFromAsset(this.mVerticalLine, this, "verticalline.png", 0,0);
		this.mVerticalLine.load();

		this.mOwnTile = new BitmapTextureAtlas(this.getTextureManager(), Constants.TILE_SIZE, Constants.TILE_SIZE, TextureOptions.REPEATING_BILINEAR);
		this.mOwnTileTexture = BitmapTextureAtlasTextureRegionFactory.createFromAsset(this.mOwnTile, this, "owntile2.png", 0,0);
		this.mOwnTileTexture.setTextureSize(Constants.TILE_SIZE, Constants.TILE_SIZE);
		this.mOwnTile.load();
		
		this.mButton = new BitmapTextureAtlas(this.getTextureManager(), 512, 256, TextureOptions.BILINEAR_PREMULTIPLYALPHA);
		this.mButtonTexture = BitmapTextureAtlasTextureRegionFactory.createFromAsset(this.mButton, this, "button_blank.png", 0, 0);
		this.mButton.load();
		
		this.mWhiteButton = new BitmapTextureAtlas(this.getTextureManager(), 512, 256, TextureOptions.BILINEAR_PREMULTIPLYALPHA);
		this.mWhiteButtonTexture = BitmapTextureAtlasTextureRegionFactory.createFromAsset(this.mWhiteButton, this, "button_white.png", 0, 0);
		this.mWhiteButton.load();
		
		this.mDune = new BitmapTextureAtlas(this.getTextureManager(), 1280, 480, TextureOptions.BILINEAR_PREMULTIPLYALPHA);
		this.mDuneTexture = BitmapTextureAtlasTextureRegionFactory.createFromAsset(this.mDune, this, "dune.png", 0, 0);
		this.mDune.load();
		
		this.mCart = new BitmapTextureAtlas(this.getTextureManager(), 256, 192, TextureOptions.BILINEAR_PREMULTIPLYALPHA);
		this.mCartTexture = BitmapTextureAtlasTextureRegionFactory.createFromAsset(this.mCart, this, "cart.png", 0, 0);
		this.mCart.load();
		
		this.mCartCircle = new BitmapTextureAtlas(this.getTextureManager(), 256, 192, TextureOptions.BILINEAR_PREMULTIPLYALPHA);
		this.mCartCircleTexture = BitmapTextureAtlasTextureRegionFactory.createFromAsset(this.mCartCircle, this, "cart_circle.png", 0, 0);
		this.mCartCircle.load();
		
		this.mCartStar = new BitmapTextureAtlas(this.getTextureManager(), 256, 192, TextureOptions.BILINEAR_PREMULTIPLYALPHA);
		this.mCartStarTexture = BitmapTextureAtlasTextureRegionFactory.createFromAsset(this.mCartStar, this, "cart_star.png", 0, 0);
		this.mCartStar.load();
		
		this.mCartSquare = new BitmapTextureAtlas(this.getTextureManager(), 256, 192, TextureOptions.BILINEAR_PREMULTIPLYALPHA);
		this.mCartSquareTexture = BitmapTextureAtlasTextureRegionFactory.createFromAsset(this.mCartSquare, this, "cart_square.png", 0, 0);
		this.mCartSquare.load();
		
		this.mCartTriangle = new BitmapTextureAtlas(this.getTextureManager(), 256, 192, TextureOptions.BILINEAR_PREMULTIPLYALPHA);
		this.mCartTriangleTexture = BitmapTextureAtlasTextureRegionFactory.createFromAsset(this.mCartTriangle, this, "cart_triangle.png", 0, 0);
		this.mCartTriangle.load();
		
		this.mCartPentagon = new BitmapTextureAtlas(this.getTextureManager(), 256, 192, TextureOptions.BILINEAR_PREMULTIPLYALPHA);
		this.mCartPentagonTexture = BitmapTextureAtlasTextureRegionFactory.createFromAsset(this.mCartPentagon, this, "cart_pentagon.png", 0, 0);
		this.mCartPentagon.load();
		
		this.mRedBar = new BitmapTextureAtlas(this.getTextureManager(), 32, 16, TextureOptions.BILINEAR_PREMULTIPLYALPHA);
		this.mRedBarTexture = BitmapTextureAtlasTextureRegionFactory.createFromAsset(this.mRedBar, this, "red_bar.png", 0, 0);
		this.mRedBar.load();
		
		this.mBlueBar = new BitmapTextureAtlas(this.getTextureManager(), 32, 16, TextureOptions.BILINEAR_PREMULTIPLYALPHA);
		this.mBlueBarTexture = BitmapTextureAtlasTextureRegionFactory.createFromAsset(this.mBlueBar, this, "blue_bar.png", 0, 0);
		this.mBlueBar.load();
		
		this.mLeatherBackground = new BitmapTextureAtlas(this.getTextureManager(), 1100, 850, TextureOptions.BILINEAR_PREMULTIPLYALPHA);
		this.mLeatherTexture = BitmapTextureAtlasTextureRegionFactory.createFromAsset(this.mLeatherBackground, this, "leather_scroll2.png",0,0);
		this.mLeatherBackground.load();
		
		this.mPaperBackground = new BitmapTextureAtlas(this.getTextureManager(), 1023, 601, TextureOptions.BILINEAR);
		this.mPaperTexture = BitmapTextureAtlasTextureRegionFactory.createFromAsset(this.mPaperBackground, this, "alternative_background.png",0,0);
		this.mPaperBackground.load();
		
		this.mBlack = new BitmapTextureAtlas(this.getTextureManager(), Constants.TILE_SIZE, Constants.TILE_SIZE, TextureOptions.REPEATING_BILINEAR_PREMULTIPLYALPHA);
		this.mBlackTexture = BitmapTextureAtlasTextureRegionFactory.createFromAsset(this.mBlack, this, "black.png", 0,0);
		this.mBlack.load();
		
		this.mCube = new BitmapTextureAtlas(this.getTextureManager(), 300, 360, TextureOptions.BILINEAR);
		this.mCubeTexture = BitmapTextureAtlasTextureRegionFactory.createTiledFromAsset(mCube, this, "cube.png",0,0,5,6);
		this.mCube.load();
		
		this.mFont = FontFactory.create(this.getFontManager(), this.getTextureManager(), 256, 256, TextureOptions.BILINEAR, Typeface.create(Typeface.DEFAULT, Typeface.BOLD),24);
		this.mFont.load();
		
		this.mBigFontBold = FontFactory.create(this.getFontManager(), this.getTextureManager(), 256, 256, TextureOptions.BILINEAR, Typeface.create(Typeface.DEFAULT, Typeface.BOLD),30);
		this.mBigFontBold.load();
		
		this.mMediumFont = FontFactory.create(this.getFontManager(), this.getTextureManager(), 256, 256, TextureOptions.BILINEAR, Typeface.create(Typeface.DEFAULT, Typeface.NORMAL),20);
		this.mMediumFont.load();
		
		this.mMediumFontBold = FontFactory.create(this.getFontManager(), this.getTextureManager(), 256, 256, TextureOptions.BILINEAR, Typeface.create(Typeface.DEFAULT, Typeface.BOLD),20);
		this.mMediumFontBold.load();
		
		this.mSmallFont = FontFactory.create(this.getFontManager(), this.getTextureManager(), 256, 256, TextureOptions.BILINEAR, Typeface.create(Typeface.DEFAULT, Typeface.BOLD),12);
		this.mSmallFont.load();

	}

	@Override
	protected Scene onCreateScene() {
		this.mHud = new LoadingHUD(mBlackTexture, mCubeTexture, getVertexBufferObjectManager());
		this.mScene = new ReceiveChallengeScene(this,mEngine);
        BattleInfoFinder finder = new BattleInfoFinder();
        finder.execute();
		return mScene;
	}
	
	/**
	 * Called when the player accepts an incoming challenge.
	 */
	public void acceptChallenge(){
//		UserFunctions.acceptChallenge(battleNr, mSelf.getName(), mOpponent.getName(), getApplicationContext());
		Intent intent = new Intent(this, BattleActivity.class);
		intent.putExtra(Constants.PARAMETER_SENT_CHALLENGE, false);
		intent.putExtra(Constants.PARAMETER_BATTLENR, battleNr);
		intent.putExtra(Constants.PARAMETER_SELF, mSelf.getName());
		intent.putExtra(Constants.PARAMETER_OPPONENT, mOpponent.getName());
		startActivity(intent);
		finish();
	}
	
	/**
	 * Called when the player rejects the incoming challenge.
	 */
	public void rejectChallenge(){
		UserFunctions.rejectChallenge(battleNr, mSelf.getName(), mOpponent.getName(), getApplicationContext());
		finish();
	}
	
	/**
	 * Displays loading-animation while the game is doing necessary background-work, like talking to the server.
	 * @param show
	 */
	public void displayLoadingAnimation(boolean show){
		mHud.displayLoadingAnimation(show);
	}
	
	/**
	 * Background-task that asks the server for information about the received challenge.
	 * @author Adrian
	 *
	 */
	private class BattleInfoFinder extends AsyncTask<Void,Void,String>{

		@Override
		protected void onPreExecute() {
			displayLoadingAnimation(true);
		}
		
		@Override
		protected String doInBackground(Void... arg0) {
			String risk = "";
			mBattleSize="";
			String self="";
			String result="";
			try {
//				UserFunctions.getOwnStatsFromServer(getApplicationContext());
				do{
					self = Server.getOwnStats(getApplicationContext());
					if(self.equals(Constants.HTTP_EXECUTE_FAILURE)){
						publishProgress();
						Thread.sleep(Constants.STANDARD_CONNECTION_WAIT_TIME);
					}
				}while(self.equals(Constants.HTTP_EXECUTE_FAILURE));
				
				JSONObject obj = new JSONObject(self);
				String name = obj.getString(Constants.PLAYER_ACCOUNT);
				String ownArea = obj.getString(Constants.PLAYER_AREA);
				int gold = Integer.parseInt(obj.getString(Constants.PLAYER_GOLD));
//				boolean busy = obj.getBoolean(Constants.PARAMETER_BUSY);
				mSelf = new Player(name, ownArea, gold);
				
				do{
					result = Server.getBattleInfo(battleNr,mCategory, getApplicationContext());
					if(result.equals(Constants.HTTP_EXECUTE_FAILURE)){
						publishProgress();
						Thread.sleep(Constants.STANDARD_CONNECTION_WAIT_TIME);
					}
				}while(result.equals(Constants.HTTP_EXECUTE_FAILURE));
				
				JSONArray ar = new JSONArray(result);
				risk = (String)ar.get(1);
				
				mBattleSize = (String)ar.get(2);
				
				JSONObject playerInfo = (JSONObject)ar.get(3);
				String account = playerInfo.getString(Constants.PLAYER_ACCOUNT);
				String area = playerInfo.getString(Constants.PLAYER_AREA);
				mOpponent = new Player(account, area, false);

				JSONObject categoryName = (JSONObject)ar.get(4);
				mCategoryName = categoryName.getString(Constants.PARAMETER_CATEGORY);
			} catch (JSONException e) {
				Log.v("Knowledgewar","ReceiveChallengeactivity: Exception thrown "+e.getLocalizedMessage());
				e.printStackTrace();
			} catch (InterruptedException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
			return risk;
		}
		
		@Override
		protected void onPostExecute(String risk){
//			mSelf=UserFunctions.self;
			int[] xy = UserFunctions.calculateAreaRanges(risk, mOpponent, mSelf);
			x=xy[0];
			y=xy[1];
			mScene.initialize(mSelf, mOpponent);
			displayLoadingAnimation(false);
		}
		
	}
	
	private class BackButtonTimer extends AsyncTask<Void, Void, Void>{

		@Override
		protected Void doInBackground(Void... arg0) {
			try {
				Thread.sleep(3000);
				isBackPressedOnce=false;
			} catch (InterruptedException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
			return null;
		}
		
	}
	
	@Override
	public void onBackPressed() {
		if(!isBackPressedOnce){
			toastOnUIThread(mRes.getString(R.string.receivechallenge_backbutton_pressed), Toast.LENGTH_SHORT);
			isBackPressedOnce=true;
			BackButtonTimer timer = new BackButtonTimer();
			timer.execute();
		}else{
			super.onBackPressed();
		}
	}
	
	@Override
	protected void onPause() {
		// TODO Auto-generated method stub
		super.onPause();
	}
	

}
