package adrian.master.knowledgewar.gameconcepts;

import java.util.ArrayList;

/**
 * The Building-class is used to keep track of the different existing buildings. Buildings are created by the BuildingFactory-class.
 * This class stores information about what kind of bonuses a building gives, and what weapons it gives access to.
 * @author Adrian
 *
 */
public class Building {
	
	private int x, y;
	private String mType;
	private String mName;
	private int mRank;
	private int mValue;
	private int mHealth;
	private int mUpgradeCost;
	private String mDescription;
	
	private float mGoldBonus;
	private int mDamageBonus;
	private int mHealthBonus;
	private float mGoldPenalty;
	
	private boolean mGlobal = false;
	private ArrayList<Weapon> mWeapons;
	
	/**
	 * This constructor is called by BuildingFactory, and is used to create the Building-templates that will later be used to create Buildings from.
	 * The templates are never used directly.
	 * @param type
	 * @param name
	 * @param rank
	 * @param value
	 * @param health
	 * @param global
	 * @param goldBonus
	 * @param damageBonus
	 * @param healthBonus
	 * @param goldPenalty
	 * @param upgradeCost
	 * @param description
	 */
	public Building(String type, String name, int rank, int value, int health, boolean global, float goldBonus, int damageBonus,
			int healthBonus, float goldPenalty, int upgradeCost, String description){
		this.mType = type;
		this.mName = name;
		this.mRank = rank;
		this.mValue = value;
		this.mHealth = health;
		this.mUpgradeCost = upgradeCost;
		this.mDescription = description;
		
		this.mGoldBonus = goldBonus;
		this.mDamageBonus = damageBonus;
		this.mHealthBonus = healthBonus;
		this.mGoldPenalty = goldPenalty;
		
		this.mGlobal = global;
		this.mWeapons = WeaponFactory.getAllWeaponsofClass(mType);
	}
	
	/**
	 * Copy constructor to make a new copy of a building template. This is used when the game needs a building for use. The templates are never used directly.
	 * @return
	 */
	public static Building getNewBuilding(Building b){
		return new Building(b.getType(),b.getName(), b.getRank(), b.getValue(), b.getHealth(), b.isGlobal(), b.getGoldBonus(),
				b.getDamageBonus(), b.getHealthBonus(), b.getGoldPenalty(), b.getUpgradeCost(), b.getDescription());
	}
	
	public String getType(){
		return mType;
	}
	
	
	/**
	 * Returns the string-representation of this building. The format is
	 * <type>:<rank>:<x-coordinate>,<y-coordinate>
	 */
	public String toString(){
		return ";"+mType+":"+mRank+":"+x+","+y;
	}
	

	public int getX(){
		return x;
	}
	
	public int getY(){
		return y;
	}
	
	public void setX(int px){
		this.x=px;
	}
	
	public void setY(int py){
		this.y=py;
	}
	
	public boolean isGlobal(){
		return mGlobal;
	}
	
	public String getName(){
		return mName;
	}
	
	public int getHealth(){
		return mHealth;
	}
	
	public int getHealthBonus(){
		return mHealthBonus;
	}
	
	public float getGoldBonus(){
		return mGoldBonus;
	}
	
	public float getGoldPenalty(){
		return mGoldPenalty;
	}
		
	public int getDamageBonus(){
		return mDamageBonus;
	}
	
	public int getValue(){
		return mValue;
	}
	
	public int getUpgradeCost(){
		return mUpgradeCost;
	}
		
	public int getRank(){
		return mRank;
	}
	
	public String getDescription(){
		return mDescription;
	}
	
	public void updatePosition(int modifier){
		x+=modifier;
		y+=modifier;
	}
	
	public ArrayList<Weapon> getWeapons(){
		return mWeapons;
	}

}
