package adrian.master.knowledgewar.gameconcepts;

import org.json.JSONException;
import org.json.JSONObject;

import adrian.master.knowledgewar.logic.Constants;

/**
 * Class that stores information about a single question, like what are the alternatives and what is the correct answer.
 * @author Adrian
 *
 */
public class Question {
	
	private int id;
	private String question;
	private String[] alternatives = new String[4];
	private int correctAlternative;
	private String answerURL;
	
	/**
	 * Extra constructor used to display question that are stored in the local database. Alternatives are not included.
	 * @param id
	 * @param question
	 * @param url
	 */
	public Question(int id, String question, String url){
		this.id=id;
		this.question=question;
		this.answerURL=url;
	}
	
	/**
	 * Main constructor. Used to create the Questions-objects that are used in battles.
	 * @param obj
	 */
	public Question(JSONObject obj){
		try {
			id = Integer.parseInt(obj.getString(Constants.QUESTION_KEY));
			question = obj.getString(Constants.QUESTION_QUESTION);
			alternatives[0] = obj.getString(Constants.QUESTION_ALT1);
			alternatives[1] = obj.getString(Constants.QUESTION_ALT2);
			alternatives[2] = obj.getString(Constants.QUESTION_ALT3);
			alternatives[3] = obj.getString(Constants.QUESTION_ALT4);
			correctAlternative = obj.getInt(Constants.QUESTION_CORRECT);
			answerURL = obj.getString(Constants.QUESTION_URL);
		} catch (JSONException e) {
			// TODO Handle JSONExcepction in Question-constructor
			e.printStackTrace();
		}
	}
	
	public int getId(){
		return id;
	}
	
	public String getQuestion(){
		return question;
	}
	
	public String getAlternative(int number){
		return alternatives[number-1];
	}
	
	public String[] getAlternatives(){
		return alternatives;
	}
	
	public String getCorrectAlternative(){
		return alternatives[correctAlternative-1];
	}
	
	public boolean isAnswerCorrect(int nr){
		return nr==correctAlternative;
	}
	
	public String getURL(){
		return answerURL;
	}
	
	@Override
	public String toString(){
		String result = "";
		result+=getQuestion()+"\n";
		for(int i=1;i<5;i++){
			result+=getAlternative(i)+"\n";
		}
		result+=correctAlternative+": "+getCorrectAlternative();
		return result;
	}

}
