package adrian.master.knowledgewar.gameconcepts;

import java.io.IOException;
import java.io.InputStream;
import java.util.ArrayList;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;
import org.xml.sax.SAXException;

import adrian.master.knowledgewar.R;
import android.content.Context;

public class WeaponFactory {

	private static ArrayList<Weapon> mWeapons;

	private static Weapon constructWeapon(String name, String desc, String attackdesc, String clas, int rank, int cost){
		return new Weapon(clas, rank, name, desc,attackdesc, cost);
	}
	/**
	 * Parses the xml-file containing information about Weapons, and creates a List containing templates that will be copied when the game needs them.
	 * @param context
	 */
	public static void parseXML(Context context){
		mWeapons  = new ArrayList<Weapon>();
		DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
		Document dom;
		try {
			InputStream catDoc = context.getResources().openRawResource(R.raw.weapons);
			DocumentBuilder db = dbf.newDocumentBuilder();
			dom = db.parse(catDoc, null);

			Element doc = dom.getDocumentElement();
			NodeList nl = doc.getElementsByTagName("weapon");

			if(nl!=null){
				for(int i=0;i<nl.getLength();i++){
					Element el = (Element)nl.item(i);
					constructWeaponTemplate(el);
				}
			}
		} catch (ParserConfigurationException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (SAXException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}

	/**
	 * Used during parsing of the xml-file.
	 * @param el
	 */
	private static void constructWeaponTemplate(Element el){
		String name = getNodeValue(el, "name");
		String description = getNodeValue(el, "description");
		String attackdescription = getNodeValue(el, "attackdescription");
		String clas = getNodeValue(el, "class");
		int rank = Integer.parseInt(getNodeValue(el, "rank"));
		int cost = Integer.parseInt(getNodeValue(el, "cost"));
		Weapon weapon = constructWeapon(name, description, attackdescription, clas, rank, cost);
		mWeapons.add(weapon);
	}

	/**
	 * Used during parsing of the xml-file
	 * @param el
	 * @param tagName
	 * @return
	 */
	private static String getNodeValue(Element el, String tagName){
		String value="";

		NodeList nl = el.getElementsByTagName(tagName);
		if(nl!=null){
			Element ele = (Element)nl.item(0);
			value = ele.getFirstChild().getNodeValue();
		}

		return value;
	}

	/**
	 * Returns all weapons that belongs to a specific building.
	 * @param buildingClass
	 * @return
	 */
	public static ArrayList<Weapon> getAllWeaponsofClass(String buildingClass){
		ArrayList<Weapon> result = new ArrayList<Weapon>();

		for(Weapon w: mWeapons){
			if(w.getmClass().equals(buildingClass)){
				result.add(Weapon.getNewWeapon(w));
			}
		}

		return result;
	}

	/**
	 * Returns a specific weapon based on a debuff-string fetched from the server. Used to display information about an attack when the player is attacked.
	 * @param debuff
	 * @return
	 */
	public static Weapon getWeaponBasedOnDebuff(String debuff){
		if(!debuff.equals("")){
			for(Weapon w: mWeapons){
				String weapon = w.getmClass()+w.getmRank();
				if(weapon.equals(debuff)){
					return w;
				}
			}
		}
		return null;
	}

}
