package adrian.master.knowledgewar.logic;

import adrian.master.knowledgewar.R;
import adrian.master.knowledgewar.gameactivities.BattleActivity;
import adrian.master.knowledgewar.gameactivities.MainActivity;
import adrian.master.knowledgewar.gameactivities.ReceiveChallengeActivity;
import adrian.master.knowledgewar.gameconcepts.Challenge;
import android.app.Notification;
import android.app.NotificationManager;
import android.app.PendingIntent;
import android.content.Context;
import android.content.Intent;
import android.media.RingtoneManager;
import android.net.Uri;
import android.os.Handler;
import android.os.Looper;
import android.support.v4.app.NotificationCompat;
import android.util.Log;
import android.widget.Toast;

/**
 * Class that handles messages pushed by the server. Messages are received by GCMIntentService.class and then this class handles the details.
 * @author Adrian
 *
 */
public class PushMessageHandler {

	/**
	 * Called when the player receives a challenge sent by another player.
	 * @param context
	 * @param intent
	 */
	public static void receiveChallenge(Context context, Intent intent){
		Log.v("Knowledgwar.pushmessagehandler", "Challenge received!");
		
		String ns = Context.NOTIFICATION_SERVICE;
		NotificationManager mNotificationManager = (NotificationManager) context.getSystemService(ns);

		int icon = R.drawable.cart;
		CharSequence tickerText = context.getString(R.string.duel);
		CharSequence contentTitle = context.getString(R.string.NOTIFICATION_CHALLENGED);
		String battleNr = intent.getStringExtra(Constants.PARAMETER_BATTLENR);
		String challenger = intent.getStringExtra(Constants.PARAMETER_CHALLENGER);
		String categoryID = intent.getStringExtra(Constants.PARAMETER_CATEGORY);

		UserFunctions.addChallengeToList(new Challenge(battleNr, challenger, categoryID));
		
		Intent notificationIntent = new Intent(context, ReceiveChallengeActivity.class);
		notificationIntent.putExtra(Constants.PARAMETER_BATTLENR, battleNr);
		notificationIntent.putExtra(Constants.PARAMETER_CATEGORY, categoryID);
		notificationIntent.addFlags(Intent.FLAG_ACTIVITY_SINGLE_TOP | Intent.FLAG_ACTIVITY_CLEAR_TOP);
		PendingIntent pendingNotificationIntent = PendingIntent.getActivity(context,0,notificationIntent,PendingIntent.FLAG_UPDATE_CURRENT);

		Uri soundUri = RingtoneManager.getDefaultUri(RingtoneManager.TYPE_NOTIFICATION);
		int ID = Integer.parseInt(battleNr);
		
		NotificationCompat.Builder mBuild = new NotificationCompat.Builder(context)
			.setSmallIcon(icon)
			.setContentTitle(contentTitle)
			.setTicker(tickerText)
			.setContentIntent(pendingNotificationIntent)
			.setSound(soundUri)
			.setAutoCancel(true);
		
		mNotificationManager.notify(ID, mBuild.build());
	}

	/**
	 * Called when both players are ready and the battle starts.
	 * @param context
	 * @param intent
	 */
	public static void startRound(Context context, Intent intent){
		String battleNr = intent.getStringExtra(Constants.PARAMETER_BATTLENR);
		BattleActivity.mThis.startBattle(battleNr);
	}

	public static void challengeAccepted(Context context, Intent intent){
		Log.v("Knowledgewar, ChallengeAccepted", "Mottatt challenge accepted notifikasjon!");
	}
	
	public static void challengeImpossible(Context context, Intent intent){
		
	}

	/**
	 * Called when the player challenged another player to a challenge, but the other challenge declined.
	 * @param context
	 * @param intent
	 */
	public static void challengeDeclined(final Context context, Intent intent){
		String challengee = intent.getStringExtra(Constants.PARAMETER_CHALLENGEE);
		String profits = intent.getStringExtra(Constants.PARAMETER_RESULT);
		UserFunctions.setUserBusy(context, false);
		BattleActivity.mThis.challengeDeclined(challengee, profits);
		
	}

	/**
	 * Called when the player declined a challenge. This contains information about the results.
	 * @param context
	 * @param intent
	 */
	public static void youDeclinedChallenge(final Context context, Intent intent){
		Log.v("Knowledgewar, YoudeclinedChallenge", "Mottatt GCM om at du declined challenge");
		String challenger = intent.getStringExtra(Constants.PARAMETER_CHALLENGER);
		String battleNr = intent.getStringExtra(Constants.PARAMETER_BATTLENR);
		String loss = intent.getStringExtra(Constants.PARAMETER_RESULT);
		int intLoss = Integer.parseInt(loss);
		intLoss = Math.abs(intLoss);
		loss = intLoss+"";
		
		final String line1 = context.getResources().getString(R.string.toast_you_declined_challenge)+" "+challenger;
		String temp = context.getResources().getString(R.string.toast_you_declined_challenge_gold);
		final String line2 = temp.replace("?", loss);
		
		
		Handler h = new Handler(Looper.getMainLooper());
		h.post(new Runnable(){

			public void run() {
				Toast.makeText(context, line1+"\n"+line2, Toast.LENGTH_LONG).show();
			}         
		});
		UserFunctions.removeChallengeNotification(context, battleNr);
		UserFunctions.getOwnStatsInBackground(context);
	}

	/**
	 * Called after a battle is finished, and sets up the battleresult-screen. If the battleresult-screen has been closed, it instead 
	 * shows a Toast containing the results.
	 * @param context
	 * @param intent
	 */
	@SuppressWarnings("deprecation")
	public static void battleFinished(Context context, Intent intent){
		String gold = intent.getStringExtra(Constants.PARAMETER_GOLD);
		String winner = intent.getStringExtra(Constants.PARAMETER_RESULT);
		String score = intent.getStringExtra(Constants.PARAMETER_SCORE);
		String opponentScore = intent.getStringExtra(Constants.PARAMETER_OPPONENTSCORE);
		String battleNr = intent.getStringExtra(Constants.PARAMETER_BATTLENR);

		if(BattleActivity.mThis != null && BattleActivity.mThis.mCurrentBattle.getBattleNr().equals(battleNr)){
			BattleActivity.mThis.updateBattleResultScene(winner, score, opponentScore, gold);
		}else if(BattleActivity.mThis == null){
			String tickerText;
			String message="";
			if(winner.equals(Constants.PARAMETER_WINNER)){
				tickerText = context.getString(R.string.notification_duel_declined);
				message = context.getString(R.string.notification_you_won_profits);
				message = message.replace("?", gold);
			}else{
				tickerText = context.getString(R.string.notification_you_lost);
				message = context.getString(R.string.notification_you_lost_loss);
				message = message.replace("?", gold);
			}
			String ns = Context.NOTIFICATION_SERVICE;
			NotificationManager mNotificationManager = (NotificationManager) context.getSystemService(ns);
			int icon = R.drawable.cart;
			long when = System.currentTimeMillis();
			Notification notification = new Notification(icon, tickerText, when);
			String contentTitle = context.getString(R.string.notification_duel_declined_2);
			notification.setLatestEventInfo(context, contentTitle, message, null);
			
			int ID = Integer.parseInt(battleNr);
			mNotificationManager.notify(ID, notification);
		}
	}
	
	private static void challengeDeclinedNotification(Context context, Intent intent){
		String ns = Context.NOTIFICATION_SERVICE;
		NotificationManager mNotificationManager = (NotificationManager) context.getSystemService(ns);

		int icon = R.drawable.cart;
		String challengee = intent.getStringExtra(Constants.PARAMETER_CHALLENGEE);
		String tickerText = context.getString(R.string.notification_duel_declined);
		tickerText = tickerText.replace("?", challengee);
		String contentTitle = context.getString(R.string.notification_duel_declined_2);
		String battleNr = intent.getStringExtra(Constants.PARAMETER_BATTLENR);

		Uri soundUri = RingtoneManager.getDefaultUri(RingtoneManager.TYPE_NOTIFICATION);
		int ID = Integer.parseInt(battleNr);
		
		NotificationCompat.Builder mBuild = new NotificationCompat.Builder(context)
		.setSmallIcon(icon)
		.setContentTitle(contentTitle)
		.setTicker(tickerText)
		.setSound(soundUri);
	
		mNotificationManager.notify(ID, mBuild.build());
	}
	
	/**
	 * Called when the players receives a chat message sent by another player.
	 * @param context
	 * @param intent
	 */
	public static void receiveChatMessage(Context context, Intent intent){
		String message = intent.getStringExtra(Constants.PARAMETER_MESSAGE);
		Log.v("Knowledgewar","Pushmessagehandler message received:"+ message);
		UserFunctions.receiveMessage(context, message);
	}

}
