package adrian.master.knowledgewar.logic;

import adrian.master.knowledgewar.gameactivities.BattleActivity;
import adrian.master.knowledgewar.gameconcepts.Question;
import adrian.master.knowledgewar.scenes.BaseDisplayQuestionScene;
import adrian.master.knowledgewar.scenes.HalfReducedScoreScene;
import adrian.master.knowledgewar.scenes.LostAlternativesScene;
import adrian.master.knowledgewar.scenes.LostQuestionScene;
import adrian.master.knowledgewar.scenes.MirroredQuestionScene;
import adrian.master.knowledgewar.scenes.NoSpacesQuestionScene;
import adrian.master.knowledgewar.scenes.ReducedTimerQuestionScene;
import adrian.master.knowledgewar.scenes.ThreeFourthsReducedScoreScene;

/**
 * Factory-class that gives the BattleActivity the next question-scene based on what weapon is being used on the player.
 * The game receives a code from the server about what is about to happen, and the factory returns a scene based on that.
 * @author Adrian
 *
 */
public class QuestionSceneFactory {
	
	public static BaseDisplayQuestionScene createNextQuestion(BattleActivity parent, Question question, String attack){
		BaseDisplayQuestionScene result;
		
		if(attack.equals(Constants.ATTACK_BUILDINGF_RANK1)){
			result = new ReducedTimerQuestionScene(parent, question, 10);
		}
		else if(attack.equals(Constants.ATTACK_BUILDINGF_RANK2)){
			result = new ReducedTimerQuestionScene(parent, question, 15);
		}
		else if(attack.equals(Constants.ATTACK_BUILDINGB_RANK1)){
			result = new NoSpacesQuestionScene(parent, question);
		}
		else if(attack.equals(Constants.ATTACK_BUILDINGB_RANK2)){
			result = new MirroredQuestionScene(parent, question);
		}
		else if(attack.equals(Constants.ATTACK_BUILDINGE_RANK1)){
			result = new ThreeFourthsReducedScoreScene(parent, question);
		}
		else if(attack.equals(Constants.ATTACK_BUILDINGE_RANK2)){
			result = new HalfReducedScoreScene(parent, question);
		}
		else if(attack.equals(Constants.ATTACK_BUILDINGC_RANK1)){
			result = new LostQuestionScene(parent, question);
		}
		else if(attack.equals(Constants.ATTACK_BUILDINGC_RANK2)){
			result = new LostAlternativesScene(parent, question);
		}
		else{
			result = new BaseDisplayQuestionScene(parent, question);
		}
		
		return result;
	}

}
