package adrian.master.knowledgewar.logic;

import java.io.IOException;
import java.io.InputStreamReader;
import java.io.Reader;
import java.io.UnsupportedEncodingException;
import java.util.ArrayList;
import java.util.List;

import org.apache.http.HttpResponse;
import org.apache.http.NameValuePair;
import org.apache.http.client.ClientProtocolException;
import org.apache.http.client.HttpClient;
import org.apache.http.client.entity.UrlEncodedFormEntity;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.impl.client.DefaultHttpClient;
import org.apache.http.message.BasicNameValuePair;
import org.apache.http.params.BasicHttpParams;
import org.apache.http.params.HttpConnectionParams;
import org.apache.http.params.HttpParams;
import org.json.JSONException;

import adrian.master.knowledgewar.gameconcepts.Building;
import android.content.Context;
import android.util.Log;

/**
 * Class that contains all communication with the server. The game never talks to the server without going through this.
 * @author Adrian
 *
 */
public class Server {

	/**
	 * Called when the user attempts to log in.
	 * @param account_name
	 * @param password
	 * @return String from the server that says whether or not the login was successful.
	 */
	public static String userLogin(String account_name, String password) {
		String serverAddress = (Constants.LOGIN_SERVLET).replace("?", UserFunctions.getServerIp());
		HttpPost httppost = new HttpPost(serverAddress); 

		List<NameValuePair> nameValuePairs = new ArrayList<NameValuePair>();   
		nameValuePairs.add(new BasicNameValuePair(Constants.COMMAND, Constants.COMMAND_LOGIN));
		nameValuePairs.add(new BasicNameValuePair(Constants.PARAMETER_ACCOUNT, account_name));
		nameValuePairs.add(new BasicNameValuePair(Constants.PARAMETER_PASSWORD, password));

		String result = executeLoginHttp(nameValuePairs, httppost);

		return result;
	}

	/**
	 * Called when the user logs out.
	 * @param account
	 * @param pass
	 */
	public static void userLogOut(String account, String pass){
		String serverAddress = (Constants.LOGIN_SERVLET).replace("?", UserFunctions.getServerIp());
		HttpPost httppost = new HttpPost(serverAddress); 

		List<NameValuePair> nameValuePairs = new ArrayList<NameValuePair>();   
		nameValuePairs.add(new BasicNameValuePair(Constants.COMMAND, Constants.COMMAND_LOGOUT));
		nameValuePairs.add(new BasicNameValuePair(Constants.PARAMETER_ACCOUNT, account));
		nameValuePairs.add(new BasicNameValuePair(Constants.PARAMETER_PASSWORD, pass));

		executeLoginHttp(nameValuePairs, httppost);
	}

	/**
	 * Called when the user attempts to register a new account.
	 * @param account
	 * @param password
	 * @return String from the server that says whether or not the registration was successful.
	 */
	public static String registerPlayer(String account, String password){
		String serverAddress = (Constants.LOGIN_SERVLET).replace("?", UserFunctions.getServerIp());
		HttpPost httppost = new HttpPost(serverAddress); 
		

		List<NameValuePair> nameValuePairs = new ArrayList<NameValuePair>();   
		nameValuePairs.add(new BasicNameValuePair(Constants.COMMAND, Constants.COMMAND_REGISTER));
		nameValuePairs.add(new BasicNameValuePair(Constants.PARAMETER_ACCOUNT, account));
		nameValuePairs.add(new BasicNameValuePair(Constants.PARAMETER_PASSWORD, password));

		String result = executeLoginHttp(nameValuePairs, httppost);

		return result;
	}

	/**
	 * Called when the user registers a new GCM id. The GCM id has to be stored on the server, and is used when pushing messages from the server.
	 * @param regId
	 * @param context
	 */
	public static void registerGCM(String regId, Context context){
		String serverAddress = (Constants.GCM_SERVLET).replace("?", UserFunctions.getServerIp());
		HttpPost httppost = new HttpPost(serverAddress); 

		List<NameValuePair> nameValuePairs = new ArrayList<NameValuePair>();
		nameValuePairs.add(new BasicNameValuePair(Constants.COMMAND, Constants.COMMAND_REGISTERGCM));
		nameValuePairs.add(new BasicNameValuePair(Constants.PARAMETER_REGID, regId));

		executeHttp(nameValuePairs, httppost, context);
	}

	/**
	 * Called when the user wants to unregister the GCM id stored on the server.
	 * @param context
	 */
	public static void unRegisterGCM(Context context){
		String serverAddress = (Constants.GCM_SERVLET).replace("?", UserFunctions.getServerIp());
		HttpPost httppost = new HttpPost(serverAddress); 

		List<NameValuePair> nameValuePairs = new ArrayList<NameValuePair>();
		nameValuePairs.add(new BasicNameValuePair(Constants.COMMAND, Constants.COMMAND_REGISTERGCM));
		nameValuePairs.add(new BasicNameValuePair(Constants.PARAMETER_REGID, "NULL"));

		executeHttp(nameValuePairs, httppost, context);
	}

	/**
	 * Called when the user logs on, to verify that the GCM is still valid. The GCM changes if the user changes devices, so the GCM must be
	 * changed also.
	 * @param regId
	 * @param context
	 */
	public static void verifyGCM(String regId, Context context){
		String serverAddress = (Constants.GCM_SERVLET).replace("?", UserFunctions.getServerIp());
		HttpPost httppost = new HttpPost(serverAddress); 

		List<NameValuePair> nameValuePairs = new ArrayList<NameValuePair>();
		nameValuePairs.add(new BasicNameValuePair(Constants.COMMAND, Constants.COMMAND_VERIFYGCM));
		nameValuePairs.add(new BasicNameValuePair(Constants.PARAMETER_REGID, regId));

		executeHttp(nameValuePairs, httppost, context);

	}

	/**
	 * Gets information about the user from the server, such as amount of money, and area.
	 * @param context
	 * @return
	 */
	public static String getOwnStats(Context context){
		String serverAddress = (Constants.PLAYER_SERVLET).replace("?", UserFunctions.getServerIp());
		HttpPost httppost = new HttpPost(serverAddress); 

		List<NameValuePair> nameValuePairs = new ArrayList<NameValuePair>();
		nameValuePairs.add(new BasicNameValuePair(Constants.COMMAND, Constants.COMMAND_GET_OWN_STATS));

		String result = executeHttp(nameValuePairs, httppost, context);
		return result;
	}

	/**
	 * Gets information about other players in the vicinity.
	 * @param context
	 * @return
	 */
	public static String getPlayers(Context context){
		String serverAddress = (Constants.PLAYER_SERVLET).replace("?", UserFunctions.getServerIp());
		HttpPost post = new HttpPost(serverAddress); 

		List<NameValuePair> pairs = new ArrayList<NameValuePair>();
		pairs.add(new BasicNameValuePair(Constants.COMMAND, Constants.COMMAND_GETPLAYERS));

		String result = executeHttp(pairs, post, context);

		return result;
	}

	/**
	 * Sends a challenge to another player.
	 * @param challengee
	 * @param risk
	 * @param size
	 * @param category
	 * @param context
	 * @return
	 */
	public static String sendChallenge(String challengee, String risk, int size,int category, Context context){
		String serverAddress = (Constants.CHALLENGE_SERVLET).replace("?", UserFunctions.getServerIp());
		HttpPost post = new HttpPost(serverAddress); 

		List<NameValuePair> pairs = new ArrayList<NameValuePair>();
		pairs.add(new BasicNameValuePair(Constants.COMMAND, Constants.COMMAND_CHALLENGE));
		pairs.add(new BasicNameValuePair(Constants.PARAMETER_CHALLENGEE, challengee));
		pairs.add(new BasicNameValuePair(Constants.PARAMETER_RISK, risk));
		pairs.add(new BasicNameValuePair(Constants.PARAMETER_SIZE, size+""));
		pairs.add(new BasicNameValuePair(Constants.PARAMETER_CATEGORY, category+""));

		String result = executeHttp(pairs, post, context);
		return result;

	}

	/**
	 * Gets information about a battle, like what area is the contested area.
	 * @param battleNr
	 * @param categoryID
	 * @param context
	 * @return
	 * @throws JSONException
	 */
	public static String getBattleInfo(String battleNr,String categoryID, Context context) throws JSONException{
		String serverAddress = (Constants.BATTLE_SERVLET).replace("?", UserFunctions.getServerIp());
		HttpPost post = new HttpPost(serverAddress); 

		List<NameValuePair> pairs = new ArrayList<NameValuePair>();
		pairs.add(new BasicNameValuePair(Constants.COMMAND, Constants.COMMAND_INFO));
		pairs.add(new BasicNameValuePair(Constants.PARAMETER_BATTLENR, battleNr));
		pairs.add(new BasicNameValuePair(Constants.PARAMETER_CATEGORY, categoryID));

		String result = executeHttp(pairs, post, context);
		//		JSONArray ar = new JSONArray(result);

		return result;

	}

	/**
	 * Accepts a received challenge.
	 * @param battleNr
	 * @param selfName
	 * @param opponentName
	 * @param context
	 * @return
	 */
	public static String acceptChallenge(String battleNr, String selfName, String opponentName, Context context){
		String serverAddress = (Constants.CHALLENGE_SERVLET).replace("?", UserFunctions.getServerIp());
		HttpPost post = new HttpPost(serverAddress); 

		List<NameValuePair> pairs = new ArrayList<NameValuePair>();
		pairs.add(new BasicNameValuePair(Constants.COMMAND, Constants.COMMAND_ACCEPT));
		pairs.add(new BasicNameValuePair(Constants.PARAMETER_CHALLENGEE, selfName));
		pairs.add(new BasicNameValuePair(Constants.PARAMETER_CHALLENGER, opponentName));
		pairs.add(new BasicNameValuePair(Constants.PARAMETER_BATTLENR, battleNr));

		String result = executeHttp(pairs, post, context);
		return result;
	}

	/**
	 * Declines a received challenge.
	 * @param battleNr
	 * @param selfName
	 * @param opponentName
	 * @param context
	 */
	public static void declineChallenge(String battleNr,  String selfName, String opponentName, Context context){
		String serverAddress = (Constants.CHALLENGE_SERVLET).replace("?", UserFunctions.getServerIp());
		HttpPost post = new HttpPost(serverAddress); 

		List<NameValuePair> pairs = new ArrayList<NameValuePair>();
		pairs.add(new BasicNameValuePair(Constants.COMMAND, Constants.COMMAND_DECLINE));
		pairs.add(new BasicNameValuePair(Constants.PARAMETER_CHALLENGEE, selfName));
		pairs.add(new BasicNameValuePair(Constants.PARAMETER_CHALLENGER, opponentName));
		pairs.add(new BasicNameValuePair(Constants.PARAMETER_BATTLENR, battleNr));

		executeHttp(pairs, post, context);
	}

	/**
	 * Gets the questions for a given battle.
	 * @param battleNr
	 * @param context
	 * @return
	 */
	public static String getQuestions(String battleNr, Context context){
		String serverAddress = (Constants.BATTLE_SERVLET).replace("?", UserFunctions.getServerIp());
		HttpPost post = new HttpPost(serverAddress); 
		List<NameValuePair> pairs = new ArrayList<NameValuePair>();
		pairs.add(new BasicNameValuePair(Constants.COMMAND, Constants.COMMAND_GETQUESTIONS));
		pairs.add(new BasicNameValuePair(Constants.PARAMETER_BATTLENR, battleNr));

		String result = executeHttp(pairs, post, context);
		return result;
	}

	/**
	 * After every question, the server has to know what weapons the player uses, and what weapon is used against them. This method 
	 * does both those things.
	 * @param context
	 * @param battleNr
	 * @param attack
	 * @return
	 */
	public static String getSetDebuffs(Context context, String battleNr, String attack){
		String serverAddress = (Constants.BATTLE_SERVLET).replace("?", UserFunctions.getServerIp());
		HttpPost post = new HttpPost(serverAddress); 

		List<NameValuePair> pairs = new ArrayList<NameValuePair>();
		pairs.add(new BasicNameValuePair("command", "answer"));
		pairs.add(new BasicNameValuePair("battlenr", battleNr));
		pairs.add(new BasicNameValuePair("attack", attack));

		String result = executeHttp(pairs, post, context);

		return result;
	}

	/**
	 * Called when the user has finished a battle.
	 * @param context
	 * @param battleNr
	 * @param score
	 * @param costOfWeapons
	 * @return
	 */
	public static String playerFinished(Context context, String battleNr, int score, int costOfWeapons){
		String serverAddress = (Constants.BATTLE_SERVLET).replace("?", UserFunctions.getServerIp());
		HttpPost post = new HttpPost(serverAddress); 

		List<NameValuePair> pairs = new ArrayList<NameValuePair>();
		pairs.add(new BasicNameValuePair(Constants.COMMAND, Constants.COMMAND_FINISH));
		pairs.add(new BasicNameValuePair(Constants.PARAMETER_BATTLENR, battleNr));
		pairs.add(new BasicNameValuePair(Constants.PARAMETER_SCORE, score+""));
		pairs.add(new BasicNameValuePair(Constants.PARAMETER_WEAPONCOST, costOfWeapons+""));

		String result = executeHttp(pairs, post, context);	
		return result;
	}

//	public static String isBattleOver(Context context, String battleNr){
//		HttpPost post = new HttpPost(Constants.BATTLE_SERVLET);
//
//		List<NameValuePair> pairs = new ArrayList<NameValuePair>();
//		pairs.add(new BasicNameValuePair(Constants.COMMAND, Constants.COMMAND_ISFINISHED));
//		pairs.add(new BasicNameValuePair(Constants.PARAMETER_BATTLENR, battleNr));
//
//		String result = executeHttp(pairs, post, context);
//		return result;
//	}

	//	public static String getBattleResult(Context context, String battleNr){
	//		HttpPost post = new HttpPost(Constants.BATTLE_SERVLET);
	//
	//		List<NameValuePair> pairs = new ArrayList<NameValuePair>();
	//		pairs.add(new BasicNameValuePair(Constants.COMMAND, Constants.COMMAND_BATTLE_RESULT));
	//		pairs.add(new BasicNameValuePair(Constants.PARAMETER_BATTLENR, battleNr));
	//
	//		String result = executeHttp(pairs, post, context);
	//		return result;
	//	}

	/**
	 * Updates the registered GPS-position on the server.
	 * @param context
	 * @param latitude
	 * @param longitude
	 */
	public static void updatePosition(Context context, double latitude, double longitude){
		String serverAddress = (Constants.LOGIN_SERVLET).replace("?", UserFunctions.getServerIp());
		HttpPost post = new HttpPost(serverAddress); 

		List<NameValuePair> pairs = new ArrayList<NameValuePair>();
		pairs.add(new BasicNameValuePair(Constants.COMMAND, Constants.COMMAND_UPDATEPOSITION));
		pairs.add(new BasicNameValuePair(Constants.PARAMETER_LATITUDE, latitude+""));
		pairs.add(new BasicNameValuePair(Constants.PARAMETER_LONGITUDE, longitude+""));

		executeHttp(pairs, post, context);
	}

	//	public static void updateStats(Context context, int gold, String area){
	//		HttpPost post = new HttpPost(Constants.PLAYER_SERVLET);
	//
	//		List<NameValuePair> pairs = new ArrayList<NameValuePair>();
	//		pairs.add(new BasicNameValuePair(Constants.COMMAND, Constants.COMMAND_UPDATESTATS));
	//		pairs.add(new BasicNameValuePair(Constants.PARAMETER_AREA, area));
	//		pairs.add(new BasicNameValuePair(Constants.PARAMETER_GOLD, gold+""));
	//
	//		executeHttp(pairs, post, context);
	//	}

	/**
	 * Sends a chat message.
	 * @param context
	 * @param message
	 */
	public static void sendChat(Context context, String message){
		String serverAddress = (Constants.CHAT_SERVLET).replace("?", UserFunctions.getServerIp());
		HttpPost post = new HttpPost(serverAddress); 

		List<NameValuePair> pairs = new ArrayList<NameValuePair>();
		pairs.add(new BasicNameValuePair(Constants.COMMAND, Constants.COMMAND_CHAT));
		pairs.add(new BasicNameValuePair(Constants.PARAMETER_MESSAGE, message));

		executeHttp(pairs, post, context);
	}

//	/
//	public static void sendHeartBeat(Context context){
//		HttpPost post = new HttpPost(Constants.CHAT_SERVLET);
//
//		List<NameValuePair> pairs = new ArrayList<NameValuePair>();
//		pairs.add(new BasicNameValuePair(Constants.COMMAND, Constants.COMMAND_HEARTBEAT));
//
//		executeHttp(pairs, post, context);
//	}

	/**
	 * Gets all the available question-categories from the server.
	 * @param context
	 * @return
	 */
	public static String getQuestionCategories(Context context){
		String serverAddress = (Constants.CHALLENGE_SERVLET).replace("?", UserFunctions.getServerIp());
		HttpPost post = new HttpPost(serverAddress); 

		List<NameValuePair> pairs = new ArrayList<NameValuePair>();
		pairs.add(new BasicNameValuePair(Constants.COMMAND, Constants.COMMAND_GETQUESTIONCATEGORIES));
		String result = executeHttp(pairs, post, context);
		return result;
	}

	/**
	 * When the user wants to build a new building in his tribe, this method tells that to the server.
	 * The server then handles the updating of the user's area, to minimize chance of cheating.
	 * @param context
	 * @param b
	 * @param x
	 * @param y
	 * @return
	 */
	public static String placeNewBuilding(Context context, Building b, int x, int y){
		String serverAddress = (Constants.PLAYER_SERVLET).replace("?", UserFunctions.getServerIp());
		HttpPost post = new HttpPost(serverAddress); 

		String result = "";
		String type = b.getType();
		int rank = b.getRank();

		List<NameValuePair> pairs = new ArrayList<NameValuePair>();
		pairs.add(new BasicNameValuePair(Constants.COMMAND, Constants.COMMAND_PLACEBUILDING));
		pairs.add(new BasicNameValuePair(Constants.PARAMETER_BUILDINGTYPE, type));
		pairs.add(new BasicNameValuePair(Constants.PARAMETER_BUILDINGRANK, rank+""));
		pairs.add(new BasicNameValuePair(Constants.PARAMETER_BUILDINGX, x+""));
		pairs.add(new BasicNameValuePair(Constants.PARAMETER_BUILDINGY, y+""));

		result = executeHttp(pairs, post, context);

		return result;
	}

	/**
	 * When the user wants to update a building in his tribe, this method tells that to the server.
	 * The server then handles the updating of the user's area, to minimize chance of cheating.
	 * @param con
	 * @param b
	 * @param x
	 * @param y
	 * @return
	 */
	public static String upgradeBuilding(Context con, Building b, int x, int y){
		String serverAddress = (Constants.PLAYER_SERVLET).replace("?", UserFunctions.getServerIp());
		HttpPost post = new HttpPost(serverAddress); 

		String result = "";
		String type = b.getType();
		int rank = b.getRank();

		List<NameValuePair> pairs = new ArrayList<NameValuePair>();
		pairs.add(new BasicNameValuePair(Constants.COMMAND, Constants.COMMAND_UPGRADEBUILDING));
		pairs.add(new BasicNameValuePair(Constants.PARAMETER_BUILDINGTYPE, type));
		pairs.add(new BasicNameValuePair(Constants.PARAMETER_BUILDINGRANK, rank+""));
		pairs.add(new BasicNameValuePair(Constants.PARAMETER_BUILDINGX, x+""));
		pairs.add(new BasicNameValuePair(Constants.PARAMETER_BUILDINGY, y+""));

		result = executeHttp(pairs, post, con);

		return result;
	}

	/**
	 * When the user wants to remove a building in his tribe, this method tells that to the server.
	 * The server then handles the updating of the user's area, to minimize chance of cheating.
	 * @param con
	 * @param x
	 * @param y
	 * @return
	 */
	public static String removeBuilding(Context con, int x, int y){
		String serverAddress = (Constants.PLAYER_SERVLET).replace("?", UserFunctions.getServerIp());
		HttpPost post = new HttpPost(serverAddress); 
		String result = "";

		List<NameValuePair> pairs = new ArrayList<NameValuePair>();
		pairs.add(new BasicNameValuePair(Constants.COMMAND, Constants.COMMAND_REMOVEBUILDING));
		pairs.add(new BasicNameValuePair(Constants.PARAMETER_BUILDINGX, x+""));
		pairs.add(new BasicNameValuePair(Constants.PARAMETER_BUILDINGY, y+""));

		result = executeHttp(pairs, post, con);

		return result;
	}

	/**
	 * Method that executes the HTTP post for a login-command.
	 * @param nameValuePairs
	 * @param post
	 * @return
	 */
	private static String executeLoginHttp(List<NameValuePair> nameValuePairs, HttpPost post){
		HttpParams httpParameters = new BasicHttpParams();
		int timeoutConnection = 8000;
		HttpConnectionParams.setConnectionTimeout(httpParameters, timeoutConnection);
		HttpClient httpclient = new DefaultHttpClient(httpParameters);

		try {
			post.setEntity(new UrlEncodedFormEntity(nameValuePairs));
			HttpResponse response = httpclient.execute(post);

			Reader reader = null;
			try {
				reader = new InputStreamReader(response.getEntity().getContent());

				StringBuffer sb = new StringBuffer();
				{
					int read;
					char[] cbuf = new char[1024];
					while ((read = reader.read(cbuf)) != -1)
						sb.append(cbuf, 0, read);
				}

				return sb.toString();

			} finally {
				if (reader != null) {
					try {
						reader.close();
					} catch (IOException e) {
						e.printStackTrace();
					}
				}
			}

		} catch (UnsupportedEncodingException e) {
			e.printStackTrace();
		} catch (ClientProtocolException e) {
			// TODO Handle ClientProtocolException
			Log.v("Knowledgewar","ExecuteHTTP: clientprotocolexc.");
			e.printStackTrace();
			return Constants.HTTP_EXECUTE_FAILURE;
		} catch (IOException e) {
			// TODO Handle IOException
			Log.v("Knowledgewar","ExecuteHTTP: IOexc.");
			e.printStackTrace();
			return Constants.HTTP_EXECUTE_FAILURE;
		} catch (Exception e){
			e.printStackTrace();
			Log.v("Knowledgewar","ExecuteHTTP: "+e.getLocalizedMessage());
			return Constants.HTTP_EXECUTE_FAILURE;
		}
		return Constants.HTTP_EXECUTE_FAILURE;
	}

	/**
	 * Method that executes the HTTP post for every other method in this class (except login). Automatically attaches information like
	 * account and password.
	 * @param nameValuePairs
	 * @param post
	 * @param context
	 * @return
	 */
	private static String executeHttp(List<NameValuePair> nameValuePairs, HttpPost post, Context context){
		HttpParams httpParameters = new BasicHttpParams();
		int timeoutConnection = 3000;
		HttpConnectionParams.setConnectionTimeout(httpParameters, timeoutConnection);
		HttpClient httpclient = new DefaultHttpClient(httpParameters);

		String account = UserFunctions.getUserAccount(context);
		String password = UserFunctions.getUserPassword(context);

		nameValuePairs.add(new BasicNameValuePair(Constants.PARAMETER_ACCOUNT, account));
		nameValuePairs.add(new BasicNameValuePair(Constants.PARAMETER_PASSWORD, password));

		try {
			post.setEntity(new UrlEncodedFormEntity(nameValuePairs));
			Log.v("Knowledgewar","ExecuteHTTP payload size: "+post.getEntity().getContentLength());
			Log.v("Knowledgewar","ExecuteHTTP payload: "+post.getEntity().toString());
			HttpResponse response = httpclient.execute(post);

			Reader reader = null;
			try {
				reader = new InputStreamReader(response.getEntity().getContent());

				StringBuffer sb = new StringBuffer();
				{
					int read;
					char[] cbuf = new char[1024];
					while ((read = reader.read(cbuf)) != -1)
						sb.append(cbuf, 0, read);
				}

				return sb.toString();

			} finally {
				if (reader != null) {
					try {
						reader.close();
					} catch (IOException e) {
						e.printStackTrace();
					}
				}
			}

		} catch (UnsupportedEncodingException e) {
			Log.v("Knowledgewar","ExecuteHTTP: unsupportedencodingexception.");
			e.printStackTrace();
			return "http_failure";
		} catch (ClientProtocolException e) {
			// TODO Handle ClientProtocolException
			Log.v("Knowledgewar","ExecuteHTTP: clientprotocolexc.");
			e.printStackTrace();
			return "http_failure";
		} catch (IOException e) {
			// TODO Handle IOException
			Log.v("Knowledgewar","ExecuteHTTP: IOexc.");
			e.printStackTrace();
			return "http_failure";
		} catch (Exception e){
			e.printStackTrace();
			Log.v("Knowledgewar","ExecuteHTTP: "+e.getLocalizedMessage());
			return "http_failure";
		}
		//		return "executeHttp failed";
	}

}
