package adrian.master.knowledgewar.logic;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.Random;

import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;

import adrian.master.knowledgewar.LocationReceiver;
import adrian.master.knowledgewar.gameconcepts.Challenge;
import adrian.master.knowledgewar.gameconcepts.Player;
import adrian.master.knowledgewar.gameconcepts.Question;
import adrian.master.knowledgewar.subscenes.ChatHUD;
import android.app.AlarmManager;
import android.app.NotificationManager;
import android.app.PendingIntent;
import android.content.Context;
import android.content.Intent;
import android.content.SharedPreferences;
import android.location.LocationManager;
import android.os.AsyncTask;
import android.os.SystemClock;
import android.util.Log;

import com.commonsware.cwac.locpoll.LocationPoller;
import com.google.android.gcm.GCMRegistrar;

/**
 * Class that contains many methods that do lots of different things. 
 * For example all methods that write and read Shared Preferences, methods that should be accessible from many different classes, and so on.
 * @author Adrian
 *
 */
public class UserFunctions {

	private static AlarmManager mgr;
	private static PendingIntent pi;
	public static Player self;
	private static Random mRand;
	private static ArrayList<Challenge> mChallenges;
	private static ArrayList<Player> mPlayerList;
	private static ArrayList<String> mMessages;
	private static Context mApplicationContext;

	public enum PlayerStrength { WEAKER, SAME, STRONGER, DEFAULT };
	
	public static void setApplicationContext(Context con){
		mApplicationContext = con;
	}
	
	public static boolean storeServerIP(Context context, String ip){
		SharedPreferences settings = context.getSharedPreferences(Constants.SHAREDPREFERENCES_NAME, 0);
		SharedPreferences.Editor editor = settings.edit();
		editor.putString(Constants.SHAREDPREFERENCES_IP, ip);

		editor.commit();
		

		return true;
	}
	
	public static String getServerIp(){
		SharedPreferences settings = mApplicationContext.getSharedPreferences(Constants.SHAREDPREFERENCES_NAME, 0);
		String ip = settings.getString(Constants.SHAREDPREFERENCES_IP, "");
		return ip;
	}
	

	/**
	 * Stores username and password in SharedPreferences, so that it can automatically log in when the game is started.
	 * @param context
	 * @param accountname
	 * @param password
	 * @return
	 */
	public static boolean logInUser(Context context, String accountname, String password){

		SharedPreferences settings = context.getSharedPreferences(Constants.SHAREDPREFERENCES_NAME, 0);
		SharedPreferences.Editor editor = settings.edit();

		editor.putBoolean(Constants.SHAREDPREFERENCES_LOGGED_IN, true);
		editor.putString(Constants.SHAREDPREFERENCES_ACCOUNTNAME, accountname);
		editor.putString(Constants.SHAREDPREFERENCES_PASSWORD, password);
//		editor.putString(Constants.SHAREDPREFERENCES_IP, ip);

		editor.commit();

		return true;
	}

	/**
	 * Deletes username and password from SharedPreferences.
	 * @param context
	 * @return
	 */
	public static boolean logOutUser(Context context){

		stopLocationPoller();
		SharedPreferences settings = context.getSharedPreferences(Constants.SHAREDPREFERENCES_NAME, 0);

		//Have to store accountname and username in temporary strings, so that we can use a background process to 
		//tell the server that the user logged out, without running into trouble when the LoginActivity is started and 
		//checks whether or not the user is logged in locally in SharedPreferences.
		String account = settings.getString(Constants.SHAREDPREFERENCES_ACCOUNTNAME, "");
		String pass = settings.getString(Constants.SHAREDPREFERENCES_PASSWORD, "");

		SharedPreferences.Editor editor = settings.edit();

		editor.putBoolean(Constants.SHAREDPREFERENCES_LOGGED_IN, false);
		editor.putString(Constants.SHAREDPREFERENCES_ACCOUNTNAME, "");
		editor.putString(Constants.SHAREDPREFERENCES_PASSWORD, "");

		editor.commit();

		UserLogOuter logout = new UserLogOuter(account, pass);
		logout.execute();

		return true;
	}

	public static boolean isUserLoggedIn(Context context){

		SharedPreferences settings = context.getSharedPreferences(Constants.SHAREDPREFERENCES_NAME, 0);

		boolean loggedIn = settings.getBoolean(Constants.SHAREDPREFERENCES_LOGGED_IN, false);

		return loggedIn;
	}

	public static boolean isSpecificUserLoggedIn(Context context, String account){
		SharedPreferences settings = context.getSharedPreferences(Constants.SHAREDPREFERENCES_NAME, 0);
		boolean loggedIn = settings.getBoolean(Constants.SHAREDPREFERENCES_LOGGED_IN, false);

		if(loggedIn){
			String userLoggedIn = getUserAccount(context);
			if(userLoggedIn.equals(account)){
				return true;
			}
		}
		return false;
	}

	public static String getUserAccount(Context context){
		SharedPreferences settings = context.getSharedPreferences(Constants.SHAREDPREFERENCES_NAME, 0);
		String account = settings.getString(Constants.SHAREDPREFERENCES_ACCOUNTNAME, "");
		return account;
	}

	public static String getUserPassword(Context context){
		SharedPreferences settings = context.getSharedPreferences(Constants.SHAREDPREFERENCES_NAME, 0);
		String password = settings.getString(Constants.SHAREDPREFERENCES_PASSWORD, "");
		return password;
	}

	public static void setUserBusy(Context context, boolean busy){
		SharedPreferences settings = context.getSharedPreferences(Constants.SHAREDPREFERENCES_NAME, 0);
		SharedPreferences.Editor edit = settings.edit();

		edit.putBoolean(Constants.SHAREDPREFERENCES_BUSY, busy);
		edit.commit();
	}

	public static boolean getUserBusy(Context context){
		SharedPreferences settings = context.getSharedPreferences(Constants.SHAREDPREFERENCES_NAME, 0);
		return settings.getBoolean(Constants.SHAREDPREFERENCES_BUSY, false);
	}

	public static void registerGCM(Context context){

		final String regId = GCMRegistrar.getRegistrationId(context);
		if (regId.equals("")) {
			GCMRegistrar.register(context, Constants.SENDER_ID);
		}
		GCMRegistrator gcm = new GCMRegistrator(regId, context);
		gcm.execute();


	}

	public static void unRegisterGCM(Context context){
		GCMRegistrar.unregister(context);
	}

	public static void startLocationPoller(Context context){
		mgr = (AlarmManager)context.getSystemService(Context.ALARM_SERVICE);

		Intent i=new Intent(context, LocationPoller.class);

		i.putExtra(LocationPoller.EXTRA_INTENT,
				new Intent(context, LocationReceiver.class));
		i.putExtra(LocationPoller.EXTRA_PROVIDER,
				LocationManager.NETWORK_PROVIDER);
//		i.putExtra(LocationPoller.EXTRA_PROVIDER, LocationManager.GPS_PROVIDER);

		pi=PendingIntent.getBroadcast(context, 0, i, 0);
		mgr.setRepeating(AlarmManager.ELAPSED_REALTIME_WAKEUP,
				SystemClock.elapsedRealtime(),
				Constants.GPS_POLLING_PERIOD,
				pi);
	}

	public static void stopLocationPoller(){
		if(pi!=null && mgr!=null)
			mgr.cancel(pi);
	}

	public static void getPlayers(Context con){
		PlayerFinder finder = new PlayerFinder(con);
		finder.execute();

	}

	public static void getOwnStatsInBackground(Context context){
		StatGetter getter = new StatGetter(context);
		getter.execute();
	}

	//	public static void updateOwnStats(Player player, Context context){
	//		Server.updateStats(context, self.getGold(), self.getAreaString());
	//		
	//	}

	//	public static int[] getOwnAreaSize(Context context){
	//		SharedPreferences settings = context.getSharedPreferences(Constants.SHAREDPREFERENCES_NAME, 0);
	//		int x = settings.getInt(Constants.SHAREDPREFERENCES_X, 0);
	//		int y = settings.getInt(Constants.SHAREDPREFERENCES_Y, 0);
	//		int[] result = {x,y};
	//		return result;
	//	}

	public static Player createPlayer(JSONObject playerRepresentation){

		try {
			Player result = new Player(playerRepresentation.getString(Constants.PLAYER_ACCOUNT),
					playerRepresentation.getString(Constants.PLAYER_AREA),
					playerRepresentation.getBoolean(Constants.PLAYER_BUSY));
			return result;
		} catch (JSONException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		return null;
	}

	public static void acceptChallenge(String battleNr, String selfName, String opponentName, Context con){
		if(mChallenges!=null){
			UserFunctions.removeChallengeFromList(battleNr);
		}
		String test = "";
		do{
			test = Server.acceptChallenge(battleNr, selfName, opponentName, con);			
		}while(test.equals("http_failure"));
	}

	public static void rejectChallenge(String battleNr, String selfName, String oppName, Context con){
		if(mChallenges!=null){
			UserFunctions.removeChallengeFromList(battleNr);
		}
		Server.declineChallenge(battleNr, selfName, oppName, con);
	}

	/**
	 * Method that creates a String representing the risk associated with a battle.
	 * The risk of a battle is stored in the database as "x,y;x1-x2,y1-y2;x1-x2,y1-y1" where the first half represents
	 * the area of the attacker in a battle, and the second half represents the area of the defender.
	 * @param context Application Context
	 * @param opponent Player that is being challenged
	 * @param x The x-coordinate where you are attacking the other player
	 * @param y The y-coordinate where you are attacking the other player
	 * @return
	 */
	public static String createRisk(Context context, Player opponent, int x, int y){
		Log.v("knowledgewar createRisk()", "X: "+x+" Y: "+y);
		String attacker = "";
		String defender = "";
		int startingX, endingX;
		int startingY, endingY;
		int ownStartingX, ownEndingX;
		int ownStartingY, ownEndingY;
		int[] ownArea = {5,5}; 

		//Find the range of the x-values for both attacker and defender
		if(x==-1){ //Attack from the left side
			ownStartingX=(ownArea[0]/2)+1;
			ownEndingX=ownArea[0]-1;

			startingX=0;
			endingX=(ownArea[0]/2)-1;
			if(endingX>opponent.getX()-1) endingX=opponent.getX()-1;
		}else{
			ownStartingX=(ownArea[0]/2)-x;
			ownEndingX=(ownArea[0]/2)+(opponent.getX()-1-x);
			if(ownStartingX<0) ownStartingX=0;
			if(ownEndingX>ownArea[0]-1) ownEndingX=ownArea[0]-1;

			startingX=x-(ownArea[0]/2);
			endingX=x+(ownArea[0]/2);
			if(startingX<0) startingX = 0;
			if(endingX>opponent.getX()-1) endingX=opponent.getX()-1;
		}

		if(y==-1){ //Attack from above
			ownStartingY=(ownArea[1]/2)+1;
			ownEndingY=ownArea[1]-1;

			startingY=0;
			endingY=(ownArea[1]/2)-1;
			if(endingY>opponent.getY()-1) endingY=opponent.getY()-1;
		}else{ 
			ownStartingY=(ownArea[1]/2)-y;
			ownEndingY=(ownArea[1]/2)+(ownArea[1]-1-y);
			if(ownStartingY<0) ownStartingY=0;
			if(ownEndingY>ownArea[1]-1) ownEndingY=ownArea[1]-1;

			startingY=y-(ownArea[1]/2);
			endingY=y+(ownArea[1]/2);
			if(startingY<0) startingY = 0;
			if(endingY>opponent.getY()-1) endingY=opponent.getY()-1;
		}

		attacker = ownStartingX+"-"+ownEndingX+","+ownStartingY+"-"+ownEndingY;
		defender = startingX+"-"+endingX+","+startingY+"-"+endingY;
		String result = x+","+y+";"+attacker+";"+defender;
		//		System.out.println(result);
		return result;
	}

	public static int[] calculateAreaRanges(String risk, Player self, Player opponent){
		int [] xy = new int[2];
		String xyString = risk.substring(0,risk.indexOf(';'));
		String xs = xyString.substring(0,risk.indexOf(','));
		String ys = xyString.substring(risk.indexOf(',')+1);

		xy[0] = Integer.parseInt(xs);
		xy[1] = Integer.parseInt(ys);

		risk = risk.substring(risk.indexOf(';')+1);
		String attackerString = risk.substring(0, risk.indexOf(';'));
		String defenderString = risk.substring(risk.indexOf(';')+1, risk.length());
		//		Log.v("UserFunctions", attackerString + " "+defenderString);

		//Calculate attackers area
		String xValues = attackerString.substring(0, attackerString.indexOf(','));
		String yValues = attackerString.substring(attackerString.indexOf(',')+1);
		self.xRange[0]=Integer.parseInt(xValues.substring(0, xValues.indexOf('-')));
		self.xRange[1]=Integer.parseInt(xValues.substring(xValues.indexOf('-')+1));	
		self.yRange[0]=Integer.parseInt(yValues.substring(0, yValues.indexOf('-')));
		self.yRange[1]=Integer.parseInt(yValues.substring(yValues.indexOf('-')+1));

		//Calculate defenders area
		xValues = defenderString.substring(0, defenderString.indexOf(','));
		yValues = defenderString.substring(defenderString.indexOf(',')+1, defenderString.length());
		//		Log.v("UserFunctions", xValues + " "+yValues);
		opponent.xRange[0]=Integer.parseInt(xValues.substring(0, xValues.indexOf('-')));
		opponent.xRange[1]=Integer.parseInt(xValues.substring(xValues.indexOf('-')+1));	
		opponent.yRange[0]=Integer.parseInt(yValues.substring(0, yValues.indexOf('-')));
		opponent.yRange[1]=Integer.parseInt(yValues.substring(yValues.indexOf('-')+1));

		return xy;
	}

	public static PlayerStrength calculatePlayerStrength(int ownRank, int enemyRank){

		int strengthDifference = ownRank - enemyRank;

		if(strengthDifference>=4){
			return PlayerStrength.WEAKER;
		}else if(strengthDifference<=-4){
			return PlayerStrength.STRONGER;
		}else{
			return PlayerStrength.SAME;
		}

	}

	public static void setPlayerList(ArrayList<Player> players){
		mPlayerList = players;
	}

	public static ArrayList<Player> getPlayerList(){
		if(mPlayerList!=null){
			return mPlayerList;
		}
		mPlayerList = new ArrayList<Player>();
		return mPlayerList;
	}

	public static Random getRandomNumberGenerator(){
		if(mRand==null){
			mRand = new Random();
		}
		return mRand;
	}

	public static void addChallengeToList(Challenge c){
		if(mChallenges==null){
			mChallenges = new ArrayList<Challenge>();
		}
		mChallenges.add(c);
	}

	public static ArrayList<Challenge> getChallengeList(){
		return mChallenges;
	}

	public static Challenge getSpecificChallenge(String nr){
		for(Challenge c:mChallenges){
			if(c.getBattleNr().equals(nr)){
				return c;
			}
		}
		return null;
	}

	public static void removeChallengeFromList(String nr){
		if(mChallenges!=null){
			Iterator<Challenge> it = mChallenges.iterator();
			while(it.hasNext()){
				Challenge c = it.next();
				if(c.getBattleNr().equals(nr)){
					it.remove();
				}
			}
		}
	}

	public static void removeChallengeNotification(Context con,String battleNr){
		String ns = Context.NOTIFICATION_SERVICE;
		NotificationManager mNotificationManager = (NotificationManager) con.getSystemService(ns);
		if(battleNr!=null){
			int ID = Integer.parseInt(battleNr);
			mNotificationManager.cancel(ID);
		}
	}

	public static void createNotification(Context con){

	}

	public static void addQuestionsToDatabase(Context con, ArrayList<Question> questions){

	}

	public static ArrayList<Question> getQuestions(Context con){


		return null;
	}

	public static void sendChatMessage(Context context, String message){
		String chatLine = UserFunctions.getUserAccount(context)+": "+message;
		MessageSender sender = new MessageSender(context, chatLine);
		sender.execute();
	}

	public static void receiveMessage(Context context, String message){
		if(mMessages==null){
			mMessages = new ArrayList<String>();
		}
		if(mMessages.size()>100){
			mMessages.remove(0);
		}
		mMessages.add(message);

		ChatHUD.getInstance().refreshList();
	}

	public static ArrayList<String> getMessageList(){
		if(mMessages==null){
			mMessages = new ArrayList<String>();
		}
		return mMessages;
	}


	private static class UserLogOuter extends AsyncTask<Void, Void, Void>{

		private String account, pass;

		public UserLogOuter(String acc, String pa){
			account=acc;
			pass=pa;
		}

		@Override
		protected Void doInBackground(Void... params) {
			Server.userLogOut(account, pass);
			return null;
		}

		@Override
		protected void onPostExecute(Void result){

		}

	}

	private static class StatGetter extends AsyncTask<Void, Void, String>{

		private Context context;

		public StatGetter(Context context){
			this.context=context;
		}

		@Override
		protected String doInBackground(Void... params) {
			String result = Server.getOwnStats(context);
			return result;
		}

		@Override
		protected void onPostExecute(String result){
			SharedPreferences settings = context.getSharedPreferences(Constants.SHAREDPREFERENCES_NAME, 0);
			SharedPreferences.Editor editor = settings.edit();

			JSONObject obj;
			try {
				obj = new JSONObject(result);
				String name = settings.getString(Constants.PARAMETER_ACCOUNT, "");
				String area = obj.getString(Constants.PLAYER_AREA);
				int gold = Integer.parseInt(obj.getString(Constants.PLAYER_GOLD));
				boolean busy = obj.getBoolean(Constants.PARAMETER_BUSY);
				Player temp = new Player(name, area, gold);
				self=temp;
				UserFunctions.setUserBusy(context, busy);

				editor.commit();
			} catch (JSONException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		}



	}

	private static class MessageSender extends AsyncTask<Void, Void, Void>{

		private Context context;
		private String chatLine;

		public MessageSender(Context con, String line){
			context=con;
			chatLine=line;
		}

		@Override
		protected Void doInBackground(Void... params) {
			Server.sendChat(context, chatLine);
			return null;
		}

	}

	private static class PlayerFinder extends AsyncTask<Void, Void, JSONArray>{

		Context context;
		ArrayList<Player> players;

		public PlayerFinder(Context con){
			context=con;
			players = new ArrayList<Player>();
		}

		@Override
		protected JSONArray doInBackground(Void... arg0) {

			try {
				String result = Server.getPlayers(context);
				JSONArray ar = new JSONArray(result);
				return ar;
			} catch (JSONException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
			return null;
		}

		protected void onPostExecute(JSONArray result) {
			JSONObject obj;
			if(result!=null){
				try {
					for (int i=0;i<result.length();i++) {
						obj=result.getJSONObject(i);
						Player temp = UserFunctions.createPlayer(obj);
						players.add(temp);
					}
				} catch (JSONException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}
			}
			UserFunctions.setPlayerList(players);
		}
	}

	private static class GCMRegistrator extends AsyncTask<Void,Void,Void>{

		private String regID;
		private Context context;

		public GCMRegistrator(String regid, Context con){
			regID = regid;
			context = con;
		}
		@Override
		protected Void doInBackground(Void... params) {
			Server.verifyGCM(regID, context);
			return null;
		}
	}

}
