package adrian.master.knowledgewar.scenes;

import java.util.Random;

import org.andengine.engine.camera.ZoomCamera;
import org.andengine.engine.camera.hud.HUD;
import org.andengine.entity.scene.Scene;
import org.andengine.entity.sprite.Sprite;
import org.andengine.entity.text.AutoWrap;
import org.andengine.entity.text.Text;
import org.andengine.entity.text.TextOptions;
import org.andengine.input.touch.TouchEvent;
import org.andengine.opengl.font.Font;
import org.andengine.opengl.texture.region.TextureRegion;
import org.andengine.opengl.vbo.VertexBufferObjectManager;
import org.andengine.util.HorizontalAlign;

import adrian.master.knowledgewar.gameactivities.BattleActivity;
import adrian.master.knowledgewar.gameconcepts.Question;
import adrian.master.knowledgewar.interfaces.BattleActivityInterface;
import adrian.master.knowledgewar.logic.Constants;
import adrian.master.knowledgewar.logic.UserFunctions;
import android.os.AsyncTask;

public class BaseDisplayQuestionScene extends Scene implements BattleActivityInterface {
	
	protected float ALTERNATIVE_WRAP, QUESTION_WRAP;
	
	protected BattleActivity mParent;
	protected ZoomCamera mCamera;
	protected HUD mHud;
	
	protected String mQuestionString, mAlternative1String, mAlternative2String, mAlternative3String, mAlternative4String;
	
	protected Text mQuestionText;
	protected Text mAlternative1, mAlternative2, mAlternative3, mAlternative4;
	protected Sprite mQuestionSprite;
	protected Sprite mAlternativeSprite1, mAlternativeSprite2, mAlternativeSprite3, mAlternativeSprite4;
	protected Sprite mSandBackgroundSprite, mDuneBackgroundSprite;
	protected Sprite mTimerBarSprite;
	protected Sprite mSelectedAlternative;
	
	protected TextureRegion mButtonTexture;
	protected TextureRegion mSandBackground, mDuneBackground;
	protected TextureRegion mRedbarTexture, mGreenbarTexture,mTimerBorder;
	protected TextureRegion mPaper, mLeather;
	
	protected Font mFont, mSmallFont;
	protected VertexBufferObjectManager mMgr;
	
	protected Timer mTimer;
	protected Question mQuestion;
	protected float[][] mAlternativeCoordinates;
	
	protected boolean mAnswerWasCorrect=false;
	protected int mTime = Constants.BASELINE_TIMER;
	protected int mTimeLeft = 0;
	
	public BaseDisplayQuestionScene(BattleActivity parent, Question question){
		this.mParent = parent;
		this.mQuestion = question;
		initializeFields();
		setupScene();
	}
	
	protected void setupScene(){
		createBackgrounds();
		createTimerSprite();
		createTexts();
		setUpQuestionSprite();
		placeQuestionSprite();
		setUpAlternativeSprites();
		createAlternativePlacements();
		placeAlternativeSprites();
		createStrings();
		displayQuestions();
		displayAlternatives();
	}
	
	protected void initializeFields(){
		this.mCamera = mParent.mCamera;
		this.mFont = mParent.mFont;
		this.mSmallFont = mParent.mSmallFont;
		this.mMgr = mParent.mMgr;
		this.mButtonTexture=mParent.mWhiteButtonTexture;
		this.mSandBackground = mParent.mSandTexture;
		this.mDuneBackground = mParent.mDuneTexture;
		this.mRedbarTexture = mParent.mRedBarTexture;
		this.mGreenbarTexture = mParent.mGreenBarTexture;
		this.mLeather = mParent.mLeatherTexture;
		this.mPaper = mParent.mPaperTexture;
		this.ALTERNATIVE_WRAP = mParent.CAMERA_WIDTH*0.4f;
		this.QUESTION_WRAP = mParent.CAMERA_WIDTH*0.6f;
		this.mTimerBorder = mParent.mTimerBorderTexture;
	}
	
	protected void createAlternativePlacements(){
		mAlternativeCoordinates = new float[4][2];
		float[][] tempCoordinates = new float[4][2];
		boolean[] isOccupied = new boolean[4];
		for(int i = 0;i<4;i++){
			isOccupied[i]=false;
		}
		tempCoordinates[0][0] = mParent.CAMERA_WIDTH/2-mAlternativeSprite1.getWidth()-10;
		tempCoordinates[0][1] = mParent.CAMERA_HEIGHT/4-mAlternativeSprite1.getHeight()-10;
		tempCoordinates[1][0] = mParent.CAMERA_WIDTH/2+10;
		tempCoordinates[1][1] = mParent.CAMERA_HEIGHT/4-mAlternativeSprite2.getHeight()-10;
		tempCoordinates[2][0] = mParent.CAMERA_WIDTH/2-mAlternativeSprite3.getWidth()-10;
		tempCoordinates[2][1] = mParent.CAMERA_HEIGHT/4+10;
		tempCoordinates[3][0] = mParent.CAMERA_WIDTH/2+10;
		tempCoordinates[3][1] = mParent.CAMERA_HEIGHT/4+10;
		
		Random rand = UserFunctions.getRandomNumberGenerator();
		int i = 0;
		
		while(i<4){
			int temp = rand.nextInt(4);
			if(!isOccupied[temp]){
				mAlternativeCoordinates[temp] = tempCoordinates[i];
				isOccupied[temp]=true;
				i++;
			}
		}
		
	}
	
	protected void createBackgrounds(){
		mSandBackground.setTextureSize(mParent.CAMERA_WIDTH, mParent.CAMERA_HEIGHT/2);
		mSandBackgroundSprite = new Sprite(0,mParent.CAMERA_HEIGHT/2, mSandBackground, mMgr);
		
		mDuneBackground.setTextureSize(mParent.CAMERA_WIDTH, mParent.CAMERA_HEIGHT/2);
		mDuneBackgroundSprite = new Sprite(0,0, mDuneBackground, mMgr);
		
		attachChild(mSandBackgroundSprite);
		attachChild(mDuneBackgroundSprite);
	}
	
	protected void createTimerSprite(){
		Sprite timerBorder = new Sprite(0,0,mTimerBorder,mMgr);
		this.mRedbarTexture.setTextureSize(mParent.CAMERA_WIDTH, 32);
		this.mGreenbarTexture.setTextureSize(mParent.CAMERA_WIDTH, 32);
		this.mTimerBarSprite = new Sprite(0,0,mGreenbarTexture, mMgr);
		this.mTimerBarSprite.setPosition(0, Constants.CAMERA_HEIGHT/2 - mTimerBarSprite.getHeight()/2);
		timerBorder.setPosition(0, Constants.CAMERA_HEIGHT/2-20);
		attachChild(mTimerBarSprite);
		attachChild(timerBorder);
	}
	
	protected void createTexts(){
		mQuestionText = new Text(0,0, mFont, "",150,new TextOptions(AutoWrap.WORDS, QUESTION_WRAP, HorizontalAlign.CENTER, Text.LEADING_DEFAULT), mMgr);
		
		mAlternative1 = new Text(0,0, mSmallFont, "",100,new TextOptions(AutoWrap.WORDS, ALTERNATIVE_WRAP, HorizontalAlign.CENTER, Text.LEADING_DEFAULT), mMgr);
		mAlternative2 = new Text(0,0, mSmallFont, "",100,new TextOptions(AutoWrap.WORDS, ALTERNATIVE_WRAP, HorizontalAlign.CENTER, Text.LEADING_DEFAULT), mMgr);
		mAlternative3 = new Text(0,0, mSmallFont, "",100,new TextOptions(AutoWrap.WORDS, ALTERNATIVE_WRAP, HorizontalAlign.CENTER, Text.LEADING_DEFAULT), mMgr);
		mAlternative4 = new Text(0,0, mSmallFont, "",100,new TextOptions(AutoWrap.WORDS, ALTERNATIVE_WRAP, HorizontalAlign.CENTER, Text.LEADING_DEFAULT), mMgr);
	}
	
	protected void setUpQuestionSprite(){
		mQuestionSprite = new Sprite(0,0,mParent.CAMERA_WIDTH*0.8f,
				mParent.CAMERA_HEIGHT*0.4f,	mLeather, mMgr);
		mDuneBackgroundSprite.attachChild(mQuestionSprite);
	}
	
	protected void placeQuestionSprite(){
		mQuestionSprite.setPosition(mParent.CAMERA_WIDTH/2-mQuestionSprite.getWidth()/2, 
				mParent.CAMERA_HEIGHT/4 - mQuestionSprite.getHeight()/2);
	}
	
	protected void setUpAlternativeSprites(){
		float width = Constants.CAMERA_WIDTH/2 - 40;
		float height = Constants.CAMERA_HEIGHT/4 - 40;
		mAlternativeSprite1 = new Sprite(0,0,
				width,height,
				mPaper, mMgr){
			@Override
			public boolean onAreaTouched(TouchEvent pSceneTouchEvent,
					float pTouchAreaLocalX, float pTouchAreaLocalY) {
				switch(pSceneTouchEvent.getAction()){
				case TouchEvent.ACTION_UP: this.setScale(1);mSelectedAlternative=this; questionAnswered(1);return true;
				case TouchEvent.ACTION_DOWN: this.setScale(Constants.BUTTONPRESSED_SCALE);return true;
				}
				return false;
			}
		};
		mAlternativeSprite2 = new Sprite(0,0,
				width,height,
				mPaper, mMgr){
			@Override
			public boolean onAreaTouched(TouchEvent pSceneTouchEvent,
					float pTouchAreaLocalX, float pTouchAreaLocalY) {
				switch(pSceneTouchEvent.getAction()){
				case TouchEvent.ACTION_UP: this.setScale(1);mSelectedAlternative=this; questionAnswered(2);return true;
				case TouchEvent.ACTION_DOWN: this.setScale(Constants.BUTTONPRESSED_SCALE);return true;
				}
				return false;
			}
		};
		mAlternativeSprite3 = new Sprite(0,0, 
				width,height, 
				mPaper, mMgr){
			@Override
			public boolean onAreaTouched(TouchEvent pSceneTouchEvent,
					float pTouchAreaLocalX, float pTouchAreaLocalY) {
				switch(pSceneTouchEvent.getAction()){
				case TouchEvent.ACTION_UP: this.setScale(1);mSelectedAlternative=this; questionAnswered(3);return true;
				case TouchEvent.ACTION_DOWN: this.setScale(Constants.BUTTONPRESSED_SCALE);return true;
				}
				return false;
			}
		};
		mAlternativeSprite4 = new Sprite(0,0, 
				width,height, 
				mPaper, mMgr){
			@Override
			public boolean onAreaTouched(TouchEvent pSceneTouchEvent,
					float pTouchAreaLocalX, float pTouchAreaLocalY) {
				switch(pSceneTouchEvent.getAction()){
				case TouchEvent.ACTION_UP: this.setScale(1);mSelectedAlternative=this; questionAnswered(4);return true;
				case TouchEvent.ACTION_DOWN: this.setScale(Constants.BUTTONPRESSED_SCALE);return true;
				}
				return false;
			}
		};
		
		
		
	}
	
	protected void placeAlternativeSprites(){
		mAlternativeSprite1.setPosition(mAlternativeCoordinates[0][0],mAlternativeCoordinates[0][1]);
		
		mAlternativeSprite2.setPosition(mAlternativeCoordinates[1][0],mAlternativeCoordinates[1][1]);
		
		mAlternativeSprite3.setPosition(mAlternativeCoordinates[2][0],mAlternativeCoordinates[2][1]);
		
		mAlternativeSprite4.setPosition(mAlternativeCoordinates[3][0],mAlternativeCoordinates[3][1]);
		
		mSandBackgroundSprite.attachChild(mAlternativeSprite1);
		mSandBackgroundSprite.attachChild(mAlternativeSprite2);
		mSandBackgroundSprite.attachChild(mAlternativeSprite3);
		mSandBackgroundSprite.attachChild(mAlternativeSprite4);
		
		registerTouchArea(mAlternativeSprite1);
		registerTouchArea(mAlternativeSprite2);
		registerTouchArea(mAlternativeSprite3);
		registerTouchArea(mAlternativeSprite4);
	}
	
	protected void createStrings(){
		mQuestionString = mQuestion.getQuestion();
		mAlternative1String = mQuestion.getAlternative(1);
		mAlternative2String = mQuestion.getAlternative(2);
		mAlternative3String = mQuestion.getAlternative(3);
		mAlternative4String = mQuestion.getAlternative(4);
	}
		
	protected void displayQuestions(){
		mQuestionText.setText(mQuestionString);
		mQuestionText.setPosition(mQuestionSprite.getWidth()/2-mQuestionText.getWidth()/2, 
				mQuestionSprite.getHeight()/2-mQuestionText.getHeight()/2);
		mQuestionSprite.attachChild(mQuestionText);
	}
	
	protected void displayAlternatives(){
		mAlternative1.setText(mAlternative1String);
		mAlternative1.setPosition(mAlternativeSprite1.getWidth()/2-mAlternative1.getWidth()/2, 
				mAlternativeSprite1.getHeight()/2-mAlternative1.getHeight()/2);
		
		mAlternative2.setText(mAlternative2String);
		mAlternative2.setPosition(mAlternativeSprite2.getWidth()/2-mAlternative2.getWidth()/2, 
				mAlternativeSprite2.getHeight()/2-mAlternative2.getHeight()/2);
				
		mAlternative3.setText(mAlternative3String);
		mAlternative3.setPosition(mAlternativeSprite3.getWidth()/2-mAlternative3.getWidth()/2, 
				mAlternativeSprite3.getHeight()/2-mAlternative3.getHeight()/2);
		
		mAlternative4.setText(mAlternative4String);
		mAlternative4.setPosition(mAlternativeSprite1.getWidth()/2-mAlternative4.getWidth()/2, 
				mAlternativeSprite4.getHeight()/2-mAlternative4.getHeight()/2);
		
		mAlternativeSprite1.attachChild(mAlternative1);
		mAlternativeSprite2.attachChild(mAlternative2);
		mAlternativeSprite3.attachChild(mAlternative3);
		mAlternativeSprite4.attachChild(mAlternative4);
	}
	
	protected void questionAnswered(int nr){
		if(!mTimer.isCancelled()){
			mTimer.cancel(true);			
		}
		unregisterAlternativeButtons();
		
		int score = returnScore(nr);
		
		mParent.questionAnswered(score);
	}
	
	protected int calculateScore(){
		int timeLeft = mTimeLeft;
		float percentTimeLeft = (float)timeLeft/(float)mTime;
		float score = percentTimeLeft*Constants.BASELINE_SCORE;
		return (int)score;
	}
	
	protected int returnScore(int nr){
		if(mQuestion.isAnswerCorrect(nr)){
			if(mSelectedAlternative!=null){
				this.mSelectedAlternative.setColor(0, 1, 0);				
			}
			this.mAnswerWasCorrect=true;
			return calculateScore();			
		}else{
			if(mSelectedAlternative!=null){
				this.mSelectedAlternative.setColor(1, 0, 0);				
			}
			this.mAnswerWasCorrect=false;
			return 0;
		}
	}
	
	public boolean isAnswerCorrect(){
		return this.mAnswerWasCorrect;
	}
	
	public void cancelTimer(){
		this.mTimer.cancel(true);
	}
	
	public void startTimer(){
		this.mTimer = new Timer(this.mTime);
		this.mTimer.execute();
	}
	
	protected void timerExpired(){
		unregisterAlternativeButtons();
		questionAnswered(-1);
	}
	
	protected void unregisterAlternativeButtons(){
		unregisterTouchArea(mAlternativeSprite1);
		unregisterTouchArea(mAlternativeSprite2);
		unregisterTouchArea(mAlternativeSprite3);
		unregisterTouchArea(mAlternativeSprite4);
	}
	
	protected class Timer extends AsyncTask<Void, Integer, Void>{
		
		private int currentTimer;
		private float barUnit;
		
		public Timer(int time){
			this.currentTimer=time;
			barUnit = mParent.CAMERA_WIDTH/time;
		}
		
		@Override
		protected void onPreExecute(){
			
		}

		@Override
		protected Void doInBackground(Void... arg0) {
			int time = currentTimer;
			do{
				try {
					Thread.sleep(1000);
					time--;
					publishProgress(time);
				} catch (InterruptedException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}
			}while(time>0 && !isCancelled());
			return null;
		}
		
		@Override
		protected void onProgressUpdate(Integer... params){
			mTimerBarSprite.setWidth(barUnit*params[0]);
			mTimeLeft = params[0];
		}
		
		@Override
		protected void onPostExecute(Void result){
			timerExpired();
		}
		
		@Override
		protected void onCancelled(){
			
		}
		
	}

	@Override
	public boolean onBackButtonPressed() {
		// TODO Auto-generated method stub
		return false;
	}

}
