package adrian.master.knowledgewar.scenes;

import org.andengine.entity.scene.Scene;
import org.andengine.entity.sprite.Sprite;
import org.andengine.entity.text.AutoWrap;
import org.andengine.entity.text.Text;
import org.andengine.entity.text.TextOptions;
import org.andengine.input.touch.TouchEvent;
import org.andengine.opengl.font.Font;
import org.andengine.opengl.texture.region.TextureRegion;
import org.andengine.opengl.vbo.VertexBufferObjectManager;
import org.andengine.util.HorizontalAlign;

import adrian.master.knowledgewar.R;
import adrian.master.knowledgewar.gameactivities.BattleActivity;
import adrian.master.knowledgewar.gameconcepts.Battle;
import adrian.master.knowledgewar.gameconcepts.Question;
import adrian.master.knowledgewar.interfaces.BattleActivityInterface;
import adrian.master.knowledgewar.logic.Constants;

public class BattleResultScene extends Scene implements BattleActivityInterface {
	
	private int CAMERA_HEIGHT = Constants.CAMERA_HEIGHT, CAMERA_WIDTH=Constants.CAMERA_WIDTH;
	
	private TextureRegion mDuneBackground, mLeather,mPaper, mGreenCheck, mRedX, mAwesome, mBibleThump;
	private Sprite mBackground, mDoneButton, mLeatherBackground, mResultFace;
	private Sprite[] mQuestions, mAnswers;
	private VertexBufferObjectManager mMgr;
	private Font mFont,mFontBold, mSmallFont;
	private Text mGoldResultText, mWinnerText, mScoreText,mOpponentScoreText, mUsedMoneyText;
	
	private Battle mBattle;
	private BattleActivity mParent;
	
	private float mQuestionButtonWidth, mButtonHeight, mAnswerButtonWidth;
	private float mRowWidth;
	private float mButtonX;
	
	
	public BattleResultScene(BattleActivity parent){
		this.mParent = parent;
		setupFields();
		setupBackground();
		setupResultText();
	}
	
	public void initialize(){	
		this.mBattle = mParent.mCurrentBattle;
		this.mQuestions = new Sprite[mBattle.mSize];
		this.mAnswers = new Sprite[mBattle.mSize];
		setupButtons();
	}
	
	private void setupFields(){
		this.mButtonHeight = Constants.TILE_SIZE*1.4f;
		this.mQuestionButtonWidth = 5*Constants.TILE_SIZE;
		this.mAnswerButtonWidth = Constants.TILE_SIZE;
		this.mRowWidth = mQuestionButtonWidth+mAnswerButtonWidth+10;
		this.mButtonX = CAMERA_WIDTH/2 - mRowWidth/2;
		this.mDuneBackground = mParent.mBigDuneTexture;
		this.mDuneBackground = mParent.mBigDuneTexture; 
		this.mFont = mParent.mFont;
		this.mFontBold = mParent.mFontBold;
		this.mSmallFont = mParent.mSmallFont;
		this.mMgr = mParent.mMgr;
		this.mLeather = mParent.mLeatherTexture;
		this.mRedX = mParent.mRedXTexture;
		this.mGreenCheck = mParent.mGreenCheckTexture;
		this.mPaper = mParent.mPaperTexture;
		this.mAwesome = mParent.mAwesomeFaceTexture;
		this.mBibleThump = mParent.mBibleThumpTexture;
	}
	
	private void setupBackground(){
		mBackground = new Sprite(0,0, CAMERA_WIDTH, CAMERA_HEIGHT, mDuneBackground, mMgr);
		attachChild(mBackground);
		
		mLeatherBackground = new Sprite(0,0, CAMERA_WIDTH-Constants.TILE_SIZE, mFontBold.getLineHeight()*7, mLeather,mMgr);
		mLeatherBackground.setPosition(CAMERA_WIDTH/2 - mLeatherBackground.getWidth()/2, 40);
		mBackground.attachChild(mLeatherBackground);
	}
	
	private void setupResultText(){
		float x_buffer=30;
		float y_position = 20;
		TextOptions right = new TextOptions(AutoWrap.WORDS, mLeatherBackground.getWidth()-60-Constants.TILE_SIZE, HorizontalAlign.LEFT);
		TextOptions center = new TextOptions(AutoWrap.WORDS, Constants.CAMERA_WIDTH-Constants.TILE_SIZE*2, HorizontalAlign.CENTER);
		mWinnerText = new Text(0,0, mFontBold, "", 30,center, mMgr);
		mWinnerText.setPosition(x_buffer, y_position);
		
		mScoreText = new Text(0,0, mSmallFont, "", 30,right, mMgr);
		mScoreText.setPosition(55+Constants.TILE_SIZE, 
				y_position+=1.5f*mFont.getLineHeight());
		mOpponentScoreText = new Text(0,0,mSmallFont,"",30,right,mMgr);
		mOpponentScoreText.setPosition(55+Constants.TILE_SIZE, 
				y_position+=mSmallFont.getLineHeight());

		mGoldResultText = new Text(0,0, mFont, "", 30,right, mMgr);
		mGoldResultText.setPosition(55+Constants.TILE_SIZE, 
				y_position+=mFont.getLineHeight());
				
		mUsedMoneyText = new Text(0,0, mFont, "", 70,center, mMgr);
		mUsedMoneyText.setPosition(x_buffer, y_position+=mFont.getLineHeight());
		
		mLeatherBackground.attachChild(mWinnerText);
		mLeatherBackground.attachChild(mGoldResultText);
		mLeatherBackground.attachChild(mScoreText);
		mLeatherBackground.attachChild(mOpponentScoreText);
		mLeatherBackground.attachChild(mUsedMoneyText);
	}
	
	private void fillOutResults(boolean winner, String score, String gold, int usedGold){
		String winnerString="";
		String goldString="";
		String scoreString = mParent.mRes.getString(R.string.battle_result_score);
		scoreString = scoreString.replace("?", score);
		String usedMoneyString = mParent.mRes.getString(R.string.battle_result_moneyused);
		usedMoneyString = usedMoneyString.replace("?", usedGold+"");
		if(winner){
			winnerString=mParent.mRes.getString(R.string.battle_result_winner);
			goldString = mParent.mRes.getString(R.string.battle_result_winner_gold);
			mResultFace = new Sprite(0,0,Constants.TILE_SIZE, Constants.TILE_SIZE,mAwesome,mMgr);
		}else{
			winnerString=mParent.mRes.getString(R.string.battle_result_loser);
			goldString = mParent.mRes.getString(R.string.battle_result_loser_gold);
			mResultFace = new Sprite(0,0,Constants.TILE_SIZE, Constants.TILE_SIZE,mBibleThump,mMgr);
		}
		goldString = goldString.replace("?", gold);
		
		mResultFace.setPosition(50, mLeatherBackground.getHeight()/2 - mResultFace.getHeight()/2);
		mWinnerText.setText(winnerString);
		mGoldResultText.setText(goldString);
		mScoreText.setText(scoreString);
		mUsedMoneyText.setText(usedMoneyString);
		
		mLeatherBackground.attachChild(mResultFace);
		
	}
	
	public void updateScreenWithResults(String winner, String score,String opponentScore, String gold, int usedGold){
//		boolean isWinner = false;
//		if(winner.equals(Constants.PARAMETER_WINNER)){
//			isWinner=true;
//		}
//		
//		fillOutResults(isWinner, score, gold, usedGold);
		
		String winnerString="";
		String goldString="";
		String scoreString = mParent.mRes.getString(R.string.battle_result_score);
		scoreString = scoreString.replace("?", score);
		String opponentScoreString = mParent.mRes.getString(R.string.battle_result_opponentscore);
		opponentScoreString = opponentScoreString.replace("?", opponentScore);
		String usedMoneyString = mParent.mRes.getString(R.string.battle_result_moneyused);
		usedMoneyString = usedMoneyString.replace("?", usedGold+"");
		
		if(winner.equals(Constants.PARAMETER_WINNER)){
			winnerString=mParent.mRes.getString(R.string.battle_result_winner);
			goldString = mParent.mRes.getString(R.string.battle_result_winner_gold);
			mResultFace = new Sprite(0,0,Constants.TILE_SIZE, Constants.TILE_SIZE,mAwesome,mMgr);
		}else if(winner.equals(Constants.PARAMETER_LOSER)){
			winnerString=mParent.mRes.getString(R.string.battle_result_loser);
			goldString = mParent.mRes.getString(R.string.battle_result_loser_gold);
			mResultFace = new Sprite(0,0,Constants.TILE_SIZE, Constants.TILE_SIZE,mBibleThump,mMgr);
		}else if(winner.equals(Constants.PARAMETER_DRAW)){
			winnerString=mParent.mRes.getString(R.string.battle_result_draw);
			goldString = mParent.mRes.getString(R.string.battle_result_winner_gold);
			mResultFace = new Sprite(0,0,Constants.TILE_SIZE, Constants.TILE_SIZE,mAwesome,mMgr);
		}
		
		goldString = goldString.replace("?", gold);
		
		mResultFace.setPosition(50, mLeatherBackground.getHeight()/2 - mResultFace.getHeight()/2);
		mWinnerText.setText(winnerString);
		mGoldResultText.setText(goldString);
		mScoreText.setText(scoreString);
		mOpponentScoreText.setText(opponentScoreString);
		mUsedMoneyText.setText(usedMoneyString);
		
		mLeatherBackground.attachChild(mResultFace);
		
	}
	
	private void setupButtons(){
		int size = mBattle.mSize;
		float yPlacement = mLeatherBackground.getY()+mLeatherBackground.getHeight()+10;
		
		for(int i=0; i<size; i++){
			mQuestions[i] = new Sprite(0,0,mQuestionButtonWidth,mButtonHeight,mLeather,mMgr);
			mQuestions[i].setPosition(mButtonX, yPlacement);
			Sprite correctIndicator;
			if(mBattle.isQuestionCorrect(i)){
				correctIndicator = new Sprite(0,0,Constants.TILE_SIZE, Constants.TILE_SIZE,mGreenCheck,mMgr);
			}else{
				correctIndicator = new Sprite(0,0,Constants.TILE_SIZE, Constants.TILE_SIZE,mRedX,mMgr);
			}
			correctIndicator.setPosition(-correctIndicator.getWidth()/2, 
					mQuestions[i].getHeight()/2-correctIndicator.getHeight()/2);
			mQuestions[i].attachChild(correctIndicator);
			
			final Question q = mBattle.getQuestion(i);
			String str = q.getQuestion();
			Text text = new Text(0,0,mSmallFont,str,new TextOptions(AutoWrap.WORDS, mQuestions[i].getWidth()-60,
					HorizontalAlign.CENTER, Text.LEADING_DEFAULT),mMgr);
			text.setPosition(30, mQuestions[i].getHeight()/2 - text.getHeight()/2);
			mQuestions[i].attachChild(text);
			
			attachChild(mQuestions[i]);
			registerTouchArea(mQuestions[i]);
			
			mAnswers[i] = new Sprite(0,0,mAnswerButtonWidth,mButtonHeight,mPaper,mMgr){
				@Override
				public boolean onAreaTouched(TouchEvent pSceneTouchEvent,
						float pTouchAreaLocalX, float pTouchAreaLocalY) {
					switch(pSceneTouchEvent.getAction()){
					case TouchEvent.ACTION_DOWN:
						this.setScale(Constants.BUTTONPRESSED_SCALE);
						return true;
					case TouchEvent.ACTION_UP:
						this.setScale(1);
						mParent.startBrowser(q.getURL());
						return true;
					}
					return false;
				}
			};
			mAnswers[i].setPosition(mButtonX+mQuestionButtonWidth+10, yPlacement);
			Text questionMark = new Text(0,0, mParent.mFontBold, "?",mMgr);
			questionMark.setPosition(mAnswers[i].getWidth()/2- questionMark.getWidth()/2, mAnswers[i].getHeight()/2 - questionMark.getHeight()/2);
			mAnswers[i].attachChild(questionMark);
			attachChild(mAnswers[i]);
			registerTouchArea(mAnswers[i]);
			
			yPlacement+=mButtonHeight;
		}
		
		mDoneButton = new Sprite(mButtonX, CAMERA_HEIGHT-mButtonHeight, mQuestionButtonWidth, mButtonHeight, mPaper,mMgr){
			@Override
			public boolean onAreaTouched(TouchEvent pSceneTouchEvent,
					float pTouchAreaLocalX, float pTouchAreaLocalY) {
				switch(pSceneTouchEvent.getAction()){
				case TouchEvent.ACTION_DOWN:
					this.setScale(Constants.BUTTONPRESSED_SCALE);
					return true;
				case TouchEvent.ACTION_UP:
					this.setScale(1);
					mParent.doneWithBattle();
					return true;
				}
				return false;
			}
		};
		
		String doneButtonText = mParent.mRes.getString(R.string.battle_result_done);
		Text doneText = new Text(0,0,mParent.mSmallFont, doneButtonText,mMgr);
		doneText.setPosition(mDoneButton.getWidth()/2-doneText.getWidth()/2, mDoneButton.getHeight()/2-doneText.getHeight()/2);
		mDoneButton.attachChild(doneText);
		mDoneButton.setScale(0.75f);
		attachChild(mDoneButton);
		registerTouchArea(mDoneButton);
	}

	@Override
	public boolean onBackButtonPressed() {
		// TODO Auto-generated method stub
		return false;
	}

}
