package adrian.master.knowledgewar.scenes;

import org.andengine.engine.Engine;
import org.andengine.engine.camera.ZoomCamera;
import org.andengine.entity.scene.IOnSceneTouchListener;
import org.andengine.entity.scene.Scene;
import org.andengine.entity.sprite.Sprite;
import org.andengine.entity.text.AutoWrap;
import org.andengine.entity.text.Text;
import org.andengine.entity.text.TextOptions;
import org.andengine.input.touch.TouchEvent;
import org.andengine.opengl.font.Font;
import org.andengine.opengl.texture.region.TextureRegion;
import org.andengine.opengl.vbo.VertexBufferObjectManager;
import org.andengine.util.HorizontalAlign;

import adrian.master.knowledgewar.R;
import adrian.master.knowledgewar.gameactivities.MainActivity;
import adrian.master.knowledgewar.gameconcepts.Building;
import adrian.master.knowledgewar.gameconcepts.Player;
import adrian.master.knowledgewar.helpscenes.ChallengeHelpScene;
import adrian.master.knowledgewar.helpscenes.OwnTribeHelpScene;
import adrian.master.knowledgewar.helpscenes.PendingChallengesHelpScene;
import adrian.master.knowledgewar.interfaces.MainActivityInterface;
import adrian.master.knowledgewar.logic.Constants;
import adrian.master.knowledgewar.logic.UserFunctions;
import android.app.AlertDialog;
import android.app.Dialog;
import android.content.res.Resources;
import android.util.Log;
import android.widget.ArrayAdapter;
import android.widget.ListView;

public class ChallengeScene extends Scene implements IOnSceneTouchListener, MainActivityInterface {

	private MainActivity mParent;
	private int CAMERA_HEIGHT = Constants.CAMERA_HEIGHT, CAMERA_WIDTH=Constants.CAMERA_WIDTH;

	private TextureRegion mBackground, mOpponent, mDuneBackground, mBlankTile, mCart;
	private ZoomCamera mCamera;
	private Engine mEngine;
	private TextureRegion mPlayerTile;
	private TextureRegion  mVerticalLine, mBlueBar, mRedBar, mLeather, mPaper;
	private VertexBufferObjectManager mMgr;
	private Font mSmallFont, mFont, mFontBold, mMediumFont, mMediumFontBold;

	private Sprite mBackgroundSprite, mDuneBackgroundSprite,mBlankTilesSprite, mPlayerSprite, mOpponentSprite;
	private Sprite mPlusSign, mMinusSign, mChallengeButton, mVerticalLineSprite, mCategories, mCategoriesBackground;
	private Sprite mOppBonusBackground, mPlayerBonusBackground, mSizeSelectBackground, mTipBackground;
	private int battleSize=2;
	private int category=0;
	private Text mBattleSizeText;

	private Resources mRes;

	private float mSpriteScale=0.5f;

	private Player mSelf, mOpponentPlayer;
	private int x,y;
	private boolean playerPlaced = false, mPlayerBusy = false, mButtonPressed=false;;

	private String mOpponentBonus, mOpponentBonusHealth, mOpponentBonusDamage, mOpponentBonusGold, mOpponentGoldPenalty;
	private Text mOpponentBonusHealthText, mOpponentBonusDamageText, mOpponentBonusGoldText, mOpponentBonusGoldPenaltyText,
	mOpponentBonusText;
	private String mSelfBonus, mSelfBonusHealth, mSelfBonusDamage, mSelfBonusGold, mSelfGoldPenalty;
	private Text mSelfBonusHealthText, mSelfBonusDamageText, mSelfBonusGoldText, mSelfGoldPenaltyText,
	mSelfBonusText;
	private Text mSizeGoldBonus;

	private boolean mIsShowingHelpScene=false;
	private Scene mHelpScene;
	private int mSelectedCategory=0;
	private String mSelectedCategoyrName="All";
	private Text mCategoryText;


	public ChallengeScene(MainActivity parent,Engine engine, int playerNr, Player opponent){
		mParent = parent;
		mEngine=engine;
		mOpponentPlayer=opponent;

		setUpFields();
		createBackgrounds();
		setupOpponent();
		createBonusList();
		createBattleSizeSelector();
		createChallengeButton();
		createTip();
		createCategoriesButton();

		setOnSceneTouchListener(this);
	}

	private void setUpFields(){
		this.mBackground = mParent.mSandTexture;
		this.mPlayerTile = mParent.mOwnTileTexture;
		this.mOpponent = mParent.mRedTileTexture;
		this.mDuneBackground = mParent.mDuneTexture;
		this.mBlankTile = mParent.mBlankTileTexture;
		this.mCart = mParent.mCartTexture;
		this.mCamera = mParent.mZoomCamera;
		this.mMgr = mParent.getVertexBufferObjectManager();
		this.mSmallFont = mParent.mSmallFontBold;
		this.mFont = mParent.mFont;
		this.mFontBold = mParent.mFontBold;
		this.mMediumFont=mParent.mMediumFont;
		this.mMediumFontBold=mParent.mMediumFontBold;
		this.mLeather = mParent.mLeatherTexture;
		this.mPaper = mParent.mPaperTexture;
		this.mVerticalLine = mParent.mVerticalLineTexture;
		this.mRes = mParent.mRes;
		this.mBlueBar = mParent.mBlueBarTexture;
		this.mRedBar = mParent.mRedBarTexture;
	}

	

	private void createBackgrounds(){
		mDuneBackgroundSprite = new Sprite(0, 0, CAMERA_WIDTH, CAMERA_HEIGHT/2,
				mDuneBackground, mMgr);
		attachChild(mDuneBackgroundSprite);

		mBackground.setTextureSize(CAMERA_WIDTH*2, CAMERA_HEIGHT);
		mBackgroundSprite = new Sprite(0, CAMERA_HEIGHT/2, CAMERA_WIDTH, 
				CAMERA_HEIGHT/2,
				mBackground, mMgr);

		attachChild(mBackgroundSprite);

		mBlankTile.setTextureSize(Constants.TILE_SIZE*7, Constants.TILE_SIZE*7);
		mBlankTilesSprite = new Sprite((CAMERA_WIDTH/2)-(Constants.TILE_SIZE*3.5f),
				(3*CAMERA_HEIGHT/4)-(Constants.TILE_SIZE*3.5f), mBlankTile, mMgr);
		mBlankTilesSprite.setScale(mSpriteScale);
		attachChild(mBlankTilesSprite);

		mVerticalLineSprite = new Sprite(CAMERA_WIDTH/2-1,10, 2,mDuneBackgroundSprite.getHeight()-80,
				mVerticalLine,mMgr);
		mVerticalLineSprite.setAlpha(0.25f);
		attachChild(mVerticalLineSprite);

		mOppBonusBackground = new Sprite(0,0, Constants.CAMERA_WIDTH/2 - 10, mMediumFont.getLineHeight()*6, 
				mLeather, mMgr);
		mOppBonusBackground.setPosition(Constants.CAMERA_WIDTH/4 - mOppBonusBackground.getWidth()/2, 5);
		attachChild(mOppBonusBackground);

		mPlayerBonusBackground = new Sprite(0,0, Constants.CAMERA_WIDTH/2 - 10, mMediumFont.getLineHeight()*6, mLeather, mMgr);
		mPlayerBonusBackground.setPosition(Constants.CAMERA_WIDTH/4 - mPlayerBonusBackground.getWidth()/2, 
				mOppBonusBackground.getY()+mOppBonusBackground.getHeight()+10);
		attachChild(mPlayerBonusBackground);

		mSizeSelectBackground = new Sprite(0,0, Constants.CAMERA_WIDTH/2 - 10, Constants.CAMERA_HEIGHT/6+10, mLeather, mMgr);
		mSizeSelectBackground.setPosition(Constants.CAMERA_WIDTH/2+5, Constants.CAMERA_HEIGHT/4-20);
		attachChild(mSizeSelectBackground);

	}

	private void createTip(){
		Text text = new Text(0,0,mSmallFont,mParent.getString(R.string.challenge_tooltip),mMgr);
		mTipBackground = new Sprite(0,0,text.getWidth()+40,mSmallFont.getLineHeight()+20,mLeather,mMgr);
		text.setPosition(mTipBackground.getWidth()/2 - text.getWidth()/2, mTipBackground.getHeight()/2 - text.getHeight()/2);
		mTipBackground.attachChild(text);
		mTipBackground.setPosition(Constants.CAMERA_WIDTH/2 - mTipBackground.getWidth()/2, Constants.CAMERA_HEIGHT/2+20);
		attachChild(mTipBackground);
	}

	private void hideTip(){
		mTipBackground.setVisible(false);
	}

	private void createBonusList(){

		mOpponentBonus = mRes.getString(R.string.challenge_bonus_opponent);
		mOpponentBonusDamage = mRes.getString(R.string.challenge_bonus_damage);
		mOpponentBonusGold = mRes.getString(R.string.challenge_bonus_gold);
		mOpponentBonusHealth = mRes.getString(R.string.challenge_bonus_health);
		mOpponentGoldPenalty = mRes.getString(R.string.challenge_bonus_goldpenalty);

		float line = mMediumFont.getLineHeight();
		float x_padding=25;
		float ycoord=10;
		mOpponentBonusText = new Text(x_padding,ycoord, mMediumFontBold, mOpponentBonus, mMgr);
		mOpponentBonusDamageText= new Text(x_padding,ycoord+=line, mMediumFont, mOpponentBonusDamage,mMgr);
		mOpponentBonusHealthText = new Text(x_padding,ycoord+=line, mMediumFont, mOpponentBonusHealth, mMgr);
		mOpponentBonusGoldText = new Text(x_padding,ycoord+=line, mMediumFont, mOpponentBonusGold, mMgr);
		mOpponentBonusGoldPenaltyText = new Text(x_padding,ycoord+=line, mMediumFont, mOpponentGoldPenalty, mMgr);

		attachChild(mOpponentBonusText);
		attachChild(mOpponentBonusDamageText);
		attachChild(mOpponentBonusGoldText);
		attachChild(mOpponentBonusHealthText);
		attachChild(mOpponentBonusGoldPenaltyText);

		mSelfBonus = mRes.getString(R.string.challenge_bonus_self);
		mSelfBonusDamage = mRes.getString(R.string.challenge_bonus_damage);
		mSelfBonusGold = mRes.getString(R.string.challenge_bonus_gold);
		mSelfBonusHealth = mRes.getString(R.string.challenge_bonus_health);
		mSelfGoldPenalty = mRes.getString(R.string.challenge_bonus_goldpenalty);

		ycoord+=(line+15);
		mSelfBonusText = new Text(x_padding,ycoord+=line, mMediumFontBold, mSelfBonus, mMgr);
		mSelfBonusDamageText = new Text(x_padding,ycoord+=line, mMediumFont, mSelfBonusDamage, mMgr);
		mSelfBonusHealthText= new Text(x_padding,ycoord+=line, mMediumFont, mSelfBonusHealth, mMgr);
		mSelfBonusGoldText = new Text(x_padding,ycoord+=line, mMediumFont, mSelfBonusGold, mMgr); 
		mSelfGoldPenaltyText = new Text(x_padding,ycoord+=line, mMediumFont, mSelfGoldPenalty, mMgr);

		attachChild(mSelfBonusText);
		attachChild(mSelfBonusDamageText);
		attachChild(mSelfBonusGoldText);
		attachChild(mSelfBonusHealthText);
		attachChild(mSelfGoldPenaltyText);
	}

	private void createBattleSizeSelector(){

		Text battleSizeText = new Text(20,5,mMediumFontBold,mRes.getString(R.string.challenge_sizeofbattle),mMgr);
		mSizeSelectBackground.attachChild(battleSizeText);

		mPlusSign = new Sprite(0,0,
				Constants.TILE_SIZE,Constants.TILE_SIZE, mPaper, mMgr){
			@Override
			public boolean onAreaTouched(TouchEvent pSceneTouchEvent,
					float pTouchAreaLocalX, float pTouchAreaLocalY) {

				switch(pSceneTouchEvent.getAction()){
				case TouchEvent.ACTION_UP:
					this.setScale(1);
					if(battleSize<5){
						battleSize++;
						mBattleSizeText.setText(battleSize+"");						
					}
					if(battleSize==5){
						this.setAlpha(0.5f);
						this.setColor(0.8f, 0.8f, 0.8f);
					}else if(battleSize>2){
						mMinusSign.setAlpha(1);
						mMinusSign.setColor(1, 1, 1);
					}
					mSizeGoldBonus.setText("Goldbonus: "+battleSize+"0%");
					return true;
				case TouchEvent.ACTION_DOWN:
					this.setScale(Constants.BUTTONPRESSED_SCALE);
					return true;
				}
				return false;
			}
		};
		mPlusSign.setPosition(battleSizeText.getX()+Constants.TILE_SIZE*2, battleSizeText.getY()+mFontBold.getLineHeight());
		Text plusSign = new Text(0, 0, mFontBold, "+",mMgr);
		plusSign.setPosition(mPlusSign.getWidth()/2-plusSign.getWidth()/2, mPlusSign.getHeight()/2-plusSign.getHeight()/2);
		mPlusSign.attachChild(plusSign);

		mMinusSign = new Sprite(0,0,
				Constants.TILE_SIZE,Constants.TILE_SIZE, mPaper, mMgr){
			@Override
			public boolean onAreaTouched(TouchEvent pSceneTouchEvent,
					float pTouchAreaLocalX, float pTouchAreaLocalY) {

				switch(pSceneTouchEvent.getAction()){
				case TouchEvent.ACTION_UP:
					this.setScale(1);
					if(battleSize>2){
						battleSize--;
						mBattleSizeText.setText(battleSize+"");						
					}
					if(battleSize==2){
						this.setAlpha(0.5f);
						this.setColor(0.8f, 0.8f, 0.8f);
					}else if(battleSize<5){
						mPlusSign.setAlpha(1);
						mPlusSign.setColor(1, 1, 1);
					}
					mSizeGoldBonus.setText("Goldbonus: "+battleSize+"0%");
					return true;
				case TouchEvent.ACTION_DOWN:
					this.setScale(Constants.BUTTONPRESSED_SCALE);
					return true;
				}
				return false;
			}
		};
		mMinusSign.setPosition(battleSizeText.getX(), battleSizeText.getY()+mFontBold.getLineHeight());
		Text minusSign = new Text(0, 0, mFont, "-",mMgr);
		minusSign.setPosition(mMinusSign.getWidth()/2-minusSign.getWidth()/2, mMinusSign.getHeight()/2-minusSign.getHeight()/2);
		mMinusSign.setAlpha(0.5f);
		mMinusSign.setColor(0.8f, 0.8f, 0.8f);
		mMinusSign.attachChild(minusSign);

		mBattleSizeText = new Text(0,0, mFontBold,battleSize+"", mMgr);
		mBattleSizeText.setPosition(mMinusSign.getX()+mMinusSign.getWidth()*1.5f-mBattleSizeText.getWidth()/2, 
				mMinusSign.getY()+mMinusSign.getHeight()/2-mBattleSizeText.getHeight()/2);

		mSizeSelectBackground.attachChild(mPlusSign);
		mSizeSelectBackground.attachChild(mMinusSign);
		mSizeSelectBackground.attachChild(mBattleSizeText);
		registerTouchArea(mPlusSign);
		registerTouchArea(mMinusSign);
		
		mSizeGoldBonus = new Text(0,0,mSmallFont,"Goldbonus: "+battleSize+"0%",mMgr);
		mSizeGoldBonus.setPosition(mSizeSelectBackground.getWidth()/2 - mSizeGoldBonus.getWidth()/2, mPlusSign.getY()+mPlusSign.getHeight()+10);
		mSizeSelectBackground.attachChild(mSizeGoldBonus);
		
		
	}
	
	private void createCategoriesButton(){
		mCategoriesBackground = new Sprite(0,0,Constants.CAMERA_WIDTH/2 - 10, Constants.CAMERA_HEIGHT/6, mLeather, mMgr);
		mCategoriesBackground.setPosition(Constants.CAMERA_WIDTH/2+5, 10);
		Text text = new Text(0,0,mFontBold,mParent.getString(R.string.challenge_category),mMgr);
		text.setPosition(mCategoriesBackground.getWidth()/2-text.getWidth()/2, 10);
		mCategoriesBackground.attachChild(text);
		
		mCategories = new Sprite(0,0,mCategoriesBackground.getWidth()-60, Constants.TILE_SIZE,mPaper,mMgr){
			@Override
			public boolean onAreaTouched(TouchEvent pSceneTouchEvent,
					float pTouchAreaLocalX, float pTouchAreaLocalY) {
				if(pSceneTouchEvent.getAction()==TouchEvent.ACTION_DOWN){
					mCategories.setScale(1.25f);
					return true;
				}else if(pSceneTouchEvent.getAction()==TouchEvent.ACTION_UP){
					mCategories.setScale(1f);
					mParent.showDialog();
					return true;					
				}
				return false;
			}
		};
		mCategories.setPosition(mCategoriesBackground.getWidth()/2 - mCategories.getWidth()/2, text.getY()+mFontBold.getLineHeight()+10);
		mCategoryText = new Text(0,0,mSmallFont,mSelectedCategoyrName,20,mMgr);
		mCategoryText.setPosition(mCategories.getWidth()/2 - mCategoryText.getWidth()/2, mCategories.getHeight()/2 - mCategoryText.getHeight()/2);
		mCategories.attachChild(mCategoryText);
		mCategoriesBackground.attachChild(mCategories);
		registerTouchArea(mCategories);
		attachChild(mCategoriesBackground);
	}

	private void createChallengeButton(){
		String challenge = mRes.getString(R.string.challenge_button_text);
		mPlayerBusy = UserFunctions.getUserBusy(mParent.getApplicationContext());
		mChallengeButton = new Sprite(0,0,
				Constants.TILE_SIZE*4, Constants.TILE_SIZE,mPaper, mMgr){
			@Override
			public boolean onAreaTouched(TouchEvent pSceneTouchEvent,
					float pTouchAreaLocalX, float pTouchAreaLocalY) {

				switch(pSceneTouchEvent.getAction()){
				case TouchEvent.ACTION_UP:
					this.setScale(1); 
					if(playerPlaced && !mPlayerBusy && !mButtonPressed){
						mButtonPressed=true;
						mParent.sendChallenge(mOpponentPlayer, x,y, battleSize,mSelectedCategory);
//						mParent.startMainMenuScene();
//						mParent.startBattleActivity(true);
						unregisterTouchArea(this);
					}
					return true;
				case TouchEvent.ACTION_DOWN:
					if(playerPlaced && !mPlayerBusy){
						this.setScale(Constants.BUTTONPRESSED_SCALE);						
					}
					return true;
				}
				return false;
			}
		};
		if(mPlayerBusy){
			mChallengeButton.setColor(0.4f, 0, 0);
		}
		updateChallengeButton();
		TextOptions header = new TextOptions(AutoWrap.WORDS, mChallengeButton.getWidth(), HorizontalAlign.CENTER, Text.LEADING_DEFAULT);
		Text challengeText = new Text(0,0,mFontBold, challenge,header, mMgr);
		challengeText.setPosition(0, mChallengeButton.getHeight()/2-challengeText.getHeight()/2);
		mChallengeButton.setPosition(CAMERA_WIDTH/2-mChallengeButton.getWidth()/2, 
				10+mDuneBackgroundSprite.getHeight()-mChallengeButton.getHeight());
		mChallengeButton.attachChild(challengeText);
		attachChild(mChallengeButton);
		registerTouchArea(mChallengeButton);
	}

	public void initialize(){
		mCamera.setZoomFactor(1f);
		mCamera.setCenter(CAMERA_WIDTH/2,CAMERA_HEIGHT/2);
		playerPlaced=false;
		setUpPlayer();
	}

	private TextureRegion getTextureBasedOnBuildingType(Building b){

		if(b.getType().equals(Constants.BUILDING_TYPES[1])){
			return mParent.mCartPentagonTexture;
		}else if(b.getType().equals(Constants.BUILDING_TYPES[2])){
			return mParent.mCartCircleTexture;
		}else if(b.getType().equals(Constants.BUILDING_TYPES[3])){
			return mParent.mCartTriangleTexture;
		}else if(b.getType().equals(Constants.BUILDING_TYPES[4])){
			return mParent.mCartStarTexture;
		}else if(b.getType().equals(Constants.BUILDING_TYPES[5])){
			return mParent.mCartSquareTexture;
		}

		return mCart;

	}

	private void setUpPlayer(){
		this.mSelf = UserFunctions.self;
		mPlayerTile.setTextureSize(Constants.TILE_SIZE*5, Constants.TILE_SIZE*5);
		this.mPlayerSprite = new Sprite(0,0, Constants.TILE_SIZE*5, Constants.TILE_SIZE*5,
				mPlayerTile, mMgr);

		if(!mSelf.getBuildings().isEmpty()){
			for(Building b: mSelf.getBuildings()){
				int x = b.getX();
				int y = b.getY();
				TextureRegion tempTex = getTextureBasedOnBuildingType(b);
				Sprite temp = new Sprite(x*Constants.TILE_SIZE, y*Constants.TILE_SIZE,
						Constants.TILE_SIZE, Constants.TILE_SIZE, tempTex, mMgr);
				mPlayerSprite.attachChild(temp);
			}
		}
	}

	private void setupOpponent(){
		mOpponentSprite = new Sprite(Constants.TILE_SIZE, Constants.TILE_SIZE, 
				Constants.TILE_SIZE*5, Constants.TILE_SIZE*5, mOpponent, mMgr);
		mOpponentSprite.setAlpha(0.15f);
		mBlankTilesSprite.attachChild(mOpponentSprite);

		if(!mOpponentPlayer.getBuildings().isEmpty()){
			for(Building b: mOpponentPlayer.getBuildings()){
				int x = b.getX();
				int y = b.getY();
				TextureRegion tempTex = getTextureBasedOnBuildingType(b);
				Sprite temp = new Sprite(x*Constants.TILE_SIZE, y*Constants.TILE_SIZE,
						Constants.TILE_SIZE, Constants.TILE_SIZE, tempTex, mMgr);
				mOpponentSprite.attachChild(temp);
			}			
		}

	}
	
	public void setSelectedCategory(int id, String name){
		mSelectedCategory = id;
		mSelectedCategoyrName = name;
		mCategoryText.setText(mSelectedCategoyrName);
		mCategoryText.setPosition(mCategories.getWidth()/2 - mCategoryText.getWidth()/2, mCategories.getHeight()/2 - mCategoryText.getHeight()/2);
	}

	@Override
	public boolean onSceneTouchEvent(Scene pScene, TouchEvent pSceneTouchEvent) {

		switch(pSceneTouchEvent.getAction()){
		case TouchEvent.ACTION_UP:
			float tempX = pSceneTouchEvent.getX();
			float tempY = pSceneTouchEvent.getY();
			float[] coord = mBlankTilesSprite.convertSceneToLocalCoordinates(tempX, tempY);

			if(isOutsideOpponent(tempX, tempY) && isWithinBounds(tempX, tempY)){
				mEngine.runSafely(new Runnable() {
					@Override
					public void run() {
						mBlankTilesSprite.detachChild(mPlayerSprite);
					}
				});
				int px = ((int)coord[0])/Constants.TILE_SIZE;
				int py = ((int)coord[1])/Constants.TILE_SIZE;
				//				Log.v("Knowledgewar onSceneTouch", "px: "+px+" py: "+py);
				this.x=px-1;
				this.y=py-1;
				px=px*Constants.TILE_SIZE;
				py=py*Constants.TILE_SIZE;
				int pWidth=mSelf.getX()/2;
				int pHeight=mSelf.getY()/2;
				px = px-Constants.TILE_SIZE*pWidth;
				py = py-Constants.TILE_SIZE*pHeight;
				mPlayerSprite.setPosition(px, py);
				mPlayerSprite.setAlpha(0.15f);
				mBlankTilesSprite.attachChild(mPlayerSprite);

				String risk = UserFunctions.createRisk(mParent, mOpponentPlayer, x, y);
				UserFunctions.calculateAreaRanges(risk, mSelf, mOpponentPlayer);
				calculateBattleBonuses();
				displayBonuses();
				playerPlaced=true;
				updateChallengeButton();
				hideTip();
				Log.v("ChallengeScene", risk);
			}

		}

		return true;
	}

	private void updateChallengeButton(){
		if(playerPlaced){
			mChallengeButton.setColor(1, 1, 1);
			mChallengeButton.setAlpha(1);
		}else{
			mChallengeButton.setColor(0.8f, 0.8f, 0.8f);
			mChallengeButton.setAlpha(0.6f);
		}
	}

	private void calculateBattleBonuses(){
		mSelf.calculateBonuses();
		mOpponentPlayer.calculateBonuses();
	}

	private void displayBonuses(){

		removeBars();

		float DAMAGE_PR_BAR = 1f;
		float HEALTH_PR_BAR = 1F;
		float GOLD_BONUS_PR_BAR = 0.25f;
		float GOLD_PENALTY_PR_BAR = 0.25f;

		createBars(mOpponentBonusDamageText, DAMAGE_PR_BAR, mOpponentPlayer.totalDamageBonus, mRedBar);
		createBars(mSelfBonusDamageText, DAMAGE_PR_BAR, mSelf.totalDamageBonus, mBlueBar);

		createBars(mOpponentBonusGoldText, GOLD_BONUS_PR_BAR, mOpponentPlayer.totalGoldModifier, mRedBar);
		createBars(mSelfBonusGoldText, GOLD_BONUS_PR_BAR, mSelf.totalGoldModifier, mBlueBar);

		createBars(mOpponentBonusHealthText, HEALTH_PR_BAR, mOpponentPlayer.totalHealthBonus, mRedBar);
		createBars(mSelfBonusHealthText, HEALTH_PR_BAR, mSelf.totalHealthBonus, mBlueBar);

		createBars(mOpponentBonusGoldPenaltyText, GOLD_PENALTY_PR_BAR, mOpponentPlayer.totalOpponentGoldModifier, mRedBar);
		createBars(mSelfGoldPenaltyText, GOLD_PENALTY_PR_BAR, mSelf.totalOpponentGoldModifier, mBlueBar);


	}

	private void createBars(Text bonusText, float stat_pr_bar, float bonus, TextureRegion bar){
		float barWidth = 10;
		float barHeight = 6;
		bonus=Math.abs(bonus);
		int nr_bars = (int)(bonus / stat_pr_bar);
		float partial_bar = ((float)bonus%stat_pr_bar)/stat_pr_bar;
		float x = bonusText.getWidth()+5;
		float y = 0;
		boolean line2=false;
		for(int i=0;i<nr_bars;i++){
			Sprite temp = new Sprite(0,0,barWidth, barHeight,bar,mMgr);
			temp.setPosition(x, y);
			bonusText.attachChild(temp);
			x+=temp.getWidth()+2;
			if(!line2 && i>5){
				y+=temp.getHeight()+2;
				x=bonusText.getWidth()+5;
				line2=true;
			}
		}
		float width = bar.getWidth();
		float newWidth = width*partial_bar;
		bar.setTextureWidth(newWidth);
		Sprite temp = new Sprite(x,y,newWidth, mMediumFontBold.getLineHeight()/2-2,bar,mMgr);
		bonusText.attachChild(temp);
		bar.setTextureWidth(width);
	}

	private void removeBars(){
		mEngine.runSafely(new Runnable() {
			@Override
			public void run() {
				mOpponentBonusDamageText.detachChildren();
				mOpponentBonusGoldPenaltyText.detachChildren();
				mOpponentBonusGoldText.detachChildren();
				mOpponentBonusHealthText.detachChildren();
				
				mSelfBonusDamageText.detachChildren();
				mSelfBonusGoldText.detachChildren();
				mSelfBonusHealthText.detachChildren();
				mSelfGoldPenaltyText.detachChildren();
				
			}
		});
	}

	private boolean isWithinBounds(float pX, float pY){
		float[] coord = mBlankTilesSprite.convertSceneToLocalCoordinates(pX, pY);
		return coord[0]>0 && coord[0]<mBlankTilesSprite.getWidth() 
				&& coord[1]>0 && coord[1]<mBlankTilesSprite.getHeight();
	}

	private boolean isOutsideOpponent(float pX, float pY){
		float[] coord = mOpponentSprite.convertSceneToLocalCoordinates(pX, pY);
		return !(coord[0]>0 && coord[1]>0 && coord[0]<(mOpponentSprite.getWidth()) && coord[1]<(mOpponentSprite.getHeight()));
	}

	@Override
	public boolean onBackButtonPressed() {
		if(mIsShowingHelpScene){
			mIsShowingHelpScene=false;
			new TransitionScene(mParent, mParent.mZoomCamera, mEngine,mHelpScene, this);
		}else{
			mParent.setMapScene();			
		}
		
		return true;
	}

	@Override
	public void showHelpScreen() {
		mIsShowingHelpScene=true;
		mHelpScene = new ChallengeHelpScene(mParent);
		new TransitionScene(mParent, mParent.mZoomCamera, mEngine, this, mHelpScene);
		mParent.updateHUD(false);
	}

	@Override
	public boolean hasHelpScene() {
		return true;
	}



}
