package adrian.master.knowledgewar.scenes;

import org.andengine.engine.Engine;
import org.andengine.entity.scene.Scene;
import org.andengine.entity.scene.background.AutoParallaxBackground;
import org.andengine.entity.scene.background.ParallaxBackground;
import org.andengine.entity.scene.background.ParallaxBackground.ParallaxEntity;
import org.andengine.entity.sprite.Sprite;
import org.andengine.entity.text.AutoWrap;
import org.andengine.entity.text.Text;
import org.andengine.entity.text.TextOptions;
import org.andengine.input.touch.TouchEvent;
import org.andengine.opengl.texture.region.TextureRegion;
import org.andengine.opengl.vbo.VertexBufferObjectManager;
import org.andengine.util.HorizontalAlign;

import adrian.master.knowledgewar.R;
import adrian.master.knowledgewar.gameactivities.BattleActivity;
import adrian.master.knowledgewar.interfaces.BattleActivityInterface;
import adrian.master.knowledgewar.logic.Constants;
import android.os.AsyncTask;

public class ChallengeSentScene extends Scene implements BattleActivityInterface {

	private ParallaxBackground mBackground;
	private BattleActivity mParent;
	private Engine mEngine;

	private TextureRegion mPaper, mLeather, mRedbarTexture, mGreenbarTexture,mTimerBorder, mAwesomeFace, mSadFace;
	private Sprite mLeatherBackground, mTimerBarSprite, mMainMenuButton;
	private int mTimeLeft;
	private Text mHeader, mMain, mProfits;

	private VertexBufferObjectManager mMgr;
	private Timer mTimer;


	public ChallengeSentScene(BattleActivity parent, Engine engine){
		mParent = parent;
		mEngine = engine;
		mMgr = mParent.mMgr;

		this.mBackground = new AutoParallaxBackground(0, 0, 0, 5);
		Sprite temp = new Sprite(0, 0, mParent.mBigDuneTexture, mMgr);
		mBackground.attachParallaxEntity(new ParallaxEntity(5, temp));
		this.setBackground(mBackground);

		mPaper = mParent.mPaperTexture;
		mLeather = mParent.mLeatherTexture;
		mRedbarTexture = mParent.mRedBarTexture;
		mGreenbarTexture = mParent.mGreenBarTexture;
		mTimerBorder = mParent.mTimerBorderTexture;
		mAwesomeFace = mParent.mAwesomeFaceTexture;
		mSadFace = mParent.mBibleThumpTexture;
		mTimeLeft = Constants.DEFAULT_WIN_TIMEOUT;

		setupBackgroundLeather();
		createTimerSprite();

		mTimer = new Timer(mTimeLeft);
		mTimer.execute();
	}

	private void setupBackgroundLeather(){
		mLeatherBackground = new Sprite(0,0,Constants.CAMERA_WIDTH - Constants.TILE_SIZE, Constants.CAMERA_HEIGHT/2,mLeather,mMgr);
		mLeatherBackground.setPosition(Constants.CAMERA_WIDTH/2 - mLeatherBackground.getWidth()/2, 50);
		attachChild(mLeatherBackground);

		TextOptions opt = new TextOptions(AutoWrap.WORDS, mLeatherBackground.getWidth()-Constants.TILE_SIZE, HorizontalAlign.CENTER);
		mHeader = new Text(0,0,mParent.mFontBold,mParent.getString(R.string.challengesent_header),opt,mMgr);
		mMain = new Text(0,0,mParent.mFont, mParent.getString(R.string.challengesent_text),opt,mMgr);

		mHeader.setPosition(mLeatherBackground.getWidth()/2-mHeader.getWidth()/2, 60);
		mMain.setPosition(mLeatherBackground.getWidth()/2-mMain.getWidth()/2, mHeader.getY()+mHeader.getHeight()+20);
		mLeatherBackground.attachChild(mHeader);
		mLeatherBackground.attachChild(mMain);
	}

	protected void createTimerSprite(){
		Sprite timerBorder = new Sprite(0,0,mTimerBorder,mMgr);
		this.mRedbarTexture.setTextureSize(mParent.CAMERA_WIDTH, 32);
		this.mGreenbarTexture.setTextureSize(mParent.CAMERA_WIDTH, 32);
		this.mTimerBarSprite = new Sprite(0,0,mGreenbarTexture, mMgr);
		this.mTimerBarSprite.setPosition(0, mLeatherBackground.getY()+mLeatherBackground.getHeight()+20);
		timerBorder.setPosition(0, mTimerBarSprite.getHeight()/2 - timerBorder.getHeight()/2);
		attachChild(mTimerBarSprite);
		mTimerBarSprite.attachChild(timerBorder);
	}

	public void cancelTimer(){
		mTimer.cancel(true);
	}
	
	public void opponentStartedOtherBattle(String challengee){
		cancelTimer();
		mEngine.runSafely(new Runnable() {
			@Override
			public void run() {
				mLeatherBackground.detachChild(mHeader);
				mLeatherBackground.detachChild(mMain);
				detachChild(mTimerBarSprite);
			}
		});

		TextOptions opt = new TextOptions(AutoWrap.WORDS, mLeatherBackground.getWidth()-Constants.TILE_SIZE, HorizontalAlign.CENTER);
		mHeader = new Text(0,0,mParent.mFontBold,mParent.getString(R.string.challengesent_declined_header),opt,mMgr);
		mHeader.setPosition(mLeatherBackground.getWidth()/2-mHeader.getWidth()/2, 60);
		
		Sprite awesomeFace = new Sprite(0,0,Constants.TILE_SIZE, Constants.TILE_SIZE,mSadFace,mMgr);
		awesomeFace.setPosition(mLeatherBackground.getWidth()/2 - awesomeFace.getWidth()/2, mHeader.getY()+mHeader.getHeight()+20);

		String mainText = mParent.getString(R.string.challengesent_acceptedother_main);
		mMain = new Text(0,0,mParent.mFont,mainText ,opt,mMgr);
		mMain.setPosition(mLeatherBackground.getWidth()/2-mMain.getWidth()/2, awesomeFace.getY()+awesomeFace.getHeight()+20);

//		String profitString = mParent.getString(R.string.challengesent_declined_profits);
//		profitString = profitString.replace("?", "0");
//		mProfits = new Text (0,0,mParent.mFont,profitString,opt,mMgr);
//		mProfits.setPosition(mLeatherBackground.getWidth()/2-mProfits.getWidth()/2, mMain.getY()+mMain.getHeight()+20);

		mLeatherBackground.attachChild(mHeader);
		mLeatherBackground.attachChild(awesomeFace);
		mLeatherBackground.attachChild(mMain);
		mLeatherBackground.attachChild(mProfits);

		mMainMenuButton = new Sprite(0,0,Constants.TILE_SIZE*4, Constants.TILE_SIZE,mPaper,mMgr){
			@Override
			public boolean onAreaTouched(TouchEvent pSceneTouchEvent,
					float pTouchAreaLocalX, float pTouchAreaLocalY) {
				if(pSceneTouchEvent.getAction() == TouchEvent.ACTION_DOWN){
					mMainMenuButton.setScale(1.25f);
					return true;
				}else if(pSceneTouchEvent.getAction()==TouchEvent.ACTION_UP){
					mMainMenuButton.setScale(1f);
					mParent.doneWithBattle();
					return true;
				}
				return false;
			}
		};
		mMainMenuButton.setPosition(Constants.CAMERA_WIDTH/2 - mMainMenuButton.getWidth()/2, 
				mLeatherBackground.getY()+mLeatherBackground.getHeight()+20);
		Text text = new Text(0,0,mParent.mFont,mParent.getString(R.string.battle_result_done),mMgr);
		text.setPosition(mMainMenuButton.getWidth()/2 - text.getWidth()/2, mMainMenuButton.getHeight()/2 - text.getHeight()/2);
		attachChild(mMainMenuButton);
		mMainMenuButton.attachChild(text);
		registerTouchArea(mMainMenuButton);
	}

	public void battleDeclined(String challengee, String profits){
		cancelTimer();
		mEngine.runSafely(new Runnable() {
			@Override
			public void run() {
				mLeatherBackground.detachChild(mHeader);
				mLeatherBackground.detachChild(mMain);
				detachChild(mTimerBarSprite);
			}
		});

		TextOptions opt = new TextOptions(AutoWrap.WORDS, mLeatherBackground.getWidth()-Constants.TILE_SIZE, HorizontalAlign.CENTER);
		mHeader = new Text(0,0,mParent.mFontBold,mParent.getString(R.string.challengesent_declined_header),opt,mMgr);
		mHeader.setPosition(mLeatherBackground.getWidth()/2-mHeader.getWidth()/2, 60);
		
		Sprite awesomeFace = new Sprite(0,0,Constants.TILE_SIZE, Constants.TILE_SIZE,mAwesomeFace,mMgr);
		awesomeFace.setPosition(mLeatherBackground.getWidth()/2 - awesomeFace.getWidth()/2, mHeader.getY()+mHeader.getHeight()+20);

		String mainText = mParent.getString(R.string.challengesent_declined_main);
		mainText = mainText.replace("?", challengee);
		mMain = new Text(0,0,mParent.mFont,mainText ,opt,mMgr);
		mMain.setPosition(mLeatherBackground.getWidth()/2-mMain.getWidth()/2, awesomeFace.getY()+awesomeFace.getHeight()+20);

		String profitString = mParent.getString(R.string.challengesent_declined_profits);
		profitString = profitString.replace("?", profits);
		mProfits = new Text (0,0,mParent.mFont,profitString,opt,mMgr);
		mProfits.setPosition(mLeatherBackground.getWidth()/2-mProfits.getWidth()/2, mMain.getY()+mMain.getHeight()+20);

		mLeatherBackground.attachChild(mHeader);
		mLeatherBackground.attachChild(awesomeFace);
		mLeatherBackground.attachChild(mMain);
		mLeatherBackground.attachChild(mProfits);

		mMainMenuButton = new Sprite(0,0,Constants.TILE_SIZE*4, Constants.TILE_SIZE,mPaper,mMgr){
			@Override
			public boolean onAreaTouched(TouchEvent pSceneTouchEvent,
					float pTouchAreaLocalX, float pTouchAreaLocalY) {
				if(pSceneTouchEvent.getAction() == TouchEvent.ACTION_DOWN){
					mMainMenuButton.setScale(1.25f);
					return true;
				}else if(pSceneTouchEvent.getAction()==TouchEvent.ACTION_UP){
					mMainMenuButton.setScale(1f);
					mParent.doneWithBattle();
					return true;
				}
				return false;
			}
		};
		mMainMenuButton.setPosition(Constants.CAMERA_WIDTH/2 - mMainMenuButton.getWidth()/2, 
				mLeatherBackground.getY()+mLeatherBackground.getHeight()+20);
		Text text = new Text(0,0,mParent.mFont,mParent.getString(R.string.battle_result_done),mMgr);
		text.setPosition(mMainMenuButton.getWidth()/2 - text.getWidth()/2, mMainMenuButton.getHeight()/2 - text.getHeight()/2);
		attachChild(mMainMenuButton);
		mMainMenuButton.attachChild(text);
		registerTouchArea(mMainMenuButton);
	}

	protected class Timer extends AsyncTask<Void, Integer, Void>{

		private int currentTimer;
		private float barUnit;

		public Timer(int time){
			this.currentTimer=time;
			barUnit = mParent.CAMERA_WIDTH/time;
		}

		@Override
		protected void onPreExecute(){

		}

		@Override
		protected Void doInBackground(Void... arg0) {
			int time = currentTimer;
			do{
				try {
					Thread.sleep(1000);
					time--;
					publishProgress(time);
				} catch (InterruptedException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}
			}while(time>0 && !isCancelled());
			return null;
		}

		@Override
		protected void onProgressUpdate(Integer... params){
			mTimerBarSprite.setWidth(barUnit*params[0]);
			mTimeLeft = params[0];
		}

		@Override
		protected void onPostExecute(Void result){
			//			timerExpired();
		}

		@Override
		protected void onCancelled(){

		}

	}

	@Override
	public boolean onBackButtonPressed() {
		// TODO Auto-generated method stub
		return false;
	}

}
