package adrian.master.knowledgewar.scenes;

import org.andengine.entity.scene.Scene;
import org.andengine.entity.sprite.Sprite;
import org.andengine.entity.text.AutoWrap;
import org.andengine.entity.text.Text;
import org.andengine.entity.text.TextOptions;
import org.andengine.opengl.font.Font;
import org.andengine.opengl.texture.region.TextureRegion;
import org.andengine.opengl.vbo.VertexBufferObjectManager;
import org.andengine.util.HorizontalAlign;

import adrian.master.knowledgewar.R;
import adrian.master.knowledgewar.gameactivities.BattleActivity;
import adrian.master.knowledgewar.gameconcepts.Weapon;
import adrian.master.knowledgewar.interfaces.BattleActivityInterface;
import adrian.master.knowledgewar.logic.Constants;
import android.os.AsyncTask;

public class DisplayAttackScene extends Scene implements BattleActivityInterface {
	
	private BattleActivity mParent;
	private TextureRegion mBackgroundTexture, mLeather;
	private Sprite mBackground, mLeatherBackground;
	private Text mAttackText, mWeaponText;
	private VertexBufferObjectManager mMgr;
	private Font mFontBold, mFont;
	private Weapon mWeapon;
	
	public DisplayAttackScene(BattleActivity parent, Weapon w, boolean isBeingAttacked){
		mParent = parent;
		mBackgroundTexture = mParent.mDuneTexture;
		mFont = mParent.mFont;
		mFontBold = mParent.mFontBold;
		mLeather = mParent.mLeatherTexture;
		mWeapon = w;
		mMgr = mParent.getVertexBufferObjectManager();
		
		createBackground(isBeingAttacked);
		createText(isBeingAttacked);
		Timer timer = new Timer();
		timer.execute();
	}
	
	private void createBackground(boolean isBeingAttacked){
		mBackground = new Sprite(0,0,Constants.CAMERA_WIDTH,Constants.CAMERA_HEIGHT,mBackgroundTexture,mMgr);
		attachChild(mBackground);
		
		if(isBeingAttacked){
			mLeatherBackground = new Sprite(0,0,Constants.CAMERA_WIDTH-Constants.TILE_SIZE,
					Constants.CAMERA_HEIGHT-Constants.TILE_SIZE*4, mLeather, mMgr);
		}else{
			mLeatherBackground = new Sprite(0,0,Constants.CAMERA_WIDTH-Constants.TILE_SIZE,
					mFontBold.getLineHeight()*3, mLeather, mMgr);
		}
		mLeatherBackground.setPosition(Constants.CAMERA_WIDTH/2 - mLeatherBackground.getWidth()/2, 
				Constants.CAMERA_HEIGHT/2 - mLeatherBackground.getHeight()/2);
		attachChild(mLeatherBackground);
	}
	
	private void createText(boolean isBeingAttacked){
		float x_buffer = 30;
		float y_buffer = 40;
		if(isBeingAttacked){
			String attackText = mParent.mRes.getString(R.string.displayattackscene_attacktext);
			mAttackText = new Text(0,0,mFontBold,attackText,
					new TextOptions(AutoWrap.WORDS, mLeatherBackground.getWidth()-60, HorizontalAlign.CENTER),mMgr);
			mAttackText.setPosition(x_buffer, y_buffer);
			mLeatherBackground.attachChild(mAttackText);
			
			Sprite icon = new Sprite(0,0,Constants.TILE_SIZE,Constants.TILE_SIZE,mParent.getTextureBasedonWeapon(mWeapon),mMgr);
			icon.setPosition(mLeatherBackground.getWidth()/2-icon.getWidth()/2, mAttackText.getY()+mAttackText.getHeight()+10);
			mLeatherBackground.attachChild(icon);
			
			String weapontext = mWeapon.getmAttackDesc();
			mWeaponText = new Text(0,0,mFont,weapontext,
					new TextOptions(AutoWrap.WORDS, mLeatherBackground.getWidth()-60, HorizontalAlign.CENTER),mMgr);
			mWeaponText.setPosition(x_buffer, icon.getY()+icon.getHeight()+10);
			mLeatherBackground.attachChild(mWeaponText);
		}else{
			String text = mParent.mRes.getString(R.string.displayattackscene_noattacktext);
			mAttackText = new Text(0,0,mFontBold,text,
					new TextOptions(AutoWrap.WORDS, mLeatherBackground.getWidth()-60, HorizontalAlign.CENTER),mMgr);
			mAttackText.setPosition(x_buffer, mLeatherBackground.getHeight()/2 - mAttackText.getHeight()/2);
			mLeatherBackground.attachChild(mAttackText);
		}
	}
	
	private class Timer extends AsyncTask<Void, Void, Void>{

		@Override
		protected Void doInBackground(Void... params) {
			try {
				Thread.sleep(3000);
			} catch (InterruptedException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
			return null;
		}
		
		@Override
		protected void onPostExecute(Void result) {
			mParent.createNextQuestionScene();
		}
		
	}

	@Override
	public boolean onBackButtonPressed() {
		// TODO Auto-generated method stub
		return false;
	}

}
