package adrian.master.knowledgewar.scenes;

import java.text.DecimalFormat;
import java.util.Random;

import org.andengine.engine.camera.hud.HUD;
import org.andengine.entity.scene.IOnSceneTouchListener;
import org.andengine.entity.scene.Scene;
import org.andengine.entity.sprite.Sprite;
import org.andengine.entity.text.Text;
import org.andengine.input.touch.TouchEvent;
import org.andengine.input.touch.detector.ScrollDetector;
import org.andengine.input.touch.detector.ScrollDetector.IScrollDetectorListener;
import org.andengine.input.touch.detector.SurfaceScrollDetector;

import adrian.master.knowledgewar.gameactivities.BattleActivity;
import adrian.master.knowledgewar.gameconcepts.Question;
import adrian.master.knowledgewar.logic.Constants;
import adrian.master.knowledgewar.logic.UserFunctions;
import android.util.Log;

public class LostAlternativesScene extends BaseDisplayQuestionScene implements IOnSceneTouchListener, IScrollDetectorListener {

	private int mAreaWidth, mAreaHeight;

	private SurfaceScrollDetector mScrollDetector;

	public LostAlternativesScene(BattleActivity parent, Question question){
		super(parent, question);
		this.mScrollDetector = new SurfaceScrollDetector(this);
		this.setOnAreaTouchTraversalFrontToBack();
		this.setOnSceneTouchListener(this);
		this.setTouchAreaBindingOnActionDownEnabled(true);
	}
	
	@Override
	protected void initializeFields() {
		super.initializeFields();
		mAreaWidth=20;
		mAreaHeight=20;
	}


	@Override
	protected void createBackgrounds() {
		//Upper part
		mHud = new HUD();
		mCamera.setHUD(mHud);
		mDuneBackgroundSprite = new Sprite(0,0, mDuneBackground, mMgr);
		mHud.attachChild(mDuneBackgroundSprite);

		//Lower part
		mSandBackground.setTextureSize(mAreaWidth*Constants.TILE_SIZE, mAreaHeight*Constants.TILE_SIZE);
		mSandBackgroundSprite = new Sprite(0,0, mAreaWidth*Constants.TILE_SIZE, mAreaHeight*Constants.TILE_SIZE, mSandBackground, mMgr);
		mCamera.setBounds(mSandBackgroundSprite.getX(), mSandBackgroundSprite.getY()-mParent.CAMERA_HEIGHT/2, 
				mSandBackgroundSprite.getX()+mSandBackgroundSprite.getWidth(), 
				mSandBackgroundSprite.getY()+mSandBackgroundSprite.getHeight());
		mCamera.setBoundsEnabled(true);

		attachChild(mSandBackgroundSprite);
	}

	@Override
	protected void createTimerSprite() {
		Sprite timerBorder = new Sprite(0,0,mTimerBorder,mMgr);
		this.mRedbarTexture.setTextureSize(mParent.CAMERA_WIDTH, 32);
		this.mGreenbarTexture.setTextureSize(mParent.CAMERA_WIDTH, 32);
		this.mTimerBarSprite = new Sprite(0,0,mGreenbarTexture, mMgr);
		this.mTimerBarSprite.setPosition(0, mParent.CAMERA_HEIGHT/2- mTimerBarSprite.getHeight()/2);
		timerBorder.setPosition(0, mTimerBarSprite.getHeight()/2-timerBorder.getHeight()/2);
		mHud.attachChild(mTimerBarSprite);
		mTimerBarSprite.attachChild(timerBorder);
	}

	@Override
	protected void placeAlternativeSprites() {
		int[][] coords = getAlternativesPlacements();
		mAlternativeSprite1.setPosition(coords[0][0],coords[0][1]);

		mAlternativeSprite2.setPosition(coords[1][0],coords[1][1]);

		mAlternativeSprite3.setPosition(coords[2][0],coords[2][1]);

		mAlternativeSprite4.setPosition(coords[3][0],coords[3][1]);

		mSandBackgroundSprite.attachChild(mAlternativeSprite1);
		mSandBackgroundSprite.attachChild(mAlternativeSprite2);
		mSandBackgroundSprite.attachChild(mAlternativeSprite3);
		mSandBackgroundSprite.attachChild(mAlternativeSprite4);
		
		registerTouchArea(mAlternativeSprite1);
		registerTouchArea(mAlternativeSprite2);
		registerTouchArea(mAlternativeSprite3);
		registerTouchArea(mAlternativeSprite4);


	}

	private int[][] getAlternativesPlacements(){
		int[][] result = new int[4][2];
		Random rand = UserFunctions.getRandomNumberGenerator();
		
		int width = mAreaWidth;
		int height = mAreaHeight;
		result[0][0] = rand.nextInt(width);
		result[0][1] = rand.nextInt(height);
		
		float alternativeWidth = mAlternativeSprite1.getWidth();
		float alternativeHeight = mAlternativeSprite1.getHeight();
		
		int coordsWidth = (int)((mAreaWidth*Constants.TILE_SIZE) / alternativeWidth);
		int coordsHeight = (int)((mAreaHeight*Constants.TILE_SIZE) / alternativeHeight);
		int[][] coords = new int[(int)coordsWidth*(int)coordsHeight][2];
		boolean[] usedCoords = new boolean[coords.length];
		int k =0;
		
		for(int i=0;i<coordsWidth;i++){
			for(int j=0;j<coordsHeight;j++){
				coords[k][0] = i*(int)alternativeWidth;
				coords[k][1] = j*(int)alternativeHeight;
				usedCoords[k] = false;
				k++;
			}
		}
		boolean finished = false;

		int i =0;
		do{
			int temp = rand.nextInt(coords.length);
			if(!usedCoords[temp]){
				result[i] = coords[temp];
				usedCoords[temp]=true;
				i++;
			}
			if(i==4)finished=true;
		}while(!finished);

		return result;
	}

	@Override
	protected void questionAnswered(int nr) {
		mDuneBackground.setTextureSize(mParent.CAMERA_WIDTH, mParent.CAMERA_HEIGHT/2);
		mSandBackground.setTextureSize(mParent.CAMERA_WIDTH, mParent.CAMERA_HEIGHT/2);
		mCamera.setHUD(null);
		super.questionAnswered(nr);
	}


	@Override
	public void onScrollStarted(ScrollDetector pScollDetector, int pPointerID,
			float pDistanceX, float pDistanceY) {
		this.mCamera.offsetCenter(-pDistanceX, -pDistanceY);

	}


	@Override
	public void onScroll(ScrollDetector pScollDetector, int pPointerID,
			float pDistanceX, float pDistanceY) {
		this.mCamera.offsetCenter(-pDistanceX, -pDistanceY);

	}


	@Override
	public void onScrollFinished(ScrollDetector pScollDetector, int pPointerID,
			float pDistanceX, float pDistanceY) {
		this.mCamera.offsetCenter(-pDistanceX, -pDistanceY);

	}


	@Override
	public boolean onSceneTouchEvent(Scene pScene, TouchEvent pSceneTouchEvent) {
		if(pSceneTouchEvent.isActionDown()) {
			this.mScrollDetector.setEnabled(true);
		}
		this.mScrollDetector.onTouchEvent(pSceneTouchEvent);

		return true;
	}

}
