package adrian.master.knowledgewar.scenes;

import java.text.DecimalFormat;
import java.util.Random;

import org.andengine.engine.camera.hud.HUD;
import org.andengine.entity.scene.IOnSceneTouchListener;
import org.andengine.entity.scene.Scene;
import org.andengine.entity.sprite.Sprite;
import org.andengine.entity.text.Text;
import org.andengine.input.touch.TouchEvent;
import org.andengine.input.touch.detector.ScrollDetector;
import org.andengine.input.touch.detector.ScrollDetector.IScrollDetectorListener;
import org.andengine.input.touch.detector.SurfaceScrollDetector;
import org.andengine.opengl.texture.region.TextureRegion;

import adrian.master.knowledgewar.R;
import adrian.master.knowledgewar.gameactivities.BattleActivity;
import adrian.master.knowledgewar.gameconcepts.Question;
import adrian.master.knowledgewar.logic.Constants;
import adrian.master.knowledgewar.logic.UserFunctions;
import android.util.Log;

public class LostQuestionScene extends BaseDisplayQuestionScene implements IOnSceneTouchListener, IScrollDetectorListener {

	private int mAreaWidth = 40;
	private int mAreaHeight = 40;
	private SurfaceScrollDetector mScrollDetector;
	private boolean mAlternativesDisplayed = false;
	private Text mDistanceText;
	private DecimalFormat mDec;
//	private TextureRegion mLeather;
	private Sprite mDistanceBackground;
	private String QUESTION_CLOSE = "Close";
	private String QUESTION_MEDIUM = "Medium";
	private String QUESTION_FARAWAY = "Far";
	private String DISTANCE="";
	private int DISTANCE_FARAWAY = 1000;
	private int DISTANCE_CLOSE = 350;

	public LostQuestionScene(BattleActivity parent, Question question){
		super(parent,question);

		this.mScrollDetector = new SurfaceScrollDetector(this);
		this.setOnAreaTouchTraversalFrontToBack();
		this.setOnSceneTouchListener(this);
		
	}

	@Override
	protected void setupScene() {
		mAreaWidth=40;
		mAreaHeight=40;
		DISTANCE = mParent.mRes.getString(R.string.battle_question_distance);
		createBackgrounds();
		createTimerSprite();
		createTexts();
		setUpQuestionSprite();
		placeQuestionSprite();
		setUpAlternativeSprites();
		placeAlternativeSprites();
		createStrings();
		displayQuestions();
	}

	@Override
	protected void createBackgrounds() {
		//Lower part
		mHud = new HUD();
		mCamera.setHUD(mHud);
		mSandBackground.setTextureSize(mParent.CAMERA_WIDTH, mParent.CAMERA_HEIGHT/2);
		mSandBackgroundSprite = new Sprite(0,mParent.CAMERA_HEIGHT/2, mSandBackground, mMgr);
		mHud.attachChild(mSandBackgroundSprite);

		mDistanceBackground = new Sprite(0,0,mParent.CAMERA_WIDTH-2*Constants.TILE_SIZE, mFont.getLineHeight()*2, mLeather, mMgr);
		mDistanceBackground.setPosition(mParent.CAMERA_WIDTH/2 - mDistanceBackground.getWidth()/2, 10);
		mHud.attachChild(mDistanceBackground);

		//Upper part
		mSandBackground.setTextureSize(mAreaWidth*Constants.TILE_SIZE*2, mAreaHeight*Constants.TILE_SIZE*2);
		mDuneBackgroundSprite = new Sprite(0,0, mAreaWidth*Constants.TILE_SIZE, mAreaHeight*Constants.TILE_SIZE, mSandBackground, mMgr);
		mCamera.setBounds(mDuneBackgroundSprite.getX(), mDuneBackgroundSprite.getY(), 
				mDuneBackgroundSprite.getX()+mDuneBackgroundSprite.getWidth(), 
				mDuneBackgroundSprite.getY()+mDuneBackgroundSprite.getHeight());
		mCamera.setBoundsEnabled(true);
		attachChild(mDuneBackgroundSprite);
	}

	@Override
	protected void createTimerSprite() {
		Sprite timerBorder = new Sprite(0,0,mTimerBorder,mMgr);
		this.mRedbarTexture.setTextureSize(mParent.CAMERA_WIDTH, 32);
		this.mGreenbarTexture.setTextureSize(mParent.CAMERA_WIDTH, 32);
		this.mTimerBarSprite = new Sprite(0,0,mGreenbarTexture, mMgr);
		this.mTimerBarSprite.setPosition(0, mParent.CAMERA_HEIGHT/2- mTimerBarSprite.getHeight()/2);
		timerBorder.setPosition(0, mTimerBarSprite.getHeight()/2-timerBorder.getHeight()/2);
		mHud.attachChild(mTimerBarSprite);
		mTimerBarSprite.attachChild(timerBorder);

		this.mDistanceText = new Text(0,0,mFont,DISTANCE,25,mMgr);
		mDistanceText.setPosition(mDistanceBackground.getWidth()/2-mDistanceText.getWidth()/2, 
				mDistanceBackground.getHeight()/2 - mDistanceText.getHeight()/2);
		mDistanceBackground.attachChild(mDistanceText);
		mDec = new DecimalFormat();
		mDec.setMaximumFractionDigits(3);
	}

	@Override
	protected void placeQuestionSprite() {
		Random rand = UserFunctions.getRandomNumberGenerator();
		int pX = rand.nextInt(mAreaWidth)*mAreaWidth;
		int pY = rand.nextInt(mAreaHeight)*mAreaHeight;
		mQuestionSprite.setPosition(pX, pY);
	}

	@Override
	protected void placeAlternativeSprites() {
		mAlternativeSprite1.setPosition(mParent.CAMERA_WIDTH/2-mAlternativeSprite1.getWidth()-10, 
				mParent.CAMERA_HEIGHT/4-mAlternativeSprite1.getHeight()-10);

		mAlternativeSprite2.setPosition(mParent.CAMERA_WIDTH/2+10, 
				mParent.CAMERA_HEIGHT/4-mAlternativeSprite2.getHeight()-10);

		mAlternativeSprite3.setPosition(mParent.CAMERA_WIDTH/2-mAlternativeSprite3.getWidth()-10, 
				mParent.CAMERA_HEIGHT/4+10);

		mAlternativeSprite4.setPosition(mParent.CAMERA_WIDTH/2+10, 
				mParent.CAMERA_HEIGHT/4+10);

		mSandBackgroundSprite.attachChild(mAlternativeSprite1);
		mSandBackgroundSprite.attachChild(mAlternativeSprite2);
		mSandBackgroundSprite.attachChild(mAlternativeSprite3);
		mSandBackgroundSprite.attachChild(mAlternativeSprite4);


	}

	@Override
	protected void questionAnswered(int nr) {
		mDuneBackground.setTextureSize(mParent.CAMERA_WIDTH, mParent.CAMERA_HEIGHT/2);
		mSandBackground.setTextureSize(mParent.CAMERA_WIDTH, mParent.CAMERA_HEIGHT/2);
		mCamera.setHUD(null);
		super.questionAnswered(nr);
	}

	@Override
	protected void displayAlternatives() {
		super.displayAlternatives();
		mHud.registerTouchArea(mAlternativeSprite1);
		mHud.registerTouchArea(mAlternativeSprite2);
		mHud.registerTouchArea(mAlternativeSprite3);
		mHud.registerTouchArea(mAlternativeSprite4);
	}

	private void calculateDistanceText(){
		float distanceX = mCamera.getCenterX() - mQuestionSprite.getX();
		float distanceY = mCamera.getCenterY() - Constants.CAMERA_HEIGHT/4 - mQuestionSprite.getY();
		double distance = Math.sqrt(distanceX*distanceX+distanceY*distanceY);

		if(distance<DISTANCE_CLOSE){
			mDistanceText.setText(DISTANCE+" "+QUESTION_CLOSE);			
		}else if(distance>DISTANCE_CLOSE && distance<DISTANCE_FARAWAY){
			mDistanceText.setText(DISTANCE+" "+QUESTION_MEDIUM);
		}else{
			mDistanceText.setText(DISTANCE+" "+QUESTION_FARAWAY);
		}
		mDistanceText.setPosition(mDistanceBackground.getWidth()/2-mDistanceText.getWidth()/2, 
				mDistanceBackground.getHeight()/2 - mDistanceText.getHeight()/2);
	}

	@Override
	public void onScrollStarted(final ScrollDetector pScollDetector, final int pPointerID, final float pDistanceX, final float pDistanceY) {
		this.mCamera.offsetCenter(-pDistanceX, -pDistanceY);
	}

	@Override
	public void onScroll(final ScrollDetector pScollDetector, final int pPointerID, final float pDistanceX, final float pDistanceY) {
		this.mCamera.offsetCenter(-pDistanceX, -pDistanceY);
		calculateDistanceText();
	}

	@Override
	public void onScrollFinished(final ScrollDetector pScollDetector, final int pPointerID, final float pDistanceX, final float pDistanceY) {
		this.mCamera.offsetCenter(-pDistanceX, -pDistanceY);
		if(mCamera.isRectangularShapeVisible(mQuestionSprite) && !mAlternativesDisplayed){
			displayAlternatives();
			mAlternativesDisplayed=true;
		}
	}

	@Override
	public boolean onSceneTouchEvent(Scene pScene, TouchEvent pSceneTouchEvent) {
		if(pSceneTouchEvent.isActionDown()) {
			this.mScrollDetector.setEnabled(true);
		}
		this.mScrollDetector.onTouchEvent(pSceneTouchEvent);

		return true;
	}

}
