package adrian.master.knowledgewar.scenes;

import org.andengine.engine.Engine;
import org.andengine.entity.scene.Scene;
import org.andengine.entity.scene.background.AutoParallaxBackground;
import org.andengine.entity.scene.background.ParallaxBackground.ParallaxEntity;
import org.andengine.entity.sprite.Sprite;
import org.andengine.entity.text.AutoWrap;
import org.andengine.entity.text.Text;
import org.andengine.entity.text.TextOptions;
import org.andengine.input.touch.TouchEvent;
import org.andengine.opengl.font.Font;
import org.andengine.opengl.texture.region.TextureRegion;
import org.andengine.opengl.vbo.VertexBufferObjectManager;
import org.andengine.util.HorizontalAlign;
import org.json.JSONException;
import org.json.JSONObject;

import adrian.master.knowledgewar.R;
import adrian.master.knowledgewar.gameactivities.MainActivity;
import adrian.master.knowledgewar.gameconcepts.Player;
import adrian.master.knowledgewar.helpscenes.MainMenuHelpScene;
import adrian.master.knowledgewar.interfaces.MainActivityInterface;
import adrian.master.knowledgewar.logic.Constants;
import adrian.master.knowledgewar.logic.Server;
import adrian.master.knowledgewar.logic.UserFunctions;
import android.content.Context;
import android.content.res.Resources;
import android.os.AsyncTask;
import android.widget.Toast;

public class MainMenuScene extends Scene implements MainActivityInterface {
	
	private TextureRegion mPaperTexture, mLeatherTexture;
	private TextureRegion mBackgroundTexture;
	private MainActivity mParent;
	private AutoParallaxBackground mBackground;
	
	private Sprite mPlayersButton, mOwnTribeButton, mChallengesButton, mLogoutButton, mHelpButton, mLeatherBackground, mLogo;
	private VertexBufferObjectManager mVertexManager;
	private Text mBusyText;
	private Engine mEngine;
	private boolean mIsShowingHelpScene;
	private Scene mHelpScene;
	private Player mSelf;
	
	public MainMenuScene(MainActivity parent, Engine engine){
		this.mParent = parent;
		this.mEngine = engine;
		this.mPaperTexture = mParent.mPaperTexture;
		this.mLeatherTexture = mParent.mLeatherTexture;
		this.mVertexManager = parent.getVertexBufferObjectManager();
		this.mBackgroundTexture = mParent.mDuneBackgroundTexture;
		this.mBackgroundTexture.setTextureHeight(Constants.CAMERA_HEIGHT);
		
		this.mBackground = new AutoParallaxBackground(0, 0, 0, 5);
		Sprite temp = new Sprite(0, 0, mBackgroundTexture, mVertexManager);
		mBackground.attachParallaxEntity(new ParallaxEntity(5, temp));
		this.setBackground(mBackground);
		setupLogo();
		
		setUpButtons();
		
		mLeatherBackground = new Sprite(0,0, Constants.CAMERA_WIDTH-Constants.TILE_SIZE, Constants.TILE_SIZE, mLeatherTexture,mVertexManager );
//		mLeatherBackground.setPosition(Constants.CAMERA_WIDTH/2 - mLeatherBackground.getWidth()/2,
//				mQuestionsButton.getY()+mChallengesButton.getHeight()+20);
//		attachChild(mLeatherBackground);
		
		
	}
	
	private void setupLogo(){
		mLogo = new Sprite(0,0,mParent.mLogoTexture,mVertexManager);
		mLogo.setPosition(MainActivity.CAMERA_WIDTH/2 - mLogo.getWidth()/2, 60);
		mLogo.setScale(0.5f);
		attachChild(mLogo);
	}
	
	private void setUpButtons(){
		mPlayersButton = new Sprite(0, 0,240,80, mPaperTexture, mVertexManager){ 
			@Override
			public boolean onAreaTouched(final TouchEvent pSceneTouchEvent, final float pTouchAreaLocalX, final float pTouchAreaLocalY) {
				switch(pSceneTouchEvent.getAction()) {
				case TouchEvent.ACTION_DOWN:
					this.setScale(Constants.BUTTONPRESSED_SCALE);
					return true;
				case TouchEvent.ACTION_UP:
					this.setScale(1);
					mParent.toastOnUIThread("Map", Toast.LENGTH_SHORT); 
					mParent.setMapScene();
					return true;
				}
				return false;
			}
		};
		mPlayersButton.setPosition(Constants.CAMERA_WIDTH/2 - mPlayersButton.getWidth()/2, 170+mLogo.getHeightScaled());
		this.attachChild(mPlayersButton);
//		this.registerTouchArea(mPlayersButton);
		
		mOwnTribeButton = new Sprite(0, 0, 240,80, mPaperTexture, mVertexManager){
			@Override
			public boolean onAreaTouched(final TouchEvent pSceneTouchEvent, final float pTouchAreaLocalX, final float pTouchAreaLocalY) {
				switch(pSceneTouchEvent.getAction()) {
				case TouchEvent.ACTION_DOWN:
					this.setScale(Constants.BUTTONPRESSED_SCALE);
					return true;
				case TouchEvent.ACTION_UP:
					this.setScale(1);
					mParent.toastOnUIThread("Own Tribe", Toast.LENGTH_SHORT); 
					mParent.setOwnTribeScene();
					return true;
				}
				return false;
			}
		};
		mOwnTribeButton.setPosition(Constants.CAMERA_WIDTH/2 - mOwnTribeButton.getWidth()/2, 
				mPlayersButton.getY()+mPlayersButton.getHeight()+10);
		this.attachChild(mOwnTribeButton);
//		this.registerTouchArea(mOwnTribeButton);
		
		mChallengesButton = new Sprite(0, 0, 240,80, mPaperTexture, mVertexManager){
			@Override
			public boolean onAreaTouched(final TouchEvent pSceneTouchEvent, final float pTouchAreaLocalX, final float pTouchAreaLocalY) {
				switch(pSceneTouchEvent.getAction()) {
				case TouchEvent.ACTION_DOWN:
					this.setScale(Constants.BUTTONPRESSED_SCALE);
					return true;
				case TouchEvent.ACTION_UP:
					this.setScale(1);
					mParent.toastOnUIThread("Pending challenges", Toast.LENGTH_SHORT); 
					mParent.setPendingChallengesScene();
					return true;
				}
				return false;
			}
		};
		mChallengesButton.setPosition(Constants.CAMERA_WIDTH/2 - mChallengesButton.getWidth()/2, 
				mOwnTribeButton.getY()+mOwnTribeButton.getHeight()+10);
		this.attachChild(mChallengesButton);
		this.registerTouchArea(mChallengesButton);
		
//		mQuestionsButton = new Sprite(0,0,240,80,mPaperTexture,mVertexManager){
//			public boolean onAreaTouched(TouchEvent pSceneTouchEvent, float pTouchAreaLocalX, float pTouchAreaLocalY) {
//				switch(pSceneTouchEvent.getAction()) {
//				case TouchEvent.ACTION_DOWN:
//					this.setScale(Constants.BUTTONPRESSED_SCALE);
//					return true;
//				case TouchEvent.ACTION_UP:
//					this.setScale(1);
//					mParent.setQuestionsScene();
//					return true;
//				}
//				return false;
//			}
//		};
//		mQuestionsButton.setPosition(Constants.CAMERA_WIDTH/2 - mQuestionsButton.getWidth()/2, 
//				mChallengesButton.getY()+mChallengesButton.getHeight()+10);
//		this.attachChild(mQuestionsButton);
//		this.registerTouchArea(mQuestionsButton);
		
		mLogoutButton = new Sprite(0,0, 128,64, mPaperTexture, mVertexManager){
			@Override
			public boolean onAreaTouched(final TouchEvent pSceneTouchEvent, final float pTouchAreaLocalX, final float pTouchAreaLocalY) {
				switch(pSceneTouchEvent.getAction()) {
				case TouchEvent.ACTION_DOWN:
					this.setScale(Constants.BUTTONPRESSED_SCALE);
					return true;
				case TouchEvent.ACTION_UP:
					this.setScale(1);
					mParent.logOut();
					return true;
				}
				return false;
			}
		};
		mLogoutButton.setPosition(0, Constants.CAMERA_HEIGHT-mLogoutButton.getHeight());
		this.attachChild(mLogoutButton);
		this.registerTouchArea(mLogoutButton);
		
//		mHelpButton = new Sprite(480-128, 720-128,128,64, mPaperTexture, mVertexManager){
//			@Override
//			public boolean onAreaTouched(final TouchEvent pSceneTouchEvent, final float pTouchAreaLocalX, final float pTouchAreaLocalY) {
//				switch(pSceneTouchEvent.getAction()) {
//				case TouchEvent.ACTION_DOWN:
//					this.setScale(Constants.BUTTONPRESSED_SCALE);
//					return true;
//				case TouchEvent.ACTION_UP:
//					this.setScale(1);
//					mParent.setInstructionScene();
//					return true;
//				}
//				return false;
//			}
//		};
//		mHelpButton.setPosition(Constants.CAMERA_WIDTH-mHelpButton.getWidth(), Constants.CAMERA_HEIGHT-mHelpButton.getHeight());
//		this.attachChild(mHelpButton);
//		this.registerTouchArea(mHelpButton);
		
		setTextOnButtons();
		
		mPlayersButton.setAlpha(0.5f);
		mOwnTribeButton.setAlpha(0.5f);
		
	}
	
	private void setTextOnButtons(){
		TextOptions buttonOpt = new TextOptions(AutoWrap.WORDS, 220, HorizontalAlign.CENTER);
		Font font = mParent.mFontBold;
		Resources res = mParent.getResources();
		String map = res.getString(R.string.mainmenu_map);
		Text mapButtonText = new Text(0,0, font,map, map.length(),buttonOpt, mVertexManager);
		mapButtonText.setPosition(10, 
				mPlayersButton.getHeight()/2 - mapButtonText.getHeight()/2);
		mPlayersButton.attachChild(mapButtonText);
		
		String ownTribe = res.getString(R.string.mainmenu_owntribe);
		Text ownTribeButtonText = new Text(0, 0, font,ownTribe, ownTribe.length(),buttonOpt, mVertexManager);
		ownTribeButtonText.setPosition(10, 
				mOwnTribeButton.getHeight()/2 - ownTribeButtonText.getHeight()/2);
		mOwnTribeButton.attachChild(ownTribeButtonText);
		
		String logout = res.getString(R.string.mainmenu_logout);
		Text logoutButtonText = new Text(0, 0, font,logout, logout.length(), mVertexManager);
		logoutButtonText.setPosition(mLogoutButton.getWidth()/2 - logoutButtonText.getWidth()/2, 
				mLogoutButton.getHeight()/2 - logoutButtonText.getHeight()/2);
		mLogoutButton.attachChild(logoutButtonText);
		
//		String help = res.getString(R.string.mainmenu_help);
//		Text helpButtonText = new Text(0, 0, font,help, help.length(), mVertexManager);
//		helpButtonText.setPosition(mHelpButton.getWidth()/2 - helpButtonText.getWidth()/2, 
//				mHelpButton.getHeight()/2 - helpButtonText.getHeight()/2);
//		mHelpButton.attachChild(helpButtonText);
		
		String pending = res.getString(R.string.mainmenu_pending_challenges);
		Text pendingText = new Text(0,0,font,pending,buttonOpt,mVertexManager);
		pendingText.setPosition(10, mChallengesButton.getHeight()/2-pendingText.getHeight()/2);
		mChallengesButton.attachChild(pendingText);
		
//		String questions = res.getString(R.string.mainmenu_questions);
//		Text questionButtonText = new Text(0,0,font,questions,buttonOpt,mVertexManager);
//		questionButtonText.setPosition(10, mQuestionsButton.getHeight()/2 - questionButtonText.getHeight()/2);
//		mQuestionsButton.attachChild(questionButtonText);
	}
	
	public void initialize(){
		if(mBusyText!=null){
			mBusyText.detachSelf();
		}
		boolean busy = UserFunctions.getUserBusy(mParent.getApplicationContext());
		String busyText="";
		if(busy){
			busyText=mParent.getString(R.string.mainmenu_youarebusy);
		}else{
			busyText=mParent.getString(R.string.mainmenu_youarenotbusy);
		}
		mBusyText = new Text(0,0,mParent.mFont,busyText,new TextOptions(AutoWrap.WORDS, mLeatherBackground.getWidth()-60, HorizontalAlign.CENTER), mVertexManager);
		mBusyText.setPosition(30,mLeatherBackground.getHeight()/2 - mBusyText.getHeight()/2);
		mLeatherBackground.attachChild(mBusyText);
		
		mPlayersButton.setAlpha(1);
		mOwnTribeButton.setAlpha(1);
		this.registerTouchArea(mPlayersButton);
		this.registerTouchArea(mOwnTribeButton);
	}
	

	@Override
	public boolean onBackButtonPressed() {
		if(mIsShowingHelpScene){
			mIsShowingHelpScene=false;
			mParent.updateHUD(hasHelpScene());
			new TransitionScene(mParent, mParent.mZoomCamera, mEngine,mHelpScene, this);
			return true;
		}
		return false;
	}

	@Override
	public void showHelpScreen() {
		mIsShowingHelpScene=true;
		mHelpScene = new MainMenuHelpScene(mParent);
		mParent.updateHUD(false);
		new TransitionScene(mParent, mParent.mZoomCamera, mEngine, this, mHelpScene);
	}

	@Override
	public boolean hasHelpScene() {
		return true;
	}

}
