package adrian.master.knowledgewar.scenes;

import java.util.ArrayList;
import java.util.Random;

import org.andengine.engine.Engine;
import org.andengine.engine.camera.ZoomCamera;
import org.andengine.entity.IEntity;
import org.andengine.entity.IEntityMatcher;
import org.andengine.entity.scene.IOnSceneTouchListener;
import org.andengine.entity.scene.ITouchArea;
import org.andengine.entity.scene.ITouchArea.ITouchAreaMatcher;
import org.andengine.entity.scene.Scene;
import org.andengine.entity.scene.background.Background;
import org.andengine.entity.sprite.Sprite;
import org.andengine.entity.text.Text;
import org.andengine.input.touch.TouchEvent;
import org.andengine.input.touch.detector.PinchZoomDetector;
import org.andengine.input.touch.detector.PinchZoomDetector.IPinchZoomDetectorListener;
import org.andengine.input.touch.detector.ScrollDetector;
import org.andengine.input.touch.detector.ScrollDetector.IScrollDetectorListener;
import org.andengine.input.touch.detector.SurfaceScrollDetector;
import org.andengine.opengl.font.Font;
import org.andengine.opengl.texture.region.TextureRegion;
import org.andengine.opengl.vbo.VertexBufferObjectManager;
import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;

import adrian.master.knowledgewar.R;
import adrian.master.knowledgewar.gameactivities.MainActivity;
import adrian.master.knowledgewar.gameconcepts.Building;
import adrian.master.knowledgewar.gameconcepts.Player;
import adrian.master.knowledgewar.helpscenes.MapHelpScene;
import adrian.master.knowledgewar.interfaces.MainActivityInterface;
import adrian.master.knowledgewar.logic.Constants;
import adrian.master.knowledgewar.logic.Server;
import adrian.master.knowledgewar.logic.UserFunctions;
import android.content.Context;
import android.os.AsyncTask;
import android.util.Log;
import android.widget.Toast;

public class MapScene extends Scene implements IOnSceneTouchListener, IScrollDetectorListener, IPinchZoomDetectorListener, MainActivityInterface {

	private SurfaceScrollDetector mScrollDetector;
	private PinchZoomDetector mPinchZoomDetector;
	private float mPinchZoomStartedCameraZoomFactor;
	private Random mRand;

	private VertexBufferObjectManager mMgr;
	private TextureRegion mCartTexture;
	private Sprite mBackground;

	private MainActivity mParent;

	private ZoomCamera mZoomCamera;
	private Engine mEngine;

	private Player mSelf;
	private float[][] mCoords;
	private boolean[] mUsedCoords;

	private ArrayList<Player> mPlayerList = new ArrayList<Player>();
	private int MAP_SIZE_X = 102;
	private int MAP_SIZE_Y = 102;
	private int mMaxNumberOfPlayersHorizontally = (MAP_SIZE_X/6)-1;
	private int mMaxNumberOfPlayersVertically = (MAP_SIZE_Y/6)-1;
	private float mBackgroundXMin, mBackgroundXMax, mBackgroundYmin, mBackgroundYMax;

	private Font mFont;
	
	private boolean mIsShowingHelpScene=false;
	private Scene mHelpScene;

	public MapScene(MainActivity parent, Engine engine){

		mParent = parent;
		mEngine = engine;
		mRand = UserFunctions.getRandomNumberGenerator();
		mMgr = mParent.getVertexBufferObjectManager();
		mCartTexture = mParent.mCartTexture;
		mFont = mParent.mFontBold;
//		createCoords();

		this.mZoomCamera=mParent.mZoomCamera;

		this.mScrollDetector = new SurfaceScrollDetector(this);
		this.mPinchZoomDetector = new PinchZoomDetector(this);
		this.setOnAreaTouchTraversalFrontToBack();

		this.setBackground(new Background(0.09804f, 0.6274f, 0.8784f));

		this.setOnSceneTouchListener(this);
		this.setTouchAreaBindingOnActionDownEnabled(true);

		this.mParent.mSandTexture.setTextureSize(MAP_SIZE_X*Constants.TILE_SIZE, MAP_SIZE_Y*Constants.TILE_SIZE);
		mBackground = new Sprite(-Constants.TILE_SIZE*25, -Constants.TILE_SIZE*25, 
				mParent.mSandTexture, mMgr);
		this.attachChild(mBackground);
		mBackgroundXMin = mBackground.getX();
		mBackgroundXMax = mBackgroundXMin+mBackground.getWidth();
		mBackgroundYmin = mBackground.getY();
		mBackgroundYMax = mBackgroundYmin+mBackground.getHeight();
	}

	private void createCoords(int numberOfPlayersHorizontally, int numberOfPlayersVertically){

//		float numberOfPlayersHorizontally = (MAP_SIZE_X/5)-1;
//		float numberOfPlayersVertically = (MAP_SIZE_Y/5)-1;
		mCoords = new float[(int)numberOfPlayersHorizontally*(int)numberOfPlayersVertically][2];
		mUsedCoords = new boolean[(int)numberOfPlayersHorizontally*(int)numberOfPlayersVertically];
		int k = 0;
		for(int i=0; i<numberOfPlayersHorizontally;i++){
			for(int j=0;j<numberOfPlayersVertically;j++){
				mCoords[k][0] = i*Constants.TILE_SIZE*6+(Constants.TILE_SIZE);
				mCoords[k][1] = j*Constants.TILE_SIZE*6+(Constants.TILE_SIZE);
				mUsedCoords[k] = false;
				k++;
			}
		}
	}

	public void startScene(){
		mSelf=UserFunctions.self;
		clearMap();
		PlayerFinder finder = new PlayerFinder();
		finder.execute();
		mZoomCamera.setZoomFactor(0.5f);
		
	}
	
	private void setCameraSettings(){
		mZoomCamera.setZoomFactor(0.5f);
		mZoomCamera.setBoundsEnabled(true);
	}
	
	private void createMapBoundaries(int numberOfPlayers){
		mZoomCamera.setBoundsEnabled(true);
		if(numberOfPlayers<10){
			createCoords(5,6);
			mZoomCamera.setBounds(mBackgroundXMin, mBackgroundYmin, mBackgroundXMin+6*5*Constants.TILE_SIZE, mBackgroundYmin+8*5*Constants.TILE_SIZE);
//			mZoomCamera.setCenter(mBackgroundXMin+MainActivity.CAMERA_WIDTH/2*mZoomCamera.getZoomFactor(), 
//					mBackgroundXMin+MainActivity.CAMERA_HEIGHT/2*mZoomCamera.getZoomFactor());
		}
		else if(numberOfPlayers>=10 && numberOfPlayers<50){
			createCoords(13, 12);
			mZoomCamera.setBounds(mBackgroundXMin, mBackgroundYmin, mBackgroundXMin+14*5*Constants.TILE_SIZE, mBackgroundYmin+14*5*Constants.TILE_SIZE);
		}
		else{
			createCoords(mMaxNumberOfPlayersHorizontally, mMaxNumberOfPlayersVertically);
			mZoomCamera.setBounds(mBackgroundXMin, mBackgroundYmin, mBackgroundXMax, mBackgroundYMax);
		}
	}
	

	private void clearMap(){
		mEngine.runSafely(new Runnable() {
			@Override
			public void run() {
				mBackground.detachChildren();
			}
		});
		unregisterTouchAreas(new ITouchAreaMatcher() {

			@Override
			public boolean matches(ITouchArea pObject) {
				return true;
			}
		});
	}

	@Override
	public void onScrollStarted(final ScrollDetector pScollDetector, final int pPointerID, final float pDistanceX, final float pDistanceY) {
		final float zoomFactor = this.mZoomCamera.getZoomFactor();
		this.mZoomCamera.offsetCenter(-pDistanceX/zoomFactor, -pDistanceY/zoomFactor);
	}

	@Override
	public void onScroll(final ScrollDetector pScollDetector, final int pPointerID, final float pDistanceX, final float pDistanceY) {
		final float zoomFactor = this.mZoomCamera.getZoomFactor();
		this.mZoomCamera.offsetCenter(-pDistanceX/zoomFactor, -pDistanceY/zoomFactor);
	}

	@Override
	public void onScrollFinished(final ScrollDetector pScollDetector, final int pPointerID, final float pDistanceX, final float pDistanceY) {
		final float zoomFactor = this.mZoomCamera.getZoomFactor();
		this.mZoomCamera.offsetCenter(-pDistanceX/zoomFactor, -pDistanceY/zoomFactor);
	}

	@Override
	public void onPinchZoomStarted(final PinchZoomDetector pPinchZoomDetector, final TouchEvent pTouchEvent) {
		this.mPinchZoomStartedCameraZoomFactor = this.mZoomCamera.getZoomFactor();
	}

	@Override
	public void onPinchZoom(final PinchZoomDetector pPinchZoomDetector, final TouchEvent pTouchEvent, final float pZoomFactor) {
		float zoom = this.mPinchZoomStartedCameraZoomFactor * pZoomFactor;

		if(zoom<0.33f){
			this.mZoomCamera.setZoomFactor(0.33f);			
		}else if(zoom>5f){
			this.mZoomCamera.setZoomFactor(5f);	
		}else{
			this.mZoomCamera.setZoomFactor(zoom);	
		}
	}

	@Override
	public void onPinchZoomFinished(final PinchZoomDetector pPinchZoomDetector, final TouchEvent pTouchEvent, final float pZoomFactor) {
		float zoom = this.mPinchZoomStartedCameraZoomFactor * pZoomFactor;

		if(zoom<0.33f){
			this.mZoomCamera.setZoomFactor(0.33f);			
		}else if(zoom>5f){
			this.mZoomCamera.setZoomFactor(5f);	
		}else{
			this.mZoomCamera.setZoomFactor(zoom);	
		}
	}

	@Override
	public boolean onSceneTouchEvent(Scene pScene, TouchEvent pSceneTouchEvent) {
		this.mPinchZoomDetector.onTouchEvent(pSceneTouchEvent);

		if(this.mPinchZoomDetector.isZooming()) {
			this.mScrollDetector.setEnabled(false);
		} else {
			if(pSceneTouchEvent.isActionDown()) {
				this.mScrollDetector.setEnabled(true);
			}
			this.mScrollDetector.onTouchEvent(pSceneTouchEvent);
		}

		return true;
	}

	private void addPlayerToBoard(final Player player, final int nr){

		int x = 5*Constants.TILE_SIZE;
		int y = 5*Constants.TILE_SIZE;

		int enemyRank = player.getRank();
		UserFunctions.PlayerStrength strength = UserFunctions.calculatePlayerStrength(mSelf.getRank(), enemyRank);
		TextureRegion enemyTiles;
		switch(strength){
		case WEAKER: enemyTiles = mParent.mGreenTileTexture;break;
		case SAME: enemyTiles = mParent.mYellowTileTexture;break;
		case STRONGER: enemyTiles = mParent.mRedTileTexture;break;
		default : enemyTiles = mParent.mRedTileTexture;break;
		}


		boolean finished = false;
		float[] coords= {0,0};
		do{
			int temp = mRand.nextInt(mCoords.length);
			if(!mUsedCoords[temp]){
				coords = mCoords[temp];
				mUsedCoords[temp]=true;
				finished=true;
				Log.v("Knowledgewar","MapScene addplayer: coords "+coords[0]+" "+coords[1]);
			}
		}while(!finished);

		enemyTiles.setTextureSize(x, y);
		Sprite temp = new Sprite(coords[0], coords[1],enemyTiles, mMgr)
		{
			long actionDown=0;
			boolean pressing=false;

			@Override
			public boolean onAreaTouched(final TouchEvent pSceneTouchEvent, final float pTouchAreaLocalX, final float pTouchAreaLocalY) {
				switch(pSceneTouchEvent.getAction()) {
				case TouchEvent.ACTION_DOWN:
					pressing=true;
					actionDown=System.currentTimeMillis();
					break;
				case TouchEvent.ACTION_UP:
					if(System.currentTimeMillis()-actionDown>100 && pressing){
						mParent.createChallengeScene(nr, player);
						actionDown=0;
					}
					pressing=false;
					break;

				}
				return true;
			}
		};

		String name = player.getName();
		Text nameText = new Text(0,0, mFont,name,mMgr);
		Sprite nameTag = null;
		if(nameText.getWidth()<Constants.TILE_SIZE*3-10){
			nameTag = new Sprite(0,0,Constants.TILE_SIZE*3-10, nameText.getHeight()+10, mParent.mLeatherTexture, mMgr);	
		}else{
			nameTag = new Sprite(0,0,nameText.getWidth()+10, nameText.getHeight()+10, mParent.mLeatherTexture, mMgr);						
		}
		nameText.setPosition(nameTag.getWidth()/2-nameText.getWidth()/2, nameTag.getHeight()/2-nameText.getHeight()/2);
		nameTag.attachChild(nameText);
		nameTag.setPosition(temp.getWidth()/2-nameTag.getWidth()/2, -nameTag.getHeight()/2);
		temp.attachChild(nameTag);

		for(Building b: player.getBuildings()){
			Sprite cart = new Sprite(b.getX()*Constants.TILE_SIZE,b.getY()*Constants.TILE_SIZE,
					Constants.TILE_SIZE,Constants.TILE_SIZE,mCartTexture, mMgr);
			temp.attachChild(cart);			
		}

		temp.setAlpha(0.15f);
		mBackground.attachChild(temp);

		registerTouchArea(temp);

	}

	private class PlayerFinder extends AsyncTask<Void, Void, JSONArray>{


		@Override
		protected void onPreExecute() {
			mParent.displayLoadingAnimation(true);
		}
		
		@Override
		protected JSONArray doInBackground(Void... arg0) {

			try {
				String result="";
				do{
					result = Server.getPlayers(mParent.getApplicationContext());	
					if(result.equals(Constants.HTTP_EXECUTE_FAILURE)){
						publishProgress();
					}
				}while(result.equals(Constants.HTTP_EXECUTE_FAILURE));
				JSONArray ar = new JSONArray(result);
				return ar;
			} catch (JSONException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
			return null;
		}
		
		@Override
		protected void onProgressUpdate(Void... values) {
			Toast.makeText(mParent.getApplicationContext(), mParent.getString(R.string.server_could_not_contact), Toast.LENGTH_SHORT).show();
		}

		protected void onPostExecute(JSONArray result) {
			JSONObject obj;
			clearMap();
			mPlayerList.clear();
			if(result!=null){
				try {
					for (int i=0;i<result.length();i++) {
						obj=result.getJSONObject(i);
						Player temp = UserFunctions.createPlayer(obj);
						mPlayerList.add(temp);
					}
				} catch (JSONException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}
			}
			createMapBoundaries(mPlayerList.size());
			for(int i=0;i<mPlayerList.size();i++){
				Player p = mPlayerList.get(i);
				if(!p.isBusy() && i<Constants.NUMBER_OF_PLAYERS_ON_MAP){
					addPlayerToBoard(p, i);							
				}				
			}
			UserFunctions.setPlayerList(mPlayerList);
			
			mParent.displayLoadingAnimation(false);
		}

	}

//	private class GetStats extends AsyncTask<Void, Void, String>{
//
//		Context context;
//
//		public GetStats(Context con){
//			context=con;
//		}
//
//		@Override
//		protected String doInBackground(Void... params) {
//			String result = Server.getOwnStats(context);
//			return result;
//		}
//
//		@Override
//		protected void onPostExecute(String result){
//
//			JSONObject obj;
//			try {
//				obj = new JSONObject(result);
//				String name = obj.getString(Constants.PLAYER_ACCOUNT);
//				String area = obj.getString(Constants.PLAYER_AREA);
//				int gold = Integer.parseInt(obj.getString(Constants.PLAYER_GOLD));
//				Player temp = new Player(name, area, gold);
//				mSelf=temp;
//				UserFunctions.self=mSelf;
//
//			} catch (JSONException e) {
//				// TODO Auto-generated catch block
//				e.printStackTrace();
//			}
//		}
//
//	}

	@Override
	public boolean onBackButtonPressed() {
		if(mIsShowingHelpScene){
			mIsShowingHelpScene=false;
			setCameraSettings();
			mParent.updateHUD(hasHelpScene());
			new TransitionScene(mParent, mParent.mZoomCamera, mEngine,mHelpScene, this);
			return true;
		}
		return false;
	}

	@Override
	public void showHelpScreen() {
		mParent.resetCameraSettings();
		mIsShowingHelpScene=true;
		mHelpScene = new MapHelpScene(mParent);
		new TransitionScene(mParent, mParent.mZoomCamera, mEngine, this, mHelpScene);
		mParent.updateHUD(false);
		
	}

	@Override
	public boolean hasHelpScene() {
		return true;
	}

}
