package adrian.master.knowledgewar.scenes;

import java.util.ArrayList;

import org.andengine.engine.Engine;
import org.andengine.entity.scene.Scene;
import org.andengine.entity.sprite.Sprite;
import org.andengine.entity.text.AutoWrap;
import org.andengine.entity.text.Text;
import org.andengine.entity.text.TextOptions;
import org.andengine.input.touch.TouchEvent;
import org.andengine.opengl.font.Font;
import org.andengine.opengl.texture.region.TextureRegion;
import org.andengine.opengl.vbo.VertexBufferObjectManager;
import org.andengine.util.HorizontalAlign;
import org.json.JSONException;
import org.json.JSONObject;

import adrian.master.knowledgewar.R;
import adrian.master.knowledgewar.gameactivities.MainActivity;
import adrian.master.knowledgewar.gameconcepts.Building;
import adrian.master.knowledgewar.gameconcepts.BuildingFactory;
import adrian.master.knowledgewar.gameconcepts.Player;
import adrian.master.knowledgewar.gameconcepts.Weapon;
import adrian.master.knowledgewar.helpscenes.OwnTribeHelpScene;
import adrian.master.knowledgewar.interfaces.MainActivityInterface;
import adrian.master.knowledgewar.interfaces.SubMenuInterface;
import adrian.master.knowledgewar.logic.Constants;
import adrian.master.knowledgewar.logic.Server;
import adrian.master.knowledgewar.logic.UserFunctions;
import adrian.master.knowledgewar.subscenes.GeneralBuildingSubMenu;
import adrian.master.knowledgewar.subscenes.SpecificBuildingSubMenu;
import android.content.Context;
import android.content.res.Resources;
import android.os.AsyncTask;
import android.util.Log;

public class OwnTribeScene extends Scene implements MainActivityInterface {

	private MainActivity mParent;
	private int CAMERA_HEIGHT = Constants.CAMERA_HEIGHT, CAMERA_WIDTH=Constants.CAMERA_WIDTH;
	private float STANDARD_BUTTON_SCALE = 0.85f;
	private float HIGHLIGHTED_BUTTON_SCALE = 1.1f;
	private float TILES_SCALE = 0.85f;
	private Resources mRes;
	private Player mSelf;
	private boolean[][] mBuildingPlacements;

	private VertexBufferObjectManager mMgr;

	private TextureRegion mSandBackground;
	private TextureRegion mTiles;
	private TextureRegion mDune;
	private TextureRegion mButton, mWhiteButton;
	private TextureRegion mBlueTile, mRedTile, mCart;
	private TextureRegion mLeather, mMoney;

	private Sprite mTilesSprite;
	private Sprite mDuneBackground, mDesertBackground, mHeader;
	private ArrayList<Sprite> mBuildingSprites;
	private Text mGoldText;
	private Font mFont;
	private Engine mEngine;

	private SubMenuInterface mChild;
	private Scene mHelpScene;
	private boolean isShowingSubMenu=false;
	private boolean isShowingHelpScene=false;
	private int mSelectedBuildingX, mSelectedBuildingY;

	public OwnTribeScene(MainActivity parent, Engine engine){
		this.initializeFields(parent, engine);
		this.createMap();
		this.createBuildingScreen();
		setTouchAreaBindingOnActionDownEnabled(true);

	}

	private void initializeFields(MainActivity parent, Engine engine){
		this.mParent = parent;
		this.mEngine = engine;
		this.mMgr = mParent.getVertexBufferObjectManager();
		this.mFont = mParent.mFontBold;
		this.mSandBackground = mParent.mSandTexture;
		this.mTiles = mParent.mBlankTileTexture;
		this.mDune = mParent.mDuneTexture;
		this.mButton = mParent.mButtonTexture;
		this.mWhiteButton = mParent.mWhiteButtonTexture;
		this.mBlueTile = mParent.mOwnTileTexture;
		this.mRedTile = mParent.mRedTileTexture;
		this.mCart = mParent.mCartTexture;
		this.mRes = mParent.getResources();
		this.mMoney = mParent.mMoneyTexture;
		this.mLeather = mParent.mLeatherTexture;
		this.mBuildingPlacements  = new boolean[5][5];

		this.mBuildingSprites = new ArrayList<Sprite>();
	}



	private void createMap(){
		this.mSandBackground.setTextureSize(CAMERA_WIDTH, CAMERA_HEIGHT/2);
		this.mTiles.setTextureSize(Constants.TILE_SIZE*5, Constants.TILE_SIZE*5);
		this.mTilesSprite = new Sprite((CAMERA_WIDTH/2)-(this.mTiles.getWidth()/2), 
				(CAMERA_HEIGHT/4)-(this.mTiles.getHeight()/2), mTiles, mMgr);

		this.mTilesSprite.setScale(TILES_SCALE);

		mDesertBackground = new Sprite(0,CAMERA_HEIGHT/2, mSandBackground, mMgr);
		this.attachChild(mDesertBackground);
		mDesertBackground.attachChild(mTilesSprite);
	}

	private void createBuildingScreen(){

		mDuneBackground=new Sprite(0,0,CAMERA_WIDTH, CAMERA_HEIGHT/2, mDune, mMgr);
		this.attachChild(mDuneBackground);
		createButtons(mDuneBackground);

		Sprite goldLeather = new Sprite(0,0,3*Constants.TILE_SIZE,mParent.mFontBold.getLineHeight()+10,mLeather,mMgr);
		Sprite gold = new Sprite(0,0,mParent.mFontBold.getLineHeight(), mParent.mFontBold.getLineHeight(),mMoney,mMgr);
		goldLeather.setPosition(CAMERA_WIDTH/2 - goldLeather.getWidth()/2, CAMERA_HEIGHT/2 - 5);
		gold.setPosition(30, goldLeather.getHeight()/2 - gold.getHeight()/2);
		goldLeather.attachChild(gold);
		mGoldText = new Text(0,0,mParent.mFontBold,"",7,mMgr);
		mGoldText.setPosition(gold.getX()+gold.getWidth()+10, goldLeather.getHeight()/2 - mGoldText.getHeight()/2);
		goldLeather.attachChild(mGoldText);

		mHeader = new Sprite(0,0,Constants.CAMERA_WIDTH-Constants.TILE_SIZE*2,Constants.TILE_SIZE, mLeather,mMgr);
		mHeader.setPosition(Constants.CAMERA_WIDTH/2 - mHeader.getWidth()/2, 5);
		mDuneBackground.attachChild(mHeader);

		Text text = new Text(0,0,mFont,mRes.getString(R.string.owntribe_header),mMgr);
		text.setPosition(mHeader.getWidth()/2 - text.getWidth()/2, mHeader.getHeight()/2 - text.getHeight()/2);
		mHeader.attachChild(text);
		attachChild(goldLeather);
	}

	private void createButtons(Sprite background){
		String[] buildings = Constants.BUILDING_TYPES;
		Sprite[] buildingButtons = new Sprite[buildings.length];
		Font font = mParent.mSmallFontBold;	

		float buttonSize = Constants.TILE_SIZE*1.5f;

		{
			BuildingButton temp = new BuildingButton(Constants.TILE_SIZE/2, Constants.TILE_SIZE, 
					buttonSize, buttonSize,
					mButton, mMgr, buildings[1]);
			String buildingName = temp.getBuildingName();
			buildingButtons[1] = temp;
			background.attachChild(temp);
			registerTouchArea(temp);

			Sprite cart = new Sprite(0,0,buttonSize,buttonSize,mParent.mCartPentagonTexture,mMgr);
			temp.attachChild(cart);

			Sprite leather = new Sprite(0,0,buttonSize+20,3*font.getLineHeight()+10,mLeather,mMgr);
			leather.setPosition(temp.getWidth()/2 - leather.getWidth()/2, buttonSize);
			temp.attachChild(leather);

			Text mapButtonText = new Text(0,0, font,buildingName, buildingName.length(),
					new TextOptions(AutoWrap.WORDS, buttonSize+20, HorizontalAlign.CENTER, Text.LEADING_DEFAULT), mMgr);
			mapButtonText.setPosition(leather.getWidth()/2-mapButtonText.getWidth()/2, 
					leather.getHeight()/2 - mapButtonText.getHeight()/2);
			leather.attachChild(mapButtonText);

			temp.setScale(STANDARD_BUTTON_SCALE);
		}

		{
			BuildingButton temp = new BuildingButton((CAMERA_WIDTH/2)-buttonSize/2, Constants.TILE_SIZE, 
					buttonSize, buttonSize,
					mButton, mMgr, buildings[2]);
			String buildingName = temp.getBuildingName();
			background.attachChild(temp);
			registerTouchArea(temp);
			buildingButtons[2] = temp;

			Sprite cart = new Sprite(0,0,buttonSize,buttonSize,mParent.mCartCircleTexture,mMgr);
			temp.attachChild(cart);

			Sprite leather = new Sprite(0,0,buttonSize+20,3*font.getLineHeight()+10,mLeather,mMgr);
			leather.setPosition(temp.getWidth()/2 - leather.getWidth()/2, buttonSize);
			temp.attachChild(leather);

			Text mapButtonText = new Text(0,0, font,buildingName, buildingName.length(),
					new TextOptions(AutoWrap.WORDS, buttonSize+20, HorizontalAlign.CENTER, Text.LEADING_DEFAULT),mMgr);
			mapButtonText.setPosition(leather.getWidth()/2-mapButtonText.getWidth()/2, 
					leather.getHeight()/2 - mapButtonText.getHeight()/2);
			leather.attachChild(mapButtonText);

			temp.setScale(STANDARD_BUTTON_SCALE);
		}

		{
			BuildingButton temp = new BuildingButton(CAMERA_WIDTH-buttonSize-Constants.TILE_SIZE/2, Constants.TILE_SIZE, 
					buttonSize, buttonSize,
					mButton, mMgr, buildings[3]);
			String buildingName = temp.getBuildingName();
			buildingButtons[3] = temp;
			background.attachChild(temp);
			registerTouchArea(temp);

			Sprite cart = new Sprite(0,0,buttonSize,buttonSize,mParent.mCartTriangleTexture,mMgr);
			temp.attachChild(cart);

			Sprite leather = new Sprite(0,0,buttonSize+20,3*font.getLineHeight()+10,mLeather,mMgr);
			leather.setPosition(temp.getWidth()/2 - leather.getWidth()/2, buttonSize);
			temp.attachChild(leather);

			Text mapButtonText = new Text(0,0, font,buildingName, buildingName.length(),
					new TextOptions(AutoWrap.WORDS, buttonSize+10, HorizontalAlign.CENTER, Text.LEADING_DEFAULT),mMgr);
			mapButtonText.setPosition(leather.getWidth()/2-mapButtonText.getWidth()/2, 
					leather.getHeight()/2 - mapButtonText.getHeight()/2);
			leather.attachChild(mapButtonText);

			temp.setScale(STANDARD_BUTTON_SCALE);
		}

		{
			BuildingButton temp = new BuildingButton((Constants.TILE_SIZE/2), 3.25f*Constants.TILE_SIZE, 
					buttonSize, buttonSize,
					mButton, mMgr, buildings[4]);
			String buildingName = temp.getBuildingName();
			buildingButtons[4] = temp;
			background.attachChild(temp);
			registerTouchArea(temp);

			Sprite cart = new Sprite(0,0,buttonSize,buttonSize,mParent.mCartStarTexture,mMgr);
			temp.attachChild(cart);

			Sprite leather = new Sprite(0,0,buttonSize+20,3*font.getLineHeight()+10,mLeather,mMgr);
			leather.setPosition(temp.getWidth()/2 - leather.getWidth()/2, buttonSize);
			temp.attachChild(leather);

			Text mapButtonText = new Text(0,0, font,buildingName, buildingName.length(),
					new TextOptions(AutoWrap.WORDS, buttonSize+20, HorizontalAlign.CENTER, Text.LEADING_DEFAULT),mMgr);
			mapButtonText.setPosition(leather.getWidth()/2-mapButtonText.getWidth()/2, 
					leather.getHeight()/2 - mapButtonText.getHeight()/2);
			leather.attachChild(mapButtonText);

			temp.setScale(STANDARD_BUTTON_SCALE);
		}

		{
			BuildingButton temp = new BuildingButton((CAMERA_WIDTH/2)-buttonSize/2, 
					3.25f*Constants.TILE_SIZE, 
					buttonSize, buttonSize,
					mButton, mMgr, buildings[5]);
			String buildingName = temp.getBuildingName();
			buildingButtons[5] = temp;
			background.attachChild(temp);
			registerTouchArea(temp);

			Sprite cart = new Sprite(0,0,buttonSize,buttonSize,mParent.mCartSquareTexture,mMgr);
			temp.attachChild(cart);

			Sprite leather = new Sprite(0,0,buttonSize+20,3*font.getLineHeight()+10,mLeather,mMgr);
			leather.setPosition(temp.getWidth()/2 - leather.getWidth()/2, buttonSize);
			temp.attachChild(leather);

			Text mapButtonText = new Text(0,0, font,buildingName, buildingName.length(),
					new TextOptions(AutoWrap.WORDS, buttonSize+20, HorizontalAlign.CENTER, Text.LEADING_DEFAULT),mMgr);
			mapButtonText.setPosition(leather.getWidth()/2-mapButtonText.getWidth()/2, 
					leather.getHeight()/2 - mapButtonText.getHeight()/2);
			leather.attachChild(mapButtonText);

			temp.setScale(STANDARD_BUTTON_SCALE);
		}


	}

	private void clearTiles(){
		mEngine.runSafely(new Runnable() {
			@Override
			public void run() {
				mTilesSprite.detachChildren();
			}
		});

	}

	private void recreateBooleanMatrix(){
		for(int i=0;i<5;i++){
			for(int j=0;j<5;j++){
				mBuildingPlacements[i][j]=false;
			}
		}
	}


	private void refresh(){
		GetStats stats = new GetStats(mParent);
		stats.execute();
	}

	private TextureRegion getTextureBasedOnBuildingType(Building b){

		if(b.getType().equals(Constants.BUILDING_TYPES[1])){
			return mParent.mCartPentagonTexture;
		}else if(b.getType().equals(Constants.BUILDING_TYPES[2])){
			return mParent.mCartCircleTexture;
		}else if(b.getType().equals(Constants.BUILDING_TYPES[3])){
			return mParent.mCartTriangleTexture;
		}else if(b.getType().equals(Constants.BUILDING_TYPES[4])){
			return mParent.mCartStarTexture;
		}else if(b.getType().equals(Constants.BUILDING_TYPES[5])){
			return mParent.mCartSquareTexture;
		}

		return mCart;

	}

	public TextureRegion getTextureBasedonWeapon(Weapon w){
		String c = w.getmClass()+w.getmRank();
		if(c.equals("B1")){
			return mParent.mNinja1Texture;
		}else if(c.equals("B3")){
			return mParent.mNinja2Texture;
		}else if(c.equals("C1")){
			return mParent.mFan1Texture;
		}else if(c.equals("C3")){
			return mParent.mFan2Texture;
		}else if(c.equals("E1")){
			return mParent.mWand1Texture;
		}else if(c.equals("E3")){
			return mParent.mWand2Texture;
		}else if(c.equals("F1")){
			return mParent.mChickenTexture;
		}else if(c.equals("F3")){
			return mParent.mCowTexture;
		}
		return mWhiteButton;
	}

	private void fillOutPlayerData(){

		for(int i =0;i<5;i++){
			for(int j=0;j<5;j++){
				mBuildingPlacements[i][j]=false;
			}
		}

		mGoldText.setText(""+mSelf.getGold());

		for(Building b: mSelf.getBuildings()){
			int x = b.getX();
			int y = b.getY();
			mRedTile.setTextureSize(Constants.TILE_SIZE, Constants.TILE_SIZE);
			TextureRegion tempTexture = getTextureBasedOnBuildingType(b);
			Sprite cart = new Sprite(x*Constants.TILE_SIZE, y*Constants.TILE_SIZE, Constants.TILE_SIZE,
					Constants.TILE_SIZE, tempTexture, mMgr);
			Sprite redTile = new Sprite(x*Constants.TILE_SIZE, y*Constants.TILE_SIZE, mRedTile, mMgr){

				final float px = this.getX();
				final float py = this.getY();
				@Override
				public boolean onAreaTouched(TouchEvent pSceneTouchEvent,
						float pTouchAreaLocalX, float pTouchAreaLocalY) {
					if(!isShowingSubMenu && pSceneTouchEvent.getAction()==TouchEvent.ACTION_UP){
						createSpecificBuildingSubMenu(px, py);
						return true;
					}
					return false;
				}
			};
			redTile.setAlpha(0.1f);
			mBuildingSprites.add(redTile);
			registerTouchArea(redTile);
			mTilesSprite.attachChild(redTile);
			mTilesSprite.attachChild(cart);
			mBuildingPlacements[x][y] = true;
		}

		if(!mBuildingPlacements[0][0]){
			placeBlueTile(0, 0);
		}
		if(!mBuildingPlacements[4][0]){
			placeBlueTile(4, 0);
		}
		if(!mBuildingPlacements[0][4]){
			placeBlueTile(0, 4);

		}
		if(!mBuildingPlacements[4][4]){
			placeBlueTile(4, 4);
		}
		if(!mBuildingPlacements[2][2]){
			placeBlueTile(2, 2);
		}
	}

	public void createSpecificBuildingSubMenu(final float px, final float py){

		isShowingSubMenu=true;
		float[] pxy = mDesertBackground.convertLocalToSceneCoordinates(mTilesSprite.getX(), mTilesSprite.getY());
		Building b=mSelf.getBuildingAtCoordinates((int)px, (int)py);
		mSelectedBuildingX = (int)px;
		mSelectedBuildingY = (int)py;
		if(b==null){
			Log.v("knowledgewar", "Owntribe buildingatcoordinates is null");
		}
		SpecificBuildingSubMenu subScene = new SpecificBuildingSubMenu(pxy[0]+((1-TILES_SCALE)*mTilesSprite.getWidth()/2), 
				pxy[1]+((1-TILES_SCALE)*mTilesSprite.getHeight()/2), mParent, b);
		this.setChildScene(subScene);
		mChild=subScene;
	}


	private void createGeneralBuildingSubMenu(Building b){
		isShowingSubMenu=true;
		GeneralBuildingSubMenu menu = new GeneralBuildingSubMenu(mParent,this, b);
		this.setChildScene(menu);
		mChild=menu;
	}

	public void removeSubMenu(){
		isShowingSubMenu=false;
		mChild=null;
		this.clearChildScene();
	}


	public void removeBuilding(int px, int py){
		//		if(mSelf.deconstructBuilding(px, py)){
		//			mBuildingPlacements[px][py]=false;
		//		}
		//		UserFunctions.updateOwnStats(mSelf, mParent);
		//		clearTiles();
		//		refresh();
		//		removeSubMenu();
		BuildingRemover remover = new BuildingRemover();
		remover.execute(px,py);
	}

	public void upgradeBuilding(float px, float py, Building building){
		int x = (int)px/Constants.TILE_SIZE;
		int y = (int)py/Constants.TILE_SIZE;
		//		if(mSelf.upgradeBuilding(x, y, newBuilding)){
		//			UserFunctions.updateOwnStats(mSelf, mParent);
		//			clearTiles();
		//			refresh();			
		//		}else{
		//			mParent.toastOnUIThread(mRes.getString(R.string.owntribe_cannot_afford_building));
		//		}

		if(mSelf.canUpgradeBuilding(building)){
			BuildingUpgrader upgrade = new BuildingUpgrader(mParent, building, x, y);
			upgrade.execute();
		}

	}


	private void placeBlueTile(int x, int y){
		mBlueTile.setTextureSize(Constants.TILE_SIZE, Constants.TILE_SIZE);
		Sprite blueTile = new Sprite(x*Constants.TILE_SIZE, y*Constants.TILE_SIZE,mBlueTile, mMgr);
		blueTile.setAlpha(0.1f);
		mTilesSprite.attachChild(blueTile);
	}

	private boolean buildingPlaced(float px, float py, Building building){
		float[] coordinates=mDesertBackground.convertLocalToSceneCoordinates(mTilesSprite.getX(), mTilesSprite.getY());
		float x1 = px - coordinates[0]-((1-TILES_SCALE)*mTilesSprite.getWidth()/2);
		float y1 = py - coordinates[1]-((1-TILES_SCALE)*mTilesSprite.getHeight()/2);

		int x = ((int)x1)/(int)(Constants.TILE_SIZE*TILES_SCALE);
		int y = ((int)y1)/(int)(Constants.TILE_SIZE*TILES_SCALE);

		float buffer = 1;
		boolean northWest = (x>=0-buffer && x<=0+buffer) && (y>=0-buffer && y<=0+buffer);
		boolean northEast = (x>=4-buffer && x<=4+buffer) && (y>=0-buffer && y<=0+buffer);
		boolean southWest = (x>=0-buffer && x<=0+buffer) && (y>=4-buffer && y<=4+buffer);
		boolean southEast = (x>=4-buffer && x<=4+buffer) && (y>=4-buffer && y<=4+buffer);
		boolean occupied;
		if(northWest){
			//			success = mSelf.addNewBuilding(building, 0, 0);
			//			if(success){
			//				UserFunctions.updateOwnStats(mSelf, mParent);
			//				clearTiles();
			//				refresh();
			//				return true;
			//			}
			occupied = mSelf.isTileOccupied(0, 0);
			if(!occupied){
				BuildingPlacer placer = new BuildingPlacer(mParent,building,0,0);
				placer.execute();				
			}
		}else if(northEast){
			//			success = mSelf.addNewBuilding(building, 4, 0);
			//			if(success){
			//				UserFunctions.updateOwnStats(mSelf, mParent);
			//				clearTiles();
			//				refresh();
			//				return true;
			//			}
			occupied = mSelf.isTileOccupied(4, 0);
			if(!occupied){
				BuildingPlacer placer = new BuildingPlacer(mParent,building,4,0);
				placer.execute();				
			}
		}else if(southWest){
			//			success = mSelf.addNewBuilding(building, 0, 4);
			//			if(success){
			//				UserFunctions.updateOwnStats(mSelf, mParent);
			//				clearTiles();
			//				refresh();
			//				return true;
			//			}
			occupied = mSelf.isTileOccupied(0, 4);
			if(!occupied){
				BuildingPlacer placer = new BuildingPlacer(mParent,building,0,4);
				placer.execute();				
			}
		}else if(southEast){
			//			success = mSelf.addNewBuilding(building, 4, 4);
			//			if(success){
			//				UserFunctions.updateOwnStats(mSelf, mParent);
			//				clearTiles();
			//				refresh();
			//				return true;
			//			}
			occupied = mSelf.isTileOccupied(4, 4);
			if(!occupied){
				BuildingPlacer placer = new BuildingPlacer(mParent,building,4,4);
				placer.execute();				
			}
		}
		return false;					

	}

	public void initializeScene(){
		refresh();
		recreateBooleanMatrix();
		clearTiles();
	}


	private class BuildingButton extends Sprite{

		private Building mBuilding;
		private boolean mGrabbed;
		private float mOriginalX, mOriginalY;
		double timer = 0;

		public BuildingButton(float pX, float pY, float pWidth, float pHeight,
				TextureRegion pTextureRegion,
				VertexBufferObjectManager pSpriteVertexBufferObject, String type) {
			super(pX, pY, pWidth, pHeight, pTextureRegion, pSpriteVertexBufferObject);
			this.mBuilding = BuildingFactory.constructBuilding(0,0,type);
			this.mOriginalX=pX;
			this.mOriginalY=pY;
		}

		public String getBuildingName(){
			return mBuilding.getName();
		}

		@Override
		public boolean onAreaTouched(TouchEvent pSceneTouchEvent,
				float pTouchAreaLocalX, float pTouchAreaLocalY) {
			if(!isShowingSubMenu){
				switch(pSceneTouchEvent.getAction()){
				case TouchEvent.ACTION_DOWN:
					timer = System.currentTimeMillis();
					mGrabbed=true;
					setScale(HIGHLIGHTED_BUTTON_SCALE);break;
				case TouchEvent.ACTION_MOVE: 
					if(this.mGrabbed) {
						this.setPosition(pSceneTouchEvent.getX() - Constants.TILE_SIZE/2, 
								pSceneTouchEvent.getY() - Constants.TILE_SIZE/2);
					}break;
				case TouchEvent.ACTION_UP: 
					//TODO Add test for building plaecement
					double newTime = System.currentTimeMillis();
					double timeButtonPressed = newTime - timer;

					if(timeButtonPressed<250 && this.mGrabbed){
						createGeneralBuildingSubMenu(mBuilding);
					}else if(this.mGrabbed){
						buildingPlaced(pSceneTouchEvent.getX(), 
								pSceneTouchEvent.getY(), mBuilding);						

					}
					mGrabbed=false;
					setScale(STANDARD_BUTTON_SCALE);
					this.setPosition(mOriginalX, mOriginalY);							
					break;
				}
			}
			return true;
		}

	}

	private class GetStats extends AsyncTask<Void, Void, String>{

		Context context;

		public GetStats(Context con){
			context=con;
		}

		@Override
		protected void onPreExecute() {
			mParent.displayLoadingAnimation(true);
		}

		@Override
		protected String doInBackground(Void... params) {
			String result="";
			try {
				do{
					result =Server.getOwnStats(mParent);
					if(result.equals(Constants.HTTP_EXECUTE_FAILURE)){
						publishProgress();
						Thread.sleep(Constants.STANDARD_CONNECTION_WAIT_TIME);
					}
				}while(result.equals(Constants.HTTP_EXECUTE_FAILURE));
			} catch (InterruptedException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
			return result;
		}

		@Override
		protected void onPostExecute(String result){

			JSONObject obj;
			try {
				obj = new JSONObject(result);
				String name = obj.getString(Constants.PLAYER_ACCOUNT);
				String area = obj.getString(Constants.PLAYER_AREA);
				int gold = Integer.parseInt(obj.getString(Constants.PLAYER_GOLD));
				boolean busy = obj.getBoolean(Constants.PARAMETER_BUSY);
				UserFunctions.setUserBusy(context, busy);
				Player temp = new Player(name, area, gold);
				mSelf=temp;
				UserFunctions.self=mSelf;
				fillOutPlayerData();

				if(isShowingSubMenu){
					removeSubMenu();
					createSpecificBuildingSubMenu(mSelectedBuildingX, mSelectedBuildingY);
				}
			} catch (JSONException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
			mParent.displayLoadingAnimation(false);
		}

	}

	private class BuildingPlacer extends AsyncTask<Void, Void, String>{

		Building b;
		int x,y;
		Context con;

		public BuildingPlacer(Context context, Building b, int px, int py){
			this.con = context;
			this.b = b;
			this.x = px;
			this.y = py;
		}

		@Override
		protected void onPreExecute() {
			mParent.displayLoadingAnimation(true);
		}

		@Override
		protected String doInBackground(Void... params) {
			String result="";

			try {
				do{
					result = Server.placeNewBuilding(con, b, x, y);
					if(result.equals(Constants.HTTP_EXECUTE_FAILURE)){
						publishProgress();
						Thread.sleep(Constants.STANDARD_CONNECTION_WAIT_TIME);
					}
				}while(result.equals(Constants.HTTP_EXECUTE_FAILURE));
			} catch (InterruptedException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}

			return result;
		}

		@Override
		protected void onPostExecute(String result) {
			mParent.displayLoadingAnimation(false);
			clearTiles();
			refresh();
		}
	}

	private class BuildingUpgrader extends AsyncTask<Void, Void, String>{

		Building b;
		Context context;
		int x,y;

		public BuildingUpgrader(Context con, Building building,int px, int py){
			b=building;
			context = con;
			x=px;
			y=py;
		}
		@Override
		protected void onPreExecute() {
			mParent.displayLoadingAnimation(true);
		}
		@Override
		protected String doInBackground(Void... params) {
			String result="";

			try {
				do{
					result =  Server.upgradeBuilding(context, b, x, y);
					if(result.equals(Constants.HTTP_EXECUTE_FAILURE)){
						publishProgress();
						Thread.sleep(Constants.STANDARD_CONNECTION_WAIT_TIME);
					}
				}while(result.equals(Constants.HTTP_EXECUTE_FAILURE));
			} catch (InterruptedException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}

			return result;
		}
		@Override
		protected void onPostExecute(String result) {
			mParent.displayLoadingAnimation(false);
			clearTiles();
			refresh();
		}
	}

	private class BuildingRemover extends AsyncTask<Integer, Void, String>{

		@Override
		protected void onPreExecute() {
			mParent.displayLoadingAnimation(true);
		}
		@Override
		protected String doInBackground(Integer... params) {
			String result="";
			try {
				do{
					result = Server.removeBuilding(mParent, params[0], params[1]);
					if(result.equals(Constants.HTTP_EXECUTE_FAILURE)){
						publishProgress();
						Thread.sleep(Constants.STANDARD_CONNECTION_WAIT_TIME);
					}
				}while(result.equals(Constants.HTTP_EXECUTE_FAILURE));
			} catch (InterruptedException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
			return result;
		}
		@Override
		protected void onPostExecute(String result) {
			mParent.displayLoadingAnimation(false);
			removeSubMenu();
			clearTiles();
			refresh();
		}
	}

	@Override
	public boolean onBackButtonPressed() {
		if(isShowingSubMenu){
			if(!mChild.onBackButtonPressed()){
				removeSubMenu();
			}
			return true;
		}else if(isShowingHelpScene){
			isShowingHelpScene=false;
			new TransitionScene(mParent, mParent.mZoomCamera, mEngine, mHelpScene, this);
			mParent.updateHUD(hasHelpScene());
			return true;
		}
		return false;
	}

	@Override
	public void showHelpScreen() {
		removeSubMenu();
		isShowingHelpScene=true;
		mHelpScene = new OwnTribeHelpScene(mParent);
		new TransitionScene(mParent, mParent.mZoomCamera, mEngine, this, mHelpScene);
		mParent.updateHUD(false);
	}

	@Override
	public boolean hasHelpScene() {
		return true;
	}


}
