package adrian.master.knowledgewar.scenes;

import java.util.ArrayList;

import org.andengine.engine.Engine;
import org.andengine.entity.scene.Scene;
import org.andengine.entity.scene.background.AutoParallaxBackground;
import org.andengine.entity.scene.background.ParallaxBackground.ParallaxEntity;
import org.andengine.entity.sprite.Sprite;
import org.andengine.entity.text.Text;
import org.andengine.input.touch.TouchEvent;
import org.andengine.opengl.font.Font;
import org.andengine.opengl.texture.region.TextureRegion;
import org.andengine.opengl.vbo.VertexBufferObjectManager;

import adrian.master.knowledgewar.R;
import adrian.master.knowledgewar.gameactivities.MainActivity;
import adrian.master.knowledgewar.gameconcepts.Challenge;
import adrian.master.knowledgewar.helpscenes.MapHelpScene;
import adrian.master.knowledgewar.helpscenes.PendingChallengesHelpScene;
import adrian.master.knowledgewar.interfaces.MainActivityInterface;
import adrian.master.knowledgewar.logic.Constants;

public class PendingChallengesScene extends Scene implements MainActivityInterface{
	
	private MainActivity mParent;
	private int CAMERA_WIDTH=Constants.CAMERA_WIDTH;
	private TextureRegion mBackground, mButton, mLeather;
	private VertexBufferObjectManager mMgr;
	private Engine mEngine;
	private Font mFont;
	private Sprite mHeader;
	
	private AutoParallaxBackground mScrollingBackground;
	
	private ArrayList<Challenge> mChallengeList;
	private ArrayList<Sprite> mSpriteList;
	
	private boolean mIsShowingHelpScene = false;
	private Scene mHelpScene;
	
	public PendingChallengesScene(MainActivity parent, Engine engine){
		this.mParent=parent;
		this.mEngine = engine;
		setupFields();
		
		setupBackground();
	}
	
	private void setupFields(){
		mBackground = mParent.mDuneBackgroundTexture;
		mButton = mParent.mPaperTexture;
		mFont = mParent.mFontBold;
		mMgr = mParent.getVertexBufferObjectManager();
		mLeather = mParent.mLeatherTexture;
	}
	
	private void setupBackground(){
		this.mScrollingBackground = new AutoParallaxBackground(0, 0, 0, 5);
		Sprite temp = new Sprite(0, 0, mBackground, mMgr);
		mScrollingBackground.attachParallaxEntity(new ParallaxEntity(5, temp));
		this.setBackground(mScrollingBackground);
		
		mHeader = new Sprite(0,0,CAMERA_WIDTH-Constants.TILE_SIZE*2,Constants.TILE_SIZE,mLeather,mMgr);
		mHeader.setPosition(CAMERA_WIDTH/2 - mHeader.getWidth()/2, 10);
		attachChild(mHeader);
		
		Text header = new Text(0,0,mFont,mParent.getString(R.string.pending_challenges_header),mMgr);
		header.setPosition(mHeader.getWidth()/2 - header.getWidth()/2, mHeader.getHeight()/2 - header.getHeight()/2);
		mHeader.attachChild(header);
	}
	
	public void initialize(ArrayList<Challenge> list){
		clearList();
		mChallengeList=list;
		if(mChallengeList!=null){
			createChallengeButtons();			
		}else{
			
		}
	}
	
	private void clearList(){
		if(mSpriteList!=null && !mSpriteList.isEmpty()){
			mEngine.runSafely(new Runnable() {
				@Override
				public void run() {
					for(Sprite s: mSpriteList){
						detachChild(s);
					}
				}
			});
		}
	}
	
	private void createChallengeButtons(){
		mSpriteList = new ArrayList<Sprite>();
		float x = CAMERA_WIDTH/2 - Constants.TILE_SIZE*2;
		float y = 120;
		for(final Challenge c: mChallengeList){
			 ChallengeButton temp = new ChallengeButton(x,y, c){
				@Override
				public boolean onAreaTouched(final TouchEvent pSceneTouchEvent, final float pTouchAreaLocalX, final float pTouchAreaLocalY) {
					switch(pSceneTouchEvent.getAction()) {
					case TouchEvent.ACTION_DOWN:
						this.setScale(Constants.BUTTONPRESSED_SCALE);
						return true;
					case TouchEvent.ACTION_UP:
						this.setScale(1);
						mParent.startReceiveChallenge(c);
						return true;
					}
					return false;
				}
			};
			mSpriteList.add(temp);
			this.attachChild(temp);
			this.registerTouchArea(temp);
			y+=temp.getHeight()+10;
		}
	}
	
	private class ChallengeButton extends Sprite{
		
		Text mChallenger;

		public ChallengeButton(float pX, float pY, Challenge c) {
			super(pX, pY, Constants.TILE_SIZE*4, Constants.TILE_SIZE*2, mButton, mMgr);
			mChallenger = new Text(0,0,mFont,c.getChallenger(),mMgr);
			mChallenger.setPosition(this.getWidth()/2-mChallenger.getWidth()/2, this.getHeight()/2-mChallenger.getHeight()/2);
			this.attachChild(mChallenger);
		}
		
	}

	@Override
	public boolean onBackButtonPressed() {
		if(mIsShowingHelpScene){
			mIsShowingHelpScene=false;
			new TransitionScene(mParent, mParent.mZoomCamera, mEngine,mHelpScene, this);
			return true;
		}
		return false;
	}

	@Override
	public void showHelpScreen() {
		mIsShowingHelpScene=true;
		mHelpScene = new PendingChallengesHelpScene(mParent);
		new TransitionScene(mParent, mParent.mZoomCamera, mEngine, this, mHelpScene);
	}

	@Override
	public boolean hasHelpScene() {
		return true;
	}

}
