package adrian.master.knowledgewar.scenes;

import java.util.LinkedList;

import org.andengine.engine.camera.Camera;
import org.andengine.entity.scene.IOnSceneTouchListener;
import org.andengine.entity.scene.Scene;
import org.andengine.entity.scene.background.AutoParallaxBackground;
import org.andengine.entity.scene.background.ParallaxBackground.ParallaxEntity;
import org.andengine.entity.sprite.Sprite;
import org.andengine.entity.text.AutoWrap;
import org.andengine.entity.text.Text;
import org.andengine.entity.text.TextOptions;
import org.andengine.input.touch.TouchEvent;
import org.andengine.input.touch.detector.ScrollDetector;
import org.andengine.input.touch.detector.ScrollDetector.IScrollDetectorListener;
import org.andengine.opengl.font.Font;
import org.andengine.opengl.texture.region.TextureRegion;
import org.andengine.opengl.vbo.VertexBufferObjectManager;
import org.andengine.util.HorizontalAlign;

import adrian.master.knowledgewar.gameactivities.MainActivity;
import adrian.master.knowledgewar.gameconcepts.Question;
import adrian.master.knowledgewar.interfaces.MainActivityInterface;
import adrian.master.knowledgewar.logic.Constants;
import adrian.master.knowledgewar.logic.Database;

public class QuestionsScene extends Scene implements MainActivityInterface, IScrollDetectorListener, IOnSceneTouchListener {
	
	private MainActivity mParent;
	private LinkedList<Question> mQuestions;
	private AutoParallaxBackground mBackground;
	private TextureRegion mBackgroundTexture;
	private TextureRegion mPaperTexture, mLeatherTexture;
	private VertexBufferObjectManager mMgr;
	
//	private Database mDatabase;
	
	private float mQuestionButtonWidth, mButtonHeight, mAnswerButtonWidth,mRowWidth;
	private Sprite[] mQuestionsSprites, mAnswers;
	private float mButtonX;
	private Font mSmallFont;
	private Camera mCamera;
	private ScrollDetector mScrollDetector;
	
	public QuestionsScene(MainActivity parent){
		mParent = parent;
		setupFields();
		
		this.setOnSceneTouchListener(this);
		this.mBackground = new AutoParallaxBackground(0, 0, 0, 5);
		Sprite temp = new Sprite(0, 0, mBackgroundTexture, mMgr);
		mBackground.attachParallaxEntity(new ParallaxEntity(5, temp));
		this.setBackground(mBackground);
	}
	
	private void setupFields(){
		mBackgroundTexture = mParent.mDuneBackgroundTexture;
		mPaperTexture = mParent.mPaperTexture;
		mLeatherTexture = mParent.mLeatherTexture;
		mMgr = mParent.getVertexBufferObjectManager();
		
//		mDatabase = mParent.mQuestionDatabase;
		
		this.mButtonHeight = Constants.TILE_SIZE*1.2f;
		this.mQuestionButtonWidth = 5*Constants.TILE_SIZE;
		this.mAnswerButtonWidth = Constants.TILE_SIZE;
		this.mRowWidth = mQuestionButtonWidth+mAnswerButtonWidth+10;
		this.mButtonX = Constants.CAMERA_WIDTH/2 - mRowWidth/2;
		this.mSmallFont = mParent.mSmallFont;
		mCamera = mParent.mZoomCamera;
		mScrollDetector = new ScrollDetector(this);
	}
	
	public void initialize(){
//		mQuestions = mDatabase.getQuestions();
		int size = mQuestions.size();
		mQuestionsSprites = new Sprite[size];
		mAnswers = new Sprite[size];
		float yPlacement = 100;
		
		for(int i=0; i<size; i++){
			mQuestionsSprites[i] = new Sprite(0,0,mQuestionButtonWidth,mButtonHeight,mLeatherTexture,mMgr);
			mQuestionsSprites[i].setPosition(mButtonX, yPlacement);
			
			final Question q = mQuestions.get(i);
			String str = q.getQuestion();
			Text text = new Text(0,0,mSmallFont,str,new TextOptions(AutoWrap.WORDS, mQuestionsSprites[i].getWidth()-60,
					HorizontalAlign.CENTER, Text.LEADING_DEFAULT),mMgr);
			text.setPosition(30, mQuestionsSprites[i].getHeight()/2 - text.getHeight()/2);
			mQuestionsSprites[i].attachChild(text);
			
			attachChild(mQuestionsSprites[i]);
			
			mAnswers[i] = new Sprite(0,0,mAnswerButtonWidth,mButtonHeight,mPaperTexture,mMgr){
				@Override
				public boolean onAreaTouched(TouchEvent pSceneTouchEvent,
						float pTouchAreaLocalX, float pTouchAreaLocalY) {
					switch(pSceneTouchEvent.getAction()){
					case TouchEvent.ACTION_DOWN:
						this.setScale(Constants.BUTTONPRESSED_SCALE);
						return true;
					case TouchEvent.ACTION_UP:
						this.setScale(1);
						mParent.startBrowser(q.getURL());
						return true;
					}
					return false;
				}
			};
			mAnswers[i].setPosition(mButtonX+mQuestionButtonWidth+10, yPlacement);
			Text questionMark = new Text(0,0, mParent.mFontBold, "?",mMgr);
			questionMark.setPosition(mAnswers[i].getWidth()/2- questionMark.getWidth()/2, mAnswers[i].getHeight()/2 - questionMark.getHeight()/2);
			mAnswers[i].attachChild(questionMark);
			attachChild(mAnswers[i]);
			registerTouchArea(mAnswers[i]);
			
			yPlacement+=mButtonHeight;
		}
	}

	@Override
	public boolean onBackButtonPressed() {
		// TODO Auto-generated method stub
		return false;
	}

	@Override
	public boolean onSceneTouchEvent(Scene pScene, TouchEvent pSceneTouchEvent) {
		mScrollDetector.onTouchEvent(pSceneTouchEvent);
		return true;
	}

	@Override
	public void onScrollStarted(ScrollDetector pScollDetector, int pPointerID,
			float pDistanceX, float pDistanceY) {
		// TODO Auto-generated method stub
		
	}

	@Override
	public void onScroll(ScrollDetector pScollDetector, int pPointerID,
			float pDistanceX, float pDistanceY) {
		if(!(mCamera.getCenterY()-pDistanceY<0)){
			this.mCamera.offsetCenter(0, -pDistanceY );				
		}
		
	}

	@Override
	public void onScrollFinished(ScrollDetector pScollDetector, int pPointerID,
			float pDistanceX, float pDistanceY) {
		// TODO Auto-generated method stub
		
	}

	@Override
	public void showHelpScreen() {
		// TODO Auto-generated method stub
	}

	@Override
	public boolean hasHelpScene() {
		// TODO Auto-generated method stub
		return false;
	}

}
