package adrian.master.knowledgewar.scenes;

import org.andengine.engine.Engine;
import org.andengine.entity.scene.Scene;
import org.andengine.entity.sprite.Sprite;
import org.andengine.entity.text.Text;
import org.andengine.input.touch.TouchEvent;
import org.andengine.opengl.font.Font;
import org.andengine.opengl.texture.region.TextureRegion;
import org.andengine.opengl.vbo.VertexBufferObjectManager;

import adrian.master.knowledgewar.R;
import adrian.master.knowledgewar.gameactivities.ReceiveChallengeActivity;
import adrian.master.knowledgewar.gameconcepts.Building;
import adrian.master.knowledgewar.gameconcepts.Player;
import adrian.master.knowledgewar.logic.Constants;
import adrian.master.knowledgewar.logic.UserFunctions;
import android.content.res.Resources;
import android.util.Log;



public class ReceiveChallengeScene extends Scene {

	private Player mSelf, mOpponentPlayer;
	private ReceiveChallengeActivity mParent;
	private Engine mEngine;
	private int CAMERA_HEIGHT = Constants.CAMERA_HEIGHT, CAMERA_WIDTH=Constants.CAMERA_WIDTH;

	private TextureRegion mBackground, mOpponent, mDuneBackground, mBlankTile, mCart;
	private TextureRegion mPlayerTile, mVerticalLine, mBlueBar, mRedBar, mLeather, mPaper;
	private VertexBufferObjectManager mMgr;
	private Font mFontBold, mFont,mBigFontBold;
	private Sprite mBackgroundSprite, mDuneBackgroundSprite,mBlankTilesSprite, mPlayerSprite, mOpponentSprite;
	private Sprite mAcceptButton, mRejectButton, mVerticalLineSprite;
	private Sprite mOppBonusBackground, mPlayerBonusBackground, mSizeSelectBackground;
	private Text mBattleSize, mBattleCategory;

	private float mSpriteScale=0.5f;
	private Resources mRes;
	private int x,y;
	private boolean mButtonPressed=false;

	private String mOpponentBonus, mOpponentBonusHealth, mOpponentBonusDamage, mOpponentBonusGold, mOpponentGoldPenalty;
	private Text mOpponentBonusHealthText, mOpponentBonusDamageText, mOpponentBonusGoldText, mOpponentGoldPenaltyText, 
	mOpponentBonusText;
	private String mSelfBonus, mSelfBonusHealth, mSelfBonusDamage, mSelfBonusGold, mSelfGoldPenalty;
	private Text mSelfBonusHealthText, mSelfBonusDamageText, mSelfBonusGoldText, mSelfGoldPenaltyText,
	mSelfBonusText;
	private Text mAcceptButtonText, mRejectButtonText, mSizeGoldBonus;
	private Sprite mCategoriesBackground;

	public ReceiveChallengeScene(ReceiveChallengeActivity parent, Engine engine){
		this.mParent = parent;
		this.mEngine = engine;
		setUpFields();
		createBackgrounds();
		createBonusList();
		createButtons();
		createBattleSizeDisplay();
		createCategoryDisplay();
	}

	private void setUpFields(){
		this.mBackground = mParent.mSandBackgroundTexture;
		this.mPlayerTile = mParent.mOwnTileTexture;
		this.mOpponent = mParent.mEnemyTileTexture;
		this.mDuneBackground = mParent.mDuneTexture;
		this.mBlankTile = mParent.mBlankTileTexture;
		this.mCart = mParent.mCartTexture;
		this.mVerticalLine = mParent.mVerticalLineTexture;
		this.mMgr = mParent.getVertexBufferObjectManager();
		this.mFontBold = mParent.mMediumFontBold;
		this.mFont = mParent.mMediumFont;
		this.mBigFontBold = mParent.mBigFontBold;
		this.mRes = mParent.mRes;
		this.mBlueBar = mParent.mBlueBarTexture;
		this.mRedBar = mParent.mRedBarTexture;
		this.mLeather = mParent.mLeatherTexture;
		this.mPaper = mParent.mPaperTexture;

	}

	private void createBackgrounds(){
		mDuneBackgroundSprite = new Sprite(0, 0, CAMERA_WIDTH, CAMERA_HEIGHT/2,
				mDuneBackground, mMgr);
		attachChild(mDuneBackgroundSprite);

		mBackgroundSprite = new Sprite(0, CAMERA_HEIGHT/2, CAMERA_WIDTH, 
				CAMERA_HEIGHT/2,
				mBackground, mMgr);

		attachChild(mBackgroundSprite);

		mBlankTile.setTextureSize(Constants.TILE_SIZE*7, Constants.TILE_SIZE*7);
		mBlankTilesSprite = new Sprite((CAMERA_WIDTH/2)-(Constants.TILE_SIZE*3.5f),
				(3*CAMERA_HEIGHT/4)-(Constants.TILE_SIZE*3.5f), mBlankTile, mMgr);
		mBlankTilesSprite.setScale(mSpriteScale);
		attachChild(mBlankTilesSprite);

		mVerticalLineSprite = new Sprite(CAMERA_WIDTH/2-1,10, 2,mDuneBackgroundSprite.getHeight()-80,
				mVerticalLine,mMgr);
		mVerticalLineSprite.setAlpha(0.25f);
		attachChild(mVerticalLineSprite);
		
		mOppBonusBackground = new Sprite(0,0, Constants.CAMERA_WIDTH/2 - 10, mFont.getLineHeight()*6, 
				mLeather, mMgr);
		mOppBonusBackground.setPosition(Constants.CAMERA_WIDTH/4 - mOppBonusBackground.getWidth()/2, 5);
		attachChild(mOppBonusBackground);
		
		mPlayerBonusBackground = new Sprite(0,0, Constants.CAMERA_WIDTH/2 - 10, mFont.getLineHeight()*6, mLeather, mMgr);
		mPlayerBonusBackground.setPosition(Constants.CAMERA_WIDTH/4 - mPlayerBonusBackground.getWidth()/2, 
				mOppBonusBackground.getY()+mOppBonusBackground.getHeight()+10);
		attachChild(mPlayerBonusBackground);
		
		mSizeSelectBackground = new Sprite(0,0, Constants.CAMERA_WIDTH/2 - 10, Constants.CAMERA_HEIGHT/6, mLeather, mMgr);
//		mSizeSelectBackground.setPosition(Constants.CAMERA_WIDTH/2+5, 5);
		mSizeSelectBackground.setPosition(Constants.CAMERA_WIDTH/2+5, Constants.CAMERA_HEIGHT/4-20);
		attachChild(mSizeSelectBackground);
		
		mCategoriesBackground = new Sprite(0,0,Constants.CAMERA_WIDTH/2 - 10, Constants.CAMERA_HEIGHT/6, mLeather, mMgr);
		mCategoriesBackground.setPosition(Constants.CAMERA_WIDTH/2+5, 10);
		attachChild(mCategoriesBackground);
	}

	private void createBonusList(){
		float line = mFont.getLineHeight();
		float x_padding=25;
		float ycoord=10;
		
		mOpponentBonus = mRes.getString(R.string.challenge_bonus_opponent);
		mOpponentBonusDamage = mRes.getString(R.string.challenge_bonus_damage);
		mOpponentBonusGold = mRes.getString(R.string.challenge_bonus_gold);
		mOpponentBonusHealth = mRes.getString(R.string.challenge_bonus_health);
		mOpponentGoldPenalty = mRes.getString(R.string.challenge_bonus_goldpenalty);

		mOpponentBonusText = new Text(x_padding,ycoord, mFontBold, mOpponentBonus, mMgr);
		mOpponentBonusDamageText= new Text(x_padding,ycoord+=line, mFont, mOpponentBonusDamage,mOpponentBonusDamage.length()+10,mMgr);
		mOpponentBonusGoldText = new Text(x_padding,ycoord+=line, mFont, mOpponentBonusGold,mOpponentBonusGold.length()+10, mMgr);
		mOpponentBonusHealthText = new Text(x_padding,ycoord+=line, mFont, mOpponentBonusHealth,mOpponentBonusHealth.length()+10, mMgr);
		mOpponentGoldPenaltyText = new Text(x_padding,ycoord+=line, mFont, mOpponentGoldPenalty,mOpponentGoldPenalty.length()+10, mMgr);

		attachChild(mOpponentBonusText);
		attachChild(mOpponentBonusDamageText);
		attachChild(mOpponentBonusGoldText);
		attachChild(mOpponentBonusHealthText);
		attachChild(mOpponentGoldPenaltyText);

		mSelfBonus = mRes.getString(R.string.challenge_bonus_self);
		mSelfBonusDamage = mRes.getString(R.string.challenge_bonus_damage);
		mSelfBonusGold = mRes.getString(R.string.challenge_bonus_gold);
		mSelfBonusHealth = mRes.getString(R.string.challenge_bonus_health);
		mSelfGoldPenalty = mRes.getString(R.string.challenge_bonus_goldpenalty);

		ycoord+=(line+15);
		mSelfBonusText = new Text(x_padding,ycoord+=line, mFontBold, mSelfBonus, mMgr);
		mSelfBonusDamageText = new Text(x_padding,ycoord+=line, mFont, mSelfBonusDamage,mSelfBonusDamage.length()+10, mMgr);
		mSelfBonusGoldText = new Text(x_padding,ycoord+=line, mFont, mSelfBonusGold,mSelfBonusGold.length()+10, mMgr); 
		mSelfBonusHealthText= new Text(x_padding,ycoord+=line, mFont, mSelfBonusHealth,mSelfBonusHealth.length()+10, mMgr);
		mSelfGoldPenaltyText = new Text(x_padding,ycoord+=line, mFont, mSelfGoldPenalty,mSelfGoldPenalty.length()+10, mMgr);

		attachChild(mSelfBonusText);
		attachChild(mSelfBonusDamageText);
		attachChild(mSelfBonusGoldText);
		attachChild(mSelfBonusHealthText);
		attachChild(mSelfGoldPenaltyText);
	}

	private void createButtons(){
		mAcceptButtonText = new Text(0,0, mFontBold, mRes.getString(R.string.receivechallenge_accept), mMgr);
		mRejectButtonText = new Text(0,0, mFontBold, mRes.getString(R.string.receivechallenge_reject), mMgr);

		mAcceptButton = new Sprite(0,0,3*Constants.TILE_SIZE, Constants.TILE_SIZE,mPaper, mMgr){
			@Override
			public boolean onAreaTouched(TouchEvent pSceneTouchEvent,
					float pTouchAreaLocalX, float pTouchAreaLocalY) {
				switch(pSceneTouchEvent.getAction()){
				case TouchEvent.ACTION_UP:
					if(!mButtonPressed){
						mButtonPressed=true;
						this.setScale(1);
						mParent.acceptChallenge();
						unregisterTouchArea(this);
						}
					return true;
				case TouchEvent.ACTION_DOWN:
					this.setScale(Constants.BUTTONPRESSED_SCALE);
					return true;
				}
				return false;
			}
		};
		mAcceptButton.setPosition(mParent.CAMERA_WIDTH/2-mAcceptButton.getWidth()-5, 
				mDuneBackgroundSprite.getHeight()-mAcceptButton.getHeight()+10);
		mAcceptButtonText.setPosition(mAcceptButton.getWidth()/2-mAcceptButtonText.getWidth()/2, 
				mAcceptButton.getHeight()/2-mAcceptButtonText.getHeight()/2);
		mAcceptButton.attachChild(mAcceptButtonText);

		mRejectButton = new Sprite(0,0,3*Constants.TILE_SIZE, Constants.TILE_SIZE, mPaper, mMgr){
			@Override
			public boolean onAreaTouched(TouchEvent pSceneTouchEvent,
					float pTouchAreaLocalX, float pTouchAreaLocalY) {
				switch(pSceneTouchEvent.getAction()){
				case TouchEvent.ACTION_UP:
					if(!mButtonPressed){
						mButtonPressed=true;
						this.setScale(1);
						mParent.rejectChallenge();
						unregisterTouchArea(this);
					}
					return true;
				case TouchEvent.ACTION_DOWN:
					this.setScale(Constants.BUTTONPRESSED_SCALE);
					return true;
				}
				return false;
			}
		};
		mRejectButton.setPosition(mParent.CAMERA_WIDTH/2+5, 
				mDuneBackgroundSprite.getHeight()-mRejectButton.getHeight()+10);
		mRejectButtonText.setPosition(mRejectButton.getWidth()/2-mRejectButtonText.getWidth()/2, 
				mRejectButton.getHeight()/2-mRejectButtonText.getHeight()/2);
		mRejectButton.attachChild(mRejectButtonText);

		attachChild(mAcceptButton);
		attachChild(mRejectButton);
		registerTouchArea(mAcceptButton);
		registerTouchArea(mRejectButton);
	}

	private void createBattleSizeDisplay(){
		Text text = new Text(0,0,mFontBold,mRes.getString(R.string.receivechallenge_size_of_battle),mMgr);
		text.setPosition(mSizeSelectBackground.getWidth()/2 - text.getWidth()/2, 20);
		mSizeSelectBackground.attachChild(text);

		mBattleSize = new Text(0,0,mBigFontBold,"","".length()+3,mMgr);
		mBattleSize.setPosition(mSizeSelectBackground.getWidth()/2 - mBattleSize.getWidth()/2,
				text.getY()+mFontBold.getLineHeight()+5);
		mSizeSelectBackground.attachChild(mBattleSize);
		
		mSizeGoldBonus = new Text(0,0,mFont,"",30,mMgr);
		mSizeGoldBonus.setPosition(mSizeSelectBackground.getWidth()/2 - mSizeGoldBonus.getWidth()/2, 
				mBattleSize.getY()+mBattleSize.getHeight()+10);
		mSizeSelectBackground.attachChild(mSizeGoldBonus);
	}
	
	private void createCategoryDisplay(){
		Text text = new Text(0,0,mFontBold,mParent.getString(R.string.challenge_category),mMgr);
		text.setPosition(mCategoriesBackground.getWidth()/2-text.getWidth()/2, 20);
		mCategoriesBackground.attachChild(text);
		
		mBattleCategory = new Text(0,0,mBigFontBold,"",20,mMgr);
		mCategoriesBackground.attachChild(mBattleCategory);
		mBattleCategory.setPosition(0, text.getY()+text.getHeight()+20);
	}

	public void initialize(Player self, Player opponent){
		mSelf=self;
		mOpponentPlayer=opponent;
		this.x=mParent.x+1;
		this.y= mParent.y+1;
		setUpSelf();
		setUpOpponent();
		mSelf.calculateBonuses();
		mOpponentPlayer.calculateBonuses();
		displayBonuses();
		mBattleSize.setText(mParent.mBattleSize);
		mSizeGoldBonus.setText("Goldbonus: "+mParent.mBattleSize+"0%");
		mSizeGoldBonus.setPosition(mSizeSelectBackground.getWidth()/2 - mSizeGoldBonus.getWidth()/2, 
				mBattleSize.getY()+mBattleSize.getHeight()+10);
		mBattleCategory.setText(mParent.mCategoryName);
		mBattleCategory.setPosition(mCategoriesBackground.getWidth()/2 - mBattleCategory.getWidth()/2, mBattleCategory.getY());
	}

	private void displayBonuses(){
		removeBars();

		float DAMAGE_PR_BAR = 1f;
		float HEALTH_PR_BAR = 1F;
		float GOLD_BONUS_PR_BAR = 0.25f;
		float GOLD_PENALTY_PR_BAR = 0.25f;

		createBars(mOpponentBonusDamageText, DAMAGE_PR_BAR, mOpponentPlayer.totalDamageBonus, mRedBar);
		createBars(mSelfBonusDamageText, DAMAGE_PR_BAR, mSelf.totalDamageBonus, mBlueBar);

		createBars(mOpponentBonusGoldText, GOLD_BONUS_PR_BAR, mOpponentPlayer.totalGoldModifier, mRedBar);
		createBars(mSelfBonusGoldText, GOLD_BONUS_PR_BAR, mSelf.totalGoldModifier, mBlueBar);

		createBars(mOpponentBonusHealthText, HEALTH_PR_BAR, mOpponentPlayer.totalHealthBonus, mRedBar);
		createBars(mSelfBonusHealthText, HEALTH_PR_BAR, mSelf.totalHealthBonus, mBlueBar);

		createBars(mOpponentGoldPenaltyText, GOLD_PENALTY_PR_BAR, mOpponentPlayer.totalOpponentGoldModifier, mRedBar);
		createBars(mSelfGoldPenaltyText, GOLD_PENALTY_PR_BAR, mSelf.totalOpponentGoldModifier, mBlueBar);

	}

	private void createBars(Text bonusText, float stat_pr_bar, float bonus, TextureRegion bar){
		bonus=Math.abs(bonus);
		float bar_height=6;
		float bar_width=10;
		int nr_bars = (int)(bonus / stat_pr_bar);
		float partial_bar = ((float)bonus%stat_pr_bar)/stat_pr_bar;
		float x = bonusText.getWidth()+5;
		float y = 0;
		boolean line2=false;
		for(int i=0;i<nr_bars;i++){
			Sprite temp = new Sprite(0,0,bar_width, bar_height,bar,mMgr);
			temp.setPosition(x, y);
			bonusText.attachChild(temp);
			x+=temp.getWidth()+2;
			if(!line2 && i>5){
				x=bonusText.getWidth()+5;
				y+=temp.getHeight()+2;
				line2=true;
			}
		}
		float width = bar.getWidth();
		float newWidth = width*partial_bar;
		bar.setTextureWidth(newWidth);
		Sprite temp = new Sprite(x,y,newWidth, mFontBold.getLineHeight()/2-2,bar,mMgr);
		bonusText.attachChild(temp);
		bar.setTextureWidth(width);
	}

	private void removeBars(){
		mEngine.runSafely(new Runnable() {
			@Override
			public void run() {
				mOpponentBonusDamageText.detachChildren();
				mOpponentGoldPenaltyText.detachChildren();
				mOpponentBonusGoldText.detachChildren();
				mOpponentBonusHealthText.detachChildren();
				
				mSelfBonusDamageText.detachChildren();
				mSelfBonusGoldText.detachChildren();
				mSelfBonusHealthText.detachChildren();
				mSelfGoldPenaltyText.detachChildren();				
			}
		});
	}

	private TextureRegion getTextureBasedOnBuildingType(Building b){

		if(b.getType().equals(Constants.BUILDING_TYPES[1])){
			return mParent.mCartPentagonTexture;
		}else if(b.getType().equals(Constants.BUILDING_TYPES[2])){
			return mParent.mCartCircleTexture;
		}else if(b.getType().equals(Constants.BUILDING_TYPES[3])){
			return mParent.mCartTriangleTexture;
		}else if(b.getType().equals(Constants.BUILDING_TYPES[4])){
			return mParent.mCartStarTexture;
		}else if(b.getType().equals(Constants.BUILDING_TYPES[5])){
			return mParent.mCartSquareTexture;
		}

		return mCart;

	}

	private void setUpSelf(){
		mPlayerTile.setTextureSize(Constants.TILE_SIZE*5, Constants.TILE_SIZE*5);
		mPlayerSprite = new Sprite(Constants.TILE_SIZE, Constants.TILE_SIZE, mPlayerTile, mMgr);
		mPlayerSprite.setAlpha(0.65f);
		mBlankTilesSprite.attachChild(mPlayerSprite);

		if(!mSelf.getBuildings().isEmpty()){
			for(Building b: mSelf.getBuildings()){
				TextureRegion tempTex = getTextureBasedOnBuildingType(b);
				int x = b.getX();
				int y = b.getY();
				Sprite temp = new Sprite(x*Constants.TILE_SIZE, y*Constants.TILE_SIZE,
						Constants.TILE_SIZE, Constants.TILE_SIZE, tempTex, mMgr);
				mPlayerSprite.attachChild(temp);
			}			
		}
	}

	private void setUpOpponent(){
		
		int enemyRank = mOpponentPlayer.getRank();
		UserFunctions.PlayerStrength strength = UserFunctions.calculatePlayerStrength(mSelf.getRank(), enemyRank);
		switch(strength){
		case WEAKER: this.mOpponent = mParent.mGreenTileTexture;break;
		case SAME: this.mOpponent = mParent.mYellowTileTexture;break;
		case STRONGER: this.mOpponent = mParent.mRedTileTexture;break;
		default : this.mOpponent = mParent.mRedTileTexture;break;
		}
		
		mOpponent.setTextureSize(Constants.TILE_SIZE*5, Constants.TILE_SIZE*5);
		this.mOpponentSprite = new Sprite(0,0, Constants.TILE_SIZE*5, Constants.TILE_SIZE*5,
				mOpponent, mMgr);
		this.mOpponentSprite.setAlpha(0.65f);

		if(!mOpponentPlayer.getBuildings().isEmpty()){
			for(Building b: mOpponentPlayer.getBuildings()){
				TextureRegion tempTex = getTextureBasedOnBuildingType(b);
				int x = b.getX();
				int y = b.getY();
				Sprite temp = new Sprite(x*Constants.TILE_SIZE, y*Constants.TILE_SIZE,
						Constants.TILE_SIZE, Constants.TILE_SIZE, tempTex, mMgr);
				mOpponentSprite.attachChild(temp);
			}
		}
		Log.v("knowledgewar ReceiveChallenge", "X og Y: "+x+" "+y);
		float px = x*Constants.TILE_SIZE;
		float py = y*Constants.TILE_SIZE;
		px= px-2*Constants.TILE_SIZE;
		py=py-2*Constants.TILE_SIZE;
		mOpponentSprite.setPosition(px, py);
		mBlankTilesSprite.attachChild(mOpponentSprite);
	}

}
