package adrian.master.knowledgewar.scenes;

import java.util.ArrayList;
import java.util.LinkedList;

import org.andengine.engine.Engine;
import org.andengine.engine.camera.Camera;
import org.andengine.engine.camera.hud.HUD;
import org.andengine.entity.scene.ITouchArea;
import org.andengine.entity.scene.ITouchArea.ITouchAreaMatcher;
import org.andengine.entity.scene.Scene;
import org.andengine.entity.sprite.Sprite;
import org.andengine.entity.text.AutoWrap;
import org.andengine.entity.text.Text;
import org.andengine.entity.text.TextOptions;
import org.andengine.input.touch.TouchEvent;
import org.andengine.opengl.font.Font;
import org.andengine.opengl.texture.region.TextureRegion;
import org.andengine.opengl.vbo.VertexBufferObjectManager;
import org.andengine.util.HorizontalAlign;

import adrian.master.knowledgewar.R;
import adrian.master.knowledgewar.gameactivities.BattleActivity;
import adrian.master.knowledgewar.gameconcepts.Building;
import adrian.master.knowledgewar.gameconcepts.Weapon;
import adrian.master.knowledgewar.interfaces.BattleActivityInterface;
import adrian.master.knowledgewar.logic.Constants;
import adrian.master.knowledgewar.subscenes.LoadingHUD;
import android.content.res.Resources;
import android.util.Log;

public class RightAnswerScene extends Scene implements BattleActivityInterface {

	private BattleActivity mParent;
	private Engine mEngine;
	private int CAMERA_HEIGHT = Constants.CAMERA_HEIGHT, CAMERA_WIDTH=Constants.CAMERA_WIDTH;

	private TextureRegion mButton, mDuneBackground, mSandBackground, mBlack, mPaper, mLeather, mRedCross, mGold;
	private Sprite mDuneBackgroundSprite, mSandBackgroundSprite, mLeatherBackgroundSprite, mHeader, mGoldSprite, mGoldDisplay;

	private Sprite mFireButton, mBlackSprite;
	private HUD mHud, mLoadingHud;
	private Camera mCamera;
	private boolean isShowingSubmenu;

	private float UNPRESSABLEBUTTON_ALPHA=0.3f;

	private Text mCorrectAnswer, mChooseWeapon, mGoldText;
	private Font mFontBold,mFont, mBigFont, mSmallFont;

	private float mWeaponButtonHeight, mWeaponButtonWidth;
	private float x_place=-1,y_place=0;
	private int mAvailableMoney;

	private VertexBufferObjectManager mMgr;
//	private Player mSelf;
	private Resources mRes;

	private Weapon mSelectedWeapon = null;
	private Sprite mSelectedWeaponButton;

	public RightAnswerScene(BattleActivity parent, Engine engine){
		this.mParent = parent;
		this.mEngine = engine;
		setUpFields();
		setUpBackground();
		setUpText();
		setUpWeaponsButtons();
		setupButtons();
		setUpGoldDisplay();
		createHUD();
	}

	private void setUpFields(){
		this.isShowingSubmenu=false;
		this.mButton = mParent.mWhiteButtonTexture;
		this.mDuneBackground = mParent.mDuneTexture;
		this.mSandBackground = mParent.mSandTexture;
		this.mCamera = mParent.mCamera;
		this.mLoadingHud = mCamera.getHUD();
		this.mMgr = mParent.getVertexBufferObjectManager();
//		this.mSelf = mParent.mSelf;
		this.mRedCross = mParent.mRedXTexture;
		this.mRes = mParent.mRes;
		this.mFontBold = mParent.mFontBold;
		this.mFont = mParent.mFont;
		this.mSmallFont = mParent.mSmallFont;
		this.mBigFont = mParent.mBigFont;
		this.mBlack = mParent.mBlackTexture;
		this.mPaper = mParent.mPaperTexture;
		this.mLeather = mParent.mLeatherTexture;
		this.mGold = mParent.mMoneyTexture;
		this.mWeaponButtonHeight = Constants.TILE_SIZE*2;
		this.mWeaponButtonWidth = Constants.TILE_SIZE*2;
	}

	private void createHUD(){
		this.mHud = new HUD();
		this.mBlackSprite = new Sprite(0,0,CAMERA_WIDTH, CAMERA_HEIGHT, mBlack, mMgr){
			@Override
			public boolean onAreaTouched(TouchEvent pSceneTouchEvent,
					float pTouchAreaLocalX, float pTouchAreaLocalY) {
				if(pSceneTouchEvent.getAction()==TouchEvent.ACTION_UP){
					clearHUD();						
					return true;
				}
				return false;
			}
		};
		this.mBlackSprite.setAlpha(0.5f);
		this.mHud.registerTouchArea(mBlackSprite);
		this.mHud.setOnAreaTouchTraversalFrontToBack();
	}

	private void setHUD(){
		isShowingSubmenu=true;
		this.mHud.attachChild(mBlackSprite);
		this.mCamera.setHUD(mHud);
	}

	private void clearHUD(){
		isShowingSubmenu=false;
		mHud.unregisterTouchAreas(new ITouchAreaMatcher() {

			@Override
			public boolean matches(ITouchArea pObject) {
				if(pObject!=mBlackSprite){
					return true;
				}
				return false;
			}
		});
		mEngine.runSafely(new Runnable() {
			@Override
			public void run() {
				mHud.detachChildren();
			}
		});
		this.mCamera.setHUD(null);
	}

	private void setUpBackground(){
		mDuneBackgroundSprite = new Sprite (0,0, CAMERA_WIDTH, CAMERA_HEIGHT/4, mDuneBackground, mMgr);
		mSandBackgroundSprite = new Sprite (0,CAMERA_HEIGHT/4, CAMERA_WIDTH, 3*CAMERA_HEIGHT/4, mSandBackground, mMgr);

		mLeatherBackgroundSprite = new Sprite(0,0,CAMERA_WIDTH-80, CAMERA_HEIGHT/4-40, mLeather,mMgr);
		mLeatherBackgroundSprite.setPosition(CAMERA_WIDTH/2 - mLeatherBackgroundSprite.getWidth()/2, 
				CAMERA_HEIGHT/8 - mLeatherBackgroundSprite.getHeight()/2);

		mHeader = new Sprite(0,0, CAMERA_WIDTH - 150, Constants.TILE_SIZE, mLeather,mMgr);
		mHeader.setPosition(CAMERA_WIDTH/2 - mHeader.getWidth()/2, mSandBackgroundSprite.getY()-mHeader.getHeight()/4);


		attachChild(mDuneBackgroundSprite);
		attachChild(mSandBackgroundSprite);
		attachChild(mLeatherBackgroundSprite);
		attachChild(mHeader);
	}
	
	private void setUpGoldDisplay(){
		mGoldDisplay = new Sprite(0,0,mWeaponButtonWidth*2,mWeaponButtonHeight/2,mLeather,mMgr);
		mGoldSprite = new Sprite(0,0,Constants.TILE_SIZE/2,Constants.TILE_SIZE/2,mGold,mMgr);
		mGoldText = new Text(0,0,mSmallFont,mAvailableMoney+"",mMgr);
		
		mGoldDisplay.setPosition(mFireButton.getX(), mFireButton.getY()-mGoldDisplay.getHeight()-10);
		mGoldSprite.setPosition(mGoldDisplay.getWidth()/2 - (mGoldText.getWidth()+mGoldSprite.getWidth())/2,
				mGoldDisplay.getHeight()/2 - mGoldSprite.getHeight()/2 );
		mGoldText.setPosition(mGoldDisplay.getWidth()/2 - (mGoldText.getWidth()+mGoldSprite.getWidth())/2 +(mGoldSprite.getWidth()+10),
				mGoldDisplay.getHeight()/2 - mGoldText.getHeight()/2);
		
		attachChild(mGoldDisplay);
		mGoldDisplay.attachChild(mGoldSprite);
		mGoldDisplay.attachChild(mGoldText);
		
	}

	private void setUpText(){
		String text = mRes.getString(R.string.battle_answer_correct);
		mCorrectAnswer = new Text(0,0, mBigFont, text, mMgr);
		mCorrectAnswer.setPosition(mLeatherBackgroundSprite.getWidth()/2 - mCorrectAnswer.getWidth()/2, 
				mLeatherBackgroundSprite.getHeight()/2 - mCorrectAnswer.getHeight()/2);
		mLeatherBackgroundSprite.attachChild(mCorrectAnswer);

		String chooseWeapon = mRes.getString(R.string.battle_answer_choose_weapon);
		mChooseWeapon = new Text(0,0, mFontBold, chooseWeapon, new TextOptions(AutoWrap.WORDS, mHeader.getWidth()-60, HorizontalAlign.CENTER), mMgr);
		mChooseWeapon.setPosition(30, mHeader.getHeight()/2 - mChooseWeapon.getHeight()/2);
		mHeader.attachChild(mChooseWeapon);
	}

	private void setUpWeaponsButtons(){
		int numberOfWeapons=0;
		mAvailableMoney = mParent.mSelf.getGold() - mParent.getMoneySpentOnWeapons();
		ArrayList<Building> buildings = mParent.mSelf.getBuildings();
		LinkedList<String> addedBuildings = new LinkedList<String>();
		boolean buildingAlreadyAdded=false;

		mEngine.runSafely(new Runnable() {
			@Override
			public void run() {
				mSandBackgroundSprite.detachChildren();
			}
		});
//		Log.v("Knowledgewar","RightAnswerScene, before buildingloop. Number of buildings: "+buildings.size());
		for(Building b: buildings){
//			Log.v("Knowledgewar","RightAnswerScene, buildingloop. Current building: "+b.getName());
			buildingAlreadyAdded=false;
			for(int i=0;i<addedBuildings.size();i++){
				if(b.getType().equals(addedBuildings.get(i))){
					buildingAlreadyAdded=true;
				}
			}
			if(!buildingAlreadyAdded){
				ArrayList<Weapon> weaponList = b.getWeapons();
				Log.v("Knowledgewar","RightAnswerScene, before weaponlopp. Current building: "+b.getName()+", number of weapons: "+weaponList.size());
				for(Weapon w: weaponList){
					Log.v("Knowledgewar","RightAnswerScene, weaponloop. Current currentweapon: "+w.getmName());
					if(w.getmRank()<=b.getRank()){
						numberOfWeapons++;
						createWeaponElement(w);	
					}
				}
				addedBuildings.add(b.getType());
			}
		}
		if(numberOfWeapons==0){
			Sprite leather = new Sprite(0,0,CAMERA_WIDTH-150,Constants.TILE_SIZE*4,mLeather,mMgr);
			leather.setPosition(CAMERA_WIDTH/2 - leather.getWidth()/2, CAMERA_HEIGHT/2 - leather.getHeight()/2);
			String string1 = mRes.getString(R.string.battle_answer_no_weapons);
			String string2 = mRes.getString(R.string.battle_answer_buy_buildings);

			Text text1 = new Text(0,0,mFont,string1,new TextOptions(AutoWrap.WORDS, leather.getWidth()-60, HorizontalAlign.CENTER),mMgr);
			Text text2 = new Text(0,0,mSmallFont,string2,new TextOptions(AutoWrap.WORDS, leather.getWidth()-60, HorizontalAlign.CENTER),mMgr);

			text1.setPosition(30, 30);
			text2.setPosition(30, text1.getY()+text1.getHeight()+10);

			leather.attachChild(text1);
			leather.attachChild(text2);
			attachChild(leather);
		}

	}

	private TextureRegion getTextureBasedonWeapon(Weapon w){
		String c = w.getmClass()+w.getmRank();
		if(c.equals("B1")){
			return mParent.mNinja1Texture;
		}else if(c.equals("B3")){
			return mParent.mNinja2Texture;
		}else if(c.equals("C1")){
			return mParent.mFan1Texture;
		}else if(c.equals("C3")){
			return mParent.mFan2Texture;
		}else if(c.equals("E1")){
			return mParent.mWand1Texture;
		}else if(c.equals("E3")){
			return mParent.mWand2Texture;
		}else if(c.equals("F1")){
			return mParent.mChickenTexture;
		}else if(c.equals("F3")){
			return mParent.mCowTexture;
		}
		return mButton;
	}

	private void setupButtons(){

		mFireButton = new Sprite(0,0,mWeaponButtonWidth*2,mWeaponButtonHeight,
				mPaper,mMgr){
			@Override
			public boolean onAreaTouched(TouchEvent pSceneTouchEvent,
					float pTouchAreaLocalX, float pTouchAreaLocalY) {

				switch(pSceneTouchEvent.getAction()){
				case TouchEvent.ACTION_UP:
					unregisterTouchArea(this);
					if(mSelectedWeapon!=null){
						this.setScale(1);
						mParent.payCostForWeapon(mSelectedWeapon.getmValue());
						mParent.nextQuestion(mSelectedWeapon.getmClass()+mSelectedWeapon.getmRank());
					}else{
						this.setScale(1);
						mParent.nextQuestion(Constants.ATTACK_NOATTACK);
					}
					return true;
				case TouchEvent.ACTION_DOWN:
					this.setScale(Constants.BUTTONPRESSED_SCALE);
					return true;
				}
				return false;
			}
		};
		mFireButton.setPosition(CAMERA_WIDTH/2 - mFireButton.getWidth()/2, 
				CAMERA_HEIGHT-mFireButton.getHeight()-20);

		setFireButtonText();

		registerTouchArea(mFireButton);
		attachChild(mFireButton);
	}

	private void setFireButtonText(){
		mEngine.runSafely(new Runnable() {
			@Override
			public void run() {
				mFireButton.detachChildren();
			}
		});
		TextOptions opt = new TextOptions(AutoWrap.WORDS, mWeaponButtonWidth*2-40, HorizontalAlign.CENTER);
		Text text,text2;
		String string, string2;
		if(mSelectedWeapon!=null){
			string = mRes.getString(R.string.battle_answer_fire);
		}else{
			string = mRes.getString(R.string.battle_answer_continue);
		}
		string2 = mRes.getString(R.string.battle_next_question);
		text = new Text(0,0,mParent.mFontBold,string, opt,mMgr);
		text2 = new Text(0,0,mParent.mFontBold,string2,opt,mMgr);
		text.setPosition(mFireButton.getWidth()/2 - text.getWidth()/2, mFireButton.getHeight()/2 - text.getHeight()-2);
		text2.setPosition(mFireButton.getWidth()/2 - text2.getWidth()/2, mFireButton.getHeight()/2 +2);
		mFireButton.attachChild(text);
		mFireButton.attachChild(text2);
	}

	private void createWeaponElement(final Weapon w){
		Log.v("knowledgewar","Create weapon element activated: "+w.getmName());
		float totalWidth = Constants.TILE_SIZE+Constants.TILE_SIZE/2+2;
		float totalHeight = Constants.TILE_SIZE + mSmallFont.getLineHeight()+10;

		x_place++;
		if(x_place>=3){
			x_place=0;
			y_place++;
		}

		int wCost = w.getmValue();
		String cost = mRes.getString(R.string.weapon_cost);
		Text tCost = new Text(0,0,mSmallFont,cost+wCost,mMgr);
		
		Sprite tCostBackground = new Sprite(0,0,tCost.getWidth()+20,tCost.getHeight()+10,mLeather,mMgr);

		Sprite button = new Sprite(0,0,Constants.TILE_SIZE, Constants.TILE_SIZE,getTextureBasedonWeapon(w),mMgr){
			@Override
			public boolean onAreaTouched(TouchEvent pSceneTouchEvent,
					float pTouchAreaLocalX, float pTouchAreaLocalY) {
				switch(pSceneTouchEvent.getAction()){
				case TouchEvent.ACTION_UP:
					if(mSelectedWeaponButton==this){
						this.setScale(1);
						mSelectedWeapon=null;
						mSelectedWeaponButton=null;
					}else if(mSelectedWeaponButton!=null){
						mSelectedWeaponButton.setScale(1);
						mSelectedWeapon=w;
						this.setScale(1.25f);
						mSelectedWeaponButton=this;
					}else{							
						mSelectedWeapon=w;
						this.setScale(1.25f);
						mSelectedWeaponButton=this;
					}
					setFireButtonText();
					return true;
				}

				return false;
			}
		};

		Sprite helpButton = new Sprite(0,0,Constants.TILE_SIZE/2,Constants.TILE_SIZE/2,mPaper,mMgr){
			@Override
			public boolean onAreaTouched(TouchEvent pSceneTouchEvent,
					float pTouchAreaLocalX, float pTouchAreaLocalY) {
				switch(pSceneTouchEvent.getAction()){
				case TouchEvent.ACTION_DOWN: 
					this.setScale(1.25f);
					break;
				case TouchEvent.ACTION_UP: 
					this.setScale(1f);
					displayWeaponSubMenu(w);
					break;
				}
				return false;
			}
		};


		float buttonPositionX=x_place*(CAMERA_WIDTH/3)+(CAMERA_WIDTH/6);
		float buttonPositionY=y_place*totalHeight+(mChooseWeapon.getY()+mChooseWeapon.getHeight()+10);

		button.setPosition(buttonPositionX-(totalWidth/2), buttonPositionY);
		tCostBackground.setPosition(buttonPositionX-tCostBackground.getWidth()/2, buttonPositionY+button.getHeight()+5);
		tCost.setPosition(tCostBackground.getWidth()/2 - tCost.getWidth()/2, tCostBackground.getHeight()/2 - tCost.getHeight()/2);
		helpButton.setPosition(button.getX()+Constants.TILE_SIZE+2, button.getY()+button.getHeight()/2-helpButton.getHeight()/2);

		Text questionMark = new Text(0,0,mSmallFont,"?",mMgr);
		questionMark.setPosition(helpButton.getWidth()/2 - questionMark.getWidth()/2, helpButton.getHeight()/2 - questionMark.getHeight()/2);
		helpButton.attachChild(questionMark);
		registerTouchArea(helpButton);

		mSandBackgroundSprite.attachChild(tCostBackground);
		tCostBackground.attachChild(tCost);
		mSandBackgroundSprite.attachChild(button);
		mSandBackgroundSprite.attachChild(helpButton);

		//Mark weapon as unavailable if player does not have enough gold
//		mAvailableMoney = mParent.mSelf.getGold() - mParent.getMoneySpentOnWeapons();
		if(mAvailableMoney>=w.getmValue()){
			registerTouchArea(button);				
		}else{
			Sprite x = new Sprite(0,0,Constants.TILE_SIZE,Constants.TILE_SIZE,mRedCross,mMgr);
			button.setAlpha(UNPRESSABLEBUTTON_ALPHA);
//			tCost.setColor(1, 0, 0);
			x.setPosition(button.getWidth()/2 - x.getWidth()/2, (button.getHeight()+tCostBackground.getHeight())/2 - x.getHeight()/2);
			button.attachChild(x);
		}
	}


	private void displayWeaponSubMenu(Weapon w){
		setHUD();
		Sprite sub = new WeaponSubMenu(w);
		mHud.attachChild(sub);
	}


	private class WeaponSubMenu extends Sprite{

		private Weapon mWeapon;

		public WeaponSubMenu(Weapon w){
			super(0,0, CAMERA_WIDTH-Constants.TILE_SIZE*3, Constants.TILE_SIZE*4, mLeather,mMgr);
			mWeapon = w;
			setPosition(CAMERA_WIDTH/2-getWidth()/2, CAMERA_HEIGHT/2-getHeight()/2);
			createText();
		}

		public void createText(){
			TextOptions main = new TextOptions(AutoWrap.WORDS, this.getWidth()-60, HorizontalAlign.CENTER, Text.LEADING_DEFAULT);
			String name = mWeapon.getmName();
			String desc = mWeapon.getmDesc();
			int cost = mWeapon.getmValue();
			String tcost = mRes.getString(R.string.weapon_cost);

			float yCoord = 25;

			Text wName = new Text(30,yCoord,mFontBold, name, main, mMgr);
			Text wDesc = new Text(30,yCoord+2*mFontBold.getLineHeight(),mSmallFont, desc, main, mMgr);
			Text wCost = new Text(30,wDesc.getY()+wDesc.getHeight()+mSmallFont.getLineHeight()*2, mSmallFont, tcost+" "+cost, main, mMgr);

			attachChild(wName);
			attachChild(wDesc);
			attachChild(wCost);
		}

	}


	@Override
	public boolean onBackButtonPressed() {
		if(isShowingSubmenu){
			clearHUD();
			return true;
		}
		return false;
	}

}
