package adrian.master.knowledgewar.scenes;

import org.andengine.entity.scene.Scene;
import org.andengine.entity.sprite.Sprite;
import org.andengine.entity.text.Text;
import org.andengine.input.touch.TouchEvent;
import org.andengine.opengl.font.Font;
import org.andengine.opengl.texture.region.TextureRegion;
import org.andengine.opengl.vbo.VertexBufferObjectManager;

import adrian.master.knowledgewar.R;
import adrian.master.knowledgewar.gameactivities.BattleActivity;
import adrian.master.knowledgewar.interfaces.BattleActivityInterface;
import adrian.master.knowledgewar.logic.Constants;
import android.content.res.Resources;

public class WrongAnswerScene extends Scene implements BattleActivityInterface {

	private int CAMERA_HEIGHT = Constants.CAMERA_HEIGHT, CAMERA_WIDTH = Constants.CAMERA_WIDTH;

	private TextureRegion mDuneBackground, mSandBackground, mPaper, mLeather;
	private Sprite mDuneBackgroundSprite, mSandBackgroundSprite, mButtonSprite, mLeatherBackground;
	private Text mWrongAnswerText, mButtonText;
	private Font mFont;

	private BattleActivity mParent;
	private Resources mRes;

	private VertexBufferObjectManager mMgr;

	public WrongAnswerScene(BattleActivity parent){
		mParent = parent;
		setupFields();
		setUpBackground();
		setUpText();
		setUpButton();
	}

	private void setupFields(){
		this.mDuneBackground = mParent.mDuneTexture;
		this.mSandBackground = mParent.mSandTexture;
		this.mMgr = mParent.mMgr;
		this.mRes = mParent.mRes;
		this.mFont = mParent.mFont;
		this.mPaper = mParent.mPaperTexture;
		this.mLeather = mParent.mLeatherTexture;
	}

	private void setUpBackground(){
		mDuneBackgroundSprite = new Sprite (0,0, CAMERA_WIDTH, CAMERA_HEIGHT/2, mDuneBackground, mMgr);
		mSandBackgroundSprite = new Sprite (0,CAMERA_HEIGHT/2, CAMERA_WIDTH, CAMERA_HEIGHT/2, mSandBackground, mMgr);
		
		mLeatherBackground = new Sprite(0,0, CAMERA_WIDTH-Constants.TILE_SIZE*2, CAMERA_HEIGHT/4 - Constants.TILE_SIZE*2, mLeather, mMgr);
		mLeatherBackground.setPosition(CAMERA_WIDTH/2 - mLeatherBackground.getWidth()/2, CAMERA_HEIGHT/4 - mLeatherBackground.getHeight()/2);

		attachChild(mDuneBackgroundSprite);
		attachChild(mSandBackgroundSprite);
		attachChild(mLeatherBackground);
	}

	private void setUpText(){
		String text = mRes.getString(R.string.battle_answer_wrong);
		mWrongAnswerText = new Text(0,0, mParent.mFontBold, text, mMgr);
		mWrongAnswerText.setPosition(mLeatherBackground.getWidth()/2 - mWrongAnswerText.getWidth()/2,
				mLeatherBackground.getHeight()/2 - mWrongAnswerText.getHeight()/2);
		mLeatherBackground.attachChild(mWrongAnswerText);

		String buttonText = mRes.getString(R.string.battle_answer_wrong_button);
		mButtonText = new Text(0,0, mFont, buttonText, mMgr);

	}

	private void setUpButton(){
		mButtonSprite = new Sprite(0,0,256,128,mPaper, mMgr){
			@Override
			public boolean onAreaTouched(TouchEvent pSceneTouchEvent,
					float pTouchAreaLocalX, float pTouchAreaLocalY) {

				switch(pSceneTouchEvent.getAction()){
				case TouchEvent.ACTION_UP:
					unregisterTouchArea(this);
					this.setScale(1);
					mParent.nextQuestion(Constants.ATTACK_NOATTACK);
					return true;
				case TouchEvent.ACTION_DOWN:
					this.setScale(Constants.BUTTONPRESSED_SCALE);
					return true;
				}
				return false;
			}
		};
		mButtonSprite.setPosition(CAMERA_WIDTH/2 - mButtonSprite.getWidth()/2
				, 3*CAMERA_HEIGHT/4 - mButtonSprite.getHeight()/2);
		attachChild(mButtonSprite);
		registerTouchArea(mButtonSprite);

		mButtonText.setPosition(mButtonSprite.getWidth()/2 - mButtonText.getWidth()/2, 
				mButtonSprite.getHeight()/2 - mButtonText.getHeight()/2);
		mButtonSprite.attachChild(mButtonText);
	}

	@Override
	public boolean onBackButtonPressed() {
		// TODO Auto-generated method stub
		return false;
	}

}
