package adrian.master.knowledgewar.subscenes;

import java.util.ArrayList;
import java.util.LinkedList;

import org.andengine.engine.Engine;
import org.andengine.engine.camera.hud.HUD;
import org.andengine.entity.scene.IOnSceneTouchListener;
import org.andengine.entity.scene.Scene;
import org.andengine.entity.sprite.AnimatedSprite;
import org.andengine.entity.sprite.Sprite;
import org.andengine.entity.text.AutoWrap;
import org.andengine.entity.text.Text;
import org.andengine.entity.text.TextOptions;
import org.andengine.input.touch.TouchEvent;
import org.andengine.input.touch.detector.ScrollDetector;
import org.andengine.input.touch.detector.ScrollDetector.IScrollDetectorListener;
import org.andengine.opengl.font.Font;
import org.andengine.opengl.texture.region.TextureRegion;
import org.andengine.opengl.texture.region.TiledTextureRegion;
import org.andengine.opengl.vbo.VertexBufferObjectManager;

import adrian.master.knowledgewar.R;
import adrian.master.knowledgewar.gameactivities.MainActivity;
import adrian.master.knowledgewar.logic.Constants;
import adrian.master.knowledgewar.logic.UserFunctions;

public class ChatHUD extends HUD implements IScrollDetectorListener, IOnSceneTouchListener {

	private MainActivity mParent;
	private ArrayList<String> mMessages;
	private LinkedList<Text>	mTexts;

	private TextureRegion mBackgroundTexture, mTabTexture,mBlackBackgroundTexture;
	private TiledTextureRegion mCubeTexture;

	private Sprite mTab, mBackground, mHelpButton,mBlackBackground;
	private AnimatedSprite mCube;
	private VertexBufferObjectManager mMgr;
	private Font mFont;
	private ScrollDetector mScroll;
	private Engine mEngine;

	private boolean isDisplayingChat = false;

	private float tab_x, tab_visible_y, tab_hidden_y;
	private float chat_x, chat_visible_y, chat_hidden_y;
	private float mChatStartingPoint = Constants.CAMERA_HEIGHT/2 - 30;

	private static ChatHUD mInstance;

	public ChatHUD(MainActivity parent, Engine engine){
		mParent = parent;
		mMessages = new ArrayList<String>();
		mBackgroundTexture = mParent.mPaperTexture;
		mTabTexture = mParent.mPaperTexture;
		mMgr = mParent.getVertexBufferObjectManager();
		mFont = mParent.mFont;
		mInstance=this;
		mEngine = engine;
		
		setupBlackBackground();

		setOnAreaTouchTraversalFrontToBack();
		setTouchAreaBindingOnActionDownEnabled(true);
		mScroll = new ScrollDetector(this);
		mScroll.setTriggerScrollMinimumDistance(11);
		mScroll.setEnabled(true);
		setOnSceneTouchListener(this);
		createSprites();
	}

	public static ChatHUD getInstance(){
		return mInstance;
	}

	public void refreshList(){
		drawMessages();
	}
	
	public boolean isDisplayingChat(){
		return isDisplayingChat;
	}
	
	private void setupBlackBackground(){
		this.mBlackBackgroundTexture = mParent.mBlackTexture;
		this.mCubeTexture = mParent.mCubeTexture;
		
		this.mCube = new AnimatedSprite(0, 0, mCubeTexture, mMgr);
		this.mCube.setPosition(Constants.CAMERA_WIDTH/2 - this.mCube.getWidth()/2, Constants.CAMERA_HEIGHT/2 - this.mCube.getHeight()/2);
		this.mCube.setScale(2);
		
		this.mBlackBackgroundTexture.setTextureSize(Constants.CAMERA_WIDTH, Constants.CAMERA_HEIGHT);
		this.mBlackBackground = new Sprite(0,0,mBlackBackgroundTexture,mMgr);
		this.mBlackBackground.setAlpha(0.5f);
		mBlackBackground.attachChild(mCube);
		attachChild(mBlackBackground);
		
		this.mBlackBackground.setVisible(false);
	}
	
	public void displayLoadingAnimation(boolean show){
		this.mBlackBackground.setVisible(show);
		if(show){
			mCube.animate(100);
//			long[] b = {40,40,40,40,40,40,40,100,40,40,40,40};
//			mCube.animate(b, 24, 35, true);
		}else{
			mCube.stopAnimation();
		}
	}

	private void createSprites(){
		mHelpButton = new Sprite(0,0,Constants.TILE_SIZE, Constants.TILE_SIZE, mBackgroundTexture,mMgr){
			@Override
			public boolean onAreaTouched(TouchEvent pSceneTouchEvent,
					float pTouchAreaLocalX, float pTouchAreaLocalY) {
				if(pSceneTouchEvent.getAction()==TouchEvent.ACTION_UP){
					this.setScale(1f);
					mParent.setHelpScene();
					return true;
				}else if(pSceneTouchEvent.getAction()==TouchEvent.ACTION_DOWN){
					this.setScale(1.25f);
					return true;
				}
				return false;
			}
		};
		
		mBackground=new Sprite(0,0,480,720,mBackgroundTexture, mMgr){
			@Override
			public boolean onAreaTouched(TouchEvent pSceneTouchEvent,
					float pTouchAreaLocalX, float pTouchAreaLocalY) {
				return false;
			}
		};
		mTab = new Sprite(0,0,120,120,mTabTexture,mMgr){
			@Override
			public boolean onAreaTouched(TouchEvent pSceneTouchEvent,
					float pTouchAreaLocalX, float pTouchAreaLocalY) {
				if(pSceneTouchEvent.getAction()==TouchEvent.ACTION_UP){
					if(isDisplayingChat){
						hideChat();
					}else{
						displayChat();
					}
					return true;
				}else if(pSceneTouchEvent.getAction()==TouchEvent.ACTION_DOWN){
					return true;
				}
				return false;
			}
		};
		Text questionMark = new Text(0,0,mParent.mFontBold, mParent.getString(R.string.HUD_help),mMgr);
		questionMark.setPosition(mHelpButton.getWidth()/2-questionMark.getWidth()/2, mHelpButton.getHeight()/2-questionMark.getHeight()/2);
		mHelpButton.attachChild(questionMark);
		mHelpButton.setPosition(Constants.CAMERA_WIDTH-mHelpButton.getWidth(), Constants.CAMERA_HEIGHT-mHelpButton.getHeight());
		attachChild(mHelpButton);

		Text arrow = new Text(0,0,mParent.mFontBold,"^",mMgr);
		Text chatText = new Text(0,0,mParent.mSmallFontBold,mParent.getString(R.string.HUD_chat),mMgr);
		arrow.setPosition(mTab.getWidth()/2 - arrow.getWidth()/2, mTab.getHeight()/5);
		chatText.setPosition(mTab.getWidth()/2 - chatText.getWidth()/2, mTab.getHeight()/3);
		mTab.attachChild(chatText);
		mTab.attachChild(arrow);

		tab_x = Constants.CAMERA_WIDTH/2 - mTab.getWidth()/2;
		tab_hidden_y = Constants.CAMERA_HEIGHT- mTab.getHeight()/2;
		tab_visible_y = tab_hidden_y - Constants.CAMERA_HEIGHT/2;

		chat_x = 0;
		chat_hidden_y = Constants.CAMERA_HEIGHT;
		chat_visible_y = Constants.CAMERA_HEIGHT/2;

		mTab.setPosition(tab_x, tab_hidden_y);
		mBackground.setPosition(chat_x, chat_hidden_y);
		attachChild(mTab);
		attachChild(mBackground);
		registerTouchArea(mTab);
		registerTouchArea(mBackground);
	}

	private void displayChat(){
		mBackground.setPosition(chat_x, chat_visible_y);
		mTab.setPosition(tab_x, tab_visible_y);
		mParent.displayChatInput(true);		
		isDisplayingChat=true;
		drawMessages();
	}

	public void hideChat(){
		removeMessages();
		mBackground.setPosition(chat_x, chat_hidden_y);
		mTab.setPosition(tab_x, tab_hidden_y);
		mParent.displayChatInput(false);
		isDisplayingChat=false;
	}

	private void drawMessages(){
		removeMessages();
		mMessages = UserFunctions.getMessageList();
		mTexts=new LinkedList<Text>();
		if(mMessages!= null){
			float y=mChatStartingPoint;
			float x=25;
			TextOptions opt = new TextOptions(AutoWrap.WORDS, mBackground.getWidth()-20);
			for(int i = mMessages.size()-1;i>=0;i--){
				String s = mMessages.get(i);
				Text message = new Text(x,y,mFont,s,opt,mMgr);
				y-=message.getHeight();
				mBackground.attachChild(message);
				mTexts.add(message);
				if(message.getY()<10){
					message.setVisible(false);
				}
			}
		}
	}

	private float getMessageListHeight(){
		float result = mFont.getLineHeight()*mTexts.size();
		return result;
	}

	private void removeMessages(){
		mEngine.runSafely(new Runnable() {
			@Override
			public void run() {
				if(mTexts!=null && !mTexts.isEmpty() ){
					for(Text t:mTexts){
						mBackground.detachChild(t);
					}	
				}
			}
		});
	}
	
	public void showHelpButton(boolean show){
		mHelpButton.setVisible(show);
		if(show){
			registerTouchArea(mHelpButton);
		}else{
			unregisterTouchArea(mHelpButton);
		}
	}

	@Override
	public void onScrollStarted(ScrollDetector pScollDetector, int pPointerID,
			float pDistanceX, float pDistanceY) {

	}

	@Override
	public void onScroll(ScrollDetector pScollDetector, int pPointerID,
			float pDistanceX, float pDistanceY) {
		if(mChatStartingPoint+pDistanceY >= Constants.CAMERA_HEIGHT/2 - mFont.getLineHeight()
				&& (mChatStartingPoint-getMessageListHeight()+pDistanceY < 10)){
			mChatStartingPoint+=pDistanceY;
			for(Text t:mTexts){
				t.setPosition(t.getX(), t.getY()+pDistanceY);
				t.setVisible(t.getY()>10 && t.getY()<Constants.CAMERA_HEIGHT/2);
			}	
		}

	}

	@Override
	public void onScrollFinished(ScrollDetector pScollDetector, int pPointerID,
			float pDistanceX, float pDistanceY) {
	}

	@Override
	public boolean onSceneTouchEvent(Scene pScene, TouchEvent pSceneTouchEvent) {
		if(isDisplayingChat){
			mScroll.onTouchEvent(pSceneTouchEvent);
			return true;			
		}
		return false;
	}

}
