package adrian.master.knowledgewar.subscenes;

import org.andengine.entity.scene.Scene;
import org.andengine.entity.sprite.Sprite;
import org.andengine.entity.text.AutoWrap;
import org.andengine.entity.text.Text;
import org.andengine.entity.text.TextOptions;
import org.andengine.input.touch.TouchEvent;
import org.andengine.opengl.texture.region.TextureRegion;
import org.andengine.opengl.vbo.VertexBufferObjectManager;
import org.andengine.util.HorizontalAlign;

import adrian.master.knowledgewar.R;
import adrian.master.knowledgewar.gameactivities.MainActivity;
import adrian.master.knowledgewar.interfaces.ConfirmationDialogInterface;
import adrian.master.knowledgewar.logic.Constants;
import android.content.res.Resources;

public class ConfirmationDialog extends Scene {
	
	float mx, my;
	Text mMainText, mYesText, mNoText;
	Sprite mYesButton, mNoButton, mWindow, mBackground;
	private MainActivity mParent;
	private ConfirmationDialogInterface mParentScene;
	private TextureRegion mPaper, mLeather;
	private Resources mRes;
	private VertexBufferObjectManager mMgr;
	private int CAMERA_HEIGHT = Constants.CAMERA_HEIGHT, CAMERA_WIDTH=Constants.CAMERA_WIDTH;
	
	public ConfirmationDialog(MainActivity parent, ConfirmationDialogInterface parentScene){
		mParent = parent;
		mPaper = mParent.mPaperTexture;
		mLeather = mParent.mLeatherTexture;
		mRes = mParent.mRes;
		mMgr = mParent.getVertexBufferObjectManager();
		mParentScene = parentScene;
		setBackgroundEnabled(false);
		setOnAreaTouchTraversalFrontToBack();
		createBackground();
		createWindow();
	}
	
	private void createBackground(){
		TextureRegion mBlack = mParent.mBlackTexture;
		mBackground=new Sprite(0,0,mBlack,mMgr){
			@Override
			public boolean onAreaTouched(TouchEvent pSceneTouchEvent,
					float pTouchAreaLocalX, float pTouchAreaLocalY) {
				mParentScene.cancel();
				return true;
			}
		};
		mBackground.setAlpha(0.3f);
		attachChild(mBackground);
		registerTouchArea(mBackground);
	}
	
	private void createWindow(){
		mWindow = new Sprite(0,0,CAMERA_WIDTH-Constants.TILE_SIZE*2, Constants.TILE_SIZE*3, mLeather,mMgr){
			@Override
			public boolean onAreaTouched(TouchEvent pSceneTouchEvent,
					float pTouchAreaLocalX, float pTouchAreaLocalY) {
				return true;
			}
		};
		mWindow.setPosition(CAMERA_WIDTH/2-mWindow.getWidth()/2, CAMERA_HEIGHT/2-mWindow.getHeight()/2);
		attachChild(mWindow);
		registerTouchArea(mWindow);
		createButtons();
		createText();
	}
	
	private void createButtons(){
		mYesButton = new Sprite(0,0,Constants.TILE_SIZE*2, Constants.TILE_SIZE, mPaper, mMgr){
			@Override
			public boolean onAreaTouched(TouchEvent pSceneTouchEvent,
					float pTouchAreaLocalX, float pTouchAreaLocalY) {

				switch(pSceneTouchEvent.getAction()){
				case TouchEvent.ACTION_DOWN:
					this.setScale(Constants.BUTTONPRESSED_SCALE);
					return true;
				case TouchEvent.ACTION_UP:
					this.setScale(1);
					mParentScene.confirmationYes();
					return true;
				}

				return false;	
			}
		};
		mNoButton = new Sprite(0,0,Constants.TILE_SIZE*2, Constants.TILE_SIZE, mPaper, mMgr){
			@Override
			public boolean onAreaTouched(TouchEvent pSceneTouchEvent,
					float pTouchAreaLocalX, float pTouchAreaLocalY) {

				switch(pSceneTouchEvent.getAction()){
				case TouchEvent.ACTION_DOWN:
					this.setScale(Constants.BUTTONPRESSED_SCALE);
					return true;
				case TouchEvent.ACTION_UP:
					this.setScale(1);
					mParentScene.confirmationNo();
					return true;
				}
				return false;	
			}
		};

		mYesButton.setPosition(mWindow.getWidth()/2-mYesButton.getWidth()-10, mWindow.getHeight()-mYesButton.getHeight()-20);
		mNoButton.setPosition(mWindow.getWidth()/2+10, mWindow.getHeight()-mNoButton.getHeight()-20);
		mWindow.attachChild(mYesButton);
		mWindow.attachChild(mNoButton);
		registerTouchArea(mYesButton);
		registerTouchArea(mNoButton);
	}
	
	private void createText(){
		TextOptions main = new TextOptions(AutoWrap.WORDS, mWindow.getWidth(), HorizontalAlign.CENTER, Text.LEADING_DEFAULT);
		TextOptions button = new TextOptions(AutoWrap.WORDS, mYesButton.getWidth(), HorizontalAlign.CENTER, Text.LEADING_DEFAULT);
		String text = mRes.getString(R.string.owntribe_confirm_removebuilding);
		String yes = mRes.getString(R.string.owntribe_confirm_yes);
		String no = mRes.getString(R.string.owntribe_confirm_no);

		mMainText = new Text(0,0,mParent.mFontBold,text,main,mMgr);
		mMainText.setPosition(mWindow.getWidth()/2 - mMainText.getWidth()/2, 15);
		mYesText = new Text(0,0,mParent.mFontBold,yes,button,mMgr);
		mYesText.setPosition(mYesButton.getWidth()/2 - mYesText.getWidth()/2, mYesButton.getHeight()/2 - mYesText.getHeight()/2);
		mNoText = new Text(0,0,mParent.mFontBold,no,button,mMgr);
		mNoText.setPosition(mNoButton.getWidth()/2 - mNoText.getWidth()/2, mNoButton.getHeight()/2 - mNoText.getHeight()/2);

		mWindow.attachChild(mMainText);
		mYesButton.attachChild(mYesText);
		mNoButton.attachChild(mNoText);	
	}

}
