package adrian.master.knowledgewar.subscenes;

import java.util.ArrayList;

import org.andengine.entity.scene.Scene;
import org.andengine.entity.sprite.Sprite;
import org.andengine.entity.text.AutoWrap;
import org.andengine.entity.text.Text;
import org.andengine.entity.text.TextOptions;
import org.andengine.input.touch.TouchEvent;
import org.andengine.opengl.texture.region.TextureRegion;
import org.andengine.opengl.vbo.VertexBufferObjectManager;
import org.andengine.util.HorizontalAlign;

import adrian.master.knowledgewar.R;
import adrian.master.knowledgewar.gameactivities.MainActivity;
import adrian.master.knowledgewar.gameconcepts.Building;
import adrian.master.knowledgewar.gameconcepts.BuildingFactory;
import adrian.master.knowledgewar.gameconcepts.Weapon;
import adrian.master.knowledgewar.gameconcepts.WeaponFactory;
import adrian.master.knowledgewar.interfaces.SubMenuInterface;
import adrian.master.knowledgewar.logic.Constants;
import adrian.master.knowledgewar.scenes.OwnTribeScene;
import android.content.res.Resources;

public class GeneralBuildingSubMenu extends Scene implements SubMenuInterface {

	private Building mBuilding;
	private ArrayList<Building> mBuildingList;
	private ArrayList<Weapon> mWeaponList;
	private boolean hasWeapons = true;
	private MainActivity mParent;
	private OwnTribeScene mParentScene;
	private VertexBufferObjectManager mMgr;
	private Resources mRes;
	private Sprite mWindow, mBackground;
	private TextureRegion mHorizontalLine,mBar, mPaper, mLeather;

	private boolean isShowingSubMenu = false;
	
	float DAMAGE_PR_BAR = 1f;
	float HEALTH_PR_BAR = 1F;
	float GOLD_BONUS_PR_BAR = 0.25f;
	float GOLD_PENALTY_PR_BAR = 0.25f;
	private int CAMERA_HEIGHT = Constants.CAMERA_HEIGHT, CAMERA_WIDTH=Constants.CAMERA_WIDTH;

	public GeneralBuildingSubMenu(MainActivity parent, OwnTribeScene parentScene, Building building){

		mBuilding = building;
		mBuildingList = BuildingFactory.getAllBuildingsOfType(mBuilding.getType());
		mWeaponList = WeaponFactory.getAllWeaponsofClass(mBuilding.getType());
		if(mWeaponList.isEmpty()){
			hasWeapons=false;
		}
		mParent = parent;
		mParentScene = parentScene;
		mMgr=mParent.getVertexBufferObjectManager();
		mRes=mParent.mRes;
		mPaper = mParent.mPaperTexture;
		mLeather=mParent.mLeatherTexture;
		mBar=mParent.mBlueBarTexture;
		this.mHorizontalLine = mParent.mHorizontalLineTexture;
		createBackground();
		this.setBackgroundEnabled(false);
		this.setOnAreaTouchTraversalFrontToBack();
		createWindowElements();
	}

	private void createBackground(){
		TextureRegion mBlack = mParent.mBlackTexture;
		mBlack.setTextureSize(Constants.CAMERA_WIDTH, Constants.CAMERA_HEIGHT);
		mBackground=new Sprite(0,0,mBlack,mMgr){
			@Override
			public boolean onAreaTouched(TouchEvent pSceneTouchEvent,
					float pTouchAreaLocalX, float pTouchAreaLocalY) {
				if(!isShowingSubMenu){
					mParentScene.removeSubMenu();
					return true;
				}
				return false;
			}
		};
		mBackground.setAlpha(0.3f);
		attachChild(mBackground);
		registerTouchArea(mBackground);
	}

	private void createWindowElements(){
		mWindow = new Sprite(0,0,
				CAMERA_WIDTH-Constants.TILE_SIZE,CAMERA_HEIGHT-Constants.TILE_SIZE*2,
				mLeather,mMgr){
			@Override
			public boolean onAreaTouched(TouchEvent pSceneTouchEvent,
					float pTouchAreaLocalX, float pTouchAreaLocalY) {
				if(!isShowingSubMenu){
					return true;					
				}
				return false;
			}
		};
		mWindow.setPosition(Constants.CAMERA_WIDTH/2 - mWindow.getWidth()/2, Constants.TILE_SIZE/2);
		attachChild(mWindow);
		registerTouchArea(mWindow);
		createInfo();
	}

	private void createInfo(){

		TextOptions header = new TextOptions(AutoWrap.WORDS, mWindow.getWidth()-60, HorizontalAlign.CENTER, Text.LEADING_DEFAULT);
		TextOptions rest = new TextOptions();
		TextOptions weaponName = new TextOptions(AutoWrap.WORDS, mWindow.getWidth()/3, HorizontalAlign.CENTER,Text.LEADING_DEFAULT);

		String rank = mRes.getString(R.string.buildinginfo_rank);
		String health = mRes.getString(R.string.buildinginfo_health);
		String value = mRes.getString(R.string.buildinginfo_value);
		String healthbonus = mRes.getString(R.string.buildinginfo_healthbonus);
		String damagebonus = mRes.getString(R.string.buildinginfo_damagebonus);
		String goldbonus = mRes.getString(R.string.buildinginfo_goldbonus);
		String goldpenalty = mRes.getString(R.string.buildinginfo_goldpenalty);

		String name = mBuilding.getName();
		String desc = mBuilding.getDescription();

		
		Text tName = new Text(0,0,mParent.mFontBold,name,header, mMgr);
		tName.setPosition(mWindow.getWidth()/2 - tName.getWidth()/2, 60);
		Text tDesc = new Text(0,0,mParent.mSmallFont,desc,header, mMgr);
		tDesc.setPosition(mWindow.getWidth()/2 - tDesc.getWidth()/2, 60+mParent.mFontBold.getLineHeight());

		mWindow.attachChild(tName);
		mWindow.attachChild(tDesc);
		Sprite line1 = new Sprite(30, tDesc.getY()+tDesc.getHeight()+mParent.mFontBold.getLineHeight(), 
				mWindow.getWidth()-60,2,mHorizontalLine,mMgr);
		line1.setAlpha(0.25f);
		mWindow.attachChild(line1);

		float x_padding=40;
		float y_padding = 6;
		float size = mParent.mSmallFontBold.getLineHeight();
		float yCoord = line1.getY()+2;
		int currentIndex=0;
		{	//Rank 1
			Building currentBuilding = mBuildingList.get(currentIndex);
			currentIndex++;
			float yCoord2 = yCoord;
			float xCoord2 = 3*mWindow.getWidth()/4;
			Text tRank = new Text(x_padding,yCoord+=y_padding,mParent.mSmallFontBold,rank+currentBuilding.getRank(),rest, mMgr);
			Text tValue = new Text(x_padding,yCoord+=size,mParent.mSmallFont,value+currentBuilding.getValue(),rest, mMgr);
			Text tHealth = new Text(x_padding,yCoord+=size,mParent.mSmallFont,health,rest, mMgr);
			Text tHealthBonus = new Text(x_padding,yCoord+=size,mParent.mSmallFont,healthbonus,rest, mMgr);
			Text tDamageBonus = new Text(x_padding,yCoord+=size,mParent.mSmallFont,damagebonus,rest, mMgr);
			Text tGoldBonus = new Text(x_padding,yCoord+=size,mParent.mSmallFont,goldbonus,rest, mMgr);
			Text tGoldPenalty = new Text(x_padding,yCoord+=size,mParent.mSmallFont,goldpenalty,rest, mMgr);

			if(hasWeapons){
				final Weapon w = mWeaponList.get(0);
				Text wName = new Text(0,0,mParent.mSmallFontBold,w.getmName(), weaponName,mMgr);
				TextureRegion texture = mParentScene.getTextureBasedonWeapon(w);
				Sprite button = new Sprite (0,0,Constants.TILE_SIZE, Constants.TILE_SIZE,mPaper,mMgr){
					@Override
					public boolean onAreaTouched(TouchEvent pSceneTouchEvent,
							float pTouchAreaLocalX, float pTouchAreaLocalY) {
						if(!isShowingSubMenu){
							switch(pSceneTouchEvent.getAction()){
							case TouchEvent.ACTION_DOWN:
								this.setScale(Constants.BUTTONPRESSED_SCALE);
								return true;
							case TouchEvent.ACTION_UP:
								this.setScale(1);
								createWeaponSubMenu(w);
								return true;
							}
						}
						return false;
					}
				};
				Sprite wIcon = new Sprite(0,0, Constants.TILE_SIZE*0.9f, Constants.TILE_SIZE*0.9f, texture, mMgr);
				button.setPosition(xCoord2-wIcon.getWidth()/2, yCoord2+wName.getHeight());
				wName.setPosition(xCoord2-wName.getWidth()/2, yCoord2+=y_padding);
				wIcon.setPosition(button.getWidth()/2-wIcon.getWidth()/2, button.getHeight()/2 - wIcon.getHeight()/2);
				mWindow.attachChild(button);
				mWindow.attachChild(wName);
				button.attachChild(wIcon);
				registerTouchArea(button);
			}

			mWindow.attachChild(tRank);
			mWindow.attachChild(tHealth);
			mWindow.attachChild(tValue);
			mWindow.attachChild(tHealthBonus);
			mWindow.attachChild(tDamageBonus);
			mWindow.attachChild(tGoldBonus);
			mWindow.attachChild(tGoldPenalty);
			
			createBars(tHealth, HEALTH_PR_BAR, currentBuilding.getHealth());
			createBars(tHealthBonus, HEALTH_PR_BAR, currentBuilding.getHealthBonus());
			createBars(tDamageBonus, DAMAGE_PR_BAR, currentBuilding.getDamageBonus());
			createBars(tGoldBonus, GOLD_BONUS_PR_BAR, currentBuilding.getGoldBonus());
			createBars(tGoldPenalty, GOLD_PENALTY_PR_BAR, currentBuilding.getGoldPenalty());
		}

		{	//Rank 2
			Sprite line = new Sprite(30, yCoord+=size+y_padding, mWindow.getWidth()-60,2,mHorizontalLine,mMgr);
			line.setAlpha(0.25f);
			Building currentBuilding = mBuildingList.get(currentIndex);
			//				yCoord +=2*size;
			Text tRank = new Text(x_padding,yCoord+=y_padding,mParent.mSmallFontBold,rank+currentBuilding.getRank(),rest, mMgr);
			Text tValue = new Text(x_padding,yCoord+=size,mParent.mSmallFont,value+(mBuildingList.get(currentIndex-1)).getUpgradeCost(),rest, mMgr);
			Text tHealth = new Text(x_padding,yCoord+=size,mParent.mSmallFont,health,rest, mMgr);
			Text tHealthBonus = new Text(x_padding,yCoord+=size,mParent.mSmallFont,healthbonus,rest, mMgr);
			Text tDamageBonus = new Text(x_padding,yCoord+=size,mParent.mSmallFont,damagebonus,rest, mMgr);
			Text tGoldBonus = new Text(x_padding,yCoord+=size,mParent.mSmallFont,goldbonus,rest, mMgr);
			Text tGoldPenalty = new Text(x_padding,yCoord+=size,mParent.mSmallFont,goldpenalty,rest, mMgr);

			mWindow.attachChild(line);
			mWindow.attachChild(tRank);
			mWindow.attachChild(tHealth);
			mWindow.attachChild(tValue);
			mWindow.attachChild(tHealthBonus);
			mWindow.attachChild(tDamageBonus);
			mWindow.attachChild(tGoldBonus);
			mWindow.attachChild(tGoldPenalty);
			
			createBars(tHealth, HEALTH_PR_BAR, currentBuilding.getHealth());
			createBars(tHealthBonus, HEALTH_PR_BAR, currentBuilding.getHealthBonus());
			createBars(tDamageBonus, DAMAGE_PR_BAR, currentBuilding.getDamageBonus());
			createBars(tGoldBonus, GOLD_BONUS_PR_BAR, currentBuilding.getGoldBonus());
			createBars(tGoldPenalty, GOLD_PENALTY_PR_BAR, currentBuilding.getGoldPenalty());
			
			currentIndex++;
		}

		{	//Rank 3
			Sprite line = new Sprite(30, yCoord+=size+y_padding, mWindow.getWidth()-60,2,mHorizontalLine,mMgr);
			line.setAlpha(0.25f);
			Building currentBuilding = mBuildingList.get(currentIndex);
			//				yCoord +=2*size;
			float yCoord2 = yCoord;
			float xCoord2 = 3*mWindow.getWidth()/4;
			Text tRank = new Text(x_padding,yCoord+=y_padding,mParent.mSmallFontBold,rank+currentBuilding.getRank(),rest, mMgr);
			Text tValue = new Text(x_padding,yCoord+=size,mParent.mSmallFont,value+(mBuildingList.get(currentIndex-1)).getUpgradeCost(),rest, mMgr);
			Text tHealth = new Text(x_padding,yCoord+=size,mParent.mSmallFont,health, rest,mMgr);
			Text tHealthBonus = new Text(x_padding,yCoord+=size,mParent.mSmallFont,healthbonus,rest, mMgr);
			Text tDamageBonus = new Text(x_padding,yCoord+=size,mParent.mSmallFont,damagebonus,rest, mMgr);
			Text tGoldBonus = new Text(x_padding,yCoord+=size,mParent.mSmallFont,goldbonus,rest, mMgr);
			Text tGoldPenalty = new Text(x_padding,yCoord+=size,mParent.mSmallFont,goldpenalty,rest, mMgr);

			if(hasWeapons){
				final Weapon w = mWeaponList.get(1);
				Text wName = new Text(0,0,mParent.mSmallFontBold,w.getmName(),weaponName,mMgr);
				TextureRegion texture = mParentScene.getTextureBasedonWeapon(w);
				
				Sprite button = new Sprite (0,0,Constants.TILE_SIZE, Constants.TILE_SIZE,mPaper,mMgr){
					@Override
					public boolean onAreaTouched(TouchEvent pSceneTouchEvent,
							float pTouchAreaLocalX, float pTouchAreaLocalY) {
						if(!isShowingSubMenu){
							switch(pSceneTouchEvent.getAction()){
							case TouchEvent.ACTION_DOWN:
								this.setScale(Constants.BUTTONPRESSED_SCALE);
								return true;
							case TouchEvent.ACTION_UP:
								this.setScale(1);
								createWeaponSubMenu(w);
								return true;
							}
						}
						return false;
					}
				};
				Sprite wIcon = new Sprite(0,0, Constants.TILE_SIZE*0.9f, Constants.TILE_SIZE*0.9f, texture, mMgr);
				button.setPosition(xCoord2-wIcon.getWidth()/2, yCoord2+wName.getHeight());
				wName.setPosition(xCoord2-wName.getWidth()/2, yCoord2+=y_padding);
				wIcon.setPosition(button.getWidth()/2-wIcon.getWidth()/2, button.getHeight()/2 - wIcon.getHeight()/2);
				mWindow.attachChild(button);
				mWindow.attachChild(wName);
				button.attachChild(wIcon);
				registerTouchArea(button);
				
				currentIndex++;
			}

			mWindow.attachChild(line);
			mWindow.attachChild(tRank);
			mWindow.attachChild(tHealth);
			mWindow.attachChild(tValue);
			mWindow.attachChild(tHealthBonus);
			mWindow.attachChild(tDamageBonus);
			mWindow.attachChild(tGoldBonus);
			mWindow.attachChild(tGoldPenalty);
			
			createBars(tHealth, HEALTH_PR_BAR, currentBuilding.getHealth());
			createBars(tHealthBonus, HEALTH_PR_BAR, currentBuilding.getHealthBonus());
			createBars(tDamageBonus, DAMAGE_PR_BAR, currentBuilding.getDamageBonus());
			createBars(tGoldBonus, GOLD_BONUS_PR_BAR, currentBuilding.getGoldBonus());
			createBars(tGoldPenalty, GOLD_PENALTY_PR_BAR, currentBuilding.getGoldPenalty());
		}

	}

	private void createWeaponSubMenu(Weapon w){
		isShowingSubMenu=true;
		WeaponSubMenu subMenu = new WeaponSubMenu(mParent, w);
		setChildScene(subMenu);
	}
	
	public void removeSubMenu(){
		isShowingSubMenu=false;
		this.clearChildScene();
	}
	
	private void createBars(Text bonusText, float stat_pr_bar, float bonus){
		bonus=Math.abs(bonus);
		int nr_bars = (int)(bonus / stat_pr_bar);
		int bars_pr_line = 5;
		float partial_bar = ((float)bonus%stat_pr_bar)/stat_pr_bar;
		float x = bonusText.getWidth()+10;
		float y = 0;
		boolean line2=false;
		for(int i=0;i<nr_bars;i++){
			Sprite temp = new Sprite(0,0,mParent.mSmallFontBold.getLineHeight()-2, mParent.mSmallFontBold.getLineHeight()/2-2,mBar,mMgr);
			temp.setPosition(x, y);
			bonusText.attachChild(temp);
			x+=temp.getWidth()+2;
			if(!line2 && i>bars_pr_line){
				y+=temp.getHeight()+2;
				line2=true;
			}
		}
		if(partial_bar>0.01){
			float width = mBar.getWidth();
			float newWidth = width*partial_bar;
			mBar.setTextureWidth(newWidth);
			Sprite temp = new Sprite(x,y,newWidth-2, mParent.mSmallFontBold.getLineHeight()/2-2,mBar,mMgr);
			bonusText.attachChild(temp);
			mBar.setTextureWidth(width);			
		}
	}

	@Override
	public boolean onBackButtonPressed() {
		if(isShowingSubMenu){
			removeSubMenu();
			return true;
		}
		return false;
	}



}
