package adrian.master.knowledgewar.subscenes;

import org.andengine.entity.scene.Scene;
import org.andengine.entity.sprite.Sprite;
import org.andengine.entity.text.AutoWrap;
import org.andengine.entity.text.Text;
import org.andengine.entity.text.TextOptions;
import org.andengine.input.touch.TouchEvent;
import org.andengine.opengl.texture.region.TextureRegion;
import org.andengine.opengl.vbo.VertexBufferObjectManager;
import org.andengine.util.HorizontalAlign;

import adrian.master.knowledgewar.R;
import adrian.master.knowledgewar.gameactivities.MainActivity;
import adrian.master.knowledgewar.gameconcepts.Building;
import adrian.master.knowledgewar.gameconcepts.BuildingFactory;
import adrian.master.knowledgewar.interfaces.ConfirmationDialogInterface;
import adrian.master.knowledgewar.interfaces.SubMenuInterface;
import adrian.master.knowledgewar.logic.Constants;
import adrian.master.knowledgewar.scenes.OwnTribeScene;
import android.content.res.Resources;

public class SpecificBuildingSubMenu extends Scene implements ConfirmationDialogInterface, SubMenuInterface {

	private Building mBuilding, mUpgradedBuilding;
	private Text mName, mRank, mHealth, mValue, mHealthBonus, mDamageBonus, mGoldBonus, mGoldPenalty;
	float coordX, coordY;
	int price=0;
	private boolean maxRank = false;
	private Sprite mWindow,upgradeButton, removeButton, mBackground;
	private Resources mRes;
	private MainActivity mParent;
	private VertexBufferObjectManager mMgr;
	private TextureRegion mRedTile, mCart, mBar, mPaper, mLeather;

	private boolean isShowingSubMenu=false;
	
	float DAMAGE_PR_BAR = 1f;
	float HEALTH_PR_BAR = 1F;
	float GOLD_BONUS_PR_BAR = 0.25f;
	float GOLD_PENALTY_PR_BAR = 0.25f;
	private int CAMERA_WIDTH=Constants.CAMERA_WIDTH;

	public SpecificBuildingSubMenu(float px, float py,MainActivity parent,Building building){
		mParent = parent;
		mRes = mParent.mRes;
		mMgr = mParent.getVertexBufferObjectManager();
		mBuilding = building;
		mPaper = mParent.mPaperTexture;
		mLeather = mParent.mLeatherTexture;
		mRedTile = mParent.mRedTileTexture;
		mCart = mParent.mCartTexture;
		mBar = mParent.mBlueBarTexture;
		coordX=px;
		coordY=py;
		setOnAreaTouchTraversalFrontToBack();
		setBackgroundEnabled(false);
		createBackground();

		mUpgradedBuilding = BuildingFactory.getUpgradedBuilding(mBuilding);
		if(mUpgradedBuilding!=null){
			price = mBuilding.getUpgradeCost();
		}else{
			maxRank=true;
		}
		mWindow = new Sprite(Constants.TILE_SIZE, Constants.TILE_SIZE,
				CAMERA_WIDTH-2*Constants.TILE_SIZE, 5*Constants.TILE_SIZE, mLeather, mMgr){
			@Override
			public boolean onAreaTouched(TouchEvent pSceneTouchEvent,
					float pTouchAreaLocalX, float pTouchAreaLocalY) {
				if(!isShowingSubMenu){
					return true;					
				}
				return false;
			}
		};
		attachChild(mWindow);
		registerTouchArea(mWindow);

		createBuildingInfo();
		createButtons();
		createHighlightedBuilding();
	}

	private void createBackground(){
		TextureRegion mBlack = mParent.mBlackTexture;
		mBlack.setTextureSize(Constants.CAMERA_WIDTH, Constants.CAMERA_HEIGHT);
		mBackground=new Sprite(0,0,mBlack,mMgr){
			@Override
			public boolean onAreaTouched(TouchEvent pSceneTouchEvent,
					float pTouchAreaLocalX, float pTouchAreaLocalY) {
				if(!isShowingSubMenu){
					((OwnTribeScene)mParentScene).removeSubMenu();
					return true;					
				}
				return false;
			}
		};
		mBackground.setAlpha(0.3f);
		attachChild(mBackground);
		registerTouchArea(mBackground);
	}
//	
//	public int getBuildingX(){
//		return mBuilding.getX();
//	}
//	
//	public int getBuildingY(){
//		return mBuilding.getY();
//	}


	private void createBuildingInfo(){
		String name = mBuilding.getName();
		TextOptions header = new TextOptions(AutoWrap.WORDS, mWindow.getWidth(), HorizontalAlign.CENTER, Text.LEADING_DEFAULT);
		String rank = mRes.getString(R.string.buildinginfo_rank);
		String health = mRes.getString(R.string.buildinginfo_health);
		String value = mRes.getString(R.string.buildinginfo_value);
		String healthbonus = mRes.getString(R.string.buildinginfo_healthbonus);
		String damagebonus = mRes.getString(R.string.buildinginfo_damagebonus);
		String goldbonus = mRes.getString(R.string.buildinginfo_goldbonus);
		String goldpenalty = mRes.getString(R.string.buildinginfo_goldpenalty);
		float size = mParent.mSmallFontBold.getLineHeight();
		float x_padding = 40;
		float yCoord = 30;
		mName = new Text(0,yCoord,mParent.mFontBold,name,header, mMgr);
		mRank = new Text(x_padding,yCoord+=2*size,mParent.mSmallFontBold,rank+mBuilding.getRank(), mMgr);
		mValue = new Text(x_padding,yCoord+=size,mParent.mSmallFont,value+mBuilding.getValue(), mMgr);
		mHealth = new Text(x_padding,yCoord+=size,mParent.mSmallFont,health, mMgr);
		mHealthBonus = new Text(x_padding,yCoord+=size,mParent.mSmallFont,healthbonus, mMgr);
		mDamageBonus = new Text(x_padding,yCoord+=size,mParent.mSmallFont,damagebonus, mMgr);
		mGoldBonus = new Text(x_padding,yCoord+=size,mParent.mSmallFont,goldbonus, mMgr);
		mGoldPenalty = new Text(x_padding,yCoord+=size,mParent.mSmallFont,goldpenalty, mMgr);

		mWindow.attachChild(mName);
		mWindow.attachChild(mRank);
		mWindow.attachChild(mHealth);
		mWindow.attachChild(mValue);
		mWindow.attachChild(mHealthBonus);
		mWindow.attachChild(mDamageBonus);
		mWindow.attachChild(mGoldBonus);
		mWindow.attachChild(mGoldPenalty);
		
		createBars(mHealth, HEALTH_PR_BAR, mBuilding.getHealth());
		createBars(mHealthBonus, HEALTH_PR_BAR, mBuilding.getHealthBonus());
		createBars(mDamageBonus, DAMAGE_PR_BAR, mBuilding.getDamageBonus());
		createBars(mGoldBonus, GOLD_BONUS_PR_BAR, mBuilding.getGoldBonus());
		createBars(mGoldPenalty, GOLD_PENALTY_PR_BAR, mBuilding.getGoldPenalty());
	}

	private void createButtons(){
		removeButton = new Sprite(0,0,
				Constants.TILE_SIZE*2, Constants.TILE_SIZE, mPaper, mMgr){
			@Override
			public boolean onAreaTouched(
					TouchEvent pSceneTouchEvent,
					float pTouchAreaLocalX, float pTouchAreaLocalY) {
				if(!isShowingSubMenu){
					switch(pSceneTouchEvent.getAction()){
					case TouchEvent.ACTION_DOWN:
						this.setScale(Constants.BUTTONPRESSED_SCALE);
						return true;
					case TouchEvent.ACTION_UP:
						this.setScale(1);
						createConfirmationDialog();
						return true;
					}
				}
				return false;
			}
		};
		removeButton.setPosition(mWindow.getWidth()/2-removeButton.getWidth()-10, mWindow.getHeight()-removeButton.getHeight()-30);

		upgradeButton = new Sprite(0,0,
				Constants.TILE_SIZE*2, Constants.TILE_SIZE, mPaper, mMgr){
			@Override
			public boolean onAreaTouched(
					TouchEvent pSceneTouchEvent,
					float pTouchAreaLocalX, float pTouchAreaLocalY) {
				if(!isShowingSubMenu){
					switch(pSceneTouchEvent.getAction()){
					case TouchEvent.ACTION_DOWN:
						this.setScale(Constants.BUTTONPRESSED_SCALE);
						return true;
					case TouchEvent.ACTION_UP:
						this.setScale(1);
						((OwnTribeScene)mParentScene).upgradeBuilding(mBuilding.getX()*Constants.TILE_SIZE, mBuilding.getY()*Constants.TILE_SIZE, mBuilding);
						return true;
					}
				}
				return false;
			}
		};
		upgradeButton.setPosition(mWindow.getWidth()/2+10, mWindow.getHeight()-upgradeButton.getHeight()-30);

		TextOptions button = new TextOptions(AutoWrap.WORDS, removeButton.getWidth(), HorizontalAlign.CENTER, Text.LEADING_DEFAULT);
		Text removeText = new Text(0,0, mParent.mSmallFontBold, mRes.getString(R.string.owntribe_building_submenu_remove), button, mMgr);
		Text upgradeText = new Text(0,0, mParent.mSmallFontBold, mRes.getString(R.string.owntribe_building_submenu_upgrade), button, mMgr);
		Text tPrice = new Text(0,mParent.mSmallFontBold.getLineHeight(), mParent.mSmallFontBold, 
				mRes.getString(R.string.owntribe_building_submenu_price)+price, button, mMgr);

		removeText.setPosition(0, removeButton.getHeight()/2-removeText.getHeight()/2);
		upgradeText.setPosition(0, upgradeButton.getHeight()/2-upgradeText.getHeight());
		tPrice.setPosition(0, upgradeText.getY()+mParent.mSmallFontBold.getLineHeight());
		removeButton.attachChild(removeText);
		upgradeButton.attachChild(upgradeText);
		upgradeButton.attachChild(tPrice);

		if(!maxRank){
			mWindow.attachChild(upgradeButton);
			registerTouchArea(upgradeButton);			
		}

		mWindow.attachChild(removeButton);
		if(!mBuilding.getType().equals(Constants.BUILDING_TYPES[0])){
			registerTouchArea(removeButton);				
		}else{
			removeButton.setAlpha(0.25f);
			removeButton.setColor(0.9f, 0.9f, 0.9f);
		}
	}

	private void createHighlightedBuilding(){
		Sprite tile = new Sprite(0,0,mRedTile,mMgr);
		Sprite cart = new Sprite(0,0,Constants.TILE_SIZE, Constants.TILE_SIZE,mCart,mMgr);
		tile.attachChild(cart);
		float x = mBuilding.getX()*Constants.TILE_SIZE*0.85f;
		float y = mBuilding.getY()*Constants.TILE_SIZE*0.85f;
		tile.setPosition(coordX+x, coordY+y);
		tile.setScale(1.1f);
		attachChild(tile);
	}
	
	private void createBars(Text bonusText, float stat_pr_bar, float bonus){
		bonus=Math.abs(bonus);
		int nr_bars = (int)(bonus / stat_pr_bar);
		float partial_bar = ((float)bonus%stat_pr_bar)/stat_pr_bar;
		float x = bonusText.getWidth()+5;
		float y = 0;
		boolean line2=false;
		for(int i=0;i<nr_bars;i++){
			Sprite temp = new Sprite(0,0,mParent.mSmallFontBold.getLineHeight()-2, mParent.mSmallFontBold.getLineHeight()/2-2,mBar,mMgr);
			temp.setPosition(x, y);
			bonusText.attachChild(temp);
			x+=temp.getWidth()+2;
			if(!line2 && i>5){
				y+=temp.getHeight()+2;
				line2=true;
			}
		}
		if(partial_bar>0.01){
			float width = mBar.getWidth();
			float newWidth = width*partial_bar;
			mBar.setTextureWidth(newWidth);
			Sprite temp = new Sprite(x,y,newWidth-2, mParent.mSmallFontBold.getLineHeight()/2-2,mBar,mMgr);
			bonusText.attachChild(temp);
			mBar.setTextureWidth(width);			
		}
	}

	private void createConfirmationDialog(){
		isShowingSubMenu=true;
		ConfirmationDialog dialog = new ConfirmationDialog(mParent, this);
		setChildScene(dialog);
	}

	public void confirmationYes(){
		isShowingSubMenu=false;
		((OwnTribeScene)mParentScene).removeBuilding(mBuilding.getX(), mBuilding.getY());
		clearChildScene();
	}

	public void confirmationNo(){
		isShowingSubMenu=false;
		clearChildScene();
		((OwnTribeScene)mParentScene).removeSubMenu();
	}

	@Override
	public void cancel() {
		isShowingSubMenu=false;
		clearChildScene();
		((OwnTribeScene)mParentScene).removeSubMenu();
	}

	@Override
	public boolean onBackButtonPressed() {
		if(isShowingSubMenu){
			cancel();
			return true;
		}
		return false;
	}

}
